function [uci,symbols,detMet] = decodeFormat2(carrier,pucch,ouci,sym,nVar,thres)
%decodeFormat2 Physical uplink control channel format 2 decoding
%
%   Note: This is an internal undocumented function and its API and/or
%   functionality may change in subsequent releases.
%
%   [UCI,SYMBOLS,DETMET] = decodeFormat2(CARRIER,PUCCH,OUCI,SYM,NVAR,THRES)
%   returns the cell containing the vector of soft bits, UCI, resulting
%   from the inverse of physical uplink control channel processing for
%   format 2, as defined in TS 38.211 6.3.2.5. The function also returns
%   the received constellation symbols, SYMBOLS. CARRIER is a scalar
%   nrCarrierConfig object. PUCCH is a scalar nrPUCCH2Config object. SYM is
%   the received equalized symbols. OUCI is the number of UCI payload bits.
%   NVAR is noise variance. THRES is the threshold value in range 0 to 1.
%   When OUCI is in range 3 to 11, the function performs discontinuous
%   transmission (DTX) detection using normalized correlation coefficient
%   metric for all the possible symbol sequences. The maximum of normalized
%   correlation coefficients is returned as the detection metric, DETMET.
%   When DETMET is greater than or equal to the THRES, the function
%   performs symbol demodulation and descrambling. When DETMET is less than
%   THRES, the function treats the input SYM as DTX and returns empty soft
%   bits. For all other values of OUCI, the function directly performs
%   symbol demodulation and descrambling, without DTX detection.

%   Copyright 2021 The MathWorks, Inc.

%#codegen

    % Cache the scrambling identity, radio network temporary identifier
    % (RNTI), and number of input symbols
    if isempty(pucch.NID)
        nid = double(carrier.NCellID);
    else
        nid = double(pucch.NID(1));
    end
    rnti = pucch.RNTI;
    Gd = size(sym,1);
    numCols = 2^double(ouci);

    % Persistent variables to store format 2 reference symbols
    persistent format2Sym format2NID format2RNTI format2Gd format2NumUCI ...
        format2Check

    % Check for non-empty symbols, when ouci is 0
    symPresence = ~isempty(sym);
    coder.internal.errorIf((ouci == 0) && symPresence, ...
        'nr5g:nrPUCCHDecode:InvalidSYMWithoutUCI');

    % Perform threshold check for PUCCH format 2 symbols, when ouci is in
    % range 3 to 11
    dtType = class(sym);
    if (ouci(1) >= 3 && ouci(1) <= 11) && symPresence
        % Check ouci and Gd values (qm*Gd must be greater than ouci)
        qm = 2;
        coder.internal.errorIf(Gd*qm <= ouci(1), ...
            'nr5g:nrPUCCHDecode:InvalidSYMLenF23',qm,Gd,ouci(1));

        % Initialize number of UCI bits for format 2
        if isempty(format2NumUCI)
            format2NumUCI = ouci(1);
        end

        % Initialize number of symbols
        if isempty(format2Gd)
            format2Gd = Gd;
        end

        % Initialize RNTI
        if isempty(format2RNTI)
            format2RNTI = rnti;
        end

        % Initialize NID
        if isempty(format2NID)
            format2NID = nid;
        end

        % Initialize this variable to check if format 2 symbols are
        % generated correctly
        if isempty(format2Check)
            format2Check = 0;
        end

        % Initialize format 2 reference symbols
        if isempty(format2Sym) || ~format2Check || ...
                (format2NumUCI ~= ouci(1)) || (format2Gd ~= Gd) || ...
                (format2NID ~= nid) || (format2RNTI ~= rnti)
            format2Check = 0; %#ok<NASGU>
            format2Sym = complex(zeros(Gd,numCols));
            for ul = 0:numCols-1
                enc = nrUCIEncode(int2bit(ul,ouci(1),false),Gd*qm);
                format2Sym(:,ul+1) = nrPUCCH(carrier,pucch,enc);
            end
            % Assign the values back, to ensure the computation of all the
            % possible symbols is performed only once and for a change in
            % one of the values from previous call
            format2NumUCI = ouci(1);
            format2Gd = Gd;
            format2NID = nid;
            format2RNTI = rnti;
            format2Check = 1;
        end

        % Get the energy of received symbols and reference symbols
        format2SymType = cast(format2Sym,dtType);
        eSym = sum(abs(sym).^2);
        eformat2Sym = sum(abs(format2SymType).^2);
        % Get the normalized correlation coefficient for all the possible
        % reference symbols. Add eps to the normalization to avoid dividing
        % by 0.
        ncc = zeros(1,numCols,dtType);
        for ul = 1:numCols
            normE = sqrt(eSym*eformat2Sym(ul));
            ncc(ul) = abs(sum(sym.*conj(format2SymType(:,ul))))/(normE+eps);
        end
        % Compare the maximum normalized correlation coefficient against a
        % threshold
        detMet = max(ncc);
        thresholdFlag = detMet(1) >= thres;
    else
        detMet = zeros(1,1,dtType);
        thresholdFlag = true;
    end

    % Perform PUCCH format 2 decoding
    symbols = sym;
    if thresholdFlag
        % Perform symbol demodulation
        symdemod = nrSymbolDemodulate(symbols,'QPSK',nVar);

        % Perform descrambling
        opts.MappingType = 'signed';
        opts.OutputDataType = dtType;
        c = nrPUCCHPRBS(nid,rnti,length(symdemod),opts);
        uci = {symdemod .* c};
    else
        % When detection metric is less than threshold, return empty UCI as
        % output
        uci = {zeros(0,1,dtType)};
    end

end
