function [uci,symbols,detMet] = decodeFormat3(carrier,pucch,ouci,sym,nVar,thres)
%decodeFormat3 Physical uplink control channel format 3 decoding
%
%   Note: This is an internal undocumented function and its API and/or
%   functionality may change in subsequent releases.
%
%   [UCI,SYMBOLS,DETMET] = decodeFormat3(CARRIER,PUCCH,OUCI,SYM,NVAR,THRES)
%   returns the cell containing the vector of soft bits, UCI, resulting
%   from the inverse of physical uplink control channel processing for
%   format 3, as defined in TS 38.211 6.3.2.6. The function also returns
%   the received constellation symbols, SYMBOLS and detection metric
%   DETMET. CARRIER is a scalar nrCarrierConfig object. PUCCH is a scalar
%   nrPUCCH3Config object. OUCI is the number of UCI payload bits. It is
%   either a scalar or two-element vector. SYM is the received equalized
%   symbols. NVAR is noise variance. THRES is the detection threshold in
%   range 0 to 1. When first element of OUCI is in range 3 to 11, the
%   function performs discontinuous transmission (DTX) detection using
%   normalized correlation coefficient metric for all the possible symbol
%   sequences. The maximum of normalized correlation coefficients is
%   returned as the detection metric, DETMET. When DETMET is greater than
%   or equal to the THRES, the function performs transform deprecoding,
%   symbol demodulation, and descrambling. When DETMET is less than THRES,
%   the function treats the input SYM as DTX and returns empty soft bits.
%   For all other values of OUCI, the function directly performs transform
%   deprecoding, symbol demodulation, and descrambling, without DTX
%   detection.

%   Copyright 2021 The MathWorks, Inc.

%#codegen

    % Cache the scrambling identity and radio network temporary identifier
    % (RNTI)
    if isempty(pucch.NID)
        nid = double(carrier.NCellID);
    else
        nid = double(pucch.NID(1));
    end
    rnti = pucch.RNTI;
    Gd = size(sym,1);
    numCols = 2^double(ouci(1));

    % Persistent variables to store format 3 reference symbols
    persistent format3Sym format3NID format3RNTI format3Gd format3NumUCI ...
        format3NumPRB format3ModScheme format3Check

    % Check for non-empty symbols, when ouci is 0
    symPresence = ~isempty(sym);
    coder.internal.errorIf(all(ouci == 0) && symPresence, ...
        'nr5g:nrPUCCHDecode:InvalidSYMWithoutUCI');

    % Get the data type of input symbols, number of unique physical
    % resource blocks (PRBs), and modulation order
    dtType = class(sym);
    mRB = numel(unique(pucch.PRBSet(:)));
    qm = nr5g.internal.getQm(pucch.Modulation);
    % Perform threshold check for PUCCH format 3 symbols, when ouci is in
    % range 3 to 11
    if (isscalar(ouci) || (ouci(2) == 0)) && (ouci(1) >= 3 && ouci(1) <= 11) && symPresence
        % Check ouci and Gd values (qm*Gd must be greater than first element of ouci)
        coder.internal.errorIf(Gd*qm <= ouci(1), ...
            'nr5g:nrPUCCHDecode:InvalidSYMLenF23',qm,Gd,ouci(1));

        % Initialize number of UCI bits for format 3
        if isempty(format3NumUCI)
            format3NumUCI = ouci(1);
        end

        % Initialize number of symbols
        if isempty(format3Gd)
            format3Gd = Gd;
        end

        % Initialize RNTI
        if isempty(format3RNTI)
            format3RNTI = rnti;
        end

        % Initialize NID
        if isempty(format3NID)
            format3NID = nid;
        end

        % Initialize number of PRBs
        if isempty(format3NumPRB)
            format3NumPRB = mRB;
        end

        % Initialize modulation scheme
        if isempty(format3ModScheme)
            format3ModScheme = pucch.Modulation;
        end

        % Initialize this variable to check if format 3 symbols are
        % generated correctly
        if isempty(format3Check)
            format3Check = 0;
        end

        % Initialize format 3 reference symbols
        if isempty(format3Sym) || ~format3Check || ...
                (format3NumUCI ~= ouci(1)) || (format3Gd ~= Gd) || ...
                (format3NID ~= nid) || (format3RNTI ~= rnti) ||  ...
                (format3NumPRB ~= mRB) || ~strcmpi(pucch.Modulation,format3ModScheme)
            format3Check = 0; %#ok<NASGU>
            format3Sym = complex(zeros(Gd,numCols));
            for ul = 0:numCols-1
                enc = nrUCIEncode(int2bit(ul,ouci(1),false),Gd*qm);
                format3Sym(:,ul+1) = nrPUCCH(carrier,pucch,enc);
            end
            % Assign the values back, to ensure the computation of all the
            % possible symbols is performed only once and for a change in
            % one of the values from previous call
            format3NumUCI = ouci(1);
            format3Gd = Gd;
            format3NID = nid;
            format3RNTI = rnti;
            format3NumPRB = mRB;
            format3ModScheme = pucch.Modulation;
            format3Check = 1;
        end

        % Get the energy of received symbols and reference symbols
        format3SymType = cast(format3Sym,dtType);
        eSym = sum(abs(sym).^2);
        eformat3Sym = sum(abs(format3SymType).^2);
        % Get the normalized correlation coefficient for all the possible
        % reference symbols. Add eps to the normalization to avoid dividing
        % by 0.
        ncc = zeros(1,numCols,dtType);
        for ul = 1:numCols
            normE = sqrt(eSym*eformat3Sym(ul));
            ncc(ul) = abs(sum(sym.*conj(format3SymType(:,ul))))/(normE+eps);
        end
        % Compare the maximum normalized correlation coefficient against a
        % threshold
        detMet = max(ncc);
        thresholdFlag = detMet(1) >= thres;
    else
        detMet = zeros(1,1,dtType);
        thresholdFlag = true;
    end

    % Perform PUCCH format 3 decoding
    if thresholdFlag
        % Perform transform deprecoding
        symbols = nrTransformDeprecode(sym,mRB);

        % Perform symbol demodulation
        symdemod = nrSymbolDemodulate(symbols(:),pucch.Modulation,nVar);

        % Perform descrambling
        opts.MappingType = 'signed';
        opts.OutputDataType = dtType;
        c = nrPUCCHPRBS(nid,rnti,length(symdemod),opts);
        uci = {symdemod .* c};
    else
        % When the detection metric is less than threshold, return empty
        % UCI and symbols as output
        uci = {zeros(0,1,dtType)};
        symbols = zeros(0,1,dtType);
    end

end
