classdef ConfigBase < nr5g.internal.pxsch.CommonConfig & nr5g.internal.FrequencyHoppingConfig
    %ConfigBase Class offering properties common between nrWavegenPUSCHConfig and nrPUSCHConfig
    %
    %   Note: This is an internal undocumented class and its API and/or
    %   functionality may change in subsequent releases.

    %   Copyright 2020-2021 The MathWorks, Inc.

    %#codegen

    % Public, tunable properties
    properties
        %Modulation Modulation scheme
        %   Specify the modulation scheme for the codeword as a character
        %   array or string scalar. The value must be one of
        %   {'pi/2-BPSK', 'QPSK', '16QAM', '64QAM', '256QAM'}. The default
        %   value is 'QPSK'.
        Modulation = 'QPSK';

        %NumLayers Number of transmission layers
        %   Specify the number of transmission layers as a scalar positive
        %   integer. The value must be in range 1...4. The default value is 1.
        NumLayers = 1;

        %TransformPrecoding Flag to enable transform precoding
        %   Specify the flag to enable or disable transform precoding as a
        %   numeric or logical scalar. The value must be binary.
        %   0 indicates that transform precoding is disabled and the
        %   waveform type is CP-OFDM. 1 indicates that transform precoding
        %   is enabled and the waveform type is DFT-s-OFDM. The default
        %   value is 0.
        TransformPrecoding = 0;

        %TransmissionScheme PUSCH transmission scheme
        %   Specify the PUSCH transmission scheme as a character vector or
        %   string scalar. The value must be one of {'nonCodebook', 'codebook'}.
        %   The default value is 'nonCodebook'.
        TransmissionScheme = 'nonCodebook';

        %NumAntennaPorts Number of antenna ports
        %   Specify the number of antenna ports for codebook based
        %   transmission scheme as a scalar positive integer. The value
        %   must be one of {1, 2, 4}. The value must be greater than or
        %   equal to the number of transmission layers. The default value
        %   is 1.
        NumAntennaPorts = 1;

        %TPMI Transmitted precoding matrix indicator
        %   Specify the transmitted precoding matrix indicator as a scalar
        %   nonnegative integer in the range 0...27. The default value is 0.
        TPMI = 0;

        %BetaOffsetACK Beta offset for HARQ-ACK
        %   Specify the beta offset for HARQ-ACK as a real scalar
        %   positive value. The default value is 20.
        BetaOffsetACK = 20;

        %BetaOffsetCSI1 Beta offset for CSI part 1
        %   Specify the beta offset for CSI part 1 as a real scalar
        %   positive value. The default value is 6.25.
        BetaOffsetCSI1 = 6.25;

        %BetaOffsetCSI2 Beta offset for CSI part 2
        %   Specify the beta offset for CSI part 2 as a real scalar
        %   positive value. The default value is 6.25.
        BetaOffsetCSI2 = 6.25;

        %UCIScaling Scaling factor for UCI-on-PUSCH resource elements
        %   Specify the scaling factor to limit the number of resource
        %   elements allocated for UCI on PUSCH as a real scalar positive
        %   value less than or equal to 1. The nominal value is one of
        %   {0.5, 0.65, 0.8, 1}. The default value is 1.
        UCIScaling = 1;

        %NRAPID Random access preamble index to initialize the scrambling sequence for msgA on PUSCH
        % Specify this index as a scalar nonnegative integer. The value
        % must be in the range 0...63. It is used to initialize the
        % scrambling sequence for msgA on PUSCH, as discussed in TS 38.211
        % Section 6.3.1.1. Use empty ([]) to disable this property. The
        % default value is [].
        NRAPID = [];

        %DMRS PUSCH-specific demodulation reference signal (DM-RS) configuration object
        %   Specify the DM-RS configuration object associated with PUSCH.
        %   The default value is a default <a href="matlab:help('nrPUSCHDMRSConfig')"
        %   >nrPUSCHDMRSConfig</a> object.
        DMRS = nrPUSCHDMRSConfig;

        %PTRS PUSCH-specific phase tracking reference signal (PT-RS)
        %configuration object
        %   Specify the PT-RS configuration object associated with PUSCH.
        %   The default value is a default <a href="matlab:help('nrPUSCHPTRSConfig')"
        %   >nrPUSCHPTRSConfig</a> object.
        PTRS = nrPUSCHPTRSConfig;
    end

    % Hidden properties
    properties (Hidden)
        %Mode Transmission mode indicates the control over visibility of
        %the properties based on the waveform type
        % Specify the transmission mode as one of (0, 1, 2). 0 displays the
        % properties specific to CP-OFDM. 1 displays the properties
        % specific to DFT-s-OFDM. 2 displays the list of properties which
        % is the union of both CP-OFDM and DFT-s-OFDM specific properties.
        Mode = 0;

        %NumPorts Number of ports
        %   Number of ports. This is a hidden property and is equal to the
        %   number of layers. This is used in updating the hidden property
        %   NLayers of DMRS property.
        NumPorts = 1;
    end

    % Constant, hidden properties
    properties (Constant,Hidden)
        Modulation_Values = {'QPSK','pi/2-BPSK','16QAM','64QAM','256QAM'};
        TransmissionScheme_Values = {'nonCodebook','codebook'};
    end

    methods
        % Constructor
        function obj = ConfigBase(varargin)
            % Get the value of DMRS from the name-value pairs
            dmrs = nr5g.internal.parseProp('DMRS',nrPUSCHDMRSConfig,varargin{:});
            % Get the value of PTRS from the name-value pairs
            ptrs = nr5g.internal.parseProp('PTRS',nrPUSCHPTRSConfig,varargin{:});

            % Support name-value pair arguments when constructing object
            obj@nr5g.internal.pxsch.CommonConfig(...
                'DMRS', dmrs,...
                'PTRS', ptrs,...
                'Modulation', 'QPSK', ...
                'TransmissionScheme', 'nonCodebook', ...
                'FrequencyHopping', 'neither', ...
                varargin{:});
        end

        % Self-validate and set properties
        function obj = set.Modulation(obj,val)
            prop = 'Modulation';
            val = validatestring(val,obj.Modulation_Values,[class(obj) '.' prop],prop);
            obj.(prop) = '';
            obj.(prop) = val;
        end

        function obj = set.NumLayers(obj,val)
            prop = 'NumLayers';
            validateattributes(val,{'numeric'},...
                {'scalar','integer','positive','<=',4},...
                [class(obj) '.' prop],prop);
            obj.(prop) = val;
            obj = updateHiddenProps(obj);
        end

        function obj = set.TransformPrecoding(obj,val)
            prop = 'TransformPrecoding';
            validateattributes(val, {'numeric','logical'},...
                {'scalar','binary'},[class(obj) '.' prop],prop);
            obj.(prop) = val;
            obj = setMode(obj);
        end

        function obj = set.TransmissionScheme(obj,val)
            prop = 'TransmissionScheme';
            val = validatestring(val,obj.TransmissionScheme_Values,[class(obj) '.' prop],prop);
            obj.(prop) = '';
            obj.(prop) = val;
        end

        function obj = set.TPMI(obj,val)
            prop = 'TPMI';
            validateattributes(val,{'numeric'},...
                {'scalar','integer','nonnegative','<=',27},...
                [class(obj) '.' prop],prop);
            obj.(prop) = val;
        end

        function obj = set.NumAntennaPorts(obj,val)
            prop = 'NumAntennaPorts';
            validateattributes(val,{'numeric'},...
                {'real','scalar'},...
                [class(obj) '.' prop],prop);
            coder.internal.errorIf((~any(val == [1,2,4])),...
                'nr5g:nrPUSCHConfig:InvalidNumAntennaPorts',sprintf('%g',double(val)));
            obj.(prop) = val;
        end

        function obj = set.BetaOffsetACK(obj,val)
            prop = 'BetaOffsetACK';
            validateattributes(val,{'numeric'},...
                {'real','scalar','positive','finite'},...
                [class(obj) '.' prop],prop);
            obj.(prop) = val;
        end

        function obj = set.BetaOffsetCSI1(obj,val)
            prop = 'BetaOffsetCSI1';
            validateattributes(val,{'numeric'},...
                {'real','scalar','positive','finite'},...
                [class(obj) '.' prop],prop);
            obj.(prop) = val;
        end

        function obj = set.BetaOffsetCSI2(obj,val)
            prop = 'BetaOffsetCSI2';
            validateattributes(val,{'numeric'},...
                {'real','scalar','positive','finite'},...
                [class(obj) '.' prop],prop);
            obj.(prop) = val;
        end

        function obj = set.UCIScaling(obj,val)
            prop = 'UCIScaling';
            validateattributes(val,{'numeric'},...
                {'real','scalar','positive','<=',1},...
                [class(obj) '.' prop],prop);
            obj.(prop) = val;
        end

        function obj = set.NRAPID(obj,val)
            prop = 'NRAPID';
            % To allow codegen for varying length in a single function script
            temp = val;
            coder.varsize('temp',[1 1],[1 1]);
            if ~(isnumeric(temp) && isempty(temp))
                validateattributes(temp,{'numeric'},{'scalar','integer','nonnegative','<=',63},[class(obj) '.' prop],prop);
            end
            obj.(prop) = temp;
        end

        function obj = set.DMRS(obj,val)
            prop = 'DMRS';
            validateattributes(val,{'nrPUSCHDMRSConfig'},{'scalar'},...
                [class(obj) '.' prop],prop);
            obj.(prop) = val;
            % Update the NLayers hidden property in DMRS object with the
            % value of NumPorts
            obj.(prop).NLayers = obj.NumPorts;  %#ok<MCSUP>

            % Update the Mode hidden property in DMRS object with the value
            % of Mode
            obj.(prop).Mode = obj.Mode; %#ok<MCSUP>
        end

        function obj = set.PTRS(obj,val)
            prop = 'PTRS';
            validateattributes(val,{'nrPUSCHPTRSConfig'},{'scalar'},...
                [class(obj) '.' prop],prop);
            obj.(prop) = val;

            % Update the Mode hidden property in PTRS object with the value
            % of Mode
            obj.(prop).Mode = obj.Mode; %#ok<MCSUP>
        end

    end

    methods(Access = public)
        function validateConfig(obj)
            % Validate DM-RS configuration object
            validateConfig(obj.DMRS);

            % Check whether the number of layers is equal to the length of
            % DM-RS port set
            emptyDMRSPortSet = isempty(obj.DMRS.DMRSPortSet);
            if ~emptyDMRSPortSet
                numDMRSPorts = numel(obj.DMRS.DMRSPortSet);
                errFlag1 = (obj.NumLayers ~= numDMRSPorts);
                coder.internal.errorIf(errFlag1,'nr5g:nrPUSCHConfig:InvalidNumLayers',obj.NumLayers,numDMRSPorts);
            end

            % Error if number of antenna ports is less than the number of
            % transmission layers, when transmission scheme is set to 'codebook'
            errFlag2 = (strcmpi(obj.TransmissionScheme,'codebook')) && (obj.NumAntennaPorts < obj.NumLayers);
            coder.internal.errorIf(errFlag2,'nr5g:nrPUSCHConfig:TooManyTxLayers',obj.NumLayers,obj.NumAntennaPorts);

            % Error if double-symbol DM-RS is configured, when intra-slot
            % frequency hopping is enabled
            intraSlotHopEnabled = strcmpi(obj.FrequencyHopping,'intraSlot');
            errFlag3 = (intraSlotHopEnabled && (isempty(obj.DMRS.CustomSymbolSet) && (obj.DMRS.DMRSLength == 2)));
            coder.internal.errorIf(errFlag3,'nr5g:nrPUSCHConfig:InvalidDMRSLength',obj.DMRS.DMRSLength);

            % Error if DMRSAdditionalPosition is configured as other than 0
            % or 1, when intra-slot frequency hopping is enabled
            errFlag4 = (intraSlotHopEnabled && (~any(obj.DMRS.DMRSAdditionalPosition == [0 1])));
            coder.internal.errorIf(errFlag4,'nr5g:nrPUSCHConfig:InvalidDMRSAdditionalPosition',obj.DMRS.DMRSAdditionalPosition);

            % Validate the combination of DM-RS port set and PT-RS port
            % set, when PT-RS is enabled and PTRSPortSet is not empty
            if obj.EnablePTRS
                validateConfig(obj.PTRS);
                if emptyDMRSPortSet
                    dmrsPorts = double(0:obj.NumLayers-1);
                else
                    dmrsPorts = double(obj.DMRS.DMRSPortSet);
                end
                DMRSPorts = dmrsPorts(:);
                upperLimit = (3 + 2*double(obj.DMRS.DMRSConfigurationType == 2));
                % Validate DMRSPortSet
                errFlag5 = any(DMRSPorts > upperLimit);
                coder.internal.errorIf(errFlag5,'nr5g:nrPUSCHConfig:InvalidDMRSPortSet',obj.DMRS.DMRSConfigurationType,max(DMRSPorts),upperLimit);

                % Validate PTRSPortSet
                if ~isempty(obj.PTRS.PTRSPortSet)
                    ptrsPorts = obj.PTRS.PTRSPortSet(:);
                    errFlag6 = ~all(sum(repmat(ptrsPorts',numel(DMRSPorts),1) == repmat(DMRSPorts,1,numel(ptrsPorts))));
                    coder.internal.errorIf(errFlag6,'nr5g:nrPUSCHConfig:InvalidPTRSPortSet');
                end
            end
            
        end
    end

    methods (Access = private)
        function obj = updateHiddenProps(obj)
            %updateHiddenProps Updates the NumPorts hidden property with
            %   the value of NumLayers and then, updates the value of
            %   NLayers hidden property of DM-RS with the value of
            %   NumLayers, if required. This is used to provide appropriate
            %   values for the read-only properties when the DM-RS port set
            %   is provided empty.
            obj.NumPorts = double(obj.NumLayers);
            obj.DMRS.NLayers = obj.NumPorts;
        end

        function obj = setMode(obj)
            %setMode Sets the value of property Mode based on the transform
            %precoding. When transform precoding is set to 1, the mode is
            %set to 1 indicating DFT-s-OFDM waveform. When transform
            %precoding is set to 0, the mode is set to 0 indicating CP-OFDM
            %waveform
            if obj.TransformPrecoding
                obj.Mode = 1;
            else
                obj.Mode = 0;
            end

            % Set the hidden property of DMRS object with the value of the
            % Mode
            obj.DMRS.Mode = obj.Mode;

            % Set the hidden property of PTRS object with the value of the
            % Mode
            obj.PTRS.Mode = obj.Mode;
        end
        
    end
end
