function [info,rinfo] = getULSCHInfo(pusch,tcr,tbs,oack,ocsi1,ocsi2)
%getULSCHInfo Returns the structural information of UCI on PUSCH
%
%   Note: This is an internal undocumented function and its API and/or
%   functionality may change in subsequent releases.
%
%   [INFO,RESINFO] = getULSCHInfo(PUSCH,TCR,TBS,OACK,OCSI1,OCSI2) returns
%   the structural informations INFO and RESINFO related to the coding
%   aspects and resourcing aspects on the physical uplink shared channel,
%   given the inputs:
%   PUSCH - Physical uplink shared channel configuration object
%   TCR   - Target code rate (0 < TCR < 1)
%   TBS   - Transport block size
%   OACK  - HARQ-ACK payload length
%   OCSI1 - CSI part 1 payload length
%   OCSI2 - CSI part 2 payload length
%
%   INFO contains the following fields:
%   CRC     - CRC polynomial selection ('16' or '24A')
%   L       - Number of CRC bits (16 or 24)
%   BGN     - LDPC base graph selection (1 or 2)
%   C       - Number of code blocks
%   Lcb     - Number of parity bits per code block (0 or 24)
%   F       - Number of <NULL> filler bits per code block
%   Zc      - Lifting size selection
%   K       - Number of bits per code block after CBS
%   N       - Number of bits per code block after LDPC coding
%   GULSCH  - Number of coded and rate matched UL-SCH data bits
%   GACK    - Number of coded and rate matched HARQ-ACK bits
%   GCSI1   - Number of coded and rate matched CSI part 1 bits
%   GCSI2   - Number of coded and rate matched CSI part 2 bits
%   GACKRvd - Number of reserved bits for HARQ-ACK
%   QdACK   - Number of coded HARQ-ACK symbols per layer (Q'_ACK)
%   QdCSI1  - Number of coded CSI part 1 symbols per layer (Q'_CSI1)
%   QdCSI2  - Number of coded CSI part 2 symbols per layer (Q'_CSI2)
%
%   RESINFO contains the following fields:
%   MULSCH         - Number of resource elements available for data
%                    transmission in each OFDM symbol
%   MUCI           - Number of resource elements available for UCI
%                    transmission in each OFDM symbol
%   PUSCHSymbolSet - 1-based OFDM symbol locations carrying data relative
%                    to the first OFDM symbol of PUSCH allocation
%                    (excluding DM-RS)
%   DMRSSymbolSet  - 1-based OFDM symbol locations carrying DM-RS relative
%                    to the first OFDM symbol of PUSCH allocation
%   PTRSSymbolSet  - 1-based OFDM symbol locations carrying PT-RS relative
%                    to the first OFDM symbol of PUSCH allocation
%   SymbolSet      - 1-based OFDM symbol locations allocated for PUSCH
%                    relative to the first OFDM symbol of PUSCH allocation
%   PRBSet         - Physical resource blocks allocated for PUSCH

%   Copyright 2020 The MathWorks, Inc.

%#codegen

    % Call the shared channel utility
    cbInfo = nr5g.internal.getSCHInfo(tbs,tcr);

    % Get the information of resource elements that are available for data
    % in each OFDM symbol, along with OFDM symbol locations used for UCI
    % transmission and DM-RS
    rinfo = nr5g.internal.pusch.resourcesInfo(pusch);
    datasymbols = rinfo.PUSCHSymbolSet;
    dmrssymbols = rinfo.DMRSSymbolSet;

    % Initialize some variables
    nlayers = double(pusch.NumLayers);
    qm = nr5g.internal.getQm(pusch.Modulation);
    nlqm = nlayers*qm;    % Product of number of layers and modulation order
    rqm = double(tcr)*qm; % Product of target code rate and modulation order
    alpha = double(pusch.UCIScaling);

    % Calculate bit capacity of PUSCH (excluding DM-RS and PT-RS)
    E = sum(rinfo.MULSCH);
    G = nlqm*E;

    % Get the first OFDM symbol (l_0) that does not carry DM-RS, after
    % the first DM-RS symbol(s) in the PUSCH transmission
    if ~isempty(dmrssymbols) && any(dmrssymbols(1)<datasymbols)
        l0 = datasymbols(find((dmrssymbols(1)<datasymbols),1)); % 1-based
    else
        % When there are no DM-RS symbol locations in the allocation,
        % the first OFDM symbol considered is the starting symbol of
        % PUSCH allocation
        if ~isempty(datasymbols)
            l0 = datasymbols(1);
        else
            l0 = zeros(0,1);
        end
    end

    % Get the total number of resource elements available for UCI
    % transmission on PUSCH
    s1 = sum(rinfo.MUCI);

    % Get the number of resource elements available for UCI transmission on
    % PUSCH from l_0
    if ~isempty(l0)
        s2 = sum(rinfo.MUCI(l0(1):end));
    else
        s2 = 0;
    end

    % Get the number of coded bits of UL-SCH based on the code block groups
    % transmission. Here, all the code blocks are assumed to be transmitted
    a = ones(cbInfo.C,1);
    K = cbInfo.K;
    numCodedBits = sum(a*K);

    % Get the number of reserved ACK bits, based on the number of
    % HARQ-ACK bits
    if oack <= 2
        % Get the reserved bits when oACK <= 2, according to TS 38.212,
        % Section 6.2.7, Step 1
        oACKrvd = 2;
    else
        oACKrvd = 0;
    end

    % Get the number of coded HARQ-ACK symbols and bits, TS 38.212,
    % Section 6.3.2.4.1.1
    [qDashACK,EuciACK] = rateMatchInfoACK(...
        tbs,oack,pusch.BetaOffsetACK,alpha,numCodedBits,s1,s2,rqm,nlqm);
    [qDashACKrvd,EuciACKRvd] = rateMatchInfoACK(...
        tbs,oACKrvd,pusch.BetaOffsetACK,alpha,numCodedBits,s1,s2,rqm,nlqm);

    % Get the number of coded CSI part 1 symbols and bits, TS 38.212,
    % Section 6.3.2.4.1.2
    if ocsi1
        if oACKrvd
            qDashACKCSI1 = qDashACKrvd;
        else
            qDashACKCSI1 = qDashACK;
        end
        firstTerm = getFirstTermOfFormula(...
            tbs,ocsi1,pusch.BetaOffsetCSI1,s1,numCodedBits,rqm);
        if tbs
            qDashCSI1 = min(firstTerm,ceil(alpha*s1)-qDashACKCSI1);
        else
            if ocsi2
                qDashCSI1 = min(firstTerm,s1-qDashACKCSI1);
            else
                qDashCSI1 = s1-qDashACKCSI1;
            end
        end
        EuciCSI = nlqm*qDashCSI1;
    else
        qDashCSI1 = 0;
        EuciCSI = 0;
    end

    % Get the number of coded CSI part 2 symbols and bits, TS 38.212,
    % Section 6.3.2.4.1.3
    if ocsi2
        qDashACKCSI2 = qDashACK;
        if oack <= 2
            qDashACKCSI2 = 0;
        end
        if tbs
            firstTerm = getFirstTermOfFormula(...
                tbs,ocsi2,pusch.BetaOffsetCSI2,s1,numCodedBits,rqm);
            qDashCSI2 = min(firstTerm,ceil(alpha*s1)-qDashACKCSI2-qDashCSI1);
        else
            qDashCSI2 = s1-qDashACKCSI2-qDashCSI1;
        end
        EuciCSI2 = nlqm*qDashCSI2;
    else
        qDashCSI2 =0;
        EuciCSI2 = 0;
    end

    % Get the bit capacity of UL-SCH
    if tbs
        gULSCH = G - EuciCSI - EuciCSI2 - EuciACK*(oACKrvd==0);
    else
        gULSCH = 0;
    end

    % Combine all the information
    info = cbInfo;
    info.GULSCH  = gULSCH;     % Bit capacity of UL-SCH
    info.GACK    = EuciACK;    % Bit capacity of HARQ-ACK
    info.GCSI1   = EuciCSI;    % Bit capacity of CSI part 1
    info.GCSI2   = EuciCSI2;   % Bit capacity of CSI part 2
    info.GACKRvd = EuciACKRvd; % Bit capacity of reserved HARQ-ACK
    info.QdACK   = qDashACK;   % Symbol capacity of HARQ-ACK
    info.QdCSI1  = qDashCSI1;  % Symbol capacity of CSI part 1
    info.QdCSI2  = qDashCSI2;  % Symbol capacity of CSI part 2

end

function [Qd,E] = rateMatchInfoACK(ulschFlag,oack,beta,alpha,sumKr,s1,s2,rqm,nlqm)
%rateMatchInfoACK Rate matching information of HARQ-ACK on PUSCH

    % Symbol and bit capacity of HARQ-ACK
    if oack
        firstTerm = getFirstTermOfFormula(ulschFlag,oack,beta,s1,sumKr,rqm);
        secondTerm = ceil(alpha*s2);
        Qd = min(firstTerm,secondTerm);
        E = nlqm*Qd;
    else
        Qd = 0;
        E = 0;
    end

end

function val = getFirstTermOfFormula(ulschFlag,ouci,beta,s1,sumKr,rqm)
%getFirstTermOfFormula First term of UCI rate match calculations

    % Value of first term in the formula
    if ulschFlag
        % In the presence of UL-SCH
        val = ceil((double(ouci)+getCRC(ouci))*double(beta)*s1/sumKr);
    else
        % In the absence of UL-SCH
        val = ceil((double(ouci)+getCRC(ouci))*double(beta)/rqm);
    end
end

function L = getCRC(oUCI)
% CRC bits for UCI information for input length oUCI, according to TS
% 38.212, Section 6.3.1.2.1
    if oUCI > 19
        L = 11;
    elseif oUCI > 11
        L = 6;
    else
        L = 0;
    end
end
