classdef CommonConfig < nr5g.internal.ConfigBase
    %CommonConfig Common configuration object for physical shared channel
    %
    %   Note: This is an internal undocumented class and its API and/or
    %   functionality may change in subsequent releases.
    %
    %   CommonConfig properties (configurable):
    %
    %   SymbolAllocation - OFDM symbols allocated for physical shared
    %                      channel within a slot (default [0 14])
    %   PRBSet           - PRBs allocated for physical shared channel
    %                      within the BWP (default 0:51)
    %   RNTI             - Radio network temporary identifier (0...65535) (default 1)
    %   EnablePTRS       - Enable or disable the PT-RS configuration (0 (default), 1)

    %   Copyright 2019-2020 The MathWorks, Inc.

    %#codegen

    % Public, tunable properties
    properties
        %MappingType Mapping type of physical shared channel
        %   Specify the mapping type of physical shared channel as a
        %   character or string scalar. The value must be one of
        %   {'A', 'B'}. The default value is 'A'.
        MappingType = 'A';

        %SymbolAllocation Physical shared channel symbol allocation within a slot
        %   Specify the physical shared channel symbol allocation within a
        %   slot as a two-element vector of nonnegative integers
        %   representing the starting symbol and symbol length. The default
        %   value is [0 14].
        SymbolAllocation = [0 14];

        %PRBSet PRBs allocated for physical shared channel within the BWP
        %   Specify the PRBs (0-based) allocated for physical shared
        %   channel within the BWP as a vector of nonnegative integers in
        %   the range 0...274. The default value is 0:51.
        PRBSet = 0:51;

        %RNTI Radio network temporary identifier
        %   Specify the radio network temporary identifier as scalar
        %   nonnegative integer. The value must be in the range 0...65535.
        %   The default value is 1.
        RNTI = 1;

        %EnablePTRS Enable or disable the PT-RS configuration
        %   Specify the flag to enable or disable PT-RS as a numeric or
        %   logical scalar. The value must be binary. 0 indicates that
        %   PT-RS is disabled. 1 indicates that PT-RS is enabled. The
        %   default value is 0.
        EnablePTRS = 0;
    end

    % Constant, hidden properties
    properties (Constant,Hidden)
        MappingType_Values = {'A','B'};
    end

    methods

        % Constructor
        function obj = CommonConfig(varargin)
            % Get the value of SymbolAllocation from the name-value pairs
            symAlloc = nr5g.internal.parseProp('SymbolAllocation',[0 14],varargin{:});
            % Get the value of PRBSet from the name-value pairs
            prbSet = nr5g.internal.parseProp('PRBSet',0:51,varargin{:});

            % Support name-value pair arguments when constructing object
            obj@nr5g.internal.ConfigBase(...
                'SymbolAllocation', symAlloc, ...
                'PRBSet', prbSet, ...
                varargin{:});
        end

        function obj = set.MappingType(obj,val)
            prop = 'MappingType';
            val = validatestring(val,obj.MappingType_Values,[class(obj) '.' prop],prop);
            obj.(prop) = val;
        end

        function obj = set.SymbolAllocation(obj,val)
            prop = 'SymbolAllocation';
            % To allow codegen for varying length in a single function script
            temp = val;
            coder.varsize('temp',[2 2],[1 1]);
            if ~isempty(temp)
                validateattributes(temp,{'numeric'},{'vector','integer','nonnegative','numel',2},[class(obj) '.' prop],prop);
                coder.internal.errorIf((temp(1) + temp(2)) > 14,'nr5g:nrPXSCH:InvalidSymbolAllocation',temp(1),temp(2),14);
            end
            obj.(prop) = temp;
        end

        function obj = set.PRBSet(obj,val)
            prop = 'PRBSet';
            % To allow codegen for varying length in a single function script
            temp = val;
            coder.varsize('temp',[Inf Inf],[1 1]);
            if ~(isnumeric(temp) && isempty(temp))
                validateattributes(temp,{'numeric'},{'vector','integer','nonnegative','<=',274},[class(obj) '.' prop],prop);
            end
            obj.(prop) = temp;
        end
        
        function obj = set.RNTI(obj,val)
            prop = 'RNTI';
            validateattributes(val,{'numeric'},{'scalar','integer','nonnegative','<=',65535},[class(obj) '.' prop],prop);
            obj.(prop) = val;
        end
        
        function obj = set.EnablePTRS (obj,val)
            prop = 'EnablePTRS';
            validateattributes(val, {'numeric','logical'},{'scalar','binary'},[class(obj) '.' prop],prop);
            obj.(prop) = val;
        end
    end
end
