classdef ConfigBase < nr5g.internal.ConfigCommon & matlab.mixin.CustomDisplay
    %ConfigBase Base object for configuration object simulation
    %
    %   Note: This is an internal undocumented class and its API and/or
    %   functionality may change in subsequent releases.

    % Copyright 2019-2021 The MathWorks, Inc.

    %#codegen

    methods
      function obj = ConfigBase(varargin)
        coder.internal.errorIf((mod(nargin,2) ~= 0),'nr5g:ConfigBase:InvalidPVPairs');

        for i = 1:2:nargin
            obj.(char(varargin{i})) = varargin{i+1};
        end
      end
    end

    methods (Access = protected)
      function groups = getPropertyGroups(obj)
        % Returns three property groups for display; normal, read-only, and
        % constant
        propName  = properties(obj);
        mc = meta.class.fromName(class(obj)); % Get metaclass
        allProperties = {mc.PropertyList.Name};
        % If defined explicitly, consider the order of properties as per
        % 'CustomPropList'
        if any(strcmpi(allProperties,'CustomPropList'))
            propName = obj.CustomPropList;
        end
        propVal = cell(numel(propName),1);
        activeIdx = true(size(propName));
        readOnlyIdx = false(size(propName));
        constIdx = false(size(propName));
        mc = meta.class.fromName(class(obj)); % Get metaclass
        % For each property...
        for n = 1:numel(propName)
            if isInactiveProperty(obj,propName{n})
                % If it is inactive then do not add it to the property list
                activeIdx(n) = false;
            elseif isUndefinedProperty(obj,propName{n})
                % If it is undefined then set the value to an empty for disp()
                propVal{n} = [];
            else
                propVal{n} = obj.(propName{n});
            end
            % Determine in property is read-only or constant from attributes
            id = cellfun(@(x)strcmp(x,propName{n}),{mc.PropertyList.Name});
            % If a read-only property then display it in a separate property group
            readOnlyIdx(n) = strcmp(mc.PropertyList(id).SetAccess,'private') && strcmp(mc.PropertyList(id).GetAccess,'public');
            % If a constant property then display it in a separate property group
            constIdx(n) = mc.PropertyList(id).Constant;
        end
        % Create three property lists, one of read-only properties, one of
        % normal properties, and one of constant properties
        readOnlyPropList = cell2struct(propVal(activeIdx&readOnlyIdx),propName(activeIdx&readOnlyIdx));
        normalPropList = cell2struct(propVal(activeIdx&~readOnlyIdx&~constIdx),propName(activeIdx&~readOnlyIdx&~constIdx));
        constPropList = cell2struct(propVal(activeIdx&constIdx),propName(activeIdx&constIdx));
        % Display non-empty lists as groups
        groups = matlab.mixin.util.PropertyGroup(normalPropList);
        if (~isempty(fields(readOnlyPropList)))
            groups = [groups matlab.mixin.util.PropertyGroup(readOnlyPropList,'Read-only properties:')];
        end
        if (~isempty(fields(constPropList)))
            groups = [groups matlab.mixin.util.PropertyGroup(constPropList,'Constant properties:')];
        end
      end

      function s = getFooter(obj)
        % If any of the properties are invalid, display the message in the
        % footer. Create a new line for each undefined property
        str = cell(0,1);
        i = 1;
        propName = properties(obj);
        % For all properties...
        for n = 1:numel(propName)
            [isUndefined, msg] = isUndefinedProperty(obj,propName{n});
            if isUndefined
                str{i} = msg;
                i = i+1;
            end
        end
        s = char(str); % Convert the cell array to a character array; each cell is a new line
        if ~isempty(s) % Wrap to command window width if there is something to print
            s = matlab.internal.display.printWrapped(char(str));
        end
      end

      function displayNonScalarObject(obj)
        % If an object array, display the size and type of object (with a link
        % to the class doc)
        fprintf('  %s %s array\n\n',matlab.mixin.CustomDisplay.convertDimensionsToString(obj), ...
            matlab.mixin.CustomDisplay.getClassNameForHeader(obj));
      end

      function displayEmptyObject(obj)
        % If an empty object array, treat as an object array for display
        displayNonScalarObject(obj);
      end
    end

    methods(Access = private, Static)
      function name = matlabCodegenRedirect(~)
        name = 'nr5g.internal.ConfigCodegenBase';
      end
    end

end
