classdef DMRSConfigBase < nr5g.internal.ConfigBase
    %DMRSConfigBase Base object for DM-RS configuration object
    %
    %   Note: This is an internal undocumented class and its API and/or
    %   functionality may change in subsequent releases.

    % Copyright 2019-2022 The MathWorks, Inc.

    %#codegen

    properties (Hidden)
        %NLayers Number of transmission layers
        NLayers = 1;
    end

    properties (Dependent, Hidden)
        %Ports The DM-RS ports configured based on the DM-RS port set and
        %the number of layers
        Ports;
    end

    properties
        %DMRSTypeAPosition Position of first DM-RS OFDM symbol in a slot
        %   Specify the position of first DM-RS OFDM symbol in a slot, as a
        %   scalar positive integer. The value must be one of {2, 3},
        %   provided by higher-layer parameter dmrs-TypeA-Position. This
        %   property is applicable only when the physical shared channel
        %   mapping type is configured to be 'A'. The default value is 2.
        DMRSTypeAPosition = 2;

        %DMRSAdditionalPosition Maximum number of DM-RS additional positions
        %   Specify the maximum number of DM-RS additional positions as a
        %   scalar nonnegative integer. The value must be one of {0, 1, 2,
        %   3}, provided by higher-layer parameter dmrs-AdditionalPosition.
        %   The default value is 0.
        DMRSAdditionalPosition = 0;

        %DMRSLength Number of consecutive DM-RS OFDM symbols
        %   Specify the number of consecutive OFDM symbols carrying DM-RS
        %   as a scalar positive integer. The value must be one of {1, 2}.
        %   The value of 1 implies single-symbol DM-RS and the value of 2
        %   implies double-symbol DM-RS. The default value is 1.
        DMRSLength = 1;

        %CustomSymbolSet Custom DM-RS symbol set (0-based)
        %   Specify the custom set of DM-RS OFDM symbol locations as a
        %   vector of nonnegative integers. Use these values to override
        %   the DM-RS symbol locations defined in the TS 38.211 DM-RS
        %   tables. Note that the values provided here are treated as
        %   single symbols. The default value is [], which corresponds to
        %   no override of the standard defined values.
        CustomSymbolSet = [];

        %DMRSPortSet DM-RS antenna ports
        %   Specify the DM-RS antenna ports as a vector of nonnegative
        %   integers. For DM-RS configuration type 1, the values must be in
        %   range 0...7. For DM-RS configuration type 2, the values must be
        %   in range 0...11. The default value is [], when the port indices
        %   are 0...NumLayers-1, where NumLayers is the number of layers
        %   defined in the shared channel configuration.
        DMRSPortSet = [];

        % NIDNSCID DM-RS scrambling identities (NID^0 and NID^1)
        %   Specify the DM-RS scrambling identities (NID^0 and NID^1). The
        %   property can be a two-element vector ([NID^0, NID^1]), a scalar
        %   (NID^0=NID^1), or empty ([]). The values must be in range 0...65535.
        %   When the property is empty, the DM-RS scrambling identity is equal
        %   to <a href="matlab:help('nrCarrierConfig/NCellID')">NCellID</a> of nrCarrierConfig.
        %   The default value is [].
        NIDNSCID = [];

        %NSCID DM-RS scrambling initialization (nSCID)
        %   Specify the DM-RS scrambling initialization as a scalar
        %   nonnegative integer. The value must be one of {0, 1}. The
        %   default value is 0.
        NSCID = 0;
    end

    properties (SetAccess = private)
        %CDMGroups CDM group number(s) corresponding to each port
        %   A row vector of CDM group number(s) corresponding to each port,
        %   depending on the DM-RS configuration type.
        CDMGroups;

        %DeltaShifts Delta shift(s) corresponding to each CDM group
        %   A row vector of delta shift(s) corresponding to each CDM group,
        %   depending on the DM-RS configuration type.
        DeltaShifts;

        %FrequencyWeights Frequency weights (w_f) corresponding to each port
        %   A matrix of frequency weights to be applied to the DM-RS
        %   symbols, depending on the DM-RS configuration type and the port
        %   set. Each column corresponds to the weights to be applied for
        %   that port.
        FrequencyWeights;

        %TimeWeights Time weights (w_t) corresponding to each port
        %   A matrix of time weights to be applied to the DM-RS symbols,
        %   depending on the DM-RS configuration type and the port set.
        %   Each column corresponds to the weights to be applied for that
        %   port.
        TimeWeights;

        %DMRSSubcarrierLocations Subcarrier locations in a resource block for each port
        %   A matrix of subcarrier locations of DM-RS symbols in a resource
        %   block for each port. Each column corresponds to the subcarrier
        %   locations for that port.
        DMRSSubcarrierLocations;

        %CDMLengths CDM lengths in frequency and time domain
        %   A two-element row vector [FD TD] specifying the length of
        %   FD-CDM and TD-CDM despreading. The values depend on the
        %   frequency and time masks to be applied to the CDM groups.
        CDMLengths;
    end

    methods
        function obj = DMRSConfigBase(varargin)
            %DMRSConfigBase Create a DMRSConfigBase object
            %   Set the property values from any name-value pairs input to
            %   the object

            % Get the value of DMRSPortSet from the name-value pairs
            dmrsPorts = nr5g.internal.parseProp('DMRSPortSet',[],varargin{:});
            % Get the value of CustomSymbolSet from the name-value pairs
            customSymbols = nr5g.internal.parseProp('CustomSymbolSet',[],varargin{:});
            % Get the value of NIDNSCID from the name-value pairs
            nidnscid = nr5g.internal.parseProp('NIDNSCID',[],varargin{:});

            % Call the base class constructor method with all the
            % name-value pairs input
            obj@nr5g.internal.ConfigBase(...
                'DMRSPortSet', dmrsPorts, ...
                'CustomSymbolSet', customSymbols, ...
                'NIDNSCID', nidnscid, ...
                varargin{:});
        end

        function obj = set.DMRSTypeAPosition(obj,val)
            prop = 'DMRSTypeAPosition';
            validateattributes(val, {'numeric'},...
                {'real','scalar'},...
                [class(obj) '.' prop], prop);
            coder.internal.errorIf(~any(val == [2 3]),...
                'nr5g:DMRSConfigBase:InvalidDMRSTypeAPosition',sprintf('%g',double(val)));
            obj.(prop) = val;
        end

        function obj = set.DMRSAdditionalPosition(obj,val)
            prop = 'DMRSAdditionalPosition';
            validateattributes(val, {'numeric'},...
                {'scalar','integer','nonnegative','<=',3},...
                [class(obj) '.' prop], prop);
            obj.(prop) = val;
        end

        function obj = set.DMRSLength(obj,val)
            prop = 'DMRSLength';
            validateattributes(val, {'numeric'},...
                {'real','scalar'},...
                [class(obj) '.' prop], prop);
            coder.internal.errorIf(~any(val == [1 2]),...
                'nr5g:DMRSConfigBase:InvalidDMRSLength',sprintf('%g',double(val)));
            obj.(prop) = val;
        end

        function obj = set.CustomSymbolSet(obj,val)
            prop = 'CustomSymbolSet';
            % To allow codegen for varying length in a single function script
            temp = val;
            coder.varsize('temp',[Inf Inf],[1 1]);
            if ~(isnumeric(temp) && isempty(temp))
                validateattributes(temp, {'numeric'},...
                    {'vector','integer','nonnegative','nonempty','<=',13},...
                    [class(obj) '.' prop],prop);
            end
            obj.(prop) = temp;
        end

        function obj = set.DMRSPortSet(obj,val)
            prop = 'DMRSPortSet';
            % To allow codegen for varying length in a single function script
            temp = val;
            coder.varsize('temp',[Inf Inf],[1 1]);
            if ~(isnumeric(temp) && isempty(temp))
                validateattributes(temp, {'numeric'},...
                    {'vector','integer','nonnegative','nonempty','<=',11},...
                    [class(obj) '.' prop], prop);
            end
            obj.(prop) = temp;
        end

        function obj = set.NIDNSCID(obj,val)
            prop = 'NIDNSCID';
            % To allow codegen for varying length in a single function script
            temp = val;
            coder.varsize('temp',[2 2],[1 1])
            % Check array length
            coder.internal.errorIf(~any(length(temp) == [0 1 2]),...
                'nr5g:DMRSConfigBase:InvalidNIDNSCIDLength');
            % Check values
            if ~(isnumeric(temp) && isempty(temp))
                validateattributes(temp, {'numeric'},...
                    {'integer','nonnegative','<=',65535},...
                    [class(obj) '.' prop], prop);
            end
            obj.(prop) = temp;
        end

        function obj = set.NSCID(obj,val)
            prop = 'NSCID';
            validateattributes(val, {'numeric'},...
                {'real','scalar'},...
                [class(obj) '.' prop], prop);
            coder.internal.errorIf(~any(val == [0 1]),...
                'nr5g:DMRSConfigBase:InvalidNSCID',sprintf('%g',double(val)));
            obj.(prop) = val;
        end

        % Dependent property
        function val = get.Ports(obj)
            % The ports configured for the shared channel. It is equal to
            % DMRSPortSet or 0:NLayers-1, based on the empty value of
            % DMRSPortSet
            if isempty(obj.DMRSPortSet)
                % Use NLayers when DMRSPortSet is empty
                val = 0:obj.NLayers-1;
            else
                val = reshape(double(obj.DMRSPortSet),1,[]);
            end
        end

        % Read-only properties
        function val = get.CDMGroups(obj)
            % The CDM group number(s) for each port
            val = reshape(...
                mod(fix(double(obj.Ports)/2),double(obj.DMRSConfigurationType)+1),1,[]);
        end

        function val = get.DeltaShifts(obj)
            % The delta shift(s) for each CDM group number
            val = double(obj.DMRSConfigurationType)*obj.CDMGroups;
        end

        function val = get.FrequencyWeights(obj)
            % The frequency weights for each port
            ports = obj.Ports;
            val = ones(2,numel(ports));
            oddPorts = mod(ports,2)==1;
            val(2,oddPorts) = -1;
        end

        function val = get.TimeWeights(obj)
            % The time weights for each port
            ports = obj.Ports;
            val = ones(2,numel(ports));
            doubleSymbolPorts = ports > (2*obj.DMRSConfigurationType+1);
            val(2,doubleSymbolPorts) = -1;
        end

        function val = get.DMRSSubcarrierLocations(obj)
            % The subcarrier locations of each port in a resource block
            deltaShifts = double(obj.DMRSConfigurationType)*obj.CDMGroups;
            if obj.DMRSConfigurationType == 2
                val = repmat([0 1 6 7]',1,numel(deltaShifts))+repmat(deltaShifts,4,1);
            else
                val = repmat((0:2:10)',1,numel(deltaShifts))+repmat(deltaShifts,6,1);
            end
        end

        function val = get.CDMLengths(obj)
            % The CDM lengths [FD, TD]

            % Initialize CDM lengths for each CDM group
            cdmgroups = obj.CDMGroups;
            cdmLen = zeros(numel(cdmgroups),2);

            % Get the number of unique frequency weights and time weights
            % applied for each CDM group, to determine lengths of FD-CDM
            % and TD-CDM despreading
            freqWeights = sum(obj.FrequencyWeights);
            timeWeights = sum(obj.TimeWeights);
            for gi = 1:numel(cdmgroups)
                cdmLen(gi,1) = numel(unique(freqWeights(cdmgroups == cdmgroups(gi))));
                cdmLen(gi,2) = numel(unique(timeWeights(cdmgroups == cdmgroups(gi))));
            end
            val = max(cdmLen,[],1);

            % Overwrite the value of TD with 1, when the DM-RS length is 1
            % or when custom DM-RS symbol set is not empty
            if ~isempty(obj.CustomSymbolSet) || obj.DMRSLength == 1
                val(end) = 1;
            end
        end
    end
end
