classdef nrPDSCHConfigBase < nr5g.internal.pxsch.CommonConfig
  % This is an internal class that may change any time. It hosts common
  % properties between nrWavegenPDSCHConfig and nrPDSCHConfig.

%   Copyright 2019-2020 The MathWorks, Inc.

    %#codegen

    % Public, tunable properties
    properties
        %Modulation Modulation scheme(s) of codeword(s)
        %   Specify the modulation scheme for the codeword(s). It must be
        %   specified as one of {'QPSK', '16QAM', '64QAM', '256QAM'}.
        %   Modulation scheme for single codeword can be specified as a
        %   character array or string scalar. Two codewords can be
        %   configured with single modulation scheme, or cell array or
        %   string array of modulation schemes where each value corresponds
        %   to a separate codeword. The default value is 'QPSK'.
        Modulation = 'QPSK';

        %NumLayers Number of PDSCH transmission layers
        %   Specify the number of PDSCH transmission layers as a scalar
        %   positive integer. The value must be in the range 1...8. The
        %   default value is 1.
        NumLayers = 1;
        
        %ReservedPRB Reserved PRBs and OFDM symbols pattern
        %   Specify the reserved PRBs and OFDM symbols pattern
        %   corresponding to CORESET and SS burst, as a cell array of
        %   object(s), of class <a href="matlab:help('nrPDSCHReservedConfig')">nrPDSCHReservedConfig</a> with the properties:
        %   PRBSet    - Reserved PRB indices (0-based) defined as a vector.
        %               The default value is []
        %   SymbolSet - OFDM symbols associated with reserved PRBs spanning
        %               over one or more slots. The default value is []
        %   Period    - Total number of slots in the pattern period. The
        %               default value is []
        %   The default value is {nrPDSCHReservedConfig}.
        ReservedPRB = {nrPDSCHReservedConfig};

        %VRBToPRBInterleaving VRB-to-PRB interleaving
        %   Specify the flag for VRB-to-PRB interleaving as a numeric or
        %   logical scalar. The value must be binary. 0 indicates that
        %   VRB-to-PRB interleaving is disabled. 1 indicates that
        %   VRB-to-PRB interleaving is enabled. The default value is 0.
        VRBToPRBInterleaving = 0;

        %VRBBundleSize Bundle size in terms of number of PRBs
        %   Specify the bundle size for VRB-to-PRB interleaving as a scalar
        %   positive integer. The value must be one of {2, 4}. The default
        %   value is 2.
        VRBBundleSize = 2;

        %DMRS PDSCH-specific DM-RS configuration object
        %   Specify the DM-RS configuration object associated with PDSCH.
        %   The default value is a default <a href="matlab:help('nrPDSCHDMRSConfig')"
        %   >nrPDSCHDMRSConfig</a> object.
        DMRS = nrPDSCHDMRSConfig;

        %PTRS PDSCH-specific PT-RS configuration object
        %   Specify the PT-RS configuration object associated with PDSCH.
        %   The default value is a default <a href="matlab:help('nrPDSCHPTRSConfig')"
        %   >nrPDSCHPTRSConfig</a> object.
        PTRS = nrPDSCHPTRSConfig;
    end

    % Hidden properties
    properties (Hidden)
        %NumPorts Number of ports
        %   Number of ports. This is a hidden property and is equal to the
        %   number of layers. This is used in updating the hidden property
        %   NLayers of DMRS property.
        NumPorts = 1;
    end

    % Constant, hidden properties
    properties (Constant,Hidden)
        Modulation_Values = {'QPSK','16QAM','64QAM','256QAM'};
    end

    methods
        % Constructor
        function obj = nrPDSCHConfigBase(varargin)
          % Get the value of ReservedPRB from the name-value pairs
            reservedPRB = nr5g.internal.parseProp('ReservedPRB',{nrPDSCHReservedConfig},varargin{:});
            dmrs = nr5g.internal.parseProp('DMRS',nrPDSCHDMRSConfig,varargin{:});
            % Get the value of PTRS from the name-value pairs
            ptrs = nr5g.internal.parseProp('PTRS',nrPDSCHPTRSConfig,varargin{:});

            % Support name-value pair arguments when constructing object
            obj@nr5g.internal.pxsch.CommonConfig(...
                'ReservedPRB',reservedPRB, ...
                'DMRS',dmrs, ...
                'PTRS',ptrs, ...
                varargin{:});
        end

        % Self-validate and set properties
        function obj = set.Modulation(obj,val)
            prop = 'Modulation';
            validateattributes(val,{'cell','char','string'},...
                {'nonempty'},[class(obj) '.' prop], prop);
            if ischar(val) || (isstring(val) && numel(val) == 1)
                % Character array or string scalar
                % Single modulation scheme
                temp = validatestring(val,obj.Modulation_Values,[class(obj) '.' prop],prop);
                obj.(prop) = '';
            else
                % Cell array or string array
                % One or two modulation schemes
                nValues = numel(val);
                coder.internal.errorIf(~any(nValues == [1 2]),'nr5g:nrPDSCHConfig:InvalidNumModulation',nValues);
                % Convert string array to cell array
                tempVal = convertStringsToChars(val);
                temp = coder.nullcopy(cell(1,nValues));
                for idx = 1:numel(tempVal)
                    temp{idx} = validatestring(tempVal{idx},obj.Modulation_Values,[class(obj) '.' prop],prop);
                    obj.(prop) = {''};
                end
            end
            obj.(prop) = temp;
        end

        function obj = set.NumLayers(obj,val)
            prop = 'NumLayers';
            validateattributes(val,{'numeric'},{'scalar','integer','positive','<=',8},[class(obj) '.' prop],prop);
            obj.(prop) = val;
            obj = updateHiddenProps(obj);
        end

        function obj = set.ReservedPRB(obj,val)
            prop = 'ReservedPRB';
            validateattributes(val,{'cell'},{},...
                [class(obj) '.' prop],prop);
            for idx = 1:numel(val)
                validateattributes(val{idx},{'nrPDSCHReservedConfig'},{'scalar'},[class(obj) '.' prop],prop);
            end
            obj.(prop) = cell(1,numel(val)+1);
            obj.(prop) = val;
        end
        
        function obj = set.VRBToPRBInterleaving(obj,val)
            prop = 'VRBToPRBInterleaving';
            validateattributes(val, {'numeric','logical'},{'scalar','binary'},[class(obj) '.' prop],prop);
            obj.(prop) = val;
        end

        function obj = set.VRBBundleSize(obj,val)
            prop = 'VRBBundleSize';
            validateattributes(val,{'numeric'},{'real','scalar'},[class(obj) '.' prop],prop);
            coder.internal.errorIf(~any(val == [2 4]),'nr5g:nrPDSCHConfig:InvalidBundleSize',sprintf('%g',double(val)));
            obj.(prop) = val;
        end

        function obj = set.DMRS(obj,val)
            prop = 'DMRS';
            validateattributes(val,{'nrPDSCHDMRSConfig'},{'scalar'},[class(obj) '.' prop],prop);
            obj.(prop) = val;
            % Update the NLayers hidden property in DM-RS with the value of
            % NumPorts
            obj.(prop).NLayers = obj.NumPorts;  %#ok<MCSUP>
        end

        function obj = set.PTRS(obj,val)
            prop = 'PTRS';
            validateattributes(val,{'nrPDSCHPTRSConfig'},{'scalar'},[class(obj) '.' prop],prop);
            obj.(prop) = val;
        end

    end

    methods(Access = public)
        function validateConfig(obj)
            % Validate DM-RS configuration object
            validateConfig(obj.DMRS);

            % Check whether the number of layers is equal to the length of
            % DM-RS port set, when DM-RS port set is not empty
            if ~isempty(obj.DMRS.DMRSPortSet)
                flag = obj.NumLayers ~= numel(obj.DMRS.DMRSPortSet);
                coder.internal.errorIf(flag,'nr5g:nrPDSCHConfig:InvalidNumLayers',obj.NumLayers,numel(obj.DMRS.DMRSPortSet));
            end

            % Validate the combination of DM-RS port set and PT-RS port
            % set, when PT-RS is enabled and PT-RS port set is not empty
            if obj.EnablePTRS
                if isempty(obj.DMRS.DMRSPortSet)
                    dmrsPorts = 0:double(obj.NumLayers)-1;
                else
                    dmrsPorts = double(obj.DMRS.DMRSPortSet);
                end
                DMRSPorts = dmrsPorts(:);
                upperLimit = (3 + 2*double(obj.DMRS.DMRSConfigurationType == 2));
                % Validate DMRSPortSet
                flag = any(DMRSPorts > upperLimit);
                coder.internal.errorIf(flag,'nr5g:nrPDSCHConfig:InvalidDMRSPortSet',obj.DMRS.DMRSConfigurationType,max(DMRSPorts),upperLimit);

                % Validate PTRSPortSet
                if ~isempty(obj.PTRS.PTRSPortSet)
                    ptrsPort = double(obj.PTRS.PTRSPortSet(1));
                    flag = ~any(ptrsPort == DMRSPorts);
                    coder.internal.errorIf(flag,'nr5g:nrPDSCHConfig:InvalidPTRSPort',ptrsPort);
                end
            end
        end
    end

    methods (Access = private)
        function obj = updateHiddenProps(obj)
            %updateHiddenProps Updates the NumPorts hidden property with
            %   the value of NumLayers and then, updates the value of
            %   NLayers hidden property of DM-RS with the value of
            %   NumPorts, if required. This is used to provide appropriate
            %   values for the read-only properties, when the DM-RS port
            %   set is provided empty.
            obj.NumPorts = double(obj.NumLayers);
            obj.DMRS.NLayers = obj.NumPorts;
        end
    end
end
