classdef nr5G_CSIRS_Tables < handle
% Creation and handling of the CSI-RS table in the CSI-RS tab of Downlink 5G

%   Copyright 2020-2021 The MathWorks, Inc.

  properties (Hidden)
    csirsFig
    csirsTable
    addCSIRSButton
    removeCSIRSButton
    selectedCSIRS = []

    % This is the internal representation of the entire CSI-RS
    % configuration(s). It is needed to cache per-CSIRS edits.
    csirsConfig = {}
  end
  
  methods
    function obj = nr5G_CSIRS_Tables()
    end
    
    function createCSIRSTables(obj)
      % Spec and creation of "CSI-RS" table
      csirsStrings = getStrings();
      if isempty(obj.csirsTable)
        spec = {csirsStrings.Enable,              'logical',                                              53;
                csirsStrings.Power,               [],                                                     53;
                csirsStrings.BWPID,               num2cell(num2str((1:size(obj.bwpTable.Data))'), 2)',    75;        
                csirsStrings.CSIRSType,           nrWavegenCSIRSConfig.CSIRSType_Values,                  53;
                csirsStrings.Periodic,            'logical',                                              62;
                csirsStrings.Period,              horzcat( 'N/A', '1', cellfun(@num2str,num2cell(nrWavegenCSIRSConfig.SlotPeriodicity_Options),'UniformOutput',false) ), 53;
                csirsStrings.SlotOffset,          [],                                                     50;
                csirsStrings.RowNumber,           cellfun(@num2str,num2cell(1:18),'UniformOutput',false),               60;
                csirsStrings.Density,             {'one', 'three', '0.5 (odd)', '0.5 (even)'},            57;
                csirsStrings.SymbolLocation1,     cellfun(@num2str,num2cell(0:13),'UniformOutput',false),               78;
                csirsStrings.SymbolLocation2,     horzcat( 'N/A', cellfun(@num2str,num2cell(2:12),'UniformOutput',false)),       78;
                csirsStrings.SubcarrierLocations, [],                                                     75;
                csirsStrings.NumRB,               [],                                                     40;
                csirsStrings.RBOffset,            [],                                                     50;
                csirsStrings.NID,                 [],                                                     33;
                csirsStrings.Label,               'char',                                                 100;
        }; %#ok<*MCNPN>
              
        obj.csirsTable = uitable('Parent', obj.csirsFig, ...
                  'Position',               [10 10 obj.csirsFig.Position(3)-20 obj.csirsFig.Position(4)/3-20], ...
                  'ColumnName',             spec(1:end, 1), ...
                  'ColumnFormat',           spec(1:end, 2)', ...
                  'ColumnEditable',         true,...
                  'RowName',                {'1'}, ...
                  'ColumnWidth',            spec(1:end, 3)', ...
                  'CellEditCallback',       @obj.csirsTableCallback, ...
                  'CellSelectionCallback',  @obj.selectCSIRSTableEntry, ...
                  'Tag',                    'CSIRSTable', ...
                  'RearrangeableColumns',   false);
        obj.csirsTable.Data = newCSIRSRow(obj);

        % add/remove buttons:
        addRemoveButtons(obj, obj.csirsFig, 'CSIRS');
        % customized remove callback:
        obj.removeCSIRSButton.Callback = @(a, b, c) removeCSIRSTableEntry(obj, [], 'CSIRS');
      end
    end

    function removeCSIRSTableEntry(obj, ~, tag)
      % remove deleted configs from internal cache:
      obj.csirsConfig(obj.selectedCSIRS) = [];
      
      removeTableEntry(obj, [], tag); % generic remove action

      updateGrid(obj);
    end

    function selectCSIRSTableEntry(obj, table, data)
      % don't perform any updates if it's not needed, as this can be a
      % considerable UX delay
      rowNames = obj.csirsTable.RowName;
      row = data.Indices(:, 1);
      if isempty(row)
        return;
      end
      newID = rowNames{row};
      prevID = num2str(obj.selectedCSIRS);
      if isequal(newID, prevID)
        noOp = true;
        % no update needed, this happens when clicking on the current CSI-RS
      else
        noOp = false;
      end
      
      % Still need to do base actions, like enabling delete button
      selectTableEntry(obj, table, data); % also highlight row
      
      if ~noOp
          % Update RE mapping plot:
          channel = ['CSI-RS ' newID];
          updateREVisual(obj, channel, row);
      end
    end
  
    function data = newCSIRSRow(obj)
      % link to the 1st defined BWP
      bwpIdx = 1;
      bwpCol = 3;
      bwps = obj.csirsTable.ColumnFormat{bwpCol};
      newBWP = bwps{bwpIdx};
                  
      % construct the new Label (smallest available CSI-RS ID)
      nextID = appendNextAvailableRowName(obj, obj.csirsTable);
      
      % get the same defaults with nrWavegenCSIRSConfig;
      c = nrDLCarrierConfig; % but get values through nrDLCarrierConfig, as Enable is customized
      c = c.CSIRS{1};
      data = {c.Enable, c.Power, newBWP, c.CSIRSType, ~strcmp(c.CSIRSPeriod, 'off'), ...
              '1', 0, num2str(c.RowNumber), c.Density, num2str(c.SymbolLocations(1)), 'N/A', ...
              ['[' num2str(c.SubcarrierLocations) ']'], c.NumRB, c.RBOffset, c.NID, ['CSIRS' num2str(nextID)]};

      % store new object in cache:
      c.BandwidthPartID = str2double(newBWP);
      obj.csirsConfig = [obj.csirsConfig {c}];

      if c.Enable
          updateGrid(obj);
      end
    end
    
    function csirsTableCallback(obj, ~, ~)
      % called when a table value has been edited
      
      % some BWP selections may have now been fixed (corrected BWP ID), see if red link can go away:
      markBrokenBWPLinks(obj);

      obj.getParent.WaveformGenerator.setStatus(''); % clear possibly previous messages

      data = obj.csirsTable.Data;
      
      try  % check each non-dropdown value, by reusing nrWavegenCSIRSConfig error messages.
           % caught errors are thrown as warnings in status bar

        % error and consistency check:
        for idx = 1:size(data, 1)
          % check each non-dropdown value, by reusing nrWavegenCSIRSConfig error messages:
          csirs = nrWavegenCSIRSConfig;

          % Enable
          csirs.Enable = data{idx,1};

          % Power
          power = data{idx, 2};
          data{idx, 2} = real(power); % do not leave a wrong value
          csirs.Power = power;

          % BandwidthPartID
          csirs.BandwidthPartID = str2double(data{idx, 3});

          % Period
          periodic = data{idx, 5};
          if ~periodic
            % If periodic is turned off, then Period and Offset do not apply
            data{idx, 6} = 'N/A'; % revert
            data{idx, 7} = 'N/A';
            csirs.CSIRSPeriod = 'off';
          else
            period = data{idx, 6};
            if strcmp(period, 'N/A')  % Periodic checkbox was just enabled
              period = '1';           % Start from a default
            end
            data{idx, 6} = period;

            offset = data{idx, 7};
            if strcmp(offset, 'N/A')  % Periodic checkbox was just enabled
              offset = 0;             % Start from a default
            end
            data{idx, 7} = min(str2double(period)-1, max(0, round(real(offset)))); % do not leave a wrong value
            if str2double(period)==1
                csirs.CSIRSPeriod = 'on';
            else
                csirs.CSIRSPeriod = [str2double(period) offset];
            end
          end

          % RowNumber
          csirs.RowNumber = str2double(data{idx,8});

          % Density
          if contains(data{idx,9},'odd')
              density = 'dot5odd';
          elseif contains(data{idx,9},'even')
              density = 'dot5even';
          else
              density = data{idx,9};
          end
          csirs.Density = density;

          % SymbolLocations
          csirs.SymbolLocations = str2double(data{idx,10});
          if ~strcmpi(data{idx,11},'N/A')
              csirs.SymbolLocations(2) = str2double(data{idx,11});
          end

          % SubcarrierLocations
          subcarriers = str2num(data{idx, 12}); %#ok<ST2NM> % vectors are displayed as strings
          data{idx, 12} = mat2str(min(11, max(0, round(real(subcarriers))))); % do not leave a wrong value
          if isscalar(subcarriers)
            data{idx, 12} = ['[' data{idx, 12} ']'];
          end
          csirs.SubcarrierLocations = subcarriers;

          % NumRB
          numRB = data{idx, 13};
          data{idx, 13} = max(1, min(275, round(real(numRB)))); % do not leave a wrong value
          csirs.NumRB = numRB;

          % RBOffset
          rbOffset = data{idx, 14};
          data{idx, 14} = max(0, min(274, round(real(rbOffset)))); % do not leave a wrong value
          csirs.RBOffset = rbOffset;

          % NID
          nid = data{idx, 15};
          data{idx, 15} = max(0, min(1023, round(real(nid)))); % do not leave a wrong value
          csirs.NID = nid;
        end
        obj.csirsConfig = getCSIRSConfig(obj); % store edits to internal cache

        obj.csirsTable.Data = data; % accept good values

        % If everything is ok, update grid plots with new CSIRS values:
        updateGrid(obj);

        % Update RE mapping plot:
        channel = 'CSI-RS';
        selectedTAG = 'selectedCSIRS';
        if isempty(obj.(selectedTAG))
            selectedRow = 1;
        else
            selectedRow = obj.(selectedTAG);
        end
        id = obj.csirsTable.RowName{selectedRow}; % Get the row name ID
        channel = [channel ' ' id];
        updateREVisual(obj, channel, selectedRow);
      catch e
        warnInStatusBar(obj, e); % throw a warning in the status bar
      end
    end
    
    function csirs = getCSIRSConfig(obj)
      % Convert CSI-RS table to a cell of nrWavegenCSIRSConfig objects
      data = obj.csirsTable.Data;
      
      csirs = cell(1, size(data, 1));
      for idx = 1:size(data, 1)
        c = nrWavegenCSIRSConfig;
        c.Enable = data{idx, 1};
        c.Power = data{idx, 2};
        c.BandwidthPartID = str2double(data{idx, 3});
        c.CSIRSType = data{idx, 4};
        
        % Period - handling is different from nrWavegenCSIRSConfig
        periodic = data{idx, 5};
        period = str2double(data{idx, 6});
        offset = data{idx, 7};
        if ~periodic
          c.CSIRSPeriod = 'off';
        elseif periodic && period == 1 || isnan(period)
          c.CSIRSPeriod = 'on';            
        else
          c.CSIRSPeriod = [period offset];
        end
        c.RowNumber = str2double(data{idx, 8});
        
        % Density:
        density = data{idx, 9};
        if strcmp(density, '0.5 (even)')
          c.Density = 'dot5even';
        elseif strcmp(density, '0.5 (odd)')
          c.Density = 'dot5odd';
        else
          c.Density = density;
        end
        
        % Symbol locations:
        symbolLoc1 = data{idx, 10};
        symbolLoc2 = data{idx, 11};
        if strcmp(symbolLoc2, 'N/A')
          c.SymbolLocations = str2double(symbolLoc1);
        else
          c.SymbolLocations = [str2double(symbolLoc1) str2double(symbolLoc2)];
        end
        
        c.SubcarrierLocations = str2num(data{idx, 12}); %#ok<ST2NM>
        c.NumRB = data{idx, 13};
        c.RBOffset = data{idx, 14};
        c.NID = data{idx, 15};
        
        c.Label = data{idx, end};
        csirs{idx} = c;
      end
    end
    
    function applyCSIRSConfig(obj, cfg)
      % Used on New and Open Session
      if isempty(obj.csirsTable)
        return; % App initialization
      end
      
      % Map a cell array of nrWavegenCSIRSConfig objects to the "CSIRS" table
      csirs = cfg.CSIRS;
      % Update internal cache:
      obj.csirsConfig = csirs;
      % Update table:
      data = cell(numel(csirs), 16);
      for idx = 1:numel(csirs)
        data{idx, 1} = csirs{idx}.Enable;
        data{idx, 2} = csirs{idx}.Power;
        data{idx, 3} = num2str(csirs{idx}.BandwidthPartID);
        data{idx, 4} = csirs{idx}.CSIRSType;
        
        % Period
        data{idx, 5} = ~strcmp('off', csirs{idx}.CSIRSPeriod);
        if strcmp('off', csirs{idx}.CSIRSPeriod)
          data{idx, 6} = 'N/A';
          data{idx, 7} = 'N/A';
        else
          if strcmp('on', csirs{idx}.CSIRSPeriod)
            data{idx, 6} = num2str(1);
            data{idx, 7} = 0;
          else
            data{idx, 6} = num2str(csirs{idx}.CSIRSPeriod(1));
            data{idx, 7} = csirs{idx}.CSIRSPeriod(2);
          end
        end
        
        data{idx, 8} = num2str(csirs{idx}.RowNumber);
        
        % Density
        if strcmp(csirs{idx}.Density, 'dot5even')
          data{idx, 9} = '0.5 (odd)';
        elseif strcmp(csirs{idx}.Density, 'dot5odd')
          data{idx, 9} = '0.5 (even)';
        else
          data{idx, 9} = csirs{idx}.Density;
        end
        
        % Symbol location
        data{idx, 10} = num2str(csirs{idx}.SymbolLocations(1));
        if numel(csirs{idx}.SymbolLocations)>1
          data{idx, 11} = num2str(csirs{idx}.SymbolLocations(2));
        else
          data{idx, 11} = 'N/A';
        end
        
        if numel(csirs{idx}.SubcarrierLocations) == 1
          data{idx, 12} = ['[' mat2str(csirs{idx}.SubcarrierLocations) ']']; % mat2str on scalars doesn't add the square brackets ([ ])
        else
          data{idx, 12} = mat2str(csirs{idx}.SubcarrierLocations);
        end
        data{idx, 13} = csirs{idx}.NumRB;
        data{idx, 14} = csirs{idx}.RBOffset;
        data{idx, 15} = csirs{idx}.NID;
        data{idx, 16} = csirs{idx}.Label;
      end
      obj.csirsTable.Data = data;
      
      obj.csirsTable.RowName = num2cell(num2str((1:numel(csirs))'));
    end 
    
    function layoutCSIRSTables(obj, margin, verScrollBarSpace)
      % This method executes everytime the App/tiles resize. Need to
      % maintain relative placement given the new "Position" of the figure
      
      if ~isempty(obj.csirsTable) && ishghandle(obj.csirsTable)
        figPos = obj.csirsTable.Parent.Position;
        if all(figPos([3 4])>margin)
          % Place CSIRS table at the top left of figure
          csirsWidth = sum([obj.csirsTable.ColumnWidth{:}])+32+verScrollBarSpace;
          obj.csirsTable.Position       = max(0, [10 figPos(4)*0.52  csirsWidth         figPos(4)*0.475-10]);
          
          % place add/remove buttons at the top-right of the table:
          layoutAddRemoveButtons(obj, obj.csirsTable, 'CSIRS');
        end
      end
    end
  end
end

function s = getStrings()

    tableName = 'CSIRSTable';
    def = {'Enable' 'Power' 'BWPID' 'CSIRSType' 'Periodic' 'Period' 'SlotOffset' 'RowNumber' 'Density' 'SymbolLocation1' 'SymbolLocation2' 'SubcarrierLocations' 'NumRB' 'RBOffset' 'NID' 'Label'};

    for i = 1:numel(def)
        s.(def{i}) = getString(message(['nr5g:waveformGeneratorApp:' tableName def{i}]));
    end

end
