classdef nr5G_CarrierBWP_Table < handle
% Creation and handling of the SCS and BWP tables in the Main tab of full 5G 

%   Copyright 2020-2021 The MathWorks, Inc.
  
  properties (Hidden)
    scsTable
    scsTitle
    addSCSButton
    removeSCSButton
    selectedSCS = []
    cachedSCS = {'15 kHz'}
        
    bwpTable
    bwpTitle
    bwpIDLabel
    addBWPButton
    removeBWPButton
    selectedBWP = []
  end
  
  methods
    function obj = nr5G_CarrierBWP_Table()
    end
    
    function createSCSTable(obj)
      % Spec and creation of "SCS Carriers" table
      scsStrings = getStrings('SCSTable');
      if isempty(obj.scsTable)
        spec = {scsStrings.SubcarrierSpacing, {'15 kHz', '30 kHz', '60 kHz'},     74;        
                scsStrings.GridSize,          [],                                 65;
                scsStrings.GridStart,         [],                                 69;
                };
      end
              
      if obj.getParent.WaveformGenerator.useAppContainer
        scsFig = obj.(mainFigTag(obj));
      else
        scsFig = obj.getParent.WaveformGenerator.pParametersFig;
      end
      obj.scsTable = uitable('Parent', scsFig, ...
                  'Position',               [10 10 scsFig.Position(3)-20 scsFig.Position(4)-20], ...
                  'ColumnName',             spec(1:end, 1), ...
                  'ColumnFormat',           spec(1:end, 2)', ...
                  'ColumnEditable',         true,...
                  'ColumnWidth',            spec(1:end, 3)', ...
                  'Data',                   newSCSRow(obj), ...
                  'CellEditCallback',       @obj.scsTableCallback, ...
                  'CellSelectionCallback',  @obj.selectTableEntry, ...
                  'Tag',                    'SCSTable', ...
                  'RearrangeableColumns',   false);

      obj.scsTitle = uicontrol('Parent', scsFig, 'Style', 'text', ...
                               'String', scsStrings.Title, 'FontWeight', 'bold');
      
      % add/remove buttons:
      addRemoveButtons(obj, scsFig, 'SCS');
      obj.addSCSButton.Callback     = @(a, b, c) addSCSTableEntry(obj, [], 'SCS');
      obj.removeSCSButton.Callback  = @(a, b, c) removeSCSTableEntry(obj, [], 'SCS');
    end
    
    function addSCSTableEntry(obj, ~, tag)
      addTableEntry(obj, [], tag); % generic actions
      
      % SCS-specific actions:
      % we have 1 carrier for every possibly SCS. No more allowed
      if size(obj.scsTable.Data, 1) == numel(obj.scsTable.ColumnFormat{1})
        % all possible already added - turn off "Add" button
        obj.addSCSButton.Enable = 'off';
      end
      
      obj.cachedSCS = obj.scsTable.Data(:, 1); % cache to support unique SCS per carrier
      
      % update SCS options in BWP table
      updateSCSoptions(obj);
      updateChannelView(obj);
    end
    
    function removeSCSTableEntry(obj, ~, tag)
      removeTableEntry(obj, [], tag); % generic remove actions
      
      % SCS-specific actions: 
      % Update SCS options in BWP table:
      updateSCSoptions(obj);
      % Highlight as red the BWP rows that point to a no longer existing SCS
      markBrokenSCSLinks(obj);
      updateGrid(obj);
      updateChannelView(obj);
    end
             
    function scsTableCallback(obj, ~, ~)
      % called when a table value has been edited
      
      % correct SCS values that should not be offered at the 1st place
      scs = obj.scsTable.Data(:, 1);
      if numel(unique(scs)) < numel(scs)
        % scs was set to an existing value. Not allowed, because only 1
        % carrier per SCS is supported. Revert:
        obj.scsTable.Data(:, 1) = obj.cachedSCS;        
      else
        obj.cachedSCS = obj.scsTable.Data(:, 1); % accept change and cache
      end
      
      updateSCSoptions(obj); % update BWP dropdown with possible new SCS
      
      % clear possibly previous messages, to prepare for error checking
      obj.getParent.WaveformGenerator.setStatus('');
      
      data = obj.scsTable.Data;
      try % check each non-dropdown value, by reusing nrSCSCarrierConfig error messages
          % caught errors are thrown as warnings in status bar

        for idx = 1:size(data, 1)
          c = nrSCSCarrierConfig;

          % Range-limit for NSizeGrid: Real integer from 1 to 275
          gridSize = data{idx, 2};
          data{idx, 2} = max(1, min(round(real(gridSize)), 275));
          c.NSizeGrid = gridSize;
          
          % Range-limit for NStartGrid: Real integer from 0 to 2199
          gridStart = data{idx, 3};
          data{idx, 3} = max(0, min(round(real(gridStart)), 2199));
          c.NStartGrid = gridStart;
        end
        % If everything is ok, update grid plots with new SCS values:
        updateGrid(obj);
        updateChannelView(obj);
      catch e
        warnInStatusBar(obj, e); % throw as a warning in the status bar
      end
      obj.scsTable.Data = data; % accept good values
    end
    
    function data = newSCSRow(obj)
      % get possible new SCS (only those that haven't been used yet for this FR)
      candidateSCS = getCandidateSCS(obj, obj.scsTable);
      newSCS = candidateSCS{1};  % get the 1st allowable SCS
      
      % Find maximum allowed NumRB for the current CBW and the new SCS
      c = nrSCSCarrierConfig;
      % try to make all SCS span the same frequency range: assign default
      % to highest frequency. Then lower frequency NRB are multiples of 2
      if strcmp(obj.FrequencyRange, 'FR1')
        if strcmp(newSCS, '15 kHz')
          gridSize = 270;
          gridStart = 3;
        elseif strcmp(newSCS, '30 kHz')
          gridSize = 132;
          gridStart = 3;
        else % 60 kHz
          gridSize = 65;
          gridStart = 2;
        end
      else
        if strcmp(newSCS, '60 kHz')
          gridSize = 66;
          gridStart = 3;
        elseif strcmp(newSCS, '120 kHz')
          gridSize = 32;
          gridStart = 2;
        else % 240 kHz
          gridSize = 16;
          gridStart = 1;
        end
      end
      
      data = {newSCS, gridSize, gridStart};
    end
    
    function candidateSCS = getCandidateSCS(obj, table)
      % get possible new SCS (only those that haven't been used yet for this FR)
      if ~isempty(table) && ~isempty(table.Data)
        % Table has already been created
        existingSCS = table.Data(:, 1);                             % currently used
        possibleSCS = table.ColumnFormat{1};                        % all possible
        candidateSCS = setdiff(possibleSCS, existingSCS, 'stable'); % remaining
      else
        % 1st time launch
        if strcmp(obj.FrequencyRange, 'FR1')
          candidateSCS = {'15 kHz'};
        else
          candidateSCS = {'60 kHz'};
        end
      end
    end
    
    function carriers = getCarrierConfig(obj)
      % Convert "SCS Carriers" table to a cell of nrSCSCarrierConfig objects
      data = obj.scsTable.Data;
      carriers = cell(1, size(data, 1));
      for idx = 1:size(data, 1) % each row
        tmp = data{idx, 1};
        carriers{idx} = nrSCSCarrierConfig;
        carriers{idx}.SubcarrierSpacing = str2double(tmp(1:end-4)); % trim ' kHz'
        carriers{idx}.NSizeGrid = data{idx, 2};
        carriers{idx}.NStartGrid = data{idx, 3};
      end
    end
    
    function applySCSConfig(obj, cfg)
      % Used on New and Open Session
      if isempty(obj.scsTable)
        return; % App initialization
      end
      
      % Map a cell array of nrSCSCarrierConfig objects to the "SCS Carriers" table
      carriers = cfg.SCSCarriers;
      data = cell(numel(carriers), 3);
      for idx = 1:numel(carriers)
        data{idx, 1} = [num2str(carriers{idx}.SubcarrierSpacing) ' kHz'];
        data{idx, 2} = carriers{idx}.NSizeGrid;
        data{idx, 3} = carriers{idx}.NStartGrid;
      end
      obj.scsTable.Data = data;
    end
        
    function updateSCSoptions(obj)
      % Set the available SCS as options in the dropdown of the BWP table,
      % i.e., make it difficult for a BWP to link to a non-present carrier.
      % Provide the options in increasing SCS order:
      scs = obj.scsTable.Data(:, 1);
      scsNum = cellfun(@(x) str2double(x(1:end-3)), scs);
      scsNum(scsNum==240) = []; % remove 240 kHz; it is only for SS Burst, not for BWP
      sortedSCS = arrayfun(@(x) [num2str(x) ' kHz'], sort(scsNum), 'UniformOutput', false);
      obj.bwpTable.ColumnFormat{1} = sortedSCS';    % set to the BWP table
    end
    
    function createBWPTable(obj)
      % Spec and creation of "Bandwidth Parts" table
      bwpStrings = getStrings('BWPTable');
      if isempty(obj.bwpTable)
        spec = {bwpStrings.SubcarrierSpacing, {'15 kHz'},                         75;        
                bwpStrings.CyclicPrefix,      {'Normal', 'Extended'},             65;
                bwpStrings.BWPSize,           [],                                 65;
                bwpStrings.BWPStart,          [],                                 70;
                bwpStrings.Label,             'char',                             80;
                };
      end

      if obj.getParent.WaveformGenerator.useAppContainer
        bwpFig = obj.(mainFigTag(obj));
      else
        bwpFig = obj.getParent.WaveformGenerator.pParametersFig;
      end
      
      obj.bwpTable = uitable('Parent', bwpFig, ...
                  'Position',               [10 10 bwpFig.Position(3)-20 bwpFig.Position(4)-20], ...
                  'ColumnName',             spec(1:end, 1), ...
                  'RowName',                {'1'}, ...
                  'ColumnFormat',           spec(1:end, 2)', ...
                  'ColumnEditable',         true,...
                  'ColumnWidth',            spec(1:end, 3)', ...
                  'CellEditCallback',       @obj.bwpTableCallback, ...
                  'CellSelectionCallback',  @obj.selectTableEntry, ...
                  'Tag',                    'BWPTable', ...
                  'RearrangeableColumns',   false); %#ok<*MCNPN>
     
      obj.bwpTable.Data = newBWPRow(obj); % default table contents
      obj.bwpTitle = uicontrol('Parent', bwpFig, 'Style', 'text', ...
                               'String', bwpStrings.Title, 'FontWeight', 'bold');
      % This is an "ID" text on top of the row numbers. Has to be done manually:
      obj.bwpIDLabel = uicontrol('Parent', bwpFig, 'Style', 'text', ...
                                'String', bwpStrings.ID, 'FontSize', 9);
                              
      % add/remove buttons:
      addRemoveButtons(obj, bwpFig, 'BWP');
      obj.addBWPButton.Callback     = @(a, b, c) addBWPTableEntry(obj, [], 'BWP');
      obj.removeBWPButton.Callback  = @(a, b, c) removeBWPTableEntry(obj, [], 'BWP');
    end
    
    function addBWPTableEntry(obj, ~, tag)
      addTableEntry(obj, [], tag); % generic actions
      
      updateChannelView(obj);
      
      % BWP-specific actions:
      % Enable new BWP visualization (separate figure)
      nextID = obj.bwpTable.RowName{end};           % The ID of this BWP
      if str2double(nextID) <= obj.maxNumBWPFigs 
        figName = ['Resource Grid (BWP#' nextID ')'];
        obj.setVisualState(figName, true);
      end
      
      % Position in the tiles:
      wavegenObj = obj.getParent.WaveformGenerator;
      if ~isempty(wavegenObj.pParameters.CurrentDialog)
        wavegenObj.setScopeLayout();
        
        if str2double(nextID) <= obj.maxNumBWPFigs
          updateGrid(obj);                              % Initial plot before generate
          ax = obj.getVisualAxes(figName);
          ax.Toolbar.Visible = 'on';                    % Enable interactive toolbar
        end

        % refocus to Main tab, until Java is removed
        figure(obj.getParent.WaveformGenerator.pParametersFig); % refocus to Main tab, until Java is removed
      end
    end
    
    function removeBWPTableEntry(obj, ~, tag)
      % Turn off corresponding Resource Grid plot:
      for idx = 1:length(obj.selectedBWP)
        % Convert selectedBWP to actual BWP IDs (row names)
        bwpIDs = cellfun(@str2double, obj.bwpTable.RowName);
        selectedID = num2str(bwpIDs(obj.selectedBWP(idx)));
        figName = ['Resource Grid (BWP#' selectedID ')'];
        obj.setVisualState(figName, false);
      end
      % Take care of tile layout:
      wavegenObj = obj.getParent.WaveformGenerator;
      wavegenObj.setScopeLayout();
      
      % refocus to Main tab, until Java is removed
      figure(obj.getParent.WaveformGenerator.pParametersFig);
      
      removeTableEntry(obj, [], tag); % generic table actions
      
      % Update BWP options in PDSCH, PDCCH, CSI-RS tables:
      updateBWPoptions(obj);
      markBrokenBWPLinks(obj);
      
      updateGrid(obj);
      updateChannelView(obj);
    end
    
    function data = newBWPRow(obj)
      % New row defaults to the 1st possible SCS
      
      scsCol = 1; % SCS column is the 1st in the BWP table
      scs = obj.bwpTable.ColumnFormat{scsCol};
      newSCS = scs{1}; % default to the 1st SCS for this FR          
      
      % Construct label as "BWPx", where x is this ID. 
      % either appending next ID, or filling up holes after deletion:
      nextID = appendNextAvailableRowName(obj, obj.bwpTable);
      label = ['BWP' num2str(nextID)];
      
      % Start with NSizeBWP equal to NSizeGrid for the same SCS (same for NStartBWP)
      scsRow = strcmp(newSCS, obj.scsTable.Data(:, 1));
      nrb = obj.scsTable.Data{scsRow, 2};
      start = obj.scsTable.Data{scsRow, 3};
      
      data = {newSCS, 'Normal', nrb, start, label};
      
      % Update BWP options in dropdowns of PXSCH and PXCCH tables:
      updateBWPoptions(obj);
      markBrokenBWPLinks(obj);
      updateGrid(obj);
    end
    
    function updateBWPoptionsBase(obj, thisTable, bwpCol)
      % Template method that can be used for different tables, to update BWP dropdowns
      if ~isempty(thisTable)
        thisTable.ColumnFormat{bwpCol} = obj.bwpTable.RowName';
      end
    end
    
    function bwp = getBWPConfig(obj)
      % Convert BWP table to a cell of nrWavegenBWPConfig objects
      
      data = obj.bwpTable.Data;
      rowNames = obj.bwpTable.RowName;

      bwp = cell(1, size(data, 1));
      for idx = 1:size(data, 1) % each row
        bwp{idx} = nrWavegenBWPConfig;
        tmp = data{idx, 1};
        bwp{idx}.BandwidthPartID = str2double(rowNames{idx});
        bwp{idx}.SubcarrierSpacing = str2double(tmp(1:end-4)); % trim ' kHz'
        bwp{idx}.CyclicPrefix = data{idx, 2};
        bwp{idx}.NSizeBWP = data{idx, 3};
        bwp{idx}.NStartBWP = data{idx, 4};
        bwp{idx}.Label = data{idx, 5};
      end
    end
    
    function applyBWPConfig(obj, cfg)
      % Used on New and Open Session
      if isempty(obj.bwpTable)
        return; % App initialization
      end
      
      % Convert a cell array of nrWavegenBWPConfig objects to the "Bandwidth parts" table
      bwp = cfg.BandwidthParts;
      data = cell(numel(bwp), 5);
      rows = cell(numel(bwp), 1);
      for idx = 1:numel(bwp)
        rows{idx} = num2str(bwp{idx}.BandwidthPartID);
        data{idx, 1} = [num2str(bwp{idx}.SubcarrierSpacing) ' kHz'];
        data{idx, 2} = bwp{idx}.CyclicPrefix;
        data{idx, 2}(1) = upper(data{idx, 2}(1)); % object uses lower case
        data{idx, 3} = bwp{idx}.NSizeBWP;
        data{idx, 4} = bwp{idx}.NStartBWP;
        data{idx, 5} = bwp{idx}.Label;
      end
      obj.bwpTable.Data = data;
      obj.bwpTable.RowName = rows;
    end

    function bwpTableCallback(obj, ~, ~)
      % executes when something changes within the data of BWP table
      
      % some SCS selections may have now been fixed with an SCS change, possibly remove red links:
      markBrokenSCSLinks(obj);

      % correct read-only values:
      % Extended CP only for 60 kHz SCS
      cpExtended = strcmpi(obj.bwpTable.Data(:, 2), 'Extended');
      scs60 = strcmpi(obj.bwpTable.Data(:, 1), '60 kHz');
      extendedNot60 = find(cpExtended & ~scs60); % indices of possibly multiple BWP
      obj.bwpTable.Data(extendedNot60 , 2) = {'Normal'}; % revert invalid Extended CPs to Normal
      
      % Check 
      data = obj.bwpTable.Data;      
      obj.getParent.WaveformGenerator.setStatus(''); % clear possibly previous messages
      
      try % catch errors from nrWavegenBWPConfig and throw these in status bar
        for idx = 1:size(data, 1)
          % check each non-dropdown value, by reusing nrWavegenBWPConfig error messages:
          bwp = nrWavegenBWPConfig;

          % Range-limit for NSizeBWP: Real integer from 1 to max NRB for
          % this carrier
          bwpSize = data{idx, 3};
          % find nrb for this carrier - don't allow NSizeBWP>NSizeGrid
          scs = data{idx, 1};
          scsIdx = contains(obj.scsTable.Data(:, 1), scs);
          numRB = obj.scsTable.Data{scsIdx, 2};
          data{idx, 3} = max(1, min(numRB, round(real(bwpSize))));
          bwp.NSizeBWP = bwpSize;

          % Range-limit for NStartBWP: Real integer from 0 to 2473
          bwpStart = data{idx, 4};
          data{idx, 4} = max(0, min(2473, round(real(bwpStart))));
          % find GridStart for this carrier - don't allow NStartBWP>NStartGrid
          carrierStart = obj.scsTable.Data{scsIdx, 3};
          data{idx, 4} = max(carrierStart, data{idx, 4});
          if bwpStart+bwpSize>numRB+carrierStart
            data{idx, 4} = carrierStart; % don't allow BWP outside of carrier. Shift to carrier start
          end
          bwp.NStartBWP = bwpStart;
        end
        % If everything is ok, update plots with new BWP values:
        updateGrid(obj);
        updateChannelView(obj);
      catch e
        warnInStatusBar(obj, e); % throw a warning in the status bar
      end
      obj.bwpTable.Data = data; % accept good values
    end
    
    function layoutSCSBWPTables(obj, margin, verScrollBarSpace)
      % This method executes everytime the App/tiles resize. Need to
      % maintain relative placement given the new "Position" of the figure
      
      if ~ishghandle(obj.(mainFigTag(obj)))
        return % App is closing
      end

      % SCS and BWP tables placement begins at the right of the Filtering Dialog:
      if obj.getParent.WaveformGenerator.useAppContainer
        figPos = obj.(mainFigTag(obj)).Position;
      else
        figPos = obj.getParent.WaveformGenerator.pParametersFig.Position;
      end
      filtDialog = obj.getParent.WaveformGenerator.pParameters.FilteringDialog;
      xOffset = 0;
      if ~isempty(filtDialog)
        p = filtDialog.getPanels;
        if ishghandle(p)
          p.Units = 'pixels';
          xOffset = sum(p.Position([1 3]));
        end
      end
      
      ySize = 151; % a common fixed height for both SCS and BWP tables
      
      % SCS Table:
      if ~isempty(obj.scsTable)
        if all(figPos([3 4])>margin)          
          % Table. Place at the right of the Filtering dialog.
          obj.scsTable.Position    = max(0, [xOffset+10 figPos(4)-ySize+10 sum([obj.scsTable.ColumnWidth{:}])+32+verScrollBarSpace ySize-margin*1.5]);
          % "SCS carriers:" label. Place on top of the SCS table:
          obj.scsTitle.Position = [obj.scsTable.Position(1) sum(obj.scsTable.Position([2 4])) 200 17];
          
          % place add/remove buttons at the top-right of the table:
          layoutAddRemoveButtons(obj, obj.scsTable, 'SCS');
        end
      end
      
      % BWP Table
      if ~isempty(obj.bwpTable)
        if all(figPos([3 4])>margin)
          xOffset = xOffset + obj.scsTable.Position(3) + margin + 20;
          % Table. Place at the right of the SCS table.
          obj.bwpTable.Position = max(0, [xOffset+10 figPos(4)-ySize+10 sum([obj.bwpTable.ColumnWidth{:}])+32+verScrollBarSpace ySize-margin*1.5]);
          % "Bandwidth parts:" label. Place on top of the BWP table:
          obj.bwpTitle.Position = [obj.bwpTable.Position(1) sum(obj.bwpTable.Position([2 4])) 230 17];
          % "ID" text at top left of BWP table:
          obj.bwpIDLabel.Position = max(0, [obj.bwpTable.Position(1)+5 sum(obj.bwpTable.Position([2 4]))-obj.bwpTitle.Position(4)-10 15 20]);
     
          % place add/remove buttons at the top-right of the table:
          layoutAddRemoveButtons(obj, obj.bwpTable, 'BWP');
        end
      end
    end
    
    function moveSCSBWPTablesOnTop(obj)
      % These commands place the SCS & BWP tables on top (depth-wise).
      % This is needed because every time there is layout on the filtering
      % dialog, a layer containing top-level 5G params and filtering is
      % placed on top, and SCS/BWP tables are covered (invisible).
      uistack(obj.scsTable, 'top'); 
      uistack([obj.addSCSButton obj.removeSCSButton], 'top');
      uistack(obj.bwpTable, 'top');
      uistack([obj.addBWPButton obj.removeBWPButton], 'top');
      uistack([obj.scsTitle obj.bwpTitle], 'top');
      uistack(obj.bwpIDLabel, 'top');
    end
    
    function frChangedForSCSBWP(obj)
      % Once FR changes, update SCS values in SCS and BWP tables.
      
      if isempty(obj.scsTable)
        return; % App initialization
      end
      
      % SCS dropdown in SCS table
      if strcmp(obj.FrequencyRange, 'FR1')
        obj.scsTable.ColumnFormat{1} = {'15 kHz', '30 kHz', '60 kHz'};
      else
        if obj.isDownlink
          obj.scsTable.ColumnFormat{1} = {'60 kHz', '120 kHz', '240 kHz'};
        else
          obj.scsTable.ColumnFormat{1} = {'60 kHz', '120 kHz'};
        end
      end
      
      % Add a new legitimate entry:
      obj.scsTable.Data = {};
      obj.scsTable.Data = obj.newSCSRow();
      if strcmp(obj.FrequencyRange, 'FR2') && obj.isDownlink
        % Add the 120 carrier, for the SS Burst not to error (no 60 kHz SS Burst)
        obj.scsTable.Data(2, :) = obj.newSCSRow();
      end
      
      % SCS dropdown in BWP table
      obj.bwpTable.Data = {};
      if strcmp(obj.FrequencyRange, 'FR1')
        obj.bwpTable.ColumnFormat{1} = {'15 kHz'}; % this will be the only possible option with 1 entry in SCS table
      else
        obj.bwpTable.ColumnFormat{1} = {'60 kHz', '120 kHz'};
      end
      % Add a new legitimate entry:
      obj.bwpTable.Data = obj.newBWPRow();
      obj.bwpTable.RowName = {'1'};

      % Make sure only one BWP grid shows
      for idx = 1:obj.maxNumBWPFigs
        figName = ['Resource Grid (BWP#' num2str(idx) ')'];
        if idx == 1 
          obj.setVisualState(figName, true);
        else
          obj.setVisualState(figName, false);
        end
      end
      % need to take care of new tiling
      wavegenObj = obj.getParent.WaveformGenerator;
      if ~isempty(wavegenObj.pParameters.CurrentDialog)
        wavegenObj.setScopeLayout();
      end
      
      % disabled (full SCS) state no longer applies, we start clean with 1 scs entry
      obj.addSCSButton.Enable = 'on';
    end
  end
end

function s = getStrings(tableName)

    switch (tableName)
        case 'SCSTable'
            def = {'Title' 'SubcarrierSpacing' 'GridSize' 'GridStart'};
        case 'BWPTable'
            def = {'Title' 'ID' 'SubcarrierSpacing' 'CyclicPrefix' 'BWPSize' 'BWPStart' 'Label'};
    end

    for i = 1:numel(def)
        s.(def{i}) = getString(message(['nr5g:waveformGeneratorApp:' tableName def{i}]));
    end

end
