classdef nr5G_Control_Tables < handle
% Creation and handling of the CORESET/SS/PDCCH tables in the PDCCH tab of Downlink 5G

%   Copyright 2020-2021 The MathWorks, Inc.
  
  properties (Hidden)
    pdcchFig
    
    % CORESET table
    coresetTable
    coresetTitle
    coresetID
    selectedCORESET = []
    addCORESETButton
    removeCORESETButton
    
    % Search space table
    ssTable
    ssTitle
    ssID
    selectedSS = []
    addSSButton
    removeSSButton
    
    % PDCCH table
    pdcchTable
    pdcchTitle
    selectedPXCCH = []
    addPDCCHButton
    removePDCCHButton

    % This is the internal representation of the entire PDCCH, CORESET, and
    % SearchSpaces configuration(s). It is needed to cache edits.
    csetConfig = {};
    ssConfig = {}
    pdcchConfig = {}
  end
  
  methods
    function obj = nr5G_Control_Tables()
    end
    
    function createPDCCHTables(obj)
      %% Spec and creation of "CORESET" table
      coresetStrings = getStrings('CORESETTable');
      if isempty(obj.coresetTable)
        spec = {coresetStrings.FrequencyResources, 'char',                               80;
                coresetStrings.Duration,           {'1', '2', '3'},                      63;
                coresetStrings.CCEREGMapping,      {'interleaved', 'noninterleaved'},    75;
                coresetStrings.REGBundleSize,      {'2', '3', '6'},                      80;
                coresetStrings.InterleaverSize,    {'2', '3', '6'},                      75;
                coresetStrings.ShiftIndex,         [],                                   45;
                coresetStrings.Label,              'char',                               100};
              
        coreset0 = newCORESETRow(obj);
        coreset0{end} = 'CORESET0';
        obj.coresetTable = uitable('Parent', obj.pdcchFig, ...
                  'Position',               [10 10 obj.pdcchFig.Position(3)-20 obj.pdcchFig.Position(4)/3-20], ...
                  'ColumnName',             spec(1:end, 1), ...
                  'ColumnFormat',           spec(1:end, 2)', ...
                  'ColumnEditable',         true,...
                  'ColumnWidth',            spec(1:end, 3)', ...
                  'Data',                   [coreset0; newCORESETRow(obj)], ...
                  'CellSelectionCallback',  @obj.selectTableEntry, ...
                  'RowName',                {'0'; '1'}, ...
                  'Tag',                    'CORESETTable', ...
                  'CellEditCallback',       @obj.coresetTableCallback, ...
                  'RearrangeableColumns',   false); %#ok<*MCNPN>
        
        % add/remove buttons:
        addRemoveButtons(obj, obj.pdcchFig, 'CORESET');
        
        % customized remove CORESET callback, to take care of CORESET dropdown in
        % SearchSpace table, and to delete the cached configuration:
        obj.removeCORESETButton.Callback = @(a, b, c) removeCORESETTableEntry(obj, [], 'CORESET');

        % The "CORESET" header at top-left
        obj.coresetTitle = uicontrol('Parent', obj.pdcchFig, 'Style', 'text', ...
                                'String', coresetStrings.Title, 'FontWeight', 'Bold');
        % An "ID" text, as a title for 1st column. Needs to be done manually:
        obj.coresetID = uicontrol('Parent', obj.pdcchFig, 'Style', 'text', ...
                                'String', coresetStrings.ID, 'FontSize', 9);
      end
      
      %% Spec and creation of "Search Spaces" table
      ssStrings = getStrings('SSTable');
      if isempty(obj.ssTable)
        spec = {ssStrings.CORESETID,             {'0', '1'},                           67;
                ssStrings.Type,                  {'ue', 'common'},                     55;
                ssStrings.StartSymbol,           cellfun(@num2str,num2cell(0:13),'UniformOutput',false),       57;
                ssStrings.SlotPeriod,            [],                                   52;
                ssStrings.SlotOffset,            [],                                   50;
                ssStrings.Duration,              [],                                   63;
                ssStrings.CandidatesPerAgLevel,  'char',                               78;
                ssStrings.Label,                 'char',                               100};
              
        obj.ssTable = uitable('Parent', obj.pdcchFig, ...
                  'Position',               [10 10 obj.pdcchFig.Position(3)-20 obj.pdcchFig.Position(4)/3-20], ...
                  'ColumnName',             spec(1:end, 1), ...
                  'ColumnFormat',           spec(1:end, 2)', ...
                  'ColumnEditable',         true,...
                  'ColumnWidth',            spec(1:end, 3)', ...
                  'CellEditCallback',       @obj.ssTableCallback, ...
                  'CellSelectionCallback',  @obj.selectTableEntry, ...
                  'RowName',                {'1'}, ...
                  'Tag',                    'SSTable', ...
                  'RearrangeableColumns',   false);
        obj.ssTable.Data = newSSRow(obj);
        
        % add/remove buttons:
        addRemoveButtons(obj, obj.pdcchFig, 'SS');
        % customized remove SearchSpace callback, to take care of
        % SearchSpace dropdown in PDCCH table and to delete the cached configuration:
        obj.removeSSButton.Callback = @(a, b, c) removeSSTableEntry(obj, [], 'SS');
        
        % The "Search space" header at top-left
        obj.ssTitle = uicontrol('Parent', obj.pdcchFig, 'Style', 'text', ...
                                'String', ssStrings.Title, 'FontWeight', 'Bold');
        % An "ID" text, as a title for 1st column. Needs to be done manually:
        obj.ssID = uicontrol('Parent', obj.pdcchFig, 'Style', 'text', ...
                                'String', ssStrings.ID, 'FontSize', 9);
      end
      
      %% Spec and creation of PDCCH table
      pdcchStrings = getStrings('PDCCHTable');
      if isempty(obj.pdcchTable)
        spec = {pdcchStrings.Enable,           'logical',                                  53;
                pdcchStrings.Power,            [],                                         53;
                pdcchStrings.BWPID,            {'1'},                                      43;
                pdcchStrings.SearchSpaceID,    {'1'},                                      65;
                pdcchStrings.AggregationLevel, {'1', '2', '4', '8', '16'},                 85;
                pdcchStrings.Candidate,        num2cell(num2str((1:8)'))',                 73;
                pdcchStrings.AllocatedSlots,   'char',                                     68;
                pdcchStrings.Period,           [],                                         53;
                pdcchStrings.Coding,           'logical',                                  55;
                pdcchStrings.PayloadSize,      [],                                         60;
                pdcchStrings.DataSource,       {'PN9-ITU', 'PN9', 'PN11', 'PN15', 'PN23'}, 55;
                pdcchStrings.RNTI,             [],                                         40;
                pdcchStrings.DMRSScramblingID, [],                                         93;
                pdcchStrings.DMRSPower,        [],                                         78;
                pdcchStrings.Label,            'char',                                     100};
              
         obj.pdcchTable = uitable('Parent', obj.pdcchFig, ...
                  'Position',               [10 10 obj.pdcchFig.Position(3)-20 obj.pdcchFig.Position(4)/3-20], ...
                  'ColumnName',             spec(1:end, 1), ...
                  'ColumnFormat',           spec(1:end, 2)', ...
                  'ColumnEditable',         true,...
                  'ColumnWidth',            spec(1:end, 3)', ...
                  'CellEditCallback',       @obj.pdcchTableCallback, ...
                  'CellSelectionCallback',  @obj.selectPDCCHTableEntry, ...
                  'RowName',                {'1'}, ...
                  'Tag',                    'PDCCHTable', ...
                  'RearrangeableColumns',   false);
          obj.pdcchTable.Data = newPDCCHRow(obj);

          % add/remove buttons:
          addRemoveButtons(obj, obj.pdcchFig, 'PDCCH');
          % customized remove PDCCH callback, to delete the cached configuration:
          obj.removePDCCHButton.Callback = @(a, b, c) removePDCCHTableEntry(obj, [], 'PDCCH');
        
          % The "PDCCH" header at top-left  
          obj.pdcchTitle = uicontrol('Parent', obj.pdcchFig, 'Style', 'text', ...
                                'String', pdcchStrings.Title, 'FontWeight', 'Bold');
      end
    end

    function data = newCORESETRow(obj)
      % construct the new Label (smallest available CORESET ID)
      nextID = appendNextAvailableRowName(obj, obj.coresetTable);

      c = nrCORESETConfig;  % get the same defaults with corresponding object
      data = {mat2str(c.FrequencyResources), num2str(c.Duration), c.CCEREGMapping, ...
              num2str(c.REGBundleSize), num2str(c.InterleaverSize), c.ShiftIndex, ['CORESET' num2str(nextID)]};
            
      % Update CORESET options in dropdown of SearchSpace table:
      updateCORESEToptions(obj);
      % Update red rows / broken links, some may now be fixed:
      markBrokenCORESETLinks(obj);

      % store new object in cache:
      obj.csetConfig = [obj.csetConfig {c}];
    end
    
    function data = newSSRow(obj)
      % link to the 1st defined CORESET that is not CORESET 0
      csetIdx = 1;
      csetCol = 1;
      csets = obj.ssTable.ColumnFormat{csetCol};
      newCSET = csets{csetIdx};
      if strcmp(newCSET, '0') && numel(csets)>1
        % avoid CORESET0 as default
        newCSET = csets{csetIdx+1}; % try the next one
      end
      
      % construct the new Label (smallest available SearchSpace ID)
      nextID = appendNextAvailableRowName(obj, obj.ssTable);
      
      c = nrSearchSpaceConfig;  % get the same defaults with corresponding object
      data = {newCSET, c.SearchSpaceType, num2str(c.StartSymbolWithinSlot), ...
            c.SlotPeriodAndOffset(1), c.SlotPeriodAndOffset(2), c.Duration, ...
            mat2str(c.NumCandidates), ['SearchSpace' (num2str(nextID))]};
      
      % Update SearchSpace options in PDCCH table:
      updateSSoptions(obj);
      % Update red rows / broken links, some may now be fixed:
      markBrokenSSLinks(obj);

      % store new object in cache:
      obj.ssConfig = [obj.ssConfig {c}];
    end
    
    function data = newPDCCHRow(obj)
      % construct the new Label (smallest available PDCCH ID)
      nextID = appendNextAvailableRowName(obj, obj.pdcchTable);
      
      % link to the 1st defined BWP
      bwpIdx = 1;
      bwpCol = 3;
      bwps = obj.pdcchTable.ColumnFormat{bwpCol};
      newBWP = bwps{bwpIdx};
      
      % link to the 1st defined Search-space
      ssIdx = 1;
      ssCol = 4;
      ss = obj.pdcchTable.ColumnFormat{ssCol};
      newSS = ss{ssIdx};
      
      c = nrWavegenPDCCHConfig;  % get the same defaults with corresponding object
      data = {c.Enable, c.Power, num2str(newBWP), num2str(newSS), ...
        num2str(c.AggregationLevel), num2str(c.AllocatedCandidate), ...
        num2str(c.SlotAllocation), c.Period, c.Coding, c.DataBlockSize, c.DataSource, ...
        c.RNTI, c.DMRSScramblingID, c.DMRSPower, ['PDCCH' num2str(nextID)]};

      % store new object in cache:
      c.BandwidthPartID = str2double(newBWP);
      obj.pdcchConfig = [obj.pdcchConfig {c}];

      if c.Enable
          updateGrid(obj);
      end
    end
    
    function removeCORESETTableEntry(obj, ~, tag)
      % remove deleted configs from internal cache:
      obj.csetConfig(obj.selectedCORESET) = [];

      removeTableEntry(obj, [], tag); % generic actions
      
      % CORESET-specific actions: 
      % Update CORESET options in SearchSpace table:
      updateCORESEToptions(obj);
      % Highlight as red the SearchSpace rows that no longer point to an existing CORESET
      markBrokenCORESETLinks(obj);
    end
    
    function updateCORESEToptions(obj)
      % Set the available CORESET as options in the dropdown of the SearchSpace table,
      % i.e., make it difficult for a SearchSpace to link to a non-present CORESET.
      if ~isempty(obj.ssTable)
        obj.ssTable.ColumnFormat{1} = obj.coresetTable.RowName';
      end
    end
    
    function removeSSTableEntry(obj, ~, tag)
      % remove deleted configs from internal cache:
      obj.ssConfig(obj.selectedSS) = [];

      removeTableEntry(obj, [], tag); % generic actions
      
      % SearchSpace-specific actions: 
      % Update SearchSpace options in PDCCH table:
      updateSSoptions(obj);
      % Highlight as red the PDCCH rows that no longer point to an existing SearchSpace
      markBrokenSSLinks(obj);
    end

    function removePDCCHTableEntry(obj, ~, tag)
      % remove deleted configs from internal cache:
      obj.pdcchConfig(obj.selectedPXCCH) = [];

      removeTableEntry(obj, [], tag); % generic actions

      updateGrid(obj); % Update the resource grid
    end

    function selectPDCCHTableEntry(obj, table, data)
      % don't perform any updates if it's not needed, as this can be a
      % considerable UX delay
      rowNames = obj.pdcchTable.RowName;
      row = data.Indices(:, 1);
      if isempty(row)
        return;
      end
      newID = rowNames{row};
      prevID = num2str(obj.selectedPXCCH);
      if isequal(newID, prevID)
        noOp = true;
        % no update needed, this happens when clicking on the current PDCCH
      else
        noOp = false;
      end
      
      % Still need to do base actions, like enabling delete button
      selectTableEntry(obj, table, data); % also highlight row
      
      if ~noOp
        updateREMapping(obj);
      end
      
    end
    
    function updateSSoptions(obj)
      % Set the available SearchSpaces as options in the dropdown of the PDCCH table,
      % i.e., make it difficult for a PDCCH to link to a non-present SearchSpace.
      if ~isempty(obj.pdcchTable)
        obj.pdcchTable.ColumnFormat{4} = obj.ssTable.RowName';
      end
    end
    
    function markBrokenCORESETLinks(obj)
      % paint red the SearchSpace rows that do not link to a valid CORESET
      markBrokenLinks(obj, obj.coresetTable, obj.ssTable, 1);
    end

    function markBrokenSSLinks(obj)
      % paint red the PDCCH rows that do not link to a valid SearchSpace
      markBrokenLinks(obj, obj.ssTable, obj.pdcchTable, 4);
    end
    
    function coresetTableCallback(obj, ~, ~)
      % executes when something changes within the data of CORESET table
      
      data = obj.coresetTable.Data;
      obj.getParent.WaveformGenerator.setStatus(''); % clear possibly previous messages
      
      try % catch errors from nrCORESETConfig and throw in status bar

        % error and consistency check:
        for idx = 1:size(data, 1)
          % check each value, by reusing nrCORESETConfig error messages:
          cset = nrCORESETConfig;
          freqRes = str2num(data{idx, 1}); %#ok<ST2NM>
          cset.FrequencyResources = freqRes;
          dur = str2double(data{idx,2});
          cset.Duration = dur;
          
          % If mapping is non-interleaved, then BundleSize,
          % InterleaverSize, ShiftIndex are N/A
          mapping = data{idx, 3};
          if strcmp(mapping, 'noninterleaved')
            data(idx, 4:6) = deal({'N/A'});
          else
            % if one of BundleSize, InterleaverSize, ShiftIndex is 'N/A'
            % then mapping just changed. Set these to default values.
            if strcmp(data{idx, 4}, 'N/A')
              data{idx, 4} = num2str(cset.REGBundleSize);
            end
            if strcmp(data{idx, 5}, 'N/A')
              data{idx, 5} = num2str(cset.InterleaverSize);
            end
            if strcmp(data{idx, 6}, 'N/A')
              data{idx, 6} = cset.ShiftIndex;
            else
              shiftIdx = data{idx, 6};
              data{idx, 6} = min(1007, max(0, round(real(shiftIdx)))); % do not leave a wrong value
              cset.ShiftIndex = shiftIdx;
            end
          end
        end
        obj.coresetTable.Data = data; % accept good values
        obj.csetConfig = getCORESETConfig(obj); % store edits to internal cache

        % If everything is ok, update grid plots with new CORESET values:
        updateGrid(obj);
        updateREMapping(obj);
      catch e
        warnInStatusBar(obj, e); % throw a warning in the status bar
      end
    end

    function ssTableCallback(obj, ~, ~)
      % executes when something changes within the data of SearchSpace table

      % Some broken CORESET selections may now have been fixed
      markBrokenCORESETLinks(obj);

      obj.getParent.WaveformGenerator.setStatus(''); % clear possibly previous messages
      
      % validate non-dropdown table entries
      data = obj.ssTable.Data;
      
      try % catch errors from nrSearchSpaceConfig and throw in status bar

        % error and consistency check:
        for idx = 1:size(data, 1)
          % check each value, by reusing nrSearchSpaceConfig error messages:
          ss = nrSearchSpaceConfig;
          
          % CORESETID
          ss.CORESETID = str2double(data{idx, 1});

          % StartSymbolWithinSlot
          ss.StartSymbolWithinSlot = str2double(data{idx, 3});

          % SlotPeriodAndOffset
          period = data{idx, 4};
          data{idx, 4} = max(1, round(real(period))); % do not leave a wrong value
          ss.SlotPeriodAndOffset(1) = period;
          
          offset = data{idx, 5};
          data{idx, 5} = min(str2double(period)-1, max(0, round(real(offset)))); % do not leave a wrong value
          ss.SlotPeriodAndOffset(2) = offset;
          
          % Duration
          duration = data{idx, 6};
          data{idx, 6} = min(str2double(period), max(1, round(real(duration)))); % do not leave a wrong value
          ss.Duration = duration;
          
          % NumCandidates
          candidates = str2num(data{idx, 7}); %#ok<ST2NM>
          ss.NumCandidates = candidates;
        end
        obj.ssTable.Data = data; % accept good values
        obj.ssConfig = getSearchSpaceConfig(obj); % store edits to internal cache
        
        % If everything is ok, update grid plots with new SearchSpace values:
        updateGrid(obj);
        updateREMapping(obj);
      catch e
        warnInStatusBar(obj, e); % throw a warning in the status bar
      end
    end

    function pdcchTableCallback(obj, ~, ~)
      % executes when something changes within the data of PDCCH table

      % some BWP selections may have now been fixed:
      markBrokenBWPLinks(obj);
      % some Search Space selections may have now been fixed:
      markBrokenSSLinks(obj);
      
      obj.getParent.WaveformGenerator.setStatus(''); % clear possibly previous messages
      
      data = obj.pdcchTable.Data;
      
      try % catch errors from nrWavegenPDCCHConfig and throw in status bar

        % error and consistency check:
        for idx = 1:size(data, 1)
          % check each non-dropdown value, by reusing nrWavegenPDCCHConfig error messages:
          pdcch = nrWavegenPDCCHConfig;

          % Enable
          pdcch.Enable = data{idx,1};

          % Power
          power = data{idx, 2};
          data{idx, 2} = real(power); % do not leave a wrong value
          pdcch.Power = power;

          % BandwidthPartID
          pdcch.BandwidthPartID = str2double(data{idx, 3});
          
          % ensure that the candidate selection is allowed based on
          % NumCandidates from the SearchSpace table:
          thisSS = data{idx, 4};
          idx2 = find(strcmp(thisSS, obj.ssTable.RowName));
          % Get NumCandidates for this aggregation level
          numCandidates = str2num(obj.ssTable.Data{idx2, 7}); %#ok<FNDSB,ST2NM>
          aggLevel = str2double(data{idx, 5});
          aggLevelIdx = 1+log2(aggLevel);
          candidate = str2double(data{idx, 6});
          maxCandidate = numCandidates(aggLevelIdx);
          data{idx, 6} = num2str(max(1, min(maxCandidate, candidate)));
          validateattributes(candidate,{'double'}, {'scalar','integer','nonnegative', '<=', maxCandidate}, '', getString(message('nr5g:waveformGeneratorApp:SSTableValidateCandidate',num2str(aggLevel),num2str(thisSS))));
          pdcch.SearchSpaceID = str2double(thisSS);
          pdcch.AggregationLevel = aggLevel;
          pdcch.AllocatedCandidate = candidate;

          % Period & SlotAllocation
          period = data{idx, 8};

          slotAllocation = str2num(data{idx, 7}); %#ok<ST2NM> % vectors are displayed as strings
          pdcch.SlotAllocation = slotAllocation;
          
          data{idx, 8} = max(0, round(real(period))); % do not leave a wrong value
          pdcch.Period = period;
          
          coding = data{idx, 9};
          % Coding rate
          if ~coding
            data{idx, 10} = 'N/A'; % data blocksize not applicable
          else
            dataSize = data{idx, 10};
            if strcmp(dataSize, 'N/A') % Coding was just enabled
              data{idx, 10} = pdcch.DataBlockSize; % restart DataBlockSize from default
              dataSize = data{idx, 10};
            else
              data{idx, 10} = min(140, max(0, round(real(dataSize)))); % do not leave a wrong value
            end
            pdcch.DataBlockSize = dataSize;
          end
          
          % RNTI
          rnti = data{idx, 12};
          data{idx, 12} = max(0, min(65535, round(real(rnti)))); % do not leave a wrong value
          pdcch.RNTI = rnti;
          
          % NID
          nid = data{idx, 13};
          data{idx, 13} = max(0, min(65535, round(real(nid)))); % do not leave a wrong value
          pdcch.DMRSScramblingID = nid;
          
          % DMRSPower
          dmrsPower = data{idx, 14};
          data{idx, 14} = real(dmrsPower); % do not leave a wrong value
          pdcch.DMRSPower = dmrsPower;
        end
        obj.pdcchTable.Data = data; % accept good values
        obj.pdcchConfig = getPDCCHConfig(obj); % store edits to internal cache
        
        % If everything is ok, update grid plots with new PDCCH values:
        updateGrid(obj);
        updateREMapping(obj);
      catch e
        warnInStatusBar(obj, e); % throw a warning in the status bar
      end
    end

    function updateREMapping(obj)
        % updates RE mapping plot for the selected PDCCH
        channel = 'PDCCH';
        selectedTAG = 'selectedPXCCH';
        if isempty(obj.(selectedTAG))
            selectedRow = 1;
        else
            selectedRow = obj.(selectedTAG);
        end
        id = obj.pdcchTable.RowName{selectedRow}; % Get the row name ID
        channel = [channel ' ' id];
        updateREVisual(obj, channel, selectedRow);
    end
    
    function coreset = getCORESETConfig(obj)
      % map CORESET table to a cell of nrCORESETConfig objects
      data = obj.coresetTable.Data;
      rowNames = obj.coresetTable.RowName;
      
      coreset = cell(1, size(data, 1));
      for idx = 1:size(data, 1)
        c = nrCORESETConfig;
        c.CORESETID = str2double(rowNames{idx});
        c.FrequencyResources = str2num(data{idx, 1}); %#ok<ST2NM>
        c.Label = data{idx, end};
        c.Duration = str2double(data{idx, 2});
        c.CCEREGMapping = data{idx, 3};
        if strcmp(c.CCEREGMapping, 'interleaved')
          c.REGBundleSize = str2double(data{idx, 4});
          c.InterleaverSize = str2double(data{idx, 5});
          c.ShiftIndex = data{idx, 6};
        end
        coreset{idx} = c;
      end
    end
    
    function applyCORESETConfig(obj, cfg)
      % Used on New and Open Session
      if isempty(obj.coresetTable)
        return; % App initialization
      end
      
      % Map a cell array of nrCORESETConfig objects to the "CORESET" table
      cset = cfg.CORESET;
      data = cell(numel(cset), 7);
      rows = cell(numel(cset), 1);
      for idx = 1:numel(cset)
        rows{idx} = num2str(cset{idx}.CORESETID);
        data{idx, 1} = mat2str(cset{idx}.FrequencyResources);
        data{idx, 2} = num2str(cset{idx}.Duration);
        data{idx, 3} = cset{idx}.CCEREGMapping;
        if strcmp(cset{idx}.CCEREGMapping, 'interleaved')
          data{idx, 4} = num2str(cset{idx}.REGBundleSize);
          data{idx, 5} = num2str(cset{idx}.InterleaverSize);
          data{idx, 6} = cset{idx}.ShiftIndex;
        else
          data{idx, 4} = 'N/A';
          data{idx, 5} = 'N/A';
          data{idx, 6} = 'N/A';
        end
        data{idx, 7} = cset{idx}.Label;
      end
      obj.coresetTable.Data = data;
      obj.coresetTable.RowName = rows;

      % Update CORESET options in SearchSpace table:
      updateCORESEToptions(obj);
    end
    
    function ss = getSearchSpaceConfig(obj)
      % map Search space table to a cell of nrSearchSpaceConfig objects
      data = obj.ssTable.Data;
      rowNames = obj.ssTable.RowName;
      
      ss = cell(1, size(data, 1));
      for idx = 1:size(data, 1)
        c = nrSearchSpaceConfig;
        c.SearchSpaceID = str2double(rowNames{idx});
        c.CORESETID = str2double(data{idx, 1});
        c.SearchSpaceType = data{idx, 2};
        c.StartSymbolWithinSlot = str2double(data{idx, 3});
        c.SlotPeriodAndOffset(1) = data{idx, 4};
        c.SlotPeriodAndOffset(2) = data{idx, 5};
        c.Duration = data{idx, 6};
        c.NumCandidates = str2num(data{idx, 7}); %#ok<ST2NM>
        c.Label = data{idx, end};
        ss{idx} = c;
      end
    end
    
    function applySearchSpaceConfig(obj, cfg)
      % Used on New and Open Session
      if isempty(obj.ssTable)
        return; % App initialization
      end
      
      % Map a cell array of nrSearchSpaceConfig objects to the "SearchSpace" table
      ss = cfg.SearchSpaces;
      % Update internal cache:
      obj.ssConfig = ss;
      % Update table:
      data = cell(numel(ss), 7);
      rows = cell(numel(ss), 1);
      for idx = 1:numel(ss)
        rows{idx} = num2str(ss{idx}.SearchSpaceID);
        data{idx, 1} = num2str(ss{idx}.CORESETID);
        data{idx, 2} = ss{idx}.SearchSpaceType;
        data{idx, 3} = num2str(ss{idx}.StartSymbolWithinSlot);
        data{idx, 4} = ss{idx}.SlotPeriodAndOffset(1);
        data{idx, 5} = ss{idx}.SlotPeriodAndOffset(2);
        data{idx, 6} = ss{idx}.Duration;
        data{idx, 7} = mat2str(ss{idx}.NumCandidates);
        data{idx, 8} = ss{idx}.Label;
      end
      obj.ssTable.Data = data;
      obj.ssTable.RowName = rows;

      % Update SearchSpace options in PDCCH table:
      updateSSoptions(obj);
    end
    
    function pdcch = getPDCCHConfig(obj)
      % map PDCCH table to a cell of nrWavegenPDCCHConfig objects
      data = obj.pdcchTable.Data;
      pdcch = cell(1, size(data, 1));
      for idx = 1:size(data, 1)
        c = nrWavegenPDCCHConfig;
        c.Enable = data{idx, 1};
        c.Label = data{idx, end};
        c.Power = data{idx, 2};
        c.BandwidthPartID = str2double(data{idx, 3});
        c.SearchSpaceID = str2double(data{idx, 4});
        c.AggregationLevel = str2double(data{idx, 5});
        c.AllocatedCandidate = str2double(data{idx, 6});
        c.SlotAllocation = str2num(data{idx, 7}); %#ok<ST2NM>
        c.Period = data{idx, 8};
        c.Coding = data{idx, 9};
        if isnumeric(data{idx, 10})
          c.DataBlockSize = data{idx, 10};
        % else it is 'N/A'
        end
        c.DataSource = data{idx, 11};
        c.RNTI = data{idx, 12};
        c.DMRSScramblingID = data{idx, 13};
        c.DMRSPower = data{idx, 14};
        pdcch{idx} = c;
      end
    end
    function applyPDCCHConfig(obj, cfg)
      % Used on New and Open Session
      if isempty(obj.pdcchTable)
        return; % App initialization
      end
      
      % Map a cell array of nrWavegenPDCCHConfig objects to the "PDCCH" table
      pdcch = cfg.PDCCH;
      % Update internal cache:
      obj.pdcchConfig = pdcch;
      % Update table:
      data = cell(numel(pdcch), 15);
      for idx = 1:numel(pdcch)
        data{idx, 1} = pdcch{idx}.Enable;
        data{idx, 2} = pdcch{idx}.Power;
        data{idx, 3} = num2str(pdcch{idx}.BandwidthPartID);
        data{idx, 4} = num2str(pdcch{idx}.SearchSpaceID);
        data{idx, 5} = num2str(pdcch{idx}.AggregationLevel);
        data{idx, 6} = num2str(pdcch{idx}.AllocatedCandidate);
        data{idx, 7} = mat2str(pdcch{idx}.SlotAllocation);
        data{idx, 8} = pdcch{idx}.Period;
        data{idx, 9} = pdcch{idx}.Coding;
        data{idx, 10} = pdcch{idx}.DataBlockSize;
        data{idx, 11} = pdcch{idx}.DataSource;
        data{idx, 12} = pdcch{idx}.RNTI;
        data{idx, 13} = pdcch{idx}.DMRSScramblingID;
        data{idx, 14} = pdcch{idx}.DMRSPower;
        data{idx, 15} = pdcch{idx}.Label;
      end
      obj.pdcchTable.Data = data;
      obj.pdcchTable.RowName = num2cell(num2str((1:numel(pdcch))'));
    end 
    
    function layoutControlTables(obj, margin, verScrollBarSpace)
      % This method executes everytime the App/tiles resize. Need to
      % maintain relative placement given the new "Position" of the figure
      
      if ~isempty(obj.coresetTable) && ishghandle(obj.coresetTable)
        figPos = obj.coresetTable.Parent.Position;
        if all(figPos([3 4])>margin)
          coresetWidth = sum([obj.coresetTable.ColumnWidth{:}])+32+verScrollBarSpace;
          % CORESET table. Place at the top left of figure
          obj.coresetTable.Position       = max(0, [10                   10+figPos(4)/2 	coresetWidth     figPos(4)/2-margin*2]);
          % CORESET label. Place at top left of CORESET table
          obj.coresetTitle.Position = [obj.coresetTable.Position(1)+3 obj.coresetTable.Position(2)+obj.coresetTable.Position(4) 170 20];
          % "ID" text. Place at top of left column.
          obj.coresetID.Position = max(0, [obj.coresetTable.Position(1)+6 sum(obj.coresetTable.Position([2 4]))-obj.coresetTitle.Position(4)-10 15 20]);

          
          % Search space table. Place right of the CORESET table.
          obj.ssTable.Position   = max(0, [sum(obj.coresetTable.Position([1 3]))+40  10+figPos(4)/2 sum([obj.ssTable.ColumnWidth{:}])+32+verScrollBarSpace     figPos(4)/2-margin*2]);
          % Search space label. Place at top left of Search space table
          obj.ssTitle.Position = [obj.ssTable.Position(1) obj.ssTable.Position(2)+obj.ssTable.Position(4) 210 20];
          % "ID" text. Place at top of left column.
          obj.ssID.Position = max(0, [obj.ssTable.Position(1)+6 sum(obj.ssTable.Position([2 4]))-obj.ssTitle.Position(4)-10 15 20]);
          
          
          % PDCCH table. Place at bottom left of figure, below CORESET table.
          obj.pdcchTable.Position         = max(0, [10                   10          sum([obj.pdcchTable.ColumnWidth{:}])+32+verScrollBarSpace     figPos(4)/2-margin*1.5]);
          % PDCCH label. Place at top left of PDCCH table
          obj.pdcchTitle.Position = [obj.pdcchTable.Position(1) obj.pdcchTable.Position(2)+obj.pdcchTable.Position(4) 165 20];
          
          % place add/remove buttons at the top-right of each table:
          layoutAddRemoveButtons(obj, obj.coresetTable, 'CORESET');
          layoutAddRemoveButtons(obj, obj.ssTable, 'SS');
          layoutAddRemoveButtons(obj, obj.pdcchTable, 'PDCCH');
        end
      end
    end
  end
end

function s = getStrings(tableName)

    switch (tableName)
        case 'CORESETTable'
            def = {'Title' 'ID' 'FrequencyResources' 'Duration' 'CCEREGMapping' 'REGBundleSize' 'InterleaverSize' 'ShiftIndex' 'Label'};
        case 'SSTable'
            def = {'Title' 'ID' 'CORESETID' 'Type' 'StartSymbol' 'SlotPeriod' 'SlotOffset' 'Duration' 'CandidatesPerAgLevel' 'Label'};
        case 'PDCCHTable'
            def = {'Title' 'Enable' 'Power' 'BWPID' 'SearchSpaceID' 'AggregationLevel' 'Candidate' 'AllocatedSlots' 'Period' 'Coding' 'PayloadSize' 'DataSource' 'RNTI' 'DMRSScramblingID' 'DMRSPower' 'Label'};
    end

    for i = 1:numel(def)
        s.(def{i}) = getString(message(['nr5g:waveformGeneratorApp:' tableName def{i}]));
    end

end
