classdef nr5G_DL_Dialog < wirelessWaveformGenerator.nr5G_Full_Base_Dialog & ...
                          wirelessWaveformGenerator.nr5G_Control_Tables & ...
                          wirelessWaveformGenerator.nr5G_PXSCH_Tables & ...
                          wirelessWaveformGenerator.nr5G_CSIRS_Tables
% Downlink-specific content of Full DL 5G wavegen

%   Copyright 2020-2021 The MathWorks, Inc.
  
  properties (Hidden)
    TitleString =  getString(message('nr5g:waveformGeneratorApp:nrDLTitle'))
    
    ssBurstFig % The fig of the 2nd tab. Other figs are brought from subclass mixins
    
    % Cache default nrDLCarrierConfig for UX speedup. Constructor takes 0.5 sec
    % because of numerous properties and validations (validatestring). CTOR
    % is exercised in updateGrid -> getConfiguration -> nrDLCarrierConfig
    cachedDefaultCfg = nrDLCarrierConfig 
    
    mainDLFig
  end
  
  methods
  
    function obj = nr5G_DL_Dialog(parent)
      obj@wirelessWaveformGenerator.nr5G_Full_Base_Dialog(parent, true); % call base constructor
      
      %% SS Burst:
      % The infrastructure takes care of the Main (1st) tab's placement.
      % Placement of secondary tabs need to be taken care of manually.
      className = 'wirelessWaveformGenerator.nr5G_SSB_Dialog';
      if ~isKey(obj.Parent.DialogsMap, className) || obj.Parent.WaveformGenerator.useAppContainer
        % create 2nd param object, so that there is no interference with
        % 1st one (Main tab) in layoutPanels()
        param2 = wirelessWaveformGenerator.Parameters(obj.Parent.WaveformGenerator);
        
        % Create Main SS Burst Dialog
        obj.Parent.DialogsMap(className) = eval([className '(param2, obj.ssBurstFig)']);
        dlg = obj.Parent.DialogsMap(className);
        param2.CurrentDialog = dlg; % needed for layoutPanels()
        
        % Create 2ndary SS Burst Dialog (Data Source configuration)
        className = 'wirelessWaveformGenerator.nr5G_SSB_DataSource';
        obj.Parent.DialogsMap(className) = eval([className '(param2, obj.ssBurstFig)']);
        dlg2 = obj.Parent.DialogsMap(className);
        
        % we won't be using the FilteringDialog created in param2, so remove:
        className = 'wirelessWaveformGenerator.FilteringDialog';
        delete(param2.DialogsMap(className).getPanels);
        
        % Fix placement of both panels:
        layoutUIControls(dlg);
        layoutUIControls(dlg2);
        layoutPanels(dlg); % same for both 
      end
      
      figResizedCallback(obj); % make sure tables are laid out properly
    end
  
    function names = figureNames(~)
      names = {'Main', 'SS Burst', 'PDSCH', 'PDCCH', 'CSI-RS'};
    end
    function tag = figTag(~, name)
      switch name
        case 'Main'
          tag = 'mainDLFig';
        case 'SS Burst'
          tag = 'ssBurstFig';
        case 'PDSCH'
          tag = 'pxschFig';
        case 'PDCCH'
          tag = 'pdcchFig';
        case 'CSI-RS'
          tag = 'csirsFig';
      end
    end

    function name = PXSCHfigureName(~)
      name = 'PDSCH';
    end
    
    function tag = mainFigTag(~)
      tag = 'mainDLFig';
    end

    function tag = pxschExtraFigTag(~)
      tag = 'pxschSingleChannelFig';
    end

    function tag = pxcchExtraFigTag(~)
      tag = 'pxcchSingleChannelFig';
    end
    
    function createFigures(obj)
      % Create figures for all 5 tabs:
      obj.createFig('SS Burst',               'ssBurstFig');
      obj.createFig('PDSCH',                  'pxschFig');
      
%       if ~obj.Parent.WaveformGenerator.useAppContainer
        % PDSCH tab has an extra side fig (right-side panel) that is not
        % visible under the Main tab:
        obj.createFig('PDSCH 1 - Advanced Configuration',                'pxschSingleChannelFig');
        if obj.Parent.WaveformGenerator.useAppContainer
          obj.Parent.WaveformGenerator.AppContainer.getDocument('figureDocumentGroup', pxschExtraFigTag(obj)).Visible = false;
        else
          obj.pxschSingleChannelFig.Visible = false;
        end
%       end
      obj.createFig('PDCCH',                  'pdcchFig');
      obj.createFig('CSI-RS',                 'csirsFig');
    end
        
    function createTables(obj)
      % Create tables for all figures:
      createSCSTable(obj);
      createBWPTable(obj);
      
      createPDCCHTables(obj);
      
      createPXSCHTables(obj);
      
      createCSIRSTables(obj);
    end
    
    function setExtraConfigFigVisibility(obj, visibility)
      % This method turns on or off all tabs if the waveform type changes
      % (e.g., Full DL -> WLAN)

      appObj = obj.Parent.WaveformGenerator;
      if appObj.useAppContainer 
        if ~visibility
          % exiting this waveform type, need to close documents
          
          appObj.AppContainer.closeDocument("figureDocumentGroup", "mainDLFig");
          appObj.AppContainer.closeDocument('figureDocumentGroup', 'ssBurstFig');
          appObj.AppContainer.closeDocument('figureDocumentGroup', 'pxschFig');
          doc = appObj.AppContainer.getDocument("figureDocumentGroup", 'pxschSingleChannelFig');
          if ~isempty(doc) && doc.Visible
            appObj.AppContainer.closeDocument('figureDocumentGroup', 'pxschSingleChannelFig');
          end
          appObj.AppContainer.closeDocument('figureDocumentGroup', 'pdcchFig');
          appObj.AppContainer.closeDocument('figureDocumentGroup', 'csirsFig');
        end
      else
        figs = [obj.ssBurstFig obj.pdcchFig obj.pxschFig obj.csirsFig];
        if ~visibility
          % Always hide right-side panel under PDSCH. It should not be
          % enabled when entering this waveform type.
          figs = [figs, obj.pxschSingleChannelFig];
        end
        set(figs, 'Visible', visibility);
        
        if visibility && strcmp(obj.lastFigInFocus, 'PDSCH')
          % We need some time for the figure to disappear before laying out
          % the tiles. This is for the case Downlink 5G (PDSCH tab) -> other
          % waveform type (e.g., WLAN) -> Downlink 5G (must be Main tab)
          pause(0.5);
        end
      end
    end

    function cleanupDlg(obj)
      % Dialog-specific cleanup when app is closing
      % Custom 5G DL has additional UI objects that need deletion when the
      % app is closing.
      delete([obj.pxschFig; obj.pxschSingleChannelFig; obj.ssBurstFig; obj.pdcchFig; obj.csirsFig])
    end
    
    function cfg = getConfiguration(obj)
      % Map all graphical content into an equivalent nrDLCarrierConfig object
      
      % Top level properties:
      cfg = obj.cachedDefaultCfg; % default nrDLCarrierConfig object
      cfg.Label             = obj.Label;
      cfg.FrequencyRange    = obj.FrequencyRange;
      cfg.ChannelBandwidth  = obj.ChannelBandwidth;
      cfg.NCellID           = obj.NCellID;
      cfg.NumSubframes      = obj.NumSubframes;

      % Windowing controls are slightly different than programmatic API
      if strcmp(obj.WindowingSource, 'Auto')
        cfg.WindowingPercent = [];
      else
        cfg.WindowingPercent = obj.WindowingPercent;
      end
      
      if strcmp(obj.SampleRateSource, 'Auto')
        cfg.SampleRate = [];
      else
        cfg.SampleRate = obj.SampleRate;
      end
      
      if obj.PhaseCompensation
        cfg.CarrierFrequency   = obj.CarrierFrequency;
      else
        cfg.CarrierFrequency   = 0;
      end
      
      % SCS carriers & BWP:
      cfg.SCSCarriers = getCarrierConfig(obj);
      cfg.BandwidthParts = getBWPConfig(obj);
      
      % SS Burst
      className = 'wirelessWaveformGenerator.nr5G_SSB_Dialog';
      if isKey(obj.Parent.DialogsMap, className)
        ssbDialog = obj.Parent.DialogsMap(className);
        cfg.SSBurst = getSSBConfig(ssbDialog);
        
        className = 'wirelessWaveformGenerator.nr5G_SSB_DataSource';
        ssbDataSourceDialog = obj.Parent.DialogsMap(className);
        cfg.SSBurst = addSSBDataSourceConfig(ssbDataSourceDialog, cfg.SSBurst);
      end
      
      % CORESET, Search spaces, PDCCH
      cfg.CORESET = getCORESETConfig(obj);
      cfg.SearchSpaces = getSearchSpaceConfig(obj);
      cfg.PDCCH = getPDCCHConfig(obj);
      
      % PDSCH
      cfg.PDSCH = getPXSCHConfig(obj);

      % CSI-RS
      cfg.CSIRS = getCSIRSConfig(obj);
    end
    
    function figResizedCallback(obj, ~)
      % This method executes when the App or its tile resize. The tables
      % (+ text labels) need to be repositioned.
      margin = 20;
      verScrollBarSpace = 20;
      
      layoutSCSBWPTables(obj, margin, verScrollBarSpace);
      
      layoutPXSCHTables(obj, margin, verScrollBarSpace);
      
      layoutControlTables(obj, margin, verScrollBarSpace);
      
      layoutCSIRSTables(obj, margin, verScrollBarSpace);
    end
    
    function markAllBrokenLinks(obj)
      % Update all broken links by highlighting red color in invalid rows
      
      markBrokenSCSLinks(obj);     % Subcarrier spacing in BWP table
      markBrokenBWPLinks(obj);     % BWP ID in PDSCH, PDCCH, CSI-RS table
      markBrokenCORESETLinks(obj); % CORESET ID in Search Space table
      markBrokenSSLinks(obj);      % SearchSpace ID in PDCCH table
    end
    
    function markBrokenBWPLinks(obj)
      % paint red the rows that do not link to a valid BWP
      markBrokenLinks(obj, obj.bwpTable, obj.pxschTable, 3);
      markBrokenLinks(obj, obj.bwpTable, obj.csirsTable, 3);
      markBrokenLinks(obj, obj.bwpTable, obj.pdcchTable, 3);
    end   
    
    function str = getIconDrawingCommand(obj)
        str = ['disp([''Freq. range: '' ' obj.configGenVar '.FrequencyRange newline ...' newline ...
                '''Bandwidth: '' num2str(' obj.configGenVar '.ChannelBandwidth) ''MHz'' newline ...' newline ...
                '''Subframes: '' num2str(' obj.configGenVar '.NumSubframes) ]);'];
    end
    
    function [blockName, maskTitleName, waveNameText] = getMaskTextWaveName(obj)
        blockName = ['5G NR ' obj.Parent.WaveformGenerator.pCurrentWaveformType];
        maskTitleName = ['5G NR ' obj.Parent.WaveformGenerator.pCurrentWaveformType ' Waveform Generator'];
        waveNameText = 'a 5G NR downlink' ;
    end

    function updateBWPoptions(obj)
      % Update the offered dropdown values for the BWP ID column in PDSCH,
      % PDCCH, CSI-RS tables, based on the row IDs in the BWP table.
      updateBWPoptionsBase(obj, obj.pxschTable, 3);
      updateBWPoptionsBase(obj, obj.pdcchTable, 3);
      updateBWPoptionsBase(obj, obj.csirsTable, 3);
    end
    
    function adjustSpec(obj)
      adjustSpec@wirelessWaveformGenerator.nr5G_Full_Base_Dialog(obj);
      
      obj.panelFixedSize = true; % prevents panel containing top-level 
                                % properties (NumSubframes etc.) from
                                % filling horizontal space and covering
                                % SCS, BWP tables
      
      % For MATLAB Code generation:
      obj.configGenFcn  = @nrDLCarrierConfig;
      obj.configGenVar  = 'cfgDL';
    end
    
    function restoreDefaults(obj)
      obj.pxschExtraTile = false;
      obj.selectedSCS = [];
      obj.selectedBWP = [];
      obj.selectedPXSCH = [];
      obj.selectedPXCCH = [];
      obj.selectedCSIRS = [];
      className = ['wirelessWaveformGenerator.nr5G_' PXSCHfigureName(obj) 'Advanced_Dialog'];
      if isKey(obj.Parent.DialogsMap, className) && ~isempty(obj.pxschTable) && ishghandle(obj.pxschTable)
        updateAdvancedPXSCH(obj);  % reset the PXSCH num to 1.
      end
      
      cfg = obj.cachedDefaultCfg; % default nrDLCarrierConfig object
      cfg.Label = 'Carrier1';
      
      % Take care of CORESET 0, which is not present in nrDLCarrierConfig
      cfg.CORESET{2} = cfg.CORESET{1};
      cfg.CORESET{1}.Label = 'CORESET0';
      cfg.CORESET{1}.CORESETID = 0;
       
      % One difference is that carriers are prepared for equal span in
      % different SCS (maximum occupancy).
      def15kHzNRB = 270;
      cfg.SCSCarriers{1}.NSizeGrid   = def15kHzNRB;
      cfg.SCSCarriers{1}.NStartGrid  = 3;
      cfg.BandwidthParts{1}.NSizeBWP = def15kHzNRB;
      cfg.BandwidthParts{1}.NStartBWP = 3;
      
      cfg.PDSCH{1}.PRBSet = 0:(def15kHzNRB-1);
      % Another difference is to start with NID = 1 instead of [] Cell ID
      cfg.PDSCH{1}.NID = 1;
      cfg.PDSCH{1}.EnablePTRS = logical(cfg.PDSCH{1}.EnablePTRS);
      
      % set to tables and panels
      applyConfiguration(obj, cfg);
    end
    
    function applyConfiguration(obj, cfg)
      % Used on New and Open Session.
      % Map an nrDLCarrierConfig object to all tables and UI elements
      
      applyConfiguration@wirelessWaveformGenerator.nr5G_Full_Base_Dialog(obj, cfg);
      
      applySSBurstConfig(obj, cfg);
      applyCORESETConfig(obj, cfg);
      applySearchSpaceConfig(obj, cfg);
      applyPDCCHConfig(obj, cfg);
      applyPXSCHConfig(obj, cfg);
      applyCSIRSConfig(obj, cfg);
    end
    
    function applySSBurstConfig(obj, cfg)
      % Map an nrDLCarrierConfig.SSBurst = nrWavegenSSBConfig object to all
      % UI elements under the SS Burst tab.
      ssb = cfg.SSBurst;
      
      % SS Burst main dialog
      className = 'wirelessWaveformGenerator.nr5G_SSB_Dialog';
      if ~isKey(obj.Parent.DialogsMap, className)
        return; % App initialization
      end
      
      % Mapping for properties that have 1-1 equivalence between GUI and programmatic API:
      dlg = obj.Parent.DialogsMap(className);
      if ~ishghandle(dlg.BlockPatternGUI)
        return; % App 2nd initialization
      end

      props = {'Enable', 'Power', 'BlockPattern', 'TransmittedBlocks', ...
                'Period', 'KSSB', 'NCRBSSB'};
      for idx = 1:numel(props)
        dlg.(props{idx}) = ssb.(props{idx});
      end
      % Custom frequency offsets are exposed differently in GUI than programmatic API
      if isempty(ssb.NCRBSSB)
        dlg.FrequencyOffset = 'Carrier center';
      else
        dlg.FrequencyOffset = 'Custom';
      end
      frequencySourceChangedGUI(dlg);
      
      % SS Burst data source dialog
      className = 'wirelessWaveformGenerator.nr5G_SSB_DataSource';
      dlg = obj.Parent.DialogsMap(className);
      % Mapping for properties that have 1-1 equivalence between GUI and programmatic API:
      props = {'DMRSTypeAPosition', 'CellBarred', 'IntraFreqReselection', ...
                'PDCCHConfigSIB1', 'SubcarrierSpacingCommon'};
      for idx = 1:numel(props)
        dlg.(props{idx}) = ssb.(props{idx});
      end
      % Custom data sources are exposed differently in GUI than programmatic API
      if ischar(ssb.DataSource)
        dlg.DataSource = ssb.DataSource;
      else
        dlg.DataSource = 'User-defined';
        dlg.Payload = ssb.DataSource;
      end
      dataSourceChangedGUI(dlg);
    end
    
    function frChanged(obj, ~)
      % Executes when Frequency Range changes (FR1<->FR2)
      
      %% SS Burst:
      % Need to take care of affected options in SS Burst tab:
      % - Block Pattern
      % - Transmitted Blocks
      % - Subcarrier Spacing Common
           
      className   = 'wirelessWaveformGenerator.nr5G_SSB_Dialog';
      classNameDS = 'wirelessWaveformGenerator.nr5G_SSB_DataSource';
      if isKey(obj.Parent.DialogsMap, className)
        ssbDialog = obj.Parent.DialogsMap(className);
        ssb_ds_Dialog = obj.Parent.DialogsMap(classNameDS);
        if ~ishghandle(ssbDialog.BlockPatternGUI)
          % Re-entering DL 5G, 2nd initialization not complete yet
          return;
        end
      
        if strcmp(obj.FrequencyRange, 'FR1')
          % Update dropdown:
          ssbDialog.BlockPatternGUI.String = {'Case A (15 kHz)', 'Case B (30 kHz)', 'Case C (30 kHz)'};
          ssbDialog.TransmittedBlocks = ones(1, 4);
          ssb_ds_Dialog.SubcarrierSpacingCommonGUI.String = {'15 kHz', '30 kHz'};
        else
          % Update dropdown:
          ssbDialog.BlockPatternGUI.Value = min(2, ssbDialog.BlockPatternGUI.Value); % 3rd option no longer available
          ssbDialog.BlockPatternGUI.String = {'Case D (120 kHz)', 'Case E (240 kHz)'};
          ssbDialog.TransmittedBlocksGUI.String = '[1:64]';
          ssb_ds_Dialog.SubcarrierSpacingCommonGUI.String = {'60 kHz', '120 kHz'};
        end
      end
      
      %% All the rest are handled commonly between DL and UL App:
      frChanged@wirelessWaveformGenerator.nr5G_Full_Base_Dialog(obj);
    end
    
    function w = getColumnWeights(obj, ~)
      % The right-side column (PDSCH Advanced config or Spectrum Analyzer)
      % is a bit smaller.
      if obj.Parent.WaveformGenerator.useAppContainer
        c = 0.7324;
        w = [nan; c; 1-c];
      else
        w = [0.725; 0.275];
      end
    end
    
    function [tileCount, tileCoverage, tileOccupancy] = getTileLayout(obj, numScopes)
      [tileCount, tileCoverage, tileOccupancy] = getTileLayout@wirelessWaveformGenerator.nr5G_Full_Base_Dialog(obj, numScopes);
     
      figs = {'ssBurstFig', 'pxschFig', 'pdcchFig', 'csirsFig'};
      for idx = 1:numel(figs)
        tileID = 1;
        documentID = ['figureDocumentGroup_' figs{idx}];
        str = struct('showOrder', 2, 'id', documentID);
        tileOccupancy(tileID).children = [tileOccupancy(tileID).children str];
      end
      
    end
    
    % Get GUI table from channel name
    function t = getChannelTable(dialog,channel)
        switch channel
            case {'PDSCH'}
                t = dialog.pxschTable;
            case 'PDCCH'
                t = dialog.pdcchTable;
            case 'CSIRS'
                t = dialog.csirsTable;
            otherwise
                t = [];
        end
    end

  end
end