classdef nr5G_Dialog < wirelessWaveformGenerator.waveformConfigurationDialog
%

%   Copyright 2018-2021 The MathWorks, Inc.
  
  properties (Dependent)
    Label             % Label
    FrequencyRange    % FR1 (450 MHz - 6 GHz) or FR2 (24.25 GHz - 52.6 GHz)
    ChannelBandwidth  % Bandwidth
    SubcarrierSpacing % Subcarrier spacing
    DuplexMode        % Duplexing mode
    NumSubframes      % Number of 1ms subframes (1,2,4,8 slots per subframe depending on SCS)
    NCellID           % Cell identity
    RNTI              % Radio Network Temporary Identifier
    AllocatedSymbols  % Symbol allocation in a slot 
    NLayers           % SCH layers
    WindowingSource   % Auto or Custom
    WindowingPercent  % Percentage of Windowing Samples / FFTLength
    SampleRateSource  % Auto or Custom
    SampleRate        % Sample rate 
    PhaseCompensation % Checkbox
    CarrierFrequency  % Carrier Frequency for Phase Compensation (Hz)
  end
  
  properties (Constant, Hidden)
    CBWFR1 = [5, 10, 15, 20, 25, 30, 40, 50, 60, 70, 80, 90, 100]
    CBWFR2 = [50, 100, 200, 400]
    
    SCSFR1 = [15, 30, 60]
    SCSFR2 = [60, 120]
  end
  
  properties (Hidden)
    configFcn = @struct
    configGenFcn = @struct
    configGenVar = 'var'

    LabelType = 'edit'
    LabelLabel
    LabelGUI
    FrequencyRangeType = 'popupmenu'
    FrequencyRangeDropDown = {'FR1 (410 MHz - 7.125 GHz)', 'FR2 (24.25 GHz - 52.6 GHz)'}
    FrequencyRangeGUI 
    FrequencyRangeLabel
    ChannelBandwidthType = 'popupmenu'
    ChannelBandwidthDropDown = num2cell(wirelessWaveformGenerator.nr5G_Dialog.CBWFR1)
    ChannelBandwidthGUI
    ChannelBandwidthLabel
    SubcarrierSpacingType = 'popupmenu'
    SubcarrierSpacingDropDown = num2cell(wirelessWaveformGenerator.nr5G_Dialog.SCSFR1)
    SubcarrierSpacingGUI
    SubcarrierSpacingLabel
    DuplexModeType = 'popupmenu'
    DuplexModeDropDown = {'FDD', 'TDD'}
    DuplexModeGUI
    DuplexModeLabel
    NumSubframesType = 'edit'
    NumSubframesGUI
    NumSubframesLabel
    NCellIDType = 'edit'
    NCellIDGUI
    NCellIDLabel
    RNTIType = 'edit'
    RNTIGUI
    RNTILabel
    AllocatedSymbolsType = 'edit'
    AllocatedSymbolsGUI
    AllocatedSymbolsLabel
    AllocatedPRBType = 'edit'
    AllocatedPRBGUI
    AllocatedPRBLabel
    
    NLayersType = 'popupmenu'
    NLayersDropDown = {'1', '2', '3', '4'}
    NLayersGUI
    NLayersLabel
    
    WindowingSourceType = 'popupmenu'
    WindowingSourceDropDown = {'Auto', 'Custom'}
    WindowingSourceGUI
    WindowingSourceLabel
    WindowingPercentType = 'edit'
    WindowingPercentGUI
    WindowingPercentLabel
    
    SampleRateSourceType = 'popupmenu'
    SampleRateSourceDropDown = {'Auto', 'Custom'}
    SampleRateSourceGUI
    SampleRateSourceLabel
    SampleRateType = 'edit'
    SampleRateGUI
    SampleRateLabel
    
    PhaseCompensationType = 'checkbox'
    PhaseCompensationGUI
    PhaseCompensationLabel
    CarrierFrequencyType = 'edit'
    CarrierFrequencyGUI
    CarrierFrequencyLabel
    
    gridSet
    isDownlink = true; % default, can be overridden
  end
  
  methods (Static)
    function hPropDb = getPropertySet(~)
      hPropDb = extmgr.PropertySet(...
        'Visualizations',   'mxArray', {'Resource Grid (BWP#1)', 'Channel View', '2D Spectrogram'});
    end
  end
  
  methods
    function obj = nr5G_Dialog(parent)
      obj@wirelessWaveformGenerator.waveformConfigurationDialog(parent); % call base constructor
      
      obj.NumSubframesGUI.Callback        = @(a,b) framesChangedGUI(obj, []);
      obj.AllocatedSymbolsGUI.Callback    = @(a,b) symbolsChanged(obj, []);
      obj.NCellIDGUI.Callback             = @(a,b) nCellChanged(obj, []);
      obj.RNTIGUI.Callback                = @(a,b) rntiChanged(obj, []);
      obj.ChannelBandwidthGUI.Callback    = @(a,b) bwChanged(obj, []);
      
      obj.WindowingSourceGUI.Callback     = @(a,b) windowingChangedGUI(obj, []);
      obj.SampleRateSourceGUI.Callback    = @(a,b) srChangedGUI(obj, []);
      
      obj.PhaseCompensationGUI.Callback   = @(a,b) phaseCompChangedGUI(obj, []);

      obj.DuplexModeGUI.Callback          = @(a,b) DuplexModeChanged(obj);
      
      obj.CCDFLegendChannelName = getString(message('nr5g:waveformGeneratorApp:CCDFLegendChannelName5G'));

    end
    
    function setupDialog(obj)
      setupDialog@wirelessWaveformGenerator.waveformConfigurationDialog(obj);
      genDialog = getGenerationDialog(obj);
      if ~isempty(genDialog)
        genDialog.InputSource = 'PN9';  % repeatable setting for T&M
        genDialog.layoutUIControls();
        genDialog.layoutPanels();
      end
      
      appObj = obj.Parent.WaveformGenerator;
      if ~appObj.useAppContainer
        % Set Help button to doc page of 5G Waveform Generator:
        setContextualHelpCallback(obj.Parent.WaveformGenerator.ToolGroup, ...
          @(h,e)helpview(fullfile(docroot,'5g','helptargets.map'), ...
          'FiveGWaveformGenerator_app'));
      end
    end
    function helpCallback(~)
      helpview(fullfile(docroot, '5g', 'ref', '5gwaveformGenerator-app.html'));
    end

    function b = exportsMLCode(~)
      b = true;
    end
    
    function bwChanged(obj, ~)
      updateInfo(obj);

      updateGrid(obj);
      updateChannelView(obj);
    end
    
    function adjustSpec(obj)
      % customization given the max label length
      obj.LabelWidth = 150;
    end
    
    function adjustDialog(obj)
      obj.FrequencyRangeGUI.Callback  = @(a,b) frChanged(obj, []);
      if any(contains(displayOrder(obj), 'SubcarrierSpacing'))
        % UL FRC does not have SCS control
        obj.SubcarrierSpacingGUI.Callback  = @(a,b) scsChanged(obj, []);
        scsChanged(obj);  % to initiate available options
      end
      % Make sure the Label tag is unique
      obj.LabelGUI.Tag = 'WaveLabel';
    end
    
    function defaultVisualLayout(obj)
      obj.setVisualState('Resource Grid (BWP#1)', true);
      obj.setVisualState('Channel View', false);
      obj.setVisualState('2D Spectrogram', false);
    end
    function tag = getFigureTag(obj, visualName)
      if strcmp(visualName, '2D Spectrogram')
        tag = 'spectrogram2D';
      else
        % Figure Tag, remove spaces
        tag = getFigureTag@wirelessWaveformGenerator.waveformConfigurationDialog(obj, visualName);
      end
      % remove '#', '(', ')' characters from field names of figures; can't
      % be saved in structure during Save Session
      tag = replace(tag, {'(',')','#'}, '');
    end
    
    function restoreDefaults(obj)
      % Label is taken care of in nr5G_DL_dialog and nr5G_UL_Dialog
      obj.FrequencyRange    = 'FR1';
      obj.ChannelBandwidth  = 40;
      obj.SubcarrierSpacing = 15;
      obj.DuplexMode        = 'FDD';
    end 
    
    function updateInfo(~)
      % no op for TM
    end

    function figureAdded(obj,figName)
      % This method initializes/updates the figure (for example the Resource 
      % Grid and the Channel View, which do not require waveform generation) 
      % before it is added to the current visuals
      if contains(figName(isletter(figName)),'ChannelView') % Java version includes spaces, Java-free does not
        % Initialize channel view
        updateChannelView(obj);
      elseif contains(figName(isletter(figName)),'ResourceGrid') % Java version includes spaces, Java-free does not
        % Initialize resource grid
        updateGrid(obj);
      end
    end
    
    function sr = getSampleRate(obj)
      if isempty(obj.gridSet)
        % trigger a dummy generation to store Sample Rate in obj.gridSet:
        generateWaveform(obj);
      end
      if isfield(obj.gridSet, 'ResourceGrids')
        % Downlink using nrWavefomGenerator()
        tmp = [obj.gridSet.ResourceGrids.Info];
        sr = max([tmp.SampleRate]);      
      else
        sr = max(obj.gridSet.Info.SamplingRate);
      end
    end
    
    function str = getSampleRateStr(~)
        str = 'info.ResourceGrids(1).Info.SampleRate';
    end
    
    function framesChangedGUI(obj, ~)
      e = framesChanged(obj);
      if isempty(e)
        % Only update the grid if the previous call didn't throw any exception
        updateGrid(obj);
      end
    end

    function [configline, configParam] = getConfigParam(obj)
        configline = '';
        configParam = obj.configGenVar;
    end

    function userDataText = getUserDataText(~)
        userDataText = 'configuration';
    end

    function AppDochyperlink = getAppLink(~)
        AppDochyperlink = '<a href="matlab:helpview(fullfile(docroot,''toolbox'',''5g'',''helptargets.map''),''FiveGWaveformGenerator_app'')">5G Waveform Generator</a>';
    end
    
    function BlockDochyperlink = getBlockLink(~)
        BlockDochyperlink = 'helpview(fullfile(docroot,''toolbox'',''5g'',''helptargets.map''),''FiveGWaveformGenerator_block'')';
    end

    function e = framesChanged(obj, ~)
      e = []; % Initialize empty exception
      try
        val = obj.NumSubframes;
        validateattributes(val, {'numeric'}, {'positive', 'integer', 'scalar', 'real'}, '', 'number of subframes');
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateStatusBar(obj,e);
    end
    
    function symbolsChanged(obj, ~)
      e = []; % Initialize empty exception
      try
        val = obj.AllocatedSymbols;
        validateattributes(val, {'numeric'}, {'nonnegative', 'integer', 'vector', '<', 14}, '', 'allocated symbols');
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateStatusBar(obj,e);
    end
    
    function nCellChanged(obj, ~)
      e = []; % Initialize empty exception
      try
        val = obj.NCellID;
        validateattributes(val, {'numeric'}, {'nonnegative', 'real', 'integer', 'scalar', '<=', 1007}, '', 'cell identity');
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateStatusBar(obj,e);
    end
    
    function rntiChanged(obj, ~)
      e = []; % Initialize empty exception
      try
        val = obj.RNTI;
        validateattributes(val, {'numeric'}, {'real', 'integer', 'nonnegative', 'scalar', '<=', 2^16-1}, '', 'radio network temporary identified (RNTI)');
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateStatusBar(obj,e);
    end
    
    function frChangedBase(obj)
      if strcmp(obj.FrequencyRange, 'FR1')
        obj.ChannelBandwidthGUI.String = num2cell(wirelessWaveformGenerator.nr5G_Dialog.CBWFR1);
        obj.ChannelBandwidthGUI.Value = 8; % default at 50 MHz
        obj.SubcarrierSpacingGUI.Value = 1;
        obj.SubcarrierSpacingGUI.String = num2cell(wirelessWaveformGenerator.nr5G_Dialog.SCSFR1);
      else
        % FR2
        obj.ChannelBandwidthGUI.Value = 1;
        obj.ChannelBandwidthGUI.String = num2cell(wirelessWaveformGenerator.nr5G_Dialog.CBWFR2);
        obj.SubcarrierSpacingGUI.Value = 1;
        obj.SubcarrierSpacingGUI.String = num2cell(wirelessWaveformGenerator.nr5G_Dialog.SCSFR2);
      end
    end
    
    function frChanged(obj, ~)
      frChangedBase(obj);

      scsChanged(obj);
      
      updateInfo(obj);

      updateGrid(obj);
      updateChannelView(obj);
    end

    function DuplexModeChanged(obj)
      updateInfo(obj);

      updateGrid(obj);
    end
    
    function scsChanged(obj, ~)
      oldCBW = obj.ChannelBandwidth;
      % remove invalid Channel Bandwidth options
      if strcmp(obj.FrequencyRange, 'FR1')
        obj.ChannelBandwidthGUI.Value = 1; % to avoid out-of-bounds from previous value
        if obj.SubcarrierSpacing == 15
          obj.ChannelBandwidthGUI.String = num2cell(wirelessWaveformGenerator.nr5G_Dialog.CBWFR1(1:8));
        elseif obj.SubcarrierSpacing == 30
          obj.ChannelBandwidthGUI.String = num2cell(wirelessWaveformGenerator.nr5G_Dialog.CBWFR1);
        else % 60
          obj.ChannelBandwidthGUI.String = num2cell(wirelessWaveformGenerator.nr5G_Dialog.CBWFR1(2:end));
        end
      else %FR2
        obj.ChannelBandwidthGUI.Value = 1; % to avoid out-of-bounds from previous value
        if obj.SubcarrierSpacing == 60
          obj.ChannelBandwidthGUI.String = num2cell(wirelessWaveformGenerator.nr5G_Dialog.CBWFR2(1:end-1));
        else % 120
          obj.ChannelBandwidthGUI.String = num2cell(wirelessWaveformGenerator.nr5G_Dialog.CBWFR2);
        end
      end
      % attempt to restore old CBW value (if still offered)
      idx = find(strcmp(obj.ChannelBandwidthGUI.String, num2str(oldCBW)));
      if ~isempty(idx)
        obj.ChannelBandwidthGUI.Value = idx;
      %else
      % obj.ChannelBandwidthGUI.Value = 1; % performed aboc
      end

      updateInfo(obj);
    end
    
    function windowingChangedGUI(obj, ~)
      if strcmp(obj.WindowingSource, 'Auto')
        set([obj.WindowingPercentLabel obj.WindowingPercentGUI], 'Visible', 'off');
      else
        set([obj.WindowingPercentLabel obj.WindowingPercentGUI], 'Visible', 'on');
      end
      
      layoutUIControls(obj);
      layoutPanels(obj);
    end
    
    function b = hasWindowing(~)
      % Remove Tukey windowing from Transmitter tab for all 5G waveforms
      b = true;
    end
    
    function srChanged(obj, ~)
      % Custom sample rate edit field shows up conditionally (non-Auto SR source)
      if strcmp(obj.SampleRateSource, 'Auto')
        set([obj.SampleRateLabel obj.SampleRateGUI], 'Visible', 'off');
      else
        set([obj.SampleRateLabel obj.SampleRateGUI], 'Visible', 'on');
        if isempty(obj.SampleRate)  % This will read indirectly the GUI text to get a dialog sample rate
          obj.SampleRate = 5e7;     % And this will write the GUI text... don't show Auto ([]) when on Custom
        end
      end
    end
    function srChangedGUI(obj, ~)
      srChanged(obj, []);
      
      layoutUIControls(obj);
      layoutPanels(obj);
    end

    function phaseCompChanged(obj, ~)
      % Carrier Frequency edit field only shows when Phase Compensation
      % checkbox is unchecked.
      set([obj.CarrierFrequencyLabel obj.CarrierFrequencyGUI], ...
        'Visible', uiservices.logicaltoonoff(obj.PhaseCompensation));
    end
    function phaseCompChangedGUI(obj, ~)
      phaseCompChanged(obj, []);
      layoutUIControls(obj);
      layoutPanels(obj);
    end

    
    function n = get.Label(obj)
      n = obj.LabelGUI.String;
    end
    function set.Label(obj, val)
      obj.LabelGUI.String = val;
    end
    
    function fr = get.FrequencyRange(obj) 
      fr = obj.FrequencyRangeGUI.String{obj.FrequencyRangeGUI.Value};
      fr = fr(1:3); % FR1 or FR2, drop parenthesis containing value
    end
    function set.FrequencyRange(obj, val) 
      obj.FrequencyRangeGUI.Value = find(startsWith(obj.FrequencyRangeGUI.String, val)); 
      
      frChanged(obj);
    end
    
    function fr = get.ChannelBandwidth(obj) 
      fr = str2double(obj.ChannelBandwidthGUI.String{obj.ChannelBandwidthGUI.Value});
    end
    function set.ChannelBandwidth(obj, val) 
      obj.ChannelBandwidthGUI.Value = find(strcmp(obj.ChannelBandwidthGUI.String, num2str(val))); 
    end
    
    function scs = get.SubcarrierSpacing(obj) 
      scs = getSCS(obj);
    end
    function scs = getSCS(obj) % allow overrides
      scs = str2double(obj.SubcarrierSpacingGUI.String{obj.SubcarrierSpacingGUI.Value});
    end
    function set.SubcarrierSpacing(obj, val) 
      obj.SubcarrierSpacingGUI.Value = find(strcmp(obj.SubcarrierSpacingGUI.String, num2str(val))); 
      
      scsChanged(obj);
    end
    
    function fr = get.DuplexMode(obj) 
      if ~strcmp(obj.DuplexModeGUI.Style, 'text')
        fr = obj.DuplexModeGUI.String{obj.DuplexModeGUI.Value}; % dropdown
      else
        fr = obj.DuplexModeGUI.String;
      end
    end
    function set.DuplexMode(obj, val) 
      if ~strcmp(obj.DuplexModeGUI.Style, 'text')
        obj.DuplexModeGUI.Value = find(strcmp(obj.DuplexModeGUI.String, val)); %dropdown
      else
        obj.DuplexModeGUI.String = val;
      end
    end
    
    function n = get.NumSubframes(obj)
      n = evalin('base', obj.NumSubframesGUI.String);
    end
    function set.NumSubframes(obj, val)
      obj.NumSubframesGUI.String = num2str(val);
    end
    
    function n = get.NCellID(obj)
      n = evalin('base', obj.NCellIDGUI.String);
    end
    function set.NCellID(obj, val)
      obj.NCellIDGUI.String = num2str(val);
    end
    
    function n = get.RNTI(obj)
      n = evalin('base', obj.RNTIGUI.String);
    end
    function set.RNTI(obj, val)
      obj.RNTIGUI.String = num2str(val);
    end
    
    function n = get.AllocatedSymbols(obj)
      n = evalin('base', obj.AllocatedSymbolsGUI.String);
    end
    function set.AllocatedSymbols(obj, val)
      obj.AllocatedSymbolsGUI.String = ['[' num2str(val) ']'];
    end
    
    function fr = get.NLayers(obj) 
      fr = str2double(obj.NLayersGUI.String{obj.NLayersGUI.Value});
    end
    function set.NLayers(obj, val) 
      obj.NLayersGUI.Value = find(strcmp(obj.NLayersGUI.String, num2str(val))); 
    end
    
    function fr = get.WindowingSource(obj) 
      fr = obj.WindowingSourceGUI.String{obj.WindowingSourceGUI.Value};
    end
    function set.WindowingSource(obj, val) 
      obj.WindowingSourceGUI.Value = find(startsWith(obj.WindowingSourceGUI.String, val));
    end
    
    function n = get.WindowingPercent(obj)
      n = evalin('base', obj.WindowingPercentGUI.String);
    end
    function set.WindowingPercent(obj, val)
      if ~ischar(val)       % Pass in any text directly
         val = mat2str(val);
      end
      obj.WindowingPercentGUI.String = val;
    end
    
    function fr = get.SampleRateSource(obj) 
      fr = obj.SampleRateSourceGUI.String{obj.SampleRateSourceGUI.Value};
    end
    function set.SampleRateSource(obj, val)
      obj.SampleRateSourceGUI.Value = find(startsWith(obj.SampleRateSourceGUI.String, val));
    end

    % SampleRate is being used to store a number, including empty
    % - The underlying string and surrounding code should be capable of capturing this, including an empty
    % - The string representation should be presentable (and not double quotes etc)
    function n = get.SampleRate(obj)
      n = evalin('base', obj.SampleRateGUI.String);
    end
    function set.SampleRate(obj, val)
      if ~ischar(val)         % The input (val) might be text or numerical 
          val = mat2str(val);
      end
      obj.SampleRateGUI.String = val;
    end
    
    function v = get.PhaseCompensation(obj) 
      v = logical(obj.PhaseCompensationGUI.Value);
    end
    function set.PhaseCompensation(obj, val) 
      obj.PhaseCompensationGUI.Value = val;
    end
    
    function n = get.CarrierFrequency(obj)
      n = evalin('base', obj.CarrierFrequencyGUI.String);
    end
    function set.CarrierFrequency(obj, val)
      obj.CarrierFrequencyGUI.String = num2str(val);
    end
    
    function customVisualizations(obj, varargin)
      % Executes upon waveform generation to update all visuals
      
      % Resource Grid, channel view and 2D Spectrogram
      updateGrid(obj);
      updateChannelView(obj);
      update2DSpectrogram(obj);
    end
      
    function [wgc,gridset,waveResources] = updateGrid(obj)
      % Live grid update when a relevant property changes or waveform is
      % generated
      obj.Parent.WaveformGenerator.setStatus(''); % clear possibly previous messages
      gridset = [];
      waveResources = [];
      wgc = [];
      try
        wgc = getConfiguration(obj);
      catch 
        % App initialization
        return;
      end
      if isempty(obj.visualStates)
        % App initialization
        return;
      end
      try
        [gridset,msg,waveResources] = wirelessWaveformGenerator.internal.computeResourceGridPRB(wgc);
        if ~isempty(msg)
          warnInStatusBar(obj,msg);
        end
            
        % Update BWP grids and textbox callback
        updateBWPGrids(obj,wgc,gridset,waveResources);
        
      catch e
        warnInStatusBar(obj,e);
      end
    end

    function updateBWPGrids(obj,wgc,gridset,waveResources)

        % Modify channel instance names if needed for text box display
        waveResources = updateChannelNames(obj,waveResources);

        % Update all activated BWP grids
        for bwpIdx = 1:length(wgc.BandwidthParts)
            ax = getResourceGridAxes(obj,wgc.BandwidthParts{bwpIdx}.BandwidthPartID);
            if ~isempty(ax)
                wirelessWaveformGenerator.internal.plotResourceGrid(ax, wgc, gridset,obj.isDownlink, bwpIdx);
                updateResourceGridTextBoxCallback(ax,wgc,waveResources);
            end
        end
    end

    function ax = getResourceGridAxes(obj,bwpID)

        ax = [];
        name = ['Resource Grid (BWP#' num2str(bwpID) ')'];
        if obj.getVisualState(name)
            ax = obj.getVisualAxes(name);
        end

        for axIdx = 1:numel(ax)
            if ~contains(ax(axIdx).Tag,'REAxes')
                % We are interested in is the axes containing the
                % resource grid and not that containing the RE
                % mapping visualization.
                break;
            end
        end
        ax = ax(axIdx);

    end

    % Modify channel names in waveform resources for display in the PRB
    % resource grid text box. This method allows to customize the text
    % displayed in the text box for each channel. Subclasses may redefine
    % this method to adapt the text to their needs.
    function waveResources = updateChannelNames(~,waveResources)
    end

    function updateChannelView(obj)
        % Live channel view update when a relevant property changes or waveform
        % is generated
        try
            cfg = getConfiguration(obj);
        catch
            % App initialization
            return;
        end
        if isempty(obj.visualStates)
          % App initialization
          return;
        end
        try  
            % Channel View:
            if obj.getVisualState('Channel View')
                ax = obj.getVisualAxes('Channel View');
                wirelessWaveformGenerator.internal.plotCarriers(ax, cfg);
                ax.Toolbar.Visible = 'on'; % interactive toolbar
            end
        catch e
            warnInStatusBar(obj,e);
        end
    end

    function warnInStatusBar(obj, in)
      % Display messages as warnings in the status bar (bottom left)
      if isa(in,'MException')
        % Display caught errors
        msg = in.message;
      else
        msg = in;
      end
      obj.Parent.WaveformGenerator.setStatus(['Warning: ' msg]);
    end

    function updateStatusBar(obj,e)
      % Update the status bar to display any warning related to the current
      % configuration or to clear any previous warning that no longer apply
      try
        if nargin==1 || isempty(e)
          getConfiguration(obj);
          obj.Parent.WaveformGenerator.setStatus(''); % clear possibly previous messages
        else
          rethrow(e);
        end
      catch ME
        warnInStatusBar(obj,ME);
      end
    end

    function update2DSpectrogram(obj)
      
      cfg = getConfiguration(obj);

      % 2D Spectrogram
      if obj.getVisualState('2D Spectrogram') 
        ax = obj.getVisualAxes('2D Spectrogram');
        % The spectrogram is only for 5G presets, so only one BWP exists
        wirelessWaveformGenerator.internal.plotSpectrogram(ax, obj.gridSet, cfg.BandwidthParts{1})
        ax.Toolbar.Visible = 'on'; % interactive toolbar
      end

    end

    function resetCustomVisuals(obj)
      % Executed during New session, to initialize desired state of visuals
      
      updateGrid(obj);

    end
    
    function b = mayHaveEmptyTimePeriods(~)
      b = true;
      % TDD mode, and partial temporal PDSCH allocation (either in slots or symbols)
      % may cause a bad spectrum plot in the end
    end
    
    function str = getCustomFilterStr(obj)
      str = getBasicLPFilterStr(obj);
    end
    
    function str = getCatalogPrefix(~)
      str = 'nr5g:waveformGeneratorApp:';
    end
    
    %% MATLAB Code generation:
    function addGenerationCode(obj, sw)
      % The waveform generation command
      addcr(sw, ['[waveform,info] = nrWaveformGenerator(' obj.configGenVar ');']);
    end
          
    function props = props2ExcludeFromConfigGeneration(~)
      % Interface differs between GUI and programmatic API
      props = {'WindowingSource', 'WindowingPercent', 'SampleRateSource', 'SampleRate', ...
               'PhaseCompensation', 'CarrierFrequency'};
    end
    
    function addConfigCode(obj,sw)
        % Get waveform configuration. This is is an nrDLCarrierConfig for
        % 5G DL or a nrULCarrierConfig for 5G UL
        carrierCfg = getConfiguration(obj); 
        
        if isstruct(carrierCfg) 
            carrierCfg = nr5g.internal.wavegen.mapStruct2Obj(carrierCfg);
        end

        mCodeCfg = getWavegenMCodeGenerationStructure(carrierCfg,obj.configGenVar);
        wirelessWaveformGenerator.internal.add5GCarrierConfigMCode(sw,carrierCfg,mCodeCfg);
    end
    
    function o = offersCCDF(~)
      o = true;
    end
  end
  
  % Protect methods for derived classes
  methods (Access=protected)
    
    % Set modulation step related common values
    function restoreMods(obj)
        obj.WindowingSource   = 'Custom';   % Auto or Custom
        obj.WindowingPercent  = 0;          % Percentage of Windowing Samples / FFTLength
        obj.SampleRateSource  = 'Auto';     % Auto or Custom
        obj.SampleRate        = 5e7;        % Sample rate
        obj.PhaseCompensation = false;      % Checkbox
        obj.CarrierFrequency  = 3.5e9;      % Carrier Frequency for Phase Compensation (Hz)
    end

    % Set channel data source from the associated dialog
    function cfg = getConfigurationDataSource(obj,dschannel,cfg)
         chnumber = 1;
         genDialog = obj.getGenerationDialog;
         if ~isempty(genDialog)
            if any(strcmp(genDialog.InputSource, {'PN9', 'PN15'}))
              cfg.(dschannel){chnumber}.DataSource = genDialog.InputSource; % PN9 or PN15 - geared for the FRC preset sources only
            else
              cfg.(dschannel){chnumber}.DataSource = genDialog.InputValue; % User defined
            end
         end
    end    
    
    % Set the modulation step related common values
    function cfg = getConfigurationMods(obj,cfg)      
         if strcmp(obj.WindowingSource, 'Auto')
            windowing = [];
         else
            windowing = obj.WindowingPercent;
         end
         % Also account for difference in Sample Rate API
         if strcmp(obj.SampleRateSource, 'Auto')
            sr = [];
         else
            sr = obj.SampleRate;
         end
         if obj.PhaseCompensation
            cf = obj.CarrierFrequency;
         else
            cf = 0;
         end
         cfg.WindowingPercent = windowing;
         cfg.SampleRate = sr;
         cfg.CarrierFrequency = cf;    
    end   

  end
  
end

function mcodeCfg = getWavegenMCodeGenerationStructure(waveCfgObj,configGenVar)
    % Create a default MATLAB code generation structure
    mcodeCfg = wirelessWaveformGenerator.internal.mCodeGenerationConfig(waveCfgObj);
    
    % Variable name of nrXLCarrierConfig object is configured by the
    % configGenVar property
    mcodeCfg.VarName = configGenVar;
    
    % Specify headers and variable names for each object property
    % Add multiple lines by concatenating strings if necessary
    mcodeCfg.SCSCarriers{1}.SectionHeader = "%% SCS specific carriers";
    mcodeCfg.SCSCarriers{1}.InstanceHeader = "% Carrier";
    mcodeCfg.SCSCarriers{1}.VarName = 'scscarrier';
    
    mcodeCfg.BandwidthParts{1}.SectionHeader =  "%% Bandwidth Parts";
    mcodeCfg.BandwidthParts{1}.InstanceHeader = "% BWP";
    mcodeCfg.BandwidthParts{1}.VarName = 'bwp';
    
    if isa(waveCfgObj,'nrDLCarrierConfig')
        mcodeCfg.SSBurst.SectionHeader = "%% Synchronization Signals Burst";
        mcodeCfg.SSBurst.VarName = 'ssburst';
        
        mcodeCfg.CORESET{1}.SectionHeader = "%% CORESET and Search Space Configuration";
        mcodeCfg.CORESET{1}.InstanceHeader = "% CORESET";
        mcodeCfg.CORESET{1}.VarName = 'coreset';
        
        mcodeCfg.SearchSpaces{1}.SectionHeader = "% Search Spaces";
        mcodeCfg.SearchSpaces{1}.InstanceHeader = "% Search Space";
        mcodeCfg.SearchSpaces{1}.VarName = 'searchspace';
        
        mcodeCfg.PDCCH{1}.SectionHeader = "%% PDCCH Instances Configuration";
        mcodeCfg.PDCCH{1}.InstanceHeader = "% PDCCH";
        mcodeCfg.PDCCH{1}.VarName = 'pdcch';
        
        mcodeCfg.PDSCH{1}.SectionHeader = "%% PDSCH Instances Configuration";
        mcodeCfg.PDSCH{1}.InstanceHeader = "% PDSCH";
        mcodeCfg.PDSCH{1}.VarName = 'pdsch';
        
        mcodeCfg.PDSCH{1}.DMRS.SectionHeader = "% PDSCH DM-RS";
        mcodeCfg.PDSCH{1}.DMRS.VarName = 'DMRS';
        mcodeCfg.PDSCH{1}.DMRS.IncludeParentName = true;
        
        mcodeCfg.PDSCH{1}.PTRS.SectionHeader = "% PDSCH PT-RS";
        mcodeCfg.PDSCH{1}.PTRS.VarName = 'PTRS';
        mcodeCfg.PDSCH{1}.PTRS.IncludeParentName = true;
        
        mcodeCfg.PDSCH{1}.ReservedPRB{1}.SectionHeader = "% PDSCH Reserved PRB";
        mcodeCfg.PDSCH{1}.ReservedPRB{1}.VarName = 'ReservedPRB';
        mcodeCfg.PDSCH{1}.ReservedPRB{1}.IncludeParentName = true;
        
        mcodeCfg.CSIRS{1}.SectionHeader = "%% CSI-RS Instances Configuration";
        mcodeCfg.CSIRS{1}.InstanceHeader = "% CSI-RS";
        mcodeCfg.CSIRS{1}.VarName = 'csirs';
        
    else % UL
        
        mcodeCfg.PUSCH{1}.SectionHeader = "%% PUSCH Instances Configuration";
        mcodeCfg.PUSCH{1}.InstanceHeader = "% PUSCH";
        mcodeCfg.PUSCH{1}.VarName = 'pusch';
        
        mcodeCfg.PUSCH{1}.DMRS.SectionHeader = "% PUSCH DM-RS";
        mcodeCfg.PUSCH{1}.DMRS.VarName = 'DMRS';
        mcodeCfg.PUSCH{1}.DMRS.IncludeParentName = true;
        
        mcodeCfg.PUSCH{1}.PTRS.SectionHeader = "% PUSCH PT-RS";
        mcodeCfg.PUSCH{1}.PTRS.VarName = 'PTRS';
        mcodeCfg.PUSCH{1}.PTRS.IncludeParentName = true;
        
        mcodeCfg.PUCCH{1}.SectionHeader = "%% PUCCH Instances Configuration";
        mcodeCfg.PUCCH{1}.InstanceHeader = "% PUCCH";
        mcodeCfg.PUCCH{1}.VarName = 'pucch';
        
        mcodeCfg.SRS{1}.SectionHeader = "%% SRS Instances Configuration";
        mcodeCfg.SRS{1}.InstanceHeader = "% SRS";
        mcodeCfg.SRS{1}.VarName = 'srs';
        
    end
end

% Update resource grid text box callback with new waveform resources
function updateResourceGridTextBoxCallback(ax,wgc,waveResources)
    
    % Update resource grid text box callback with conflicts
    fig = ax.Parent;
    textBoxCallbackFun = @wirelessWaveformGenerator.internal.resourceGridTextboxCallback;
    fig.WindowButtonMotionFcn = {textBoxCallbackFun,wgc,waveResources};

end