classdef nr5G_FRC_UL_Dialog < wirelessWaveformGenerator.nr5G_FRC_Dialog
% Uplink FRC extension for the Wireless Waveform Generator App

%   Copyright 2019-2021 The MathWorks, Inc.
    
  properties (Dependent)
    PTRS
  end

  properties (Hidden)
    TitleString = getString(message('nr5g:waveformGeneratorApp:nrULFRCTitle'))    
    
    PTRSType = 'checkbox'
    PTRSGUI
    PTRSLabel
    
    infoClass = 'wirelessWaveformGenerator.nr5G_UL_InfoDialog'
  end
  
  methods (Static)
    function frcDropdown = getFRCOptions(mcsOrder, frStr)
      % construct customer visible text of FRCs, e.g., 'G-FR1-A1-1'
            
      frNum = str2double(frStr(end));
      
      % numMCS = 5; % the sections of Annex A in TS 38.104, e.g., A.1
      % row is section/MCS, column is FR (1 or 2)
                    %  FR1 FR2
      numFRCsPerMCS = [ 9   5 ;
                        6   0 ; 
                       32  12 ;
                       28  10 ;
                       14   5];
      
      numFRCs = numFRCsPerMCS(mcsOrder, frNum);
      frcDropdown = cell(numFRCs, 1);
      for frcNum = 1:numFRCs
        frcDropdown{frcNum} = ['G-' frStr '-A' num2str(mcsOrder) '-' num2str(frcNum) ' (' ...
          num2str(wirelessWaveformGenerator.internal.getSCSUplinkFRC(frNum, mcsOrder, frcNum)) ' kHz SCS, ' ...
          num2str(wirelessWaveformGenerator.internal.getNRBUplinkFRC(frNum, mcsOrder, frcNum)) ' RBs)'];
      end
    end
  end
  
  methods
    function obj = nr5G_FRC_UL_Dialog(parent)
      obj@wirelessWaveformGenerator.nr5G_FRC_Dialog(parent); % call base constructor
      obj.isDownlink = false;

      % Enable toolbar in grid fig if using Java version
      useJava = ~obj.Parent.WaveformGenerator.useAppContainer;
      if useJava
        % Enable interactive toolbars for grid fig:
        figName = 'Resource Grid (BWP#1)';
        ax = obj.getVisualAxes(figName);
        ax.Toolbar.Visible = 'on'; % interactive toolbar
      end
    end 
    
    function str = getIconDrawingCommand(obj)
        str = ['disp([' obj.configGenVar '.Label newline ...' newline ...
                '''Modulation: '' ' obj.configGenVar '.PUSCH{1}.Modulation newline ...' newline ...
                '''Bandwidth: '' num2str(' obj.configGenVar '.ChannelBandwidth) ''MHz'' newline ...' newline ...
                '''Subframes: '' num2str(' obj.configGenVar '.NumSubframes) ]);'];
    end
    
    function [blockName, maskTitleName, waveNameText] = getMaskTextWaveName(obj)
        blockName = ['5G NR ' obj.Parent.WaveformGenerator.pCurrentWaveformType];
        maskTitleName = ['5G NR ' obj.Parent.WaveformGenerator.pCurrentWaveformType ' Waveform Generator'];
        waveNameText = 'a 5G NR uplink FRC';
    end
    
    function adjustSpec(obj)
      obj.FRCDropDown = obj.getFRCOptions(1, 'FR1');
      
      % For MATLAB script generation
      obj.configGenFcn  = @nrULCarrierConfig;
      obj.configGenVar  = 'cfgULFRC';
    end
    
    function props = displayOrder(~)
      props = {'FrequencyRange'; 'MCS'; 'FRC'; 'ChannelBandwidth'; 'DuplexMode'; ...
        'NumSubframes'; 'NCellID'; 'RNTI'; 'PTRS'; ...
        'WindowingSource'; 'WindowingPercent'; 'SampleRateSource'; ...
        'SampleRate'; 'PhaseCompensation'; 'CarrierFrequency'};
    end

    function restoreDefaults(obj)
      obj.FrequencyRange    = 'FR1';
      obj.FRC               = 'G-FR1-A1-1';
      obj.MCS               = 'QPSK, R=1/3';
      obj.ChannelBandwidth  = 40;
      obj.DuplexMode        = 'FDD';
      obj.NumSubframes      = 10;
      obj.NCellID           = 1;
      obj.RNTI              = 1;
      obj.PTRS              = false;
      
      restoreMods(obj); 
    end
    
    function updateInfo(obj)
      try
        try
          cfg = getConfiguration(obj);
        catch
          % Initialization; possibly inconsistent state
          return
        end
        ch = cfg.PUSCH{1};
        
        infoDialog = obj.Parent.DialogsMap('wirelessWaveformGenerator.nr5G_UL_InfoDialog');
        infoDialog.SubcarrierSpacing      = cfg.SCSCarriers{1}.SubcarrierSpacing;
        infoDialog.AllocatedRBs           = length(ch.PRBSet);
        infoDialog.Modulation             = ch.Modulation;
        
        mcs = obj.MCS;
        idx = strfind(mcs, '=');
        rate = mcs(idx(1)+1:end);
        infoDialog.TargetCodeRateGUI.String = rate;
        
        infoDialog.DMRSConfigurationType  = ch.DMRS.DMRSConfigurationType;
        infoDialog.PUSCHMappingType       = ch.MappingType;
        infoDialog.DMRSAdditionalPosition = ch.DMRS.DMRSAdditionalPosition;
        if ch.MappingType == 'A'
          infoDialog.DMRSTypeAPosition = ch.DMRS.DMRSTypeAPosition;
        else
          infoDialog.DMRSTypeAPosition = 0;
        end
        infoDialog.NLayers                = ch.NumLayers;
        if ch.TransformPrecoding
          infoDialog.TransformPrecodingInfo = 'enabled';
        else
          infoDialog.TransformPrecodingInfo = 'disabled';
        end
        
        infoDialog.AllocatedSymbolsGUI.String = sprintf('[%d %d]',ch.SymbolAllocation);
        
        % Calculate Payload (bits) / Transport Block Size (TBS)
        % Use the same methodology used by waveform generators
        bwp = cfg.BandwidthParts{1};
        carrier = nrCarrierConfig('NCellID', cfg.NCellID, ...
                                  'SubcarrierSpacing', bwp.SubcarrierSpacing, ...
                                  'CyclicPrefix', bwp.CyclicPrefix, ...
                                  'NSizeGrid', bwp.NSizeBWP, ...
                                  'NStartGrid', bwp.NStartBWP);
        
        pusch = nrPUSCHConfig('NSizeBWP', bwp.NSizeBWP, ...
                              'NStartBWP', bwp.NStartBWP, ...
                              'Modulation', ch.Modulation, ...
                              'NumLayers', ch.NumLayers, ...
                              'NumAntennaPorts', ch.NumLayers, ...
                              'MappingType', ch.MappingType, ...
                              'SymbolAllocation', ch.SymbolAllocation, ...
                              'TransformPrecoding', ch.TransformPrecoding, ...
                              'TransmissionScheme', ch.TransmissionScheme, ...
                              'PRBSet', ch.PRBSet, ...
                              'NID', ch.NID, ...
                              'RNTI', ch.RNTI, ...
                              'DMRS', ch.DMRS);
                              
        [~, modinfo] = nrPUSCHIndices(carrier, pusch);
        infoDialog.PayloadSize  = nrTBS(ch.Modulation, ch.NumLayers, length(ch.PRBSet), modinfo.NREPerPRB, ch.TargetCodeRate, ch.XOverhead);
        
      catch exc
        % nrFRCUL may error
        obj.errorFromException(exc);
      end
    end
    
    % Get waveform parameter configuration
    function cfg = getConfiguration(obj)
        
      refObj = wirelessWaveformGenerator.internal.hNRReferenceWaveformGenerator(obj.FRC,obj.ChannelBandwidth,[],obj.DuplexMode,double(obj.NCellID));
      cfg = refObj.Config;
      
      % Applied additional properties
      cfg.ChannelBandwidth    = double(obj.ChannelBandwidth);
      cfg.NumSubframes        = double(obj.NumSubframes);
      cfg.PUSCH{1}.RNTI       = double(obj.RNTI);
      cfg.PUSCH{1}.EnablePTRS = logical(obj.PTRS);

      % Set the PUSCH data source 
      cfg = getConfigurationDataSource(obj,'PUSCH',cfg);
      % Set the modulation related parameters
      cfg = getConfigurationMods(obj,cfg);

    end
    
    function frChanged(obj, ~)
      % handle offered MCS:
      obj.MCSGUI.Value = 1;
      if strcmp(obj.FrequencyRange, 'FR1')
        obj.MCSGUI.String = obj.MCSDropDown;
      else
        % '16QAM, R=2/3' has no FRCs for FR2, do not offer this option
        obj.MCSGUI.String = obj.MCSDropDown([1 3:end]);
      end
      
      % update list of TMs for this frequency range:
      obj.FRCGUI.Value = 1; % to avoid out-of-bound from previous setting
      obj.FRCGUI.String = obj.getFRCOptions(obj.MCSGUI.Value, obj.FrequencyRange);
    
      frChanged@wirelessWaveformGenerator.nr5G_Dialog(obj);
      
      updateInfo(obj);
    end
    
    function mcsChanged(obj, ~)
      % update list of TMs for this frequency range:
      obj.FRCGUI.Value = 1; % to avoid out-of-bound from previous setting
      if strcmp(obj.FrequencyRange, 'FR1') || obj.MCSGUI.Value == 1
        trueIndex = obj.MCSGUI.Value;
      else
        trueIndex = obj.MCSGUI.Value + 1; % because the 2nd MCS is not offered for FR2
      end
      
      obj.FRCGUI.String = obj.getFRCOptions(trueIndex, obj.FrequencyRange);
      
      frcChanged(obj); % update Channel Bandwidth dropdown
      
      updateInfo(obj);

    end
    
    function frcChanged(obj, ~)
      % Get minimum number of RBs for the FRC and use that to determine the
      % minimum bandwidth allowed
      refObj = wirelessWaveformGenerator.internal.hNRReferenceWaveformGenerator(obj.FRC, [], [], [], [], "15.2.0");
      cfg = refObj.Config;
      numRBs = length(cfg.PUSCH{1}.PRBSet);
      scs = cfg.SCSCarriers{1}.SubcarrierSpacing;
      
      % find minimum BW needed
      if strcmp(obj.FrequencyRange, 'FR1')
        t = nr5g.internal.wavegen.getFR1BandwidthTable;
        bws = wirelessWaveformGenerator.nr5G_Dialog.CBWFR1;
      else % FR2
        t = nr5g.internal.wavegen.getFR2BandwidthTable;
        bws = wirelessWaveformGenerator.nr5G_Dialog.CBWFR2;
      end
      t2 = t([num2str(scs) 'kHz'], :);
      idx = find(t2.Variables>=numRBs, 1);
      BWstr = t2.Properties.VariableNames{idx};
      mHzIndex = strfind(BWstr, 'M');
      minBW = str2double(BWstr(4:mHzIndex-1));
      % Display only sufficiently large BWs for holding the RBs:
      vals = num2cell(bws((bws>=minBW) & (~isnan(t2.Variables)))); % do not include non-valid (NaN) CBW
      % Try to retain old value, in the new set of available options. If
      % not possible, default to the 1st one.
      valIdx = find(obj.ChannelBandwidth==[vals{:}]);
      if isempty(valIdx) 
        obj.ChannelBandwidthGUI.Value = 1;
      else
        obj.ChannelBandwidthGUI.Value = valIdx;
      end
      obj.ChannelBandwidthGUI.String = vals;            
      
      updateInfo(obj);

      updateGrid(obj);
      updateChannelView(obj);
    end
    
    function scs = getSCS(obj)
      refObj = wirelessWaveformGenerator.internal.hNRReferenceWaveformGenerator(obj.FRC);
      scs = refObj.Config.SCSCarriers{1}.SubcarrierSpacing;
    end
        
    function v = get.PTRS(obj) 
      if obj.PTRSGUI.Value
        v = true;
      else
        v = false;
      end
    end
    function set.PTRS(obj, val) 
      obj.PTRSGUI.Value = val;
    end
  end
end
