classdef nr5G_Full_Base_Dialog < wirelessWaveformGenerator.nr5G_Dialog & ...
                          wirelessWaveformGenerator.nr5G_CarrierBWP_Table                          
% Common functionality for both Downlink and Uplink Full 5G wavegen

%   Copyright 2020-2021 The MathWorks, Inc.
  
  properties (Hidden)
    % Handle tab transitions:
    ClientActionListener;   % Listener tracking clicks on specific tabs
    pxschExtraTile = false  % internal state indicating if right-side panel under PXSCH has/should be launched
    pxcchExtraTile = false  % internal state indicating if right-side panel under PUCCH has/should be launched
    lastFigInFocus = 'Main' % last figure to be in focus (before any possible future clicks)
    lastResourceGridInFocus = 'Resource Grid (BWP#1)';
    ErrorColor = [1 0.5 0.5];
  end
  
  properties (Constant)
    buttonWidth = 21        % Dimension(s) of add/remove buttons for all tables
    maxNumBWPFigs = 9       % An upper bound on BWP figs to be visualized
  end
  
  methods(Abstract)
    % DL/UL subclasses must implement these methods:
    createFigures(obj)
    createTables(obj)
    setExtraConfigFigVisibility(obj, visibility)
    figResizedCallback(obj)
    names = figureNames(obj)
    tag = figTag(obj, name)
    name = PXSCHfigureName(obj)
    mainFigTag(obj)
    pxschExtraFigTag(obj)
    pxcchExtraFigTag(obj)
    getChannelTable(obj,channel)
  end
  
  methods (Static)
    function hPropDb = getPropertySet(~)
      % Define visualizations that are specific to this waveform extension.
      % In this case, we have 
      % - one resource grid plot for each BWP (up to maxNumBWPFigs)
      % - Plus the "Channel View" plot
      resGrids = cellstr(strcat('Resource Grid (BWP#', num2str((1:wirelessWaveformGenerator.nr5G_Full_Base_Dialog.maxNumBWPFigs)'), ')'))';
      hPropDb = extmgr.PropertySet(...
        'Visualizations',   'mxArray', [resGrids {'Channel View'}]);
    end
  end
  
  methods
    function obj = nr5G_Full_Base_Dialog(parent, isDownlink)
      obj@wirelessWaveformGenerator.nr5G_Dialog(parent); % call base constructor
      
      obj.isDownlink = isDownlink;
      useJava = ~obj.Parent.WaveformGenerator.useAppContainer;
      
      %% Figures:
      createFigures(obj);
      mainFig = obj.Parent.WaveformGenerator.pParametersFig;
      mainFig.Name = 'Main'; % rename from "Waveform" to "Main"
      mainFig.AutoResizeChildren = 'off';
      mainFig.SizeChangedFcn = @(a, b) figResizedCallback(obj, []);
      obj.setExtraConfigFigVisibility(true);
      
      % Create listenener for understanding which tab we click:
      if ~useJava
        obj.ClientActionListener = addlistener(obj.Parent.WaveformGenerator.AppContainer, 'PropertyChanged', @(event, data) propertyChanged(obj, event, data));

        % Turn off panel folding; panels are stacked horizontally, not vertically
        cb = findall(obj.TitleGUI, 'Tag', 'PanelUnfoldedLabel');
        cb.ValueChangedFcn = @(a, b) keepPanelPresent(obj);
        img = findall(obj.TitleGUI, 'Tag', 'PanelUnfoldedImage');
        img.ImageClickedFcn = @(a, b) keepPanelPresent(obj);
      else
      obj.ClientActionListener = event.listener(obj.Parent.WaveformGenerator.ToolGroup, 'ClientAction', @obj.onClientAction);
        % Turn off panel folding; panels are stacked horizontally, not vertically
        obj.TitleGUI.Callback = @(a, b) keepPanelPresent(obj);
      end
      
      %% Tables:
      restoreDefaults(obj); % set ChannelBandwidth before creating tables, which need Carriers.NRB
      createTables(obj);
      
      %% Make Filtering dialog fixed-width. This is needed to prevent its
      % width to cover the SCS & BWP tables when it folds and then unfolds
      className = 'wirelessWaveformGenerator.FilteringDialog';
      dlg = obj.Parent.DialogsMap(className);
      dlg.panelFixedSize = true;
      
      if useJava
        % customize fold/unfold callback so that SCS/BWP tables are not hidden
        dlg.TitleGUI.Callback = @(hh, e) customFilterFoldCallback(obj);
        % same challenge when the Filter type changes:
        dlg.FilteringGUI.Callback = @(hh, e) customFilteringChangedGUI(obj);
      else
        % Turn off panel folding; panels are stacked horizontally, not vertically
        cb = findall(dlg.TitleGUI, 'Tag', 'PanelUnfoldedLabel');
        cb.ValueChangedFcn = @(a, b) keepPanelPresent(obj);
        img = findall(dlg.TitleGUI, 'Tag', 'PanelUnfoldedImage');
        img.ImageClickedFcn = @(a, b) keepPanelPresent(obj);
      end
      
      % SR is Auto ([]) by default:
      srChanged(obj, []);
      phaseCompChanged(obj, []);

      if useJava
        % Enable interactive toolbars for BWP#1 fig:
        figName = 'Resource Grid (BWP#1)';
        ax = obj.getVisualAxes(figName);
        ax.Toolbar.Visible = 'on'; % interactive toolbar 
      end
    end
    
    function adjustSpec(obj)
      adjustSpec@wirelessWaveformGenerator.nr5G_Dialog(obj);
      
      if obj.Parent.WaveformGenerator.useAppContainer
        % redirect top-level parameters (e.g., FR, CBW, Windowing +
        % Filtering) to dedicated main figure instead of left-side panel
        obj.createFig('Main',  mainFigTag(obj));

        updateParametersFig(obj);
      end
    end
    
    function updateParametersFig(obj)
      obj.Figure = obj.(mainFigTag(obj)); % this is where create UI Controls will place items
    end
    
    function createFig(obj, figTitle, figName, varargin)
      if isempty(varargin)
        figTag = figName;
      else
        figTag = varargin{1};
      end
      
      % Reusable figure constructor for all tabs
      if isempty(obj.(figName))
        appObj = obj.Parent.WaveformGenerator;
        if appObj.useAppContainer
          doc = appObj.AppContainer.getDocument("figureDocumentGroup", figTag);
          if isempty(doc)
            document = matlab.ui.internal.FigureDocument(...
                  'Title',  figTitle, ...
                  'Tag', figTag, ...
                  'DocumentGroupTag', 'figureDocumentGroup', ...
                  'Closable', false);
            addDocument(obj.Parent.WaveformGenerator.AppContainer, document);
            document.Figure.Tag = figTag;
            obj.(figName) = document.Figure;
            obj.(figName).AutoResizeChildren = 'off';
            obj.(figName).SizeChangedFcn = @(a, b) figResizedCallback(obj, []);
          else
            doc.Visible = true;
            obj.(figName) = doc.Figure;
          end
        else
          obj.(figName) = figure('Name', figTitle, 'NumberTitle','off', ...
              'HandleVisibility', 'off', 'Tag', figTag, 'SizeChangedFcn', @(a, b) figResizedCallback(obj, []));
          obj.Parent.WaveformGenerator.ToolGroup.addFigure(obj.(figName));
        end
      end
    end
    
    function config = getConfigurationForSave(obj)
      % Info to save is:
      % - 1 nrXLCarrierConfig object (workspace variables are not saved)
      % - plus filtering info (which retains workspace variables)
      config.waveform = getConfiguration(obj);
      config.filtering = getConfigurationForSave(obj.Parent.FilteringDialog);
      
      % Extra field needed for save/load sessions
      config.selectedPXSCH = obj.selectedPXSCH;
      config.selectedPXCCH = obj.selectedPXCCH;
    end
    
    function sOut = appendExportData(~, sIn)
      % Remove selectedPXSCH and selectedPXCCH fields from the exported data
      sOut = sIn;
      if isfield(sOut,'config')
          sOut.config = rmfield(sOut.config,{'selectedPXSCH','selectedPXCCH'});
      else
          % When exported to .bb, sIn does not have the config field
          sOut = rmfield(sOut,{'selectedPXSCH','selectedPXCCH'});
      end
    end
    
    function customLoadActions(obj, newData)
      obj.selectedPXSCH = newData.Waveform.Configuration.selectedPXSCH;
      if isfield(newData.Waveform.Configuration,'selectedPXCCH')
          obj.selectedPXCCH = newData.Waveform.Configuration.selectedPXCCH;
      else
          % A pre-21b session is loaded
          obj.selectedPXCCH = [];
      end
    end
    
    function setupDialog(obj)
      % Actions performed when entering this Full DL/UL wavegen extension:
      
      % turn off the listener while figures are enabled, so that client
      % actions (similar to clicking) are not triggered
      obj.ClientActionListener.Enabled = false; 
      obj.setExtraConfigFigVisibility(true);
      
      % Make SCS & BWP tables re-appear
      if ishghandle(obj.scsTable)
        set([obj.scsTable obj.bwpTable obj.addSCSButton obj.removeSCSButton ...
          obj.addBWPButton obj.removeBWPButton obj.bwpIDLabel obj.scsTitle obj.bwpTitle], 'Visible', 'on');
        moveSCSBWPTablesOnTop(obj);
      end
      
      useJava = ~obj.Parent.WaveformGenerator.useAppContainer;     
      if ~useJava 
        if any(obj.Parent.WaveformGenerator.AppContainer.getPanels{1}.WindowBounds<0)
          % App started from 5G shortcut in Apps gallery (not a transition from other waveform types).
          % Wait for AppContainer to load, so that hiding the left-side panel has an effect
          drawnow;
        end
        obj.Parent.WaveformGenerator.AppContainer.getPanels{1}.Opened = false; % hide the left-side panel
      end
    
      % start from the Main tab if user goes back to DL 5G from the in-App gallery:
      obj.lastFigInFocus = 'Main';
      figResizedCallback(obj); % needed once to size after figure placement
      if ishghandle(obj.scsTable)
        setupDialog@wirelessWaveformGenerator.nr5G_Dialog(obj);
      end
      
      figure(obj.Parent.WaveformGenerator.pParametersFig); % start from the Main tab
      drawnow; % so that PXSCH tab is not launched when we enter DL/UL      
      obj.ClientActionListener.Enabled = true; % now we can reenable the listener
      
      % Customize displayed visualization options (1 fig per BWP) + no TimeScope/ConstellationDiagram. 
      % 5G-specific behavior can possibly be merged with
      % wirelessWaveformGenerator.waveformGenerator.updateScopeOptions
      visualizeBtn = find(obj.Parent.WaveformGenerator.pWavegenTab, 'plots');
      visualizeBtn.DynamicPopupFcn =  @(a, b) updateScopeOptions5GFull(obj, []);
    end
    
    function resetCustomVisuals(obj)
      % Executed during New session, to initialize desired state of visuals
      resetCustomVisuals@wirelessWaveformGenerator.nr5G_Dialog(obj);
      
      markAllBrokenLinks(obj); 
      updateGrid(obj);
      updateChannelView(obj);
      updateREVisual(obj, 'Main', [], 'reset');
      
      % Enable interactive toolbar here, after CLF from super resetCustomVisual
      figName = 'Resource Grid (BWP#1)';
      ax = obj.getVisualAxes(figName);
      if ~isempty(ax.Toolbar) && ishghandle(ax.Toolbar)
        ax.Toolbar.Visible = 'on';
      end
      % only 1st BWP needs to be taken care, because this is called in new/waveformTypeChange

      % Make sure the Advanced configuration tabs for PXSCH and PUCCH are
      % not visible
      updatePXSCHSidePanel(obj, false)
      updatePXCCHSidePanel(obj, false)
    end

    function updateDisabled(obj)
        % Reset the Enable property of the 5G DL/UL add/remove buttons for
        % SCS and BWP after the app disabled them (e.g., after waveform
        % generation or export to Simulink)
        gobj = obj.Parent.WaveformGenerator.pParametersFig.Children;
        for idx = 1:length(gobj)
            % Disable the object for any of the following, otherwise enable it:
            % 1. addSCSButton: all possible SCS carriers are already added
            % 2. removeSCSButton: only a single SCS carrier is available
            % 3. removeBWPButton: only a single BWP is available
            if (contains(gobj(idx).Tag,'addSCS') && size(obj.scsTable.Data, 1) == numel(obj.scsTable.ColumnFormat{1})) || ...
                   (contains(gobj(idx).Tag,'removeSCS') && size(obj.scsTable.Data, 1) == 1) || ...
                   (contains(gobj(idx).Tag,'removeBWP') && size(obj.bwpTable.Data, 1) == 1)

                set(gobj(idx),'Enable',uiservices.logicalToOnOff(false));
            else
                set(gobj(idx),'Enable',uiservices.logicalToOnOff(true));
            end
        end
    end
    
    function outro(obj, newDialog)
      % Executed when moving to a new waveform type (e.g., 5G -> QAM), i.e., Cleanup
      
      if obj.Parent.WaveformGenerator.useAppContainer
        if ~isa(newDialog, 'wirelessWaveformGenerator.nr5G_DL_Dialog') && ...
           ~isa(newDialog, 'wirelessWaveformGenerator.nr5G_UL_Dialog')
           obj.Parent.WaveformGenerator.AppContainer.getPanels{1}.Opened = true; % restore the left-side panel
        end
        
        updateParametersFig(newDialog);
        obj.Parent.CurrentDialog = newDialog;
      end
      
      % Turn of figures specific to this feature when switching to a different waveform type
      obj.setExtraConfigFigVisibility(false);
      if obj.Parent.WaveformGenerator.useAppContainer
        % now Filtering Dialog got destroyed. Re-create it
        createFilteringDialog(obj.Parent);
      end
      
      % Hide tables from Main tab
      if ~obj.Parent.WaveformGenerator.useAppContainer
      set([obj.scsTable obj.bwpTable obj.addSCSButton obj.removeSCSButton ...
        obj.addBWPButton obj.removeBWPButton obj.bwpIDLabel obj.scsTitle obj.bwpTitle], 'Visible', 'off');
      end
      
      % Restore generic visualization options:
      visualizeBtn = find(obj.Parent.WaveformGenerator.pWavegenTab, 'plots');
      visualizeBtn.DynamicPopupFcn =  @(a, b) updateScopeOptions(obj.Parent.WaveformGenerator, []);
    end
    
    function popup = updateScopeOptions5GFull(obj, ~)
      % Executed when the "Visualize" button is clicked on the toolstrip
      % - do not offer post-OFDM time scope and constellation diagram
      % - also only offer already created BWP
      
      wavegenObj = obj.Parent.WaveformGenerator;
      
      popup = matlab.ui.internal.toolstrip.PopupList();
      sub_item1 = matlab.ui.internal.toolstrip.ListItemWithCheckBox('Spectrum Analyzer');
      sub_item1.ShowDescription = false;
      sub_item1.Value = wavegenObj.pPlotSpectrum;
      sub_item1.Tag = 'spectrumAnalyzer';
      sub_item1.ValueChangedFcn = @(a, b) visualChanged(wavegenObj, sub_item1);
      popup.add(sub_item1);
      
      bwps = cellfun(@str2double, obj.bwpTable.RowName);
      
      propSet = obj.getPropertySet();
      if ~isempty(propSet.findProperty('Visualizations'))
        visuals = propSet.getPropValue('Visualizations');
        for idx = 1:length(visuals)
          thisVis = visuals{idx};
          if startsWith(thisVis, 'Resource Grid') && ~any(str2double(thisVis(end-1)) == bwps)
            % do not offer Grid options for non-defined BWPs
            continue;
          end
          sub_item4 = matlab.ui.internal.toolstrip.ListItemWithCheckBox(thisVis);
          sub_item4.ShowDescription = false;
          sub_item4.Tag = obj.getFigureTag(thisVis);
          sub_item4.Value = obj.getVisualState(thisVis);
          sub_item4.ValueChangedFcn = @(a, b) visualChanged(wavegenObj, sub_item4);
          popup.add(sub_item4);
        end
        
        % CCDF
        sub_itemCCDF = matlab.ui.internal.toolstrip.ListItemWithCheckBox('CCDF');
        sub_itemCCDF.ShowDescription = false;
        sub_itemCCDF.Value = obj.Parent.WaveformGenerator.pPlotCCDF;
        sub_itemCCDF.Tag = 'CCDF';
        sub_itemCCDF.ValueChangedFcn = @(a, b) visualChanged(obj.Parent.WaveformGenerator, sub_itemCCDF);
        popup.add(sub_itemCCDF);
      end
    end
    
    function b = mayHaveEmptyTimePeriods(~)
      % This is needed to avoid finishing with a non-meaningful Spectrum visualization 
      b = true;
    end
    
    function defaultVisualLayout(obj)
      % By default, only 1 resource grid is shown as only 1 BWP exists
      % Also Channel View is turned off by default
      
      obj.setVisualState('Resource Grid (BWP#1)', true);
      for idx = 2:obj.maxNumBWPFigs
        obj.setVisualState(['Resource Grid (BWP#' num2str(idx) ')'], false);
      end
      obj.setVisualState('Channel View', true);
      
      % Set Main tab to focus on startup
      figure(obj.Parent.WaveformGenerator.pParametersFig);
    end
    
    function addRemoveButtons(obj, fig, tag)
      % Create 1 add and 1 remove button for a given table (tag)
      appObj = obj.Parent.WaveformGenerator;
      addCData = imread(getIconFile(matlab.ui.internal.toolstrip.Icon.ADD_16), 'BackgroundColor', fig.Color);
      obj.(['add' tag 'Button'])    = uicontrol('Parent', fig, 'Tag', ['add' tag 'Button'], 'Tooltip', getMsgString(obj, 'addButtonTT', tag), ...
        'Style', 'pushbutton', 'CData', addCData, 'Callback', @(a, b, c) addTableEntry(obj, [], tag));
      obj.(['remove' tag 'Button']) = uicontrol('Parent', fig, 'Tag', ['remove' tag 'Button'], 'Tooltip', getMsgString(obj, 'removeButtonTT', tag),...
        'Style', 'pushbutton', 'CData', getIcon(appObj, 'delete16'), 'Enable', 'off', 'Callback', @(a, b, c) removeTableEntry(obj, [], tag));
    end
    
    function layoutAddRemoveButtons(obj, table, tag)
      % For a given table (tag), position its add/remove buttons next to
      % its top-right corner.
      tableW  = table.Position(1) + table.Position(3);
      top     = table.Position(2) + table.Position(4);
      obj.(['add' tag 'Button']).Position     = [2+tableW  top-obj.buttonWidth     obj.buttonWidth obj.buttonWidth];
      obj.(['remove' tag 'Button']).Position  = [2+tableW  top-obj.buttonWidth*2.2 obj.buttonWidth obj.buttonWidth];
    end
    
    function addTableEntry(obj, ~, tag)
      % Executes when the Add button is clicked. Expects a newXRow implementation.
      newDefault = str2func(['new' tag 'Row']);
      obj.([lower(tag) 'Table']).Data = [obj.([lower(tag) 'Table']).Data; newDefault(obj)];

      % Update linking status for all tables. Not the minimum set of
      % operations, but delay is not noticeable
      markAllBrokenLinks(obj); 

      % Update grid and retrieve config conflicts
      updateGrid(obj);

    end
    
    function markAllBrokenLinks(~)
      % can be overriden by children classes
    end
    
    function selectTableEntry(obj, table, data)
      % Executes upon selection of table row(s). Cache selected row(s). If the
      % row(s) is/are valid (no broken link), then highlight as blue. If
      % the row(s) are red (have broken links) then highlight as purple, to
      % distinguish from non-selected red rows.
      
      str = 'Table';
      tag = table.Tag(1:end-length(str));
      if any(strcmp(tag,{'PDCCH','PUCCH'}))
          selectedTAG = 'selectedPXCCH';
      else
          selectedTAG = ['selected' tag];
      end
      if ~isempty(data.Indices)
        % Initialize colors etc.
        obj.(selectedTAG) = data.Indices(:, 1);
        N = size(obj.([lower(tag) 'Table']).Data, 1);
        colors = get([obj.([lower(tag) 'Table'])], 'BackgroundColor');
        errColor = obj.ErrorColor;
        selectedErrColor = obj.ErrorColor - [0.2 0 0];
        selectedColor = [0.97    0.97    1]; % blue-ish gray
        
        % Identify currently error rows
        errRows = find(ismember(colors, errColor, 'rows'));
        selectedErrRows = find(ismember(colors, selectedErrColor, 'rows'));
        
        % Initialize new output, all rows are white
        newColors = ones(N, 3);  % delete previous selection
        
        % maintain previous error rows
        newColors(errRows, :) = repmat(errColor, length(errRows), 1);
        
        % highlight selections with blue. some error rows are now overwritten
        newColors(obj.(selectedTAG), :) = repmat(selectedColor, length(obj.(selectedTAG)), 1);
        
        % Now combine error and blue rows to mark purple rows
        combinedErrRows = intersect(union(errRows, selectedErrRows), obj.(selectedTAG));
        newColors(combinedErrRows, :)  = repmat(selectedErrColor, length(combinedErrRows), 1);
        
        % deselected error rows now need to change color
        deselectedErrRows = setdiff(selectedErrRows, obj.(selectedTAG));
        newColors(deselectedErrRows, :)  = repmat(errColor, length(deselectedErrRows), 1);
        
        set([obj.([lower(tag) 'Table'])], 'BackgroundColor', newColors);        
      end
      
      % Disable the remove button when no row is selected, or when only 1 row exists.
      N = size(obj.([lower(tag) 'Table']).Data, 1);
      obj.(['remove' tag 'Button']).Enable = uiservices.logicaltoonoff(~isempty(obj.(selectedTAG)) && N>1 && N>size(obj.(selectedTAG), 1));
    end
    
    function removeTableEntry(obj, ~, tag)
      % Executes when the Remove button is clicked. Remove row and possibly
      % shift contents up. Also handle selections.
      
      if any(strcmp(tag,{'PDCCH','PUCCH'}))
          selectedTAG = 'selectedPXCCH';
      else
          selectedTAG = ['selected' tag];
      end
      obj.([lower(tag) 'Table']).Data(obj.(selectedTAG), :) = [];
      if ~strcmp(obj.([lower(tag) 'Table']).RowName, 'numbered') 
        % adjust IDs (e.g., in Bandwidth Parts table, without breaking links)
        obj.([lower(tag) 'Table']).RowName(obj.(selectedTAG), :) = [];
      end
      N = size(obj.([lower(tag) 'Table']).Data, 1);
            
      colors = ones(N, 3);

      % clear selections:
      obj.(selectedTAG) = [];
      set([obj.([lower(tag) 'Table'])], 'BackgroundColor', colors);
      obj.(['remove' tag 'Button']).Enable = 'off';
      
      % enable additions
      obj.(['add' tag 'Button']).Enable = 'on';

      % update broken links for shifted entries
      markAllBrokenLinks(obj);

      % Update grid 
      updateGrid(obj);
      updateREVisual(obj,tag);

    end        
    
    function markBrokenLinks(obj, tableToLink, tableThatLinks, colWithLinks)
      
      % paint red the rows that do not link to a valid ID (e.g., BWP)
      if ~isempty(tableThatLinks)
        possibleLinks = str2double(tableToLink.RowName(:)');
        currLinks = str2double(tableThatLinks.Data(:, colWithLinks))';
        d = setdiff(currLinks, possibleLinks);
        if isempty(d)
          idx = [];
        else
          idx = ismember(currLinks, d);
        end
        colors = ones(size(tableThatLinks.Data, 1), 3); % start white
        errorColor = obj.ErrorColor;
        colors(idx, :) = repmat(errorColor, sum(idx), 1); % red
        set(tableThatLinks,       'BackgroundColor', colors);
      end
    end
    function markBrokenSCSLinks(obj)
      % Needs a slightly different implementation than markBrokenLinks, as
      % we are not looking for IDs in row numbers; instead we are looking
      % for SCS column in scs Table.
      
      % paint red the rows that do not link to a valid SCS
      scs = (obj.scsTable.Data(:, 1)');
      if ~isempty(obj.bwpTable)
        scsIdx = 1;
        usedSCS = (obj.bwpTable.Data(:, scsIdx))';
        d = setdiff(usedSCS, scs);
        if isempty(d)
          idx = [];
        else
          idx = contains(usedSCS, setdiff(usedSCS, scs));
        end
        colors = ones(size(obj.bwpTable.Data, 1), 3); % start white
        errColor = obj.ErrorColor;
        colors(idx, :) = repmat(errColor, sum(idx), 1); % red
        set(obj.bwpTable, 'BackgroundColor', colors);
      end
    end

    
    function nextID = appendNextAvailableRowName(~, table)
      % This methods finds the smallest available row ID number. For
      % example, if 3 rows are added and the 2nd is deleted, then adding 
      % new rows will fill the gap, i.e., [1; 3; 2; 4].
      
      if ~isempty(table) && ~isempty(table.Data)
        % find the smallest available ID:
        curRows = strtrim(table.RowName);
        curRowsNum = cellfun(@(c) str2num(c), curRows); %#ok<ST2NM>
        possibleRows = 1:(length(curRows)+1);
        % Find which row numbers can now be used:
        nextID = setdiff(possibleRows, curRowsNum, 'stable');
        nextID = nextID(1); % get the 1st available one
      else
        nextID = 1;
        curRows = {};
      end
      table.RowName = [curRows; num2str(nextID)]; % Associate unique ID
    end
    
    
    function props = displayOrder(~)
      props = {'Label'; 'FrequencyRange'; 'ChannelBandwidth'; 'NCellID'; 'NumSubframes'; ...
               'WindowingSource'; 'WindowingPercent'; 'SampleRateSource'; ...
               'SampleRate'; 'PhaseCompensation'; 'CarrierFrequency'};
    end
    
    function waveform = generateWaveform(obj)
      % The actual waveform generation! Using nrWaveformGenerator.
      cfg = getConfiguration(obj);
      [waveform, obj.gridSet] = nrWaveformGenerator(cfg);
    end
    
    function applyConfiguration(obj, cfg)
      % Used on New and Open Session. 
      % Map a saved nrXLCarrierConfig object to the GUI elements of the
      % Main tab (only - for right here).
      obj.Label             = cfg.Label;
      obj.FrequencyRange    = cfg.FrequencyRange;
      obj.ChannelBandwidth  = cfg.ChannelBandwidth;
      obj.NCellID           = cfg.NCellID;
      obj.NumSubframes      = cfg.NumSubframes;
      
      % There are 3 differences between programmatic API and GUI:
      % 1. Windowing
      if isempty(cfg.WindowingPercent)
        obj.WindowingSource   = 'Auto';
        obj.WindowingPercent  = [];
      else
        obj.WindowingSource   = 'Custom';
        obj.WindowingPercent  = cfg.WindowingPercent;
      end
      set([obj.WindowingPercentLabel obj.WindowingPercentGUI], 'Visible', ...
        uiservices.logicaltoonoff(~isequal(cfg.WindowingPercent, [])));
      % 2. Sample Rate
      if isempty(cfg.SampleRate)
        obj.SampleRateSource   = 'Auto';
        obj.SampleRate         = [];
      else
        obj.SampleRateSource   = 'Custom';
        obj.SampleRate         = cfg.SampleRate;
      end
      set([obj.SampleRateLabel obj.SampleRateGUI], 'Visible', ...
        uiservices.logicaltoonoff(~isequal(cfg.SampleRate, [])));
      % 3. Carrier Frequency
      if cfg.CarrierFrequency == 0
        obj.PhaseCompensation  = false;
        obj.CarrierFrequency   = 3.5e9; % 3.5 GHz
      else
        obj.PhaseCompensation  = true;
        obj.CarrierFrequency   = cfg.CarrierFrequency;
      end
      set([obj.CarrierFrequencyLabel obj.CarrierFrequencyGUI], 'Visible', ...
        uiservices.logicaltoonoff(cfg.CarrierFrequency ~= 0));
      
      applySCSConfig(obj, cfg);
      applyBWPConfig(obj, cfg);
    end
        
    function frChanged(obj, ~)
      % Executed when frequency range changes (FR1<->FR2)
      frChangedBase(obj); % Actions that are common with 5G presets
      
      % Actions that specific to Full 5G wavegen:
      frChangedForSCSBWP(obj);
      
      % Make PXSCH full-band:
      if ~isempty(obj.scsTable) % not init
        prbSetColIdx = 11;
        gridSize = obj.scsTable.Data{1, 2};
        obj.pxschTable.Data{1, prbSetColIdx} = ['0:' num2str(gridSize-1)];
      end
      
      updateGrid(obj);
      updateChannelView(obj);
    end
    
    function bwChanged(obj, ~)
      updateChannelView(obj);
    end
    

    %% Tile Handling
    function cols = getNumTileColumns(obj, ~)
      % Tile architecture always uses 2 columns (under all tabs)
      cols = 2 + obj.Parent.WaveformGenerator.useAppContainer; % AppContainer layout removes 1 column, which is for the left-side panel
    end
    function rows = getNumTileRows(~, ~)
      % Tile architecture always uses 2 rows (under all tabs)
      rows = 2;
    end
    
    function w = getRowWeights(~, ~)
      % Rows are split evenly
      w = [0.5 0.5];
    end
    
    function tiles = getNumTiles(obj, ~)
      configTab = 1;
      resourceGrid = false;
      for idx = 1:obj.maxNumBWPFigs
        resourceGrid = resourceGrid || obj.getVisualState(['Resource Grid (BWP#' num2str(idx) ')']);
      end
      advancedRightPane = obj.pxschExtraTile || obj.pxcchExtraTile;
      spectrum = obj.Parent.WaveformGenerator.pPlotSpectrum;
      channelView = obj.getVisualState('Channel View');
      
      visualOrInfo = 1;
      tiles = configTab + visualOrInfo + double(advancedRightPane) + double(~advancedRightPane && resourceGrid && (spectrum || channelView));
    end
    
    function n = numVisibleFigs(obj)
      n = numVisibleFigs@wirelessWaveformGenerator.nr5G_Dialog(obj);
      n = n+obj.pxschExtraTile;
    end
    
    function sw = getTileOrientation(obj, sw, ~, ~)
      % Create the segment of an XML file, that creates tiles and their dimensions
      
      % Top-level Config (Main, PXSCH etc.)
      w = 2 - double(obj.pxschExtraTile || obj.pxcchExtraTile);
      sw.addcr(['<Tile Height="1" Width="' num2str(w) '" X="0" Y="0"/>']);
      
      % Visualizations Tile #1 (Resource Grid)
      % See if at least one resource grid is activated
      resourceGrid = false;
      for idx = 1:obj.maxNumBWPFigs
        resourceGrid = resourceGrid || obj.getVisualState(['Resource Grid (BWP#' num2str(idx) ')']);
      end
      secondVis = resourceGrid && (obj.Parent.WaveformGenerator.pPlotSpectrum || obj.getVisualState('Channel View'));
      if obj.pxschExtraTile || obj.pxcchExtraTile || secondVis
        sw.addcr('<Tile Height="1" Width="1" X="0" Y="1"/>');
      else
        sw.addcr('<Tile Height="1" Width="2" X="0" Y="1"/>');
      end
      
      if obj.pxschExtraTile || obj.pxcchExtraTile
        % PXSCH or PUCCH Advanced (right-side panel)
        sw.addcr('<Tile Height="2" Width="1" X="1" Y="0"/>');
      elseif secondVis
        % Spectrum Analyzer under all non-PXSCH, non-PUCCH tabs
        sw.addcr('<Tile Height="1" Width="1" X="1" Y="1"/>');
      end
    end
    
    function sw = getCustomTilePlacement(obj, sw)
      % Create the segment of an XML file, that specifies in which tile
      % each visual is positioned
      
      % Configuration tabs (Main, PXSCH etc.) go to the 1st tile (top left)
      wavegenObj = obj.Parent.WaveformGenerator;            
      names = figureNames(obj);
      inFocus = repmat({'no'}, 1, numel(names));      
      inFocus(strcmpi(names, obj.lastFigInFocus)) = {'yes'};
      for idx = 1:numel(names)
        sw.addcr(['<Occupant InFront="' inFocus{idx} '" Name="' names{idx} '" Tile="0"/>']);
      end
      
      % PXSCH extra tile (right-side advanced config) goes to right side col
      if obj.pxschExtraTile
        if ~isscalar(obj.selectedPXSCH)
          % no PXSCH or multiple PXSCH selected
          pxschNum = 1;
        else
          pxschNum = obj.selectedPXSCH;
        end
        sw.addcr(['<Occupant InFront="yes" Name="' PXSCHfigureName(obj) ' ' num2str(pxschNum) ' - Advanced Configuration" Tile="2"/>']);        
      end
      
      % PUCCH extra tile (right-side advanced config) goes to right side col
      if obj.pxcchExtraTile
        if ~isscalar(obj.selectedPXCCH)
            % no PXCCH or multiple PXCCH selected
          pucchNum = 1;
        else
          pucchNum = obj.selectedPXCCH;
        end
        sw.addcr(['<Occupant InFront="yes" Name="PUCCH ' num2str(pucchNum) ' - Advanced Configuration" Tile="2"/>']);        
      end
      
      % Dedicated tile for Resource Grid
      scopesTile = 1;
      for idx = 1:obj.maxNumBWPFigs
        figName = ['Resource Grid (BWP#' num2str(idx) ')'];
        if obj.getVisualState(figName)
            inFront = '"no"';
            if strcmpi(obj.lastResourceGridInFocus,figName)
                inFront = '"yes"';
            end
            sw.addcr(['<Occupant InFront=' inFront ' Name="Resource Grid (BWP#' num2str(idx) ')" Tile="' num2str(scopesTile) '"/>']);  
        end
      end
      
      % All visuals (Spectrum, Channel Plot) except for resource grid, bundled together in the last tile
      % See if any resource grid is enabled, to see if a secondary tile will be needed for Spectrum
      resourceGrid = false;
      for idx = 1:obj.maxNumBWPFigs
        resourceGrid = resourceGrid || obj.getVisualState(['Resource Grid (BWP#' num2str(idx) ')']);
      end
      % Spectrum is located in its own tile that is different than resource
      % grid, unless no grid is enabled, or PXSCH or PUCCH is activated
      scopesTile = 2-double(obj.pxschExtraTile || obj.pxcchExtraTile || ~resourceGrid);      
      
      sw.addcr('<Occupant InFront="yes" Name="Info" Tile="1"/>');
      if wavegenObj.pPlotSpectrum
        sw.addcr(['<Occupant InFront="no" Name="Spectrum Analyzer" Tile="' num2str(scopesTile) '"/>']);
      end
      
      if wavegenObj.pPlotCCDF
        sw.addcr(['<Occupant InFront="no" Name="CCDF" Tile="' num2str(scopesTile) '"/>']);
      end
     
      names = {'Channel View'};
      for k = 1:length(names)
        if obj.getVisualState(names{k})
            if obj.pxcchExtraTile || obj.pxschExtraTile
                inFront = 'no';
            else
                inFront = 'yes';
            end
            sw.addcr(['<Occupant InFront="' inFront '" Name="' obj.getFigureName(names{k}) '" Tile="' num2str(scopesTile) '"/>']);  
        end
      end
    end
    
    function updatePXCCHSidePanel(~, ~)
    end

    function updatePXSCHSidePanel(obj, flag)
      % Update the visibility of the PXSCH side panel containing the
      % advanced configuration tab
      obj.pxschExtraTile = flag;
      if obj.Parent.WaveformGenerator.useAppContainer
        obj.Parent.WaveformGenerator.AppContainer.getDocument('figureDocumentGroup', pxschExtraFigTag(obj)).Visible = flag;
      else
        obj.pxschSingleChannelFig.Visible = flag;
      end
    end
    
    function [tileCount, tileCoverage, tileOccupancy] = getTileLayout(obj, numScopes)
      
      tileCount = getNumTiles(obj);

      numTiles = getNumTiles(obj);
      if obj.pxschExtraTile
        tileCoverage = [1 2; 3 2];
      else
        if numTiles == 3
          tileCoverage = [1 1; 2 3];
        else
          tileCoverage = [1; 2];
        end
      end
      
      tileOccupancy = repmat(struct('children', []), numTiles, 1);
      
      tileID = 1;
      documentID = ['figureDocumentGroup_' mainFigTag(obj)];
      str = struct('showOrder', 1, 'id', documentID);
      tileOccupancy(tileID).children = [tileOccupancy(tileID).children str];
      tileOccupancy(tileID).showingChildId = ['figureDocumentGroup_' figTag(obj, obj.lastFigInFocus)];
      
      tileID = tileID + 1;
      
      if obj.pxschExtraTile
        if strcmp(obj.PXSCHfigureName, 'PDSCH')
          tag = 'pxsch';
        else
          tag = 'pusch';
        end
        documentID = ['figureDocumentGroup_' tag 'SingleChannelFig'];
        str = struct('showOrder', 1, 'id', documentID);
        tileOccupancy(tileID).children = [tileOccupancy(tileID).children str];
        tileOccupancy(tileID).showingChildId = documentID;
        
        tileOccupancy(1).showingChildId = ['figureDocumentGroup_' tag 'Fig'];

        tileID = tileID + 1;
      end

      if obj.pxcchExtraTile
        tileID = tileID + 1;
      end
      if ishghandle(obj.bwpTable)
        numBWP = min(obj.maxNumBWPFigs, size(obj.bwpTable.Data, 1));
      else
        numBWP = 1;
      end
      someBWPenabled = false;
      for bwpNo = 1:numBWP
        if obj.getVisualState(['Resource Grid (BWP#' num2str(bwpNo) ')'])
          documentID = ['figureDocumentGroup_ResourceGridBWP' num2str(bwpNo) ''];
          str = struct('showOrder', bwpNo, 'id', documentID);
          tileOccupancy(tileID).children = [tileOccupancy(tileID).children str];
          tileOccupancy(tileID).showingChildId = documentID;

          someBWPenabled = true;
        end
      end
      
      if ~obj.pxschExtraTile && ~obj.pxcchExtraTile && someBWPenabled
        tileID = tileID + 1;
      end     
  
      appObj = obj.Parent.WaveformGenerator;
      if appObj.pPlotSpectrum
        documentID = ['WorkingArea_ScopeComponent:' getClientId(appObj.pSpectrum1)];
        str = struct('showOrder', 1, 'id', documentID);
        tileOccupancy(tileID).children = [tileOccupancy(tileID).children str];
        tileOccupancy(tileID).showingChildId = documentID;
      end

      for k = 1:length(obj.visualNames)
        if obj.getVisualState(obj.visualNames{k})
          documentID = ['figureDocumentGroup_' obj.getFigureTag(obj.visualNames{k})];
          str = struct('showOrder', 1, 'id', documentID);
          tileOccupancy(tileID).children = [tileOccupancy(tileID).children str];
          tileOccupancy(tileID).showingChildId = documentID;
        end
      end
    end
    
    %%
    
    function customVisualizations(obj, varargin)
      % Executes upon waveform generation to update all visuals
      
      % Resource Grid and channel view
      updateGrid(obj);
      updateChannelView(obj);
    end

    function updateREVisual(obj, channelName, varargin)
        % Live update of the RE mapping grid when a relevant property
        % changes, waveform is generated, or the tab in focus changes

        resetFlag = false;
        if nargin==2 || isempty(varargin{1})
            % This is the result of a tab change. We need to retrieve the
            % selected row number and proper channel ID
            if contains(channelName,{'PDSCH','PUSCH'})
                selectedTAG = 'selectedPXSCH';
                tableName = 'pxschTable';
            elseif contains(channelName,{'PDCCH','PUCCH'})
                selectedTAG = 'selectedPXCCH';
                if contains(channelName,'PD')
                    tableName = 'pdcchTable';
                else
                    tableName = 'pucchTable';
                end
            elseif contains(channelName,'CSI-RS')
                selectedTAG = 'selectedCSIRS';
                tableName = 'csirsTable';
            elseif contains(channelName,'SRS')
                selectedTAG = 'selectedSRS';
                tableName = 'srsTable';
            else
                % Selected tab is Main or SS Burst
                selectedTAG = '';
            end
            if ~isempty(selectedTAG)
                selectedRow = obj.(selectedTAG);
                if isempty(selectedRow)
                    selectedRow = 1;
                end
                if ~isprop(obj,tableName) || ~ishghandle(obj.(tableName))
                    return
                end
                id = obj.(tableName).RowName{selectedRow}; % Get the row name ID
                channelName = [channelName ' ' id];
            else
                % Tab is Main or SS Burst: selectedRow is not needed.
                selectedRow = [];
            end

            if nargin==4
                resetFlag = matches(varargin{2},'reset');
            end
        else
            selectedRow = varargin{1};
        end

        try
            % Update all activated BWP grids
            for idx = 1:obj.maxNumBWPFigs
                name = ['Resource Grid (BWP#' num2str(idx) ')'];
                if obj.getVisualState(name)
                    ax = obj.getVisualAxes(name);
                    bwpIdx = find(strcmp(obj.bwpTable.RowName, num2str(idx)), 1);
                    if ~isempty(ax) && ~isempty(bwpIdx)
                        for axIdx = 1:numel(ax)
                            if ~contains(ax(axIdx).Tag,'REAxes')
                                % We are interested in the axes containing
                                % the resource grid and not that containing
                                % the RE mapping visualization.
                                break;
                            end
                        end
                        useJava = ~obj.Parent.WaveformGenerator.useAppContainer;
                        wirelessWaveformGenerator.internal.updateREMapping(obj,ax(axIdx),bwpIdx,channelName,selectedRow,resetFlag,useJava);
                    end
                end
            end
        catch e
            warnInStatusBar(obj,e);
        end
    end
    
    function updateGrid(obj)
        % Live grid update when a relevant property changes or waveform is
        % generated
        [wgc,gridset,waveResources] = updateGrid@wirelessWaveformGenerator.nr5G_Dialog(obj);

        try
            % If updateGrid in superclass succeeded
            if ~isempty(gridset)
                % Detect conflicts among channels/signals
                conflicts = nr5g.internal.wavegen.detectConflict(wgc,waveResources);

                % Update conflict visualization and textbox callback
                updateConflictResourceGrids(obj,wgc,conflicts);
                
                % Highlight conflicts in configuration tables
                markConflictInConfigTables(obj,conflicts);
            end

        catch e
            warnInStatusBar(obj,e);
        end

    end

    function figureAdded(obj,figName)
        % This method initializes/updates the figure before it is added to
        % the current visuals.
        figureAdded@wirelessWaveformGenerator.nr5G_Dialog(obj,figName);

        % Update the RE mapping too.
        if contains(figName(isletter(figName)),'ResourceGrid') % Java version includes spaces, Java-free does not
            % Initialize RE mapping
            updateREVisual(obj,obj.lastFigInFocus);
        end
    end
    
    % Modify channel names in waveform resources for display in the PRB
    % resource grid text box. The channel instances displayed reflect the
    % row number in the corresponding configuration table instead of the
    % position in the nrXLCarrierConfig object.
    function waveResources = updateChannelNames(dlg,waveResources)

        channelTypes = fieldnames(waveResources);

        for c = 1:numel(channelTypes)
            chType = channelTypes{c};
            channels = waveResources.(chType);

            t = getChannelTable(dlg,chType);
            if ~isempty(t)
                for ch = 1:length(channels)
                    channels(ch).Name = [chType,t.RowName{ch}];
                end
            end
            waveResources.(channelTypes{c}) = channels;
        end

    end

    function updateConflictResourceGrids(obj,wgc,conflicts)

        % Update conflict grids in activated BWPs
        for bwpIdx = 1:length(wgc.BandwidthParts)
            ax = getResourceGridAxes(obj,wgc.BandwidthParts{bwpIdx}.BandwidthPartID);
            if ~isempty(ax)
                wirelessWaveformGenerator.internal.plotResourceGridConflicts(ax,wgc,conflicts,bwpIdx,obj.ErrorColor);
                
                % Update resource grid text box callback with conflicts
                fig = ax.Parent;
                fig.WindowButtonMotionFcn{end+1} = conflicts;
            end
        end
        
    end
    
    function customFilterFoldCallback(obj)
      % Default layoutPanels creates a panel that is on top and hides SCS & BWP tables
      className = 'wirelessWaveformGenerator.FilteringDialog';
      dlg = obj.Parent.DialogsMap(className);
      
%       callbackCheckbox(dlg); % the actually intended method
      
      % move SCS & BWP tables on top (depth-wise):
      moveSCSBWPTablesOnTop(obj);
    end
    
    function customFilteringChangedGUI(obj)
      % Default layoutPanels creates a panel that is on top and hides SCS & BWP tables
      className = 'wirelessWaveformGenerator.FilteringDialog';
      dlg = obj.Parent.DialogsMap(className);
      
      filteringChangedGUI(dlg); % the actually intended method
      
      % move SCS & BWP tables on top (depth-wise):
      moveSCSBWPTablesOnTop(obj);
    end
    
    function layoutPanels(obj)
      layoutPanels@wirelessWaveformGenerator.nr5G_Dialog(obj);
      % layoutPanels sets a Panel covering the whole figure, hiding tables
      
      % move SCS & BWP tables on top (depth-wise):
      moveSCSBWPTablesOnTop(obj);

      % position SCS & BWP tables at right level:
      if obj.Parent.WaveformGenerator.useAppContainer
        drawnow; % for layoutPanels to finish
        pause(1);
      end
      margin = 20;
      verScrollBarSpace = 20;
      layoutSCSBWPTables(obj, margin, verScrollBarSpace);
      if obj.Parent.WaveformGenerator.useAppContainer
        drawnow;
      end
    end
    
    function parent = getParent(obj)
      % needed for distributed classes (nr5g_CarrierBWP_Table) to fetch protected property
      parent = obj.Parent;
    end
    
    function b = spectrumEnabled(~)
      b = true;
    end
    
    function cellDialogs = getDialogsPerColumn(obj)
      % Filtering panel on separate column, to handle for new top-level properties (Windowing etc.)
      cellDialogs{1} = {obj}; 
      cellDialogs{2} = {obj.Parent.FilteringDialog};
    end
    
    %% Listener handling
    function names = notPXSCHfigureNames(obj)
      % remove PXSCHfigureName(obj) from figureNames(obj)
      names = figureNames(obj);
      idx = strcmp(names, PXSCHfigureName(obj));
      names(idx) = [];
    end
    
    function names = notPUCCHfigureNames(obj)
      % remove 'PUCCH' from figureNames(obj)
      names = figureNames(obj);
      idx = strcmp(names, 'PUCCH');
      names(idx) = [];
    end
    
    function propertyChanged(obj, ~, data)
      
      if ~(strcmp(data.EventName, 'PropertyChanged') && strcmp(data.PropertyName, 'SelectedChild')) || ...
         ~(isa(obj.Parent.CurrentDialog, 'wirelessWaveformGenerator.nr5G_DL_Dialog')  || ...
           isa(obj.Parent.CurrentDialog, 'wirelessWaveformGenerator.nr5G_UL_Dialog')) || ...
          ~isa(obj.Parent.CurrentDialog,class(obj)) || ...
           isvalid(obj.Parent.WaveformGenerator.FreezeHandle) % still launching
        return;
      end

      newTab = obj.Parent.WaveformGenerator.AppContainer.SelectedChild.tag;
      switch newTab
        case {'mainDLFig', 'mainULFig'}
          tabName = 'Main';
        case 'ssBurstFig'
          tabName = 'SS Burst';
        case 'pdcchFig'
          tabName = 'PDCCH';
        case 'csirsFig'
          tabName = 'CSI-RS';
        case 'pxschFig'
          tabName = PXSCHfigureName(obj); % PDSCH or PUSCH
        case 'puschFig'
          tabName = 'PUSCH';
        case 'pucchFig'
          tabName = 'PUCCH';
        case 'srsFig'
          tabName = 'SRS';
        otherwise 
          tabName = 'Spectrum Analyzer'; % all other cases (e.g., Resource Grid) are treated the same
      end
      onTabChange(obj, tabName);
    end
    
    function onClientAction(obj, ~, ev)
      % Executes when, e.g., a click happens. Need to filter the clicks that
      % correspond to a tab change, so that we possibly change the PDSCH layout.
      
      if strcmpi(ev.EventData.EventType, 'ACTIVATED')
        if ~(isa(obj.Parent.CurrentDialog, 'wirelessWaveformGenerator.nr5G_DL_Dialog')  || ...
             isa(obj.Parent.CurrentDialog, 'wirelessWaveformGenerator.nr5G_UL_Dialog')) || ...
            ~isa(obj.Parent.CurrentDialog,class(obj)) || ...
             isempty(ev.EventData.Client)
          return;
        end

        newTabName = ev.EventData.Client.Name;
        onTabChange(obj, newTabName);
      end

    end
      
    function onTabChange(obj, newTabName)

        if strcmp(obj.lastFigInFocus, newTabName)
            % If the tab has not changed, no need to do anything
            return;
        end

        pxsch = PXSCHfigureName(obj);
        useJava = ~obj.Parent.WaveformGenerator.useAppContainer;

        firstTimeRightSide = false;
        if any(strcmpi(newTabName, notPXSCHfigureNames(obj)))
            % We clicked on a tab that is not PXSCH.
            if any(strcmpi(newTabName, 'PUCCH'))
                % We clicked on the PUCCH tab.
                updatePXCCHSidePanel(obj, true)

                % Create the PUCCH's right-side panels if this is the 1st time we
                % reach here.
                className = 'wirelessWaveformGenerator.nr5G_PUCCHAdvanced_Dialog';
                if ~isKey(obj.Parent.DialogsMap, className)
                    createPXCCHAdvancedTab(obj, 'PUCCH', className);
                    firstTimeRightSide = true;
                end
            else
                % We clicked on tab that is not PUCCH.
                updatePXCCHSidePanel(obj, false)
            end

            % Hide PUSCH after PUCCH is made visible. Thus, the right side
            % panel will always be maintained, and we can avoid 1 expensive
            % setScopeLayout call.
            updatePXSCHSidePanel(obj, false)

        elseif any(strcmpi(newTabName, pxsch))
            % We clicked on the PXSCH tab.
            updatePXSCHSidePanel(obj, true)
            updatePXCCHSidePanel(obj, false)

            % Create the PXSCH's right-side panels if this is the 1st time we
            % reach here.
            className = ['wirelessWaveformGenerator.nr5G_' pxsch 'Advanced_Dialog'];
            if ~isKey(obj.Parent.DialogsMap, className)
                firstTimeRightSide = true;
                % create 2nd param object, so that there is no interference with
                % 1st one in layoutPanels()
                param2 = wirelessWaveformGenerator.Parameters(obj.Parent.WaveformGenerator);

                % Advanced PXSCH properties:
                obj.Parent.DialogsMap(className) = eval([className '(param2, obj.pxschSingleChannelFig)']);
                dlg = obj.Parent.DialogsMap(className);
                param2.CurrentDialog = dlg; % needed for layoutPanels()

                % DMRS panel:
                className = ['wirelessWaveformGenerator.nr5G_' pxsch 'DMRS_Dialog'];
                obj.Parent.DialogsMap(className) = eval([className '(param2, obj.pxschSingleChannelFig)']);
                dlg2 = obj.Parent.DialogsMap(className);

                % PTRS panel:
                className = ['wirelessWaveformGenerator.nr5G_' pxsch 'PTRS_Dialog'];
                obj.Parent.DialogsMap(className) = eval([className '(param2, obj.pxschSingleChannelFig)']);
                dlg3 = obj.Parent.DialogsMap(className);

                % Arrange placements of all right-side PXSCH panels
                layoutUIControls(dlg);
                layoutUIControls(dlg2);
                layoutUIControls(dlg3);
                layoutPanels(dlg); % same for all 3
            end
        end

        % Do the re-layout but only when it is needed (towards or from
        % PXSCH or PUCCH but not in the PUSCH<->PUCCH transition)
        if ~any(startsWith(newTabName, {'Spectrum Analyzer', 'Resource Grid (BWP#1)', 'Channel ', [pxsch ' '], 'PUCCH '}))
            notPXSCHOrPUCCH = intersect(notPXSCHfigureNames(obj), notPUCCHfigureNames(obj));
            if firstTimeRightSide || ((any(strcmp(obj.lastFigInFocus, {pxsch,'PUCCH'})) && any(strcmp(newTabName, notPXSCHOrPUCCH))) || ...
                    (any(strcmp(newTabName, {pxsch, 'PUCCH'})) && any(strcmp(obj.lastFigInFocus, notPXSCHOrPUCCH))))
                % Transitions from Main <-> SSBurst <-> PDCCH <-> CSIRS do not cause layout changes
                % Similarly, PUSCH <-> PUCCH do not cause layout changes
                % Now we either entered PXSCH or PUCCH tab, or we left it for one of the
                % other configuration tabs.

                % Re-layout the tiles, but only if we are not in a startup/shut down state
                if isvalid(obj.Parent.WaveformGenerator) && ~(useJava && (~isvalid(obj.Parent.WaveformGenerator.ToolGroup) || ...
                        obj.Parent.WaveformGenerator.ToolGroup.isWaiting || isempty(getFramework(obj.Parent.WaveformGenerator.pSpectrum1))))
                    % do not attempt re-layout during startup/closing, java warnings may be thrown
                    % the spectrum analyzer check is also used to identify closing states
                    if any(strcmpi(newTabName, figureNames(obj)))
                        % do not do these when someone clicks on a scope

                        obj.lastFigInFocus = newTabName;
                        setScopeLayout(obj.Parent.WaveformGenerator);
                        if isgraphics(obj.Panel) % avoid calling layoutPanels during shutdown
                            layoutPanels(obj);
                            if strcmp(newTabName, pxsch)
                                className = ['wirelessWaveformGenerator.nr5G_' PXSCHfigureName(obj) 'Advanced_Dialog'];
                                dlg = obj.getParent.DialogsMap(className);
                                layoutPanels(dlg); % prevent messed up layout during render
                            elseif strcmp(newTabName, 'PUCCH')
                                className = 'wirelessWaveformGenerator.nr5G_PUCCHAdvanced_Dialog';
                                dlg = obj.getParent.DialogsMap(className);
                                layoutPanels(dlg); % prevent messed up layout during render
                            end
                        end
                    end
                end
            end
            obj.lastFigInFocus = newTabName;
        elseif any(startsWith(newTabName, 'Resource Grid'))
          obj.lastResourceGridInFocus = newTabName;
        end

        % Update RE mapping visualization, if the selected tab is a channel
        % or signal
        if ~any(startsWith(newTabName, {'Spectrum Analyzer', 'Resource Grid (BWP#', 'Channel ', [pxsch ' '], 'PUCCH '}))
            updateREVisual(obj, newTabName);
        end
    end
    
    function postSelectedTabChanged(obj)

        appObj = obj.Parent.WaveformGenerator;
        if ~appObj.useAppContainer
            % These are custom follow up actions when the Toolstrip tabs change (Generator <-> Transmitter)
            % Basically, we want to configure the visibility of all 2ndary
            % configuration tabs (e.g., PXSCH).

            inRadioTab = strcmp(appObj.ToolGroup.SelectedTab, 'transmitterTab');

            % Hide supplemental figures when entering the Transmitter Tab
            appObj.pParametersFig.Visible = ~inRadioTab;
            obj.setExtraConfigFigVisibility(~inRadioTab);
            layoutPanels(obj);
            if ~inRadioTab
                % bring back Main tab to focus
                figure(appObj.pParametersFig);
            end
        else
            % Java-free / AppContainer
            currTab = appObj.AppContainer.SelectedToolstripTab.tag;
            inRadioTab = strcmp(currTab, 'transmitterTab');
            if ~inRadioTab
                % Tranmitter->Generator => hide the left-side panel again
                for idx = 1:numel(appObj.AppContainer.getPanels)
                    appObj.AppContainer.getPanels{idx}.Opened = false;
                end
            end
        end
    end
    
    function createPXSCHAdvancedTab(obj, pxsch, className)
        % create 2nd param object, so that there is no interference with
        % 1st one in layoutPanels()
        param2 = wirelessWaveformGenerator.Parameters(obj.Parent.WaveformGenerator);

        % Advanced PXSCH properties:
        obj.Parent.DialogsMap(className) = eval([className '(param2, obj.pxschSingleChannelFig)']);
        dlg = obj.Parent.DialogsMap(className);
        param2.CurrentDialog = dlg; % needed for layoutPanels()

        % DMRS panel:
        className = ['wirelessWaveformGenerator.nr5G_' pxsch 'DMRS_Dialog'];
        obj.Parent.DialogsMap(className) = eval([className '(param2, obj.pxschSingleChannelFig)']);
        dlg2 = obj.Parent.DialogsMap(className);

        % PTRS panel:
        className = ['wirelessWaveformGenerator.nr5G_' pxsch 'PTRS_Dialog'];
        obj.Parent.DialogsMap(className) = eval([className '(param2, obj.pxschSingleChannelFig)']);
        dlg3 = obj.Parent.DialogsMap(className);

        % Arrange placements of all right-side PXSCH panels
        layoutUIControls(dlg);
        layoutUIControls(dlg2);
        layoutUIControls(dlg3);
        layoutPanels(dlg); % same for all 3
    end
    
    function createPXCCHAdvancedTab(obj, pxcch, className)
        % create 2nd param object, so that there is no interference with
        % 1st one in layoutPanels()
        param2 = wirelessWaveformGenerator.Parameters(obj.Parent.WaveformGenerator);

        % Advanced PUCCH properties:
        obj.Parent.DialogsMap(className) = eval([className '(param2, obj.pxcchSingleChannelFig)']);
        dlg = obj.Parent.DialogsMap(className);
        param2.CurrentDialog = dlg; % needed for layoutPanels()

        % UCI panel:
        className = ['wirelessWaveformGenerator.nr5G_' pxcch 'UCI_Dialog'];
        obj.Parent.DialogsMap(className) = eval([className '(param2, obj.pxcchSingleChannelFig)']);
        dlg2 = obj.Parent.DialogsMap(className);

        % Arrange placements of all right-side PXCCH panels
        layoutUIControls(dlg);
        layoutUIControls(dlg2);
        layoutPanels(dlg);
    end

    function markConflictInConfigTables(dialog,conflicts)
        % Change color of tables involved in a conflict among
        % channels/signals
        
        conflictColor = dialog.ErrorColor;
        for c = 1:length(conflicts)
            cfl = conflicts(c);

            % Change color of table row of the first config in conflict
            t = getChannelTable(dialog,cfl.ChannelType{1});
            if ~isempty(t)
                t.BackgroundColor(cfl.ChannelIdx(1),:) = conflictColor;
            end

            % Change color of table row of the second config in conflict
            t = getChannelTable(dialog,cfl.ChannelType{2});
            if ~isempty(t)
                t.BackgroundColor(cfl.ChannelIdx(2),:) = conflictColor;
            end
        end

    end
    
  end
end