classdef nr5G_PDSCHAdvanced_Dialog < wirelessWaveformGenerator.nr5G_PXSCHAdvanced_Dialog
% Interface to properties that are unique for PDSCH and are not present in the basic table

%   Copyright 2020-2021 The MathWorks, Inc.

  properties (Dependent)
    VRBToPRBInterleaving
    VRBBundleSize
    ReservedCORESET
    ReservedPRB
    ReservedSymbols
    ReservedPeriod
    TBScaling
  end  
  
  properties (Hidden)
    TitleString = 'PDSCH 1 (Advanced Configuration)'
    
    VRBToPRBInterleavingType = 'checkbox'
    VRBToPRBInterleavingLabel
    VRBToPRBInterleavingGUI
    VRBBundleSizeType = 'popupmenu'
    VRBBundleSizeDropDown = {'2', '4'}
    VRBBundleSizeLabel
    VRBBundleSizeGUI
    ReservedCORESETType = 'edit'
    ReservedCORESETLabel
    ReservedCORESETGUI
    ReservedPRBType = 'edit'
    ReservedPRBLabel
    ReservedPRBGUI
    ReservedSymbolsType = 'edit'
    ReservedSymbolsLabel
    ReservedSymbolsGUI
    ReservedPeriodType = 'edit'
    ReservedPeriodLabel
    ReservedPeriodGUI
    TBScalingType = 'popupmenu'
    TBScalingDropDown = {'0.25', '0.5', '1'}
    TBScalingLabel
    TBScalingGUI
  end
  
  methods
    function obj = nr5G_PDSCHAdvanced_Dialog(parent, fig)
      obj@wirelessWaveformGenerator.nr5G_PXSCHAdvanced_Dialog(parent, fig); % call base constructor
      
      % Callbacks needed either for validation of edit fields or for
      % changes to visibility of other controls
      obj.VRBToPRBInterleavingGUI.Callback  =  @(a,b) vrbChangedGUI(obj);
      obj.ReservedCORESETGUI.Callback       =  @(a,b) resCORESETChangedGUI(obj);
      obj.ReservedPRBGUI.Callback           =  @(a,b) resPRBChangedGUI(obj);
      obj.ReservedSymbolsGUI.Callback       =  @(a,b) resSymbolsChangedGUI(obj);
      obj.ReservedPeriodGUI.Callback        =  @(a,b) resPeriodChangedGUI(obj);
      
      vrbChangedGUI(obj);
    end
    
    function adjustDialog(obj)
      % Make sure all tags are unique. Otherwise there is a conflict with SSB
      obj.DataSourceGUI.Tag = 'PDSCHDataSource';
    end
    
    function vrbChangedGUI(obj, ~)
      % VRB Bundle Size is not visible when VRB interleaving is off
      set([obj.VRBBundleSizeGUI obj.VRBBundleSizeLabel], ...
                      'Visible', uiservices.logicalToOnOff(obj.VRBToPRBInterleaving));
      
      layoutUIControls(obj);
      layoutPanels(obj);
    end
    
    function resCORESETChangedGUI(obj, ~)
      % validation
      try
        % use error throwing behavior of nrWavegenPDSCHConfig
        c = nrWavegenPDSCHConfig;
        c.ReservedCORESET = obj.ReservedCORESET;
        
        % Do live grid update:
        updateGrids(obj);
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateAppStatusBar(obj);
    end
    function resPRBChangedGUI(obj, ~)
      % validation
      try
        % use error throwing behavior of nrPDSCHReservedConfig
        c = nrPDSCHReservedConfig;
        c.PRBSet = obj.ReservedPRB;
        
        % Do live grid update:
        updateGrids(obj);
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateAppStatusBar(obj);
    end
    function resSymbolsChangedGUI(obj, ~)
      % validation
      try
        % use error throwing behavior of nrPDSCHReservedConfig
        c = nrPDSCHReservedConfig;
        c.SymbolSet = obj.ReservedSymbols;
        
        % Do live grid update:
        updateGrids(obj);
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateAppStatusBar(obj);
    end
    function resPeriodChangedGUI(obj, ~)
      % validation
      try
        % use error throwing behavior of nrPDSCHReservedConfig
        c = nrPDSCHReservedConfig;
        c.Period = obj.ReservedPeriod;
        
        % Do live grid update:
        updateGrids(obj);
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateAppStatusBar(obj);
    end
    
    function props = displayOrder(~)
      props = {'Label'; 'VRBToPRBInterleaving'; 'VRBBundleSize'; 'ReservedCORESET'; ...
              'ReservedPRB'; 'ReservedSymbols'; 'ReservedPeriod'; 'TBScaling'; ...
              'XOverhead';  'RVSequence'; 'DataSource'; 'CustomDataSource'};
    end
    
    function restoreDefaults(obj)
      % Get the same defaults with programmatic objects
      c = nrWavegenPDSCHConfig;
      obj.Label                 = c.Label;
      obj.RVSequence            = c.RVSequence;
      obj.VRBToPRBInterleaving  = c.VRBToPRBInterleaving;
      obj.VRBBundleSize         = c.VRBBundleSize;
      obj.ReservedCORESET       = c.ReservedCORESET;
      obj.ReservedPRB           = c.ReservedPRB{1}.PRBSet;
      obj.ReservedSymbols       = c.ReservedPRB{1}.SymbolSet;
      obj.ReservedPeriod        = c.ReservedPRB{1}.Period;
      obj.TBScaling             = c.TBScaling;
      obj.XOverhead             = c.XOverhead;
      obj.DataSource            = c.DataSource;
      obj.CustomDataSource      = [1; 0; 0; 1];
    end
    
    % Getters/setters
    function v = get.VRBToPRBInterleaving(obj) 
      v = logical(obj.VRBToPRBInterleavingGUI.Value);
    end
    function set.VRBToPRBInterleaving(obj, val) 
      obj.VRBToPRBInterleavingGUI.Value = val;
    end
    
    function fr = get.VRBBundleSize(obj) 
      fr = str2double(obj.VRBBundleSizeGUI.String{obj.VRBBundleSizeGUI.Value});
    end
    function set.VRBBundleSize(obj, val) 
      obj.VRBBundleSizeGUI.Value = find(strcmp(obj.VRBBundleSizeGUI.String, num2str(val))); 
    end
        
    function n = get.ReservedCORESET(obj)
      n = evalin('base', obj.ReservedCORESETGUI.String);
    end
    function set.ReservedCORESET(obj, val)
      obj.ReservedCORESETGUI.String = mat2str(val);
    end
    
    function n = get.ReservedPRB(obj)
      n = evalin('base', obj.ReservedPRBGUI.String);
    end
    function set.ReservedPRB(obj, val)
      obj.ReservedPRBGUI.String = mat2str(val);
    end
    
    function n = get.ReservedSymbols(obj)
      n = evalin('base', obj.ReservedSymbolsGUI.String);
    end
    function set.ReservedSymbols(obj, val)
      obj.ReservedSymbolsGUI.String = mat2str(val);
    end
    
    function n = get.ReservedPeriod(obj)
      n = evalin('base', obj.ReservedPeriodGUI.String);
    end
    function set.ReservedPeriod(obj, val)
      obj.ReservedPeriodGUI.String = mat2str(val);
    end
    
    function fr = get.TBScaling(obj) 
      fr = str2double(obj.TBScalingGUI.String{obj.TBScalingGUI.Value});
    end
    function set.TBScaling(obj, val) 
      obj.TBScalingGUI.Value = find(strcmp(obj.TBScalingGUI.String, num2str(val))); 
    end
    
    % Stack Advanced/DMRS/PTRS vertically:
    function cellDialogs = getDialogsPerColumn(obj)
      cellDialogs{1} = {obj ...
        obj.Parent.WaveformGenerator.pParameters.DialogsMap('wirelessWaveformGenerator.nr5G_PDSCHDMRS_Dialog') ...
        obj.Parent.WaveformGenerator.pParameters.DialogsMap('wirelessWaveformGenerator.nr5G_PDSCHPTRS_Dialog') };    
    end
  end
end
