classdef nr5G_PDSCHPTRS_Dialog < wirelessWaveformGenerator.Dialog
% Interface to PDSCH PTRS properties

%   Copyright 2020-2021 The MathWorks, Inc.
  
  properties (Dependent)
    Enable
    TimeDensity
    FrequencyDensity
    REOffset
    PTRSPortSet
    Power
  end  
  
  properties (Hidden)
    TitleString = 'PDSCH 1 PT-RS'
    configFcn = @struct
    
    EnableType = 'checkbox'
    EnableGUI
    EnableLabel
    
    TimeDensityType = 'popupmenu'
    TimeDensityDropDown = {'1', '2', '4'}
    TimeDensityLabel
    TimeDensityGUI
    
    FrequencyDensityType = 'popupmenu'
    FrequencyDensityDropDown = {'2', '4'}
    FrequencyDensityLabel
    FrequencyDensityGUI
    
    REOffsetType = 'popupmenu'
    REOffsetDropDown = {'00', '01', '10', '11'}
    REOffsetLabel
    REOffsetGUI
    
    PTRSPortSetType = 'edit'    
    PTRSPortSetLabel
    PTRSPortSetGUI
        
    PowerType = 'edit'
    PowerGUI
    PowerLabel
  end
  
  methods
    function obj = nr5G_PDSCHPTRS_Dialog(parent, fig)
      obj@wirelessWaveformGenerator.Dialog(parent, fig); % call base constructor
      
      % Callbacks needed either for validation of edit fields, for
      % changes to visibility of other controls, or for grid live update
      obj.EnableGUI.Callback           = @(a,b) enabledChangedGUI(obj);
      obj.PowerGUI.Callback            = @(a,b) powerChangedGUI(obj);
      obj.PTRSPortSetGUI.Callback      = @(a,b) ptrsPortChangedGUI(obj);
      obj.TimeDensityGUI.Callback      = @(a,b) ptrsGridChangedGUI(obj);
      obj.REOffsetGUI.Callback         = @(a,b) ptrsGridChangedGUI(obj);
    end
    
    function adjustSpec(obj)
      % customization given the PDSCH max label length
      obj.LabelWidth = 150;
    end
    
    function adjustDialog(obj)
      % PT-RS is turned off by default, so hide everything
      set([obj.PowerLabel obj. PowerGUI obj.TimeDensityLabel obj.TimeDensityGUI ...
           obj.FrequencyDensityLabel obj.FrequencyDensityGUI...
           obj.REOffsetLabel obj.REOffsetGUI obj.PTRSPortSetLabel obj.PTRSPortSetGUI], ...
                      'Visible', 'off');
    end
    
    function props = displayOrder(~)
      props = {'Enable'; 'Power'; 'TimeDensity'; 'FrequencyDensity'; 'REOffset'; 'PTRSPortSet'};
    end
    
    function restoreDefaults(obj)
      % Get the same defaults with programmatic objects
      c = nrWavegenPDSCHConfig;
      obj.Power             = c.PTRSPower;
      obj.Enable            = c.EnablePTRS;
      
      c = nrPDSCHPTRSConfig;
      obj.TimeDensity       = c.TimeDensity;
      obj.FrequencyDensity  = c.FrequencyDensity;
      obj.REOffset          = c.REOffset;
      obj.PTRSPortSet       = c.PTRSPortSet;
    end
    
    function enabledChangedGUI(obj, ~)
      enabledChanged(obj);
      
      % synchronize with Enable checkbox in the PDSCH table
      dlg = obj.Parent.WaveformGenerator.pParameters.CurrentDialog;
      displayedID = getAdvancedPXSCHID(dlg);
      rowNames = dlg.pxschTable.RowName;
      idx = find(strcmp(num2str(displayedID), rowNames));
      dlg.pxschTable.Data{idx, end} = obj.Enable; %#ok<FNDSB>
    end
    
    function enabledChanged(obj)
      % Turn on or off all other knobs, based on the Enable flag
      oldVal = strcmp(obj.PowerGUI.Visible, 'on');
      props = displayOrder(obj);
      for idx = 1:numel(props)
        if ~strcmp(props{idx}, 'Enable')
          set([obj.([props{idx} 'Label']) obj.([props{idx} 'GUI'])], ...
                      'Visible', uiservices.logicalToOnOff(obj.Enable));
        end
      end
      
      if xor(oldVal, obj.Enable) 
        % do re-layout only when it is needed, because it is expensive
        layoutUIControls(obj);
        layoutPanels(obj);
      end

      % Update resource grid and RE mapping
      ptrsGridChangedGUI(obj);
    end
    
    function ptrsGridChangedGUI(obj, ~)
        % A property that affects the PT-RS RE mapping is changed.
        % Do live RE mapping update:
        dlg = obj.Parent.WaveformGenerator.pParameters.CurrentDialog;
        str = getTitle(obj);
        pxschID = extract(str,digitsPattern);
        channel = [str(1:length('PDSCH ')) pxschID{1}]; % channelName can be either PDSCH or PUSCH
        selectedRow = find(strcmpi(pxschID,dlg.pxschTable.RowName)); % Get the row name ID
        dlg.updateREVisual(channel, selectedRow);

        % Do live grid update to trigger conflict detection:
        dlg.updateGrid();
    end
    
    %% Validators
    function powerChangedGUI(obj)
      try
        % use error throwing behavior of nrWavegenPDSCHConfig
        c = nrWavegenPDSCHConfig;
        c.PTRSPower = obj.Power;
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateAppStatusBar(obj);
    end
    function ptrsPortChangedGUI(obj, ~)
      try
        % use error throwing behavior of nrPDSCHPTRSConfig
        c = nrPDSCHPTRSConfig;
        c.PTRSPortSet = obj.PTRSPortSet;
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateAppStatusBar(obj);
    end

    function updateAppStatusBar(obj)
      % Update the status bar to display any warning related to the current
      % configuration or to clear any previous warning that no longer apply
      dlg = obj.Parent.WaveformGenerator.pParameters.CurrentDialog;
      updateStatusBar(dlg);
    end
    
    %% Getters/setters
    function v = get.Enable(obj) 
      v = logical(obj.EnableGUI.Value);
    end
    function set.Enable(obj, val) 
      obj.EnableGUI.Value = val;
      
      enabledChanged(obj);
    end
                   
    function fr = get.TimeDensity(obj) 
      fr = str2double(obj.TimeDensityGUI.String{obj.TimeDensityGUI.Value});
    end
    function set.TimeDensity(obj, val) 
      obj.TimeDensityGUI.Value = find(strcmp(obj.TimeDensityGUI.String, num2str(val))); 
    end
    
    function fr = get.FrequencyDensity(obj) 
      fr = str2double(obj.FrequencyDensityGUI.String{obj.FrequencyDensityGUI.Value});
    end
    function set.FrequencyDensity(obj, val) 
      obj.FrequencyDensityGUI.Value = find(strcmp(obj.FrequencyDensityGUI.String, num2str(val))); 
    end
    
    function fr = get.REOffset(obj) 
      fr = obj.REOffsetGUI.String{obj.REOffsetGUI.Value};
    end
    function set.REOffset(obj, val) 
      obj.REOffsetGUI.Value = find(strcmp(obj.REOffsetGUI.String, val));
    end
    
    function n = get.PTRSPortSet(obj)
      n = evalin('base', obj.PTRSPortSetGUI.String);
    end
    function set.PTRSPortSet(obj, val)
      obj.PTRSPortSetGUI.String = mat2str(val);
    end
    
    function n = get.Power(obj)
      n = evalin('base', obj.PowerGUI.String);
    end
    function set.Power(obj, val)
      obj.PowerGUI.String = num2str(val);
    end

    function str = getCatalogPrefix(~)
      str = 'nr5g:waveformGeneratorApp:';
    end
  end
end
