classdef nr5G_PUCCHAdvanced_Dialog < wirelessWaveformGenerator.Dialog
% Interface to all PUCCH properties that are not present in the basic table

%   Copyright 2021 The MathWorks, Inc.
  
    properties (Dependent)
        % Common to all formats
        Format
        Label
        
        % Format-specific properties
        Modulation_f0
        Modulation_f1
        Modulation_f2
        Modulation_f34
        FrequencyHopping
        SecondHopStartPRB
        PUCCHGroupHopping
        HoppingID
        InitialCyclicShift
        SpreadingFactor
        OCCI_f1
        OCCI_f4
        NID0
        AdditionalDMRS
        DMRSPower
    end
  
  properties (Hidden)
    TitleString = 'PUCCH 1 (Advanced Configuration)'
    configFcn = @struct
    
    FormatType = 'text'
    FormatGUI
    FormatLabel
    
    LabelType = 'edit'
    LabelLabel
    LabelGUI
    
    Modulation_f0Type = 'popupmenu'
    Modulation_f0DropDown = {'Z-Chu'}
    Modulation_f0GUI
    Modulation_f0Label
    
    Modulation_f1Type = 'popupmenu'
    Modulation_f1DropDown = {'BPSK', 'QPSK'}
    Modulation_f1GUI
    Modulation_f1Label
    
    Modulation_f2Type = 'popupmenu'
    Modulation_f2DropDown = {'QPSK'}
    Modulation_f2GUI
    Modulation_f2Label
    
    Modulation_f34Type = 'popupmenu'
    Modulation_f34DropDown = {'pi/2-BPSK', 'QPSK'}
    Modulation_f34Label
    Modulation_f34GUI
    
    FrequencyHoppingType = 'popupmenu'
    FrequencyHoppingDropDown = {'neither', 'intraSlot', 'interSlot'}
    FrequencyHoppingLabel
    FrequencyHoppingGUI
    
    SecondHopStartPRBType = 'edit'
    SecondHopStartPRBLabel
    SecondHopStartPRBGUI
    
    PUCCHGroupHoppingType = 'popupmenu'
    PUCCHGroupHoppingDropDown = {'neither', 'enable', 'disable'}
    PUCCHGroupHoppingLabel
    PUCCHGroupHoppingGUI
    
    HoppingIDType = 'edit'
    HoppingIDLabel
    HoppingIDGUI
    
    InitialCyclicShiftType = 'popupmenu'
    InitialCyclicShiftDropDown = {'0','1','2','3','4','5','6','7','8','9','10','11'}
    InitialCyclicShiftLabel
    InitialCyclicShiftGUI
    
    SpreadingFactorType = 'popupmenu'
    SpreadingFactorDropDown = {'2', '4'}
    SpreadingFactorLabel
    SpreadingFactorGUI
    
    OCCI_f1Type = 'popupmenu'
    OCCI_f1DropDown = {'0','1','2','3','4','5','6'}
    OCCI_f1Label
    OCCI_f1GUI
    
    OCCI_f4Type = 'popupmenu'
    OCCI_f4DropDown = {'0','1','2','3'}
    OCCI_f4Label
    OCCI_f4GUI
    
    NID0Type = 'edit'
    NID0Label
    NID0GUI
    
    AdditionalDMRSType = 'checkbox'
    AdditionalDMRSLabel
    AdditionalDMRSGUI
    
    DMRSPowerType = 'edit'
    DMRSPowerLabel
    DMRSPowerGUI
  end
  
  methods
    function obj = nr5G_PUCCHAdvanced_Dialog(parent, fig)
      obj@wirelessWaveformGenerator.Dialog(parent, fig); % call base constructor
      
      % Callbacks needed either for validation of edit fields or for
      % changes to visibility of other controls
      obj.LabelGUI.Callback              = @(a,b) labelChangedGUI(obj);
      obj.Modulation_f34GUI.Callback     = @(a,b) modChangedGUI(obj);
      obj.FrequencyHoppingGUI.Callback   = @(a,b) freqHopChangedGUI(obj);
      obj.SecondHopStartPRBGUI.Callback  = @(a,b) secondHopChangedGUI(obj);
      obj.PUCCHGroupHoppingGUI.Callback  = @(a,b) groupHopChangedGUI(obj);
      obj.HoppingIDGUI.Callback          = @(a,b) hoppingIDChangedGUI(obj);
      obj.InitialCyclicShiftGUI.Callback = @(a,b) initCSChangedGUI(obj);
      obj.OCCI_f1GUI.Callback            = @(a,b) occiChangedGUI(obj);
      obj.OCCI_f4GUI.Callback            = @(a,b) occiChangedGUI(obj);
      obj.NID0GUI.Callback               = @(a,b) nid0ChangedGUI(obj);
      obj.AdditionalDMRSGUI.Callback     = @(a,b) additionalDMRSChangedGUI(obj);
      obj.DMRSPowerGUI.Callback          = @(a,b) dmrsPowerChangedGUI(obj);
      
      % Disable user interaction for the Modulation dropdown menu for
      % formats 0, 1, and 2.
      obj.Modulation_f0GUI.Enable = 'off';
      obj.Modulation_f1GUI.Enable = 'off';
      obj.Modulation_f2GUI.Enable = 'off';
      
      % Set default visibility
      formatChangedGUI(obj, 1);
    end
    
    function adjustSpec(obj)
      % customization given the PUCCH max label length
      obj.LabelWidth = 150;
    end
    
    function adjustDialog(obj)
      % SecondHopStart, HoppingID do not appear by default:
      set([obj.SecondHopStartPRBLabel obj.SecondHopStartPRBGUI, ...
           obj.HoppingIDLabel obj.HoppingIDGUI], ...
                 'Visible', 'off');
      % Make sure all tags are unique. Otherwise there is a conflict with
      % other channels
      obj.LabelGUI.Tag = 'PUCCHLabel';
      obj.FrequencyHoppingGUI.Tag = 'PUCCHFrequencyHopping';
      obj.SecondHopStartPRBGUI.Tag = 'PUCCHSecondHopStartPRB';
      % Decorate the tags for the remaining properties with the channel name
      obj.Modulation_f0GUI.Tag = 'PUCCHModulation_f0';
      obj.Modulation_f1GUI.Tag = 'PUCCHModulation_f1';
      obj.Modulation_f2GUI.Tag = 'PUCCHModulation_f2';
      obj.Modulation_f34GUI.Tag = 'PUCCHModulation_f34';
      obj.OCCI_f1GUI.Tag = 'PUCCHOCCI_f1';
      obj.OCCI_f4GUI.Tag = 'PUCCHOCCI_f4';
      obj.FormatGUI.Tag = 'PUCCHFormat';
      obj.HoppingIDGUI.Tag = 'PUCCHHoppingID';
      obj.InitialCyclicShiftGUI.Tag = 'PUCCHInitialCyclicShift';
      obj.SpreadingFactorGUI.Tag = 'PUCCHSpreadingFactor';
      obj.NID0GUI.Tag = 'PUCCHNID0';
      obj.AdditionalDMRSGUI.Tag = 'PUCCHAdditionalDMRS';
      obj.DMRSPowerGUI.Tag = 'PUCCHDMRSPower';
    end
    
    function props = displayOrder(~)
      props = {'Format'; 'Label'; 'Modulation_f0'; 'Modulation_f1'; 'Modulation_f2'; 'Modulation_f34'; ...
               'FrequencyHopping'; 'SecondHopStartPRB'; 'PUCCHGroupHopping'; 'HoppingID'; ...
               'InitialCyclicShift'; 'SpreadingFactor'; 'OCCI_f1'; 'OCCI_f4'; ...
               'NID0'; 'DMRSPower'; 'AdditionalDMRS'};
    end
    
    function restoreDefaults(obj)
      % Get the same defaults with programmatic objects
      c = nrWavegenPUCCH0Config;
      c2 = nrWavegenPUCCH2Config;
      c4 = nrWavegenPUCCH4Config;
      obj.Format                  = 'Format 0';
      obj.Label                   = 'PUCCH1';
      obj.Modulation_f0           = obj.Modulation_f0DropDown{1};
      obj.Modulation_f1           = obj.Modulation_f1DropDown{1};
      obj.Modulation_f2           = obj.Modulation_f2DropDown{1};
      obj.Modulation_f34          = obj.Modulation_f34DropDown{2};
      obj.FrequencyHopping        = c.FrequencyHopping;
      obj.SecondHopStartPRB       = c.SecondHopStartPRB;
      obj.PUCCHGroupHopping       = c.GroupHopping;
      obj.HoppingID               = c.HoppingID;
      obj.InitialCyclicShift      = c.InitialCyclicShift;
      obj.SpreadingFactor         = c4.SpreadingFactor;
      obj.OCCI_f1                 = c4.OCCI; % Default for format 1 and 4 are the same
      obj.OCCI_f4                 = c4.OCCI;
      obj.NID0                    = c2.NID0;
      obj.AdditionalDMRS          = c4.AdditionalDMRS;
      obj.DMRSPower               = c4.DMRSPower;
      
      obj.FormatGUI.String = obj.Format(end);
      formatChangedGUI(obj, 1);
      labelChangedGUI(obj, []);
    end
    
    function labelChangedGUI(obj,~)
        % Update the Label read-only cell in the main PUCCH table
        dlg = obj.Parent.WaveformGenerator.pParameters.CurrentDialog;
        str = getTitle(obj);
        id = extract(str,digitsPattern);
        id = strcmpi(id,dlg.pucchTable.RowName); % Get the row name ID
        dlg.pucchTable.Data{id,1} = obj.Label;
    end
    
    function modChangedGUI(obj,~)
        switch obj.FormatGUI.String
            case '0'
                modulation = obj.Modulation_f0;
            case '1'
                modulation = obj.Modulation_f1;
            case '2'
                modulation = obj.Modulation_f2;
            otherwise
                modulation = obj.Modulation_f34;
        end
        % Update the Modulation read-only cell in the main PUCCH table
        dlg = obj.Parent.WaveformGenerator.pParameters.CurrentDialog;
        str = getTitle(obj);
        id = extract(str,digitsPattern);
        id = strcmpi(id,dlg.pucchTable.RowName); % Get the row name ID
        dlg.pucchTable.Data{id,6} = modulation;
    end
    
    function formatChangedGUI(obj,isNewRow)

        allProps = displayOrder(obj);
        switch obj.FormatGUI.String
            case '0'
                % Property list of format 0
                props = {'Format', 'FrequencyHopping', 'SecondHopStartPRB', 'PUCCHGroupHopping', 'HoppingID', 'InitialCyclicShift', 'Modulation_f0'};
                
                if isNewRow
                    % Assign the default value to format 0 properties
                    c = nrWavegenPUCCH0Config;
                    fnames = fieldnames(c);
                    fnames = fnames(ismember(fnames,props));
                    for n = 1:length(fnames)
                        obj.(fnames{n}) = c.(fnames{n});
                    end
                    obj.PUCCHGroupHopping = c.GroupHopping;
                    obj.Modulation_f0 = obj.Modulation_f0DropDown{1};
                end
            case '1'
                % Property list of format 1
                props = {'Format', 'FrequencyHopping', 'SecondHopStartPRB', 'PUCCHGroupHopping', 'HoppingID', ...
                    'InitialCyclicShift', 'OCCI_f1', 'DMRSPower', 'Modulation_f1'};
                
                if isNewRow
                    % Assign the default value to format 1 properties
                    c = nrWavegenPUCCH1Config;
                    fnames = fieldnames(c);
                    fnames = fnames(ismember(fnames,props));
                    for n = 1:length(fnames)
                        obj.(fnames{n}) = c.(fnames{n});
                    end
                    obj.PUCCHGroupHopping = c.GroupHopping;
                    obj.OCCI_f1 = c.OCCI;

                    % Get the modulation scheme based on the number of UCI bits
                    className = 'wirelessWaveformGenerator.nr5G_PUCCHUCI_Dialog';
                    if ~isKey(obj.Parent.WaveformGenerator.pParameters.DialogsMap, className)
                        % Assume default value for NumUCIBits of 1
                        obj.Modulation_f1 = 'BPSK';
                    else
                        % Get the number of UCI bits from the UCI panel
                        dlg = obj.Parent.WaveformGenerator.pParameters.DialogsMap(className);
                        if dlg.NumUCIBits_f01==2
                            obj.Modulation_f1 = 'QPSK';
                        else
                            obj.Modulation_f1 = 'BPSK';
                        end
                    end
                end
                
            case '2'
                % Property list of format 2
                props = {'Format', 'FrequencyHopping', 'SecondHopStartPRB', ...
                    'DMRSPower', 'NID0', 'Modulation_f2'};
                
                if isNewRow
                    % Assign the default value to format 2 properties
                    c = nrWavegenPUCCH2Config;
                    fnames = fieldnames(c);
                    fnames = fnames(ismember(fnames,props));
                    for n = 1:length(fnames)
                        obj.(fnames{n}) = c.(fnames{n});
                    end
                    obj.Modulation_f2 = 'QPSK';
                end
            case '3'
                % Property list of format 3
                props = {'Format', 'Modulation_f34', 'FrequencyHopping', 'SecondHopStartPRB', ...
                    'DMRSPower', 'AdditionalDMRS', 'PUCCHGroupHopping', 'HoppingID'};
                
                if isNewRow
                    % Assign the default value to format 3 properties
                    c = nrWavegenPUCCH3Config;
                    fnames = fieldnames(c);
                    fnames = fnames(ismember(fnames,props));
                    for n = 1:length(fnames)
                        obj.(fnames{n}) = c.(fnames{n});
                    end
                    obj.PUCCHGroupHopping = c.GroupHopping;
                    obj.Modulation_f34 = c.Modulation;
                end
            otherwise % format 4
                % Property list of format 4
                props = {'Format', 'Modulation_f34', 'FrequencyHopping', 'SecondHopStartPRB', ...
                    'DMRSPower', 'AdditionalDMRS', 'PUCCHGroupHopping', 'HoppingID', ...
                    'SpreadingFactor', 'OCCI_f4'};
                
                if isNewRow
                    % Assign the default value to format 4 properties
                    c = nrWavegenPUCCH4Config;
                    fnames = fieldnames(c);
                    fnames = fnames(ismember(fnames,props));
                    for n = 1:length(fnames)
                        obj.(fnames{n}) = c.(fnames{n});
                    end
                    obj.PUCCHGroupHopping = c.GroupHopping;
                    obj.Modulation_f34 = c.Modulation;
                    obj.OCCI_f4 = c.OCCI;
                end
        end
        
        % Make applicable properties visible
        props{end+1} = 'Label';
        for n = 1:length(props)
            set([obj.([props{n} 'Label']) obj.([props{n} 'GUI'])], ...
                'Visible', uiservices.logicalToOnOff(true));
        end

        % Hide non-applicable properties
        notProps = allProps(~ismember(allProps,props));
        for n = 1:length(notProps)
            set([obj.([notProps{n} 'Label']) obj.([notProps{n} 'GUI'])], ...
                'Visible', uiservices.logicalToOnOff(false));
        end

        % Adjust visibility for dependent properties common to all formats
        freqHopChangedGUI(obj, []);
        if ~strcmp(obj.FormatGUI.String, '2')
            groupHopChangedGUI(obj, []);
        end
        
        % do re-layout only when it is needed, because it is expensive
        layoutUIControls(obj);
        layoutPanels(obj);
    end
    
    function freqHopChangedGUI(obj, ~)
      oldVis = strcmpi(obj.SecondHopStartPRBGUI.Visible, 'on');
      newVis = ~strcmpi(obj.FrequencyHopping, 'neither');
      if xor(oldVis, newVis)
        set([obj.SecondHopStartPRBLabel obj.SecondHopStartPRBGUI], ...
                 'Visible', uiservices.logicalToOnOff(newVis));
        
        % do re-layout only when it is needed, because it is expensive
        layoutUIControls(obj);
        layoutPanels(obj);
      end
      
      % Do live grid update:
      updateGrids(obj);
    end
    
    function secondHopChangedGUI(obj, ~)
      try
        % use error throwing behavior of nrWavegenPUCCH0Config
        c = nrWavegenPUCCH0Config;
        c.SecondHopStartPRB = obj.SecondHopStartPRB;
        
        % Do live grid update:
        updateGrids(obj);
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateAppStatusBar(obj);
    end
    
    function groupHopChangedGUI(obj, ~)
      oldVis = strcmpi(obj.HoppingIDGUI.Visible, 'on');
      newVis = ~strcmpi(obj.PUCCHGroupHopping, 'neither');
      if xor(oldVis, newVis)
        set([obj.HoppingIDLabel obj.HoppingIDGUI], ...
                 'Visible', uiservices.logicalToOnOff(newVis));
        
        % do re-layout only when it is needed, because it is expensive
        layoutUIControls(obj);
        layoutPanels(obj);
      end
    end
    
    function hoppingIDChangedGUI(obj, ~)
      try
        % use error throwing behavior of nrWavegenPUCCH0Config as all
        % formats will have the same behaviour
        c = nrWavegenPUCCH0Config;
        c.HoppingID = obj.HoppingID;
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateAppStatusBar(obj);
    end
    
    function initCSChangedGUI(obj, ~)
      try
        % use error throwing behavior of nrWavegenPUCCH0Config as all
        % formats will have the same behaviour
        c = nrWavegenPUCCH0Config;
        c.InitialCyclicShift = obj.InitialCyclicShift;
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateAppStatusBar(obj);
    end
    
    function occiChangedGUI(obj, ~)
      try
        % use error throwing behavior of nrWavegenPUCCH1Config and
        % nrWavegenPUCCH4Config, depending on which format is selected
        if strcmp(obj.Format,'1')
            c = nrWavegenPUCCH1Config;
            c.OCCI = obj.OCCI_f1;
        else % format 4
            c = nrWavegenPUCCH4Config;
            c.SpreadingFactor = obj.SpreadingFactor;
            c.OCCI = obj.OCCI_f4;
            validateConfig(c);
        end
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateAppStatusBar(obj);
    end
    
    function nid0ChangedGUI(obj, ~)
      try
        % use error throwing behavior of nrWavegenPUCCH2Config
        c = nrWavegenPUCCH2Config;
        c.NID0 = obj.NID0;
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateAppStatusBar(obj);
    end
    
    function additionalDMRSChangedGUI(obj, ~)
      try
        % use error throwing behavior of nrWavegenPUCCH3Config
        c = nrWavegenPUCCH3Config;
        c.AdditionalDMRS = obj.AdditionalDMRS;
        
        % Do live grid update:
        updateGrids(obj);
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateAppStatusBar(obj);
    end
    
    function dmrsPowerChangedGUI(obj, ~)
      try
        % use error throwing behavior of nrWavegenPUCCH1Config
        c = nrWavegenPUCCH1Config;
        c.DMRSPower = obj.DMRSPower;
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateAppStatusBar(obj);
    end

    function updateGrids(obj)
      % A property that affects the resource grid or the RE mapping is changed.
      dlg = obj.Parent.WaveformGenerator.pParameters.CurrentDialog;

      % Do live grid update:
      dlg.updateGrid();

      % Do live RE mapping update:
      str = getTitle(obj);
      pucchID = extract(str,digitsPattern);
      channel = [str(1:length('PUCCH ')) pucchID{1}];
      selectedRow = find(strcmpi(pucchID,dlg.pucchTable.RowName)); % Get the row name ID
      dlg.updateREVisual(channel,selectedRow);
    end

    function updateAppStatusBar(obj)
      % Update the status bar to display any warning related to the current
      % configuration or to clear any previous warning that no longer apply
      dlg = obj.Parent.WaveformGenerator.pParameters.CurrentDialog;
      updateStatusBar(dlg);
    end
    
    
    %% Getters/setters
    function n = get.Format(obj)
      n = obj.FormatGUI.String;
    end
    function set.Format(obj, val)
      obj.FormatGUI.String = val;
    end
    
    function n = get.Label(obj)
      n = obj.LabelGUI.String;
    end
    function set.Label(obj, val)
      obj.LabelGUI.String = val;
    end
    
    function fr = get.Modulation_f0(obj) 
      fr = obj.Modulation_f0GUI.String{obj.Modulation_f0GUI.Value};
    end
    function set.Modulation_f0(obj, val) 
      obj.Modulation_f0GUI.Value = find(strcmp(obj.Modulation_f0GUI.String, val)); 
    end
    
    function fr = get.Modulation_f1(obj) 
      fr = obj.Modulation_f1GUI.String{obj.Modulation_f1GUI.Value};
    end
    function set.Modulation_f1(obj, val) 
      obj.Modulation_f1GUI.Value = find(strcmp(obj.Modulation_f1GUI.String, val)); 
    end
    
    function fr = get.Modulation_f2(obj) 
      fr = obj.Modulation_f2GUI.String{obj.Modulation_f2GUI.Value};
    end
    function set.Modulation_f2(obj, val) 
      obj.Modulation_f2GUI.Value = find(strcmp(obj.Modulation_f2GUI.String, val)); 
    end
    
    function fr = get.Modulation_f34(obj) 
      fr = obj.Modulation_f34GUI.String{obj.Modulation_f34GUI.Value};
    end
    function set.Modulation_f34(obj, val) 
      obj.Modulation_f34GUI.Value = find(strcmp(obj.Modulation_f34GUI.String, val)); 
    end
    
    function fr = get.FrequencyHopping(obj) 
      fr = obj.FrequencyHoppingGUI.String{obj.FrequencyHoppingGUI.Value};
    end
    function set.FrequencyHopping(obj, val) 
      obj.FrequencyHoppingGUI.Value = find(strcmp(obj.FrequencyHoppingGUI.String, val)); 
    end
    
    function n = get.SecondHopStartPRB(obj)
      n = evalin('base', obj.SecondHopStartPRBGUI.String);
    end
    function set.SecondHopStartPRB(obj, val)
      obj.SecondHopStartPRBGUI.String = num2str(val);
    end
    
    function fr = get.PUCCHGroupHopping(obj) 
      fr = obj.PUCCHGroupHoppingGUI.String{obj.PUCCHGroupHoppingGUI.Value};
    end
    function set.PUCCHGroupHopping(obj, val) 
      obj.PUCCHGroupHoppingGUI.Value = find(strcmp(obj.PUCCHGroupHoppingGUI.String, val)); 
    end
    
    function n = get.HoppingID(obj)
      n = evalin('base', obj.HoppingIDGUI.String);
    end
    function set.HoppingID(obj, val)
      obj.HoppingIDGUI.String = mat2str(val);
    end
    
    function n = get.InitialCyclicShift(obj)
      n = str2double(obj.InitialCyclicShiftGUI.String{obj.InitialCyclicShiftGUI.Value});
    end
    function set.InitialCyclicShift(obj, val)
      obj.InitialCyclicShiftGUI.Value = find(strcmp(obj.InitialCyclicShiftGUI.String, num2str(val)));
    end
    
    function n = get.SpreadingFactor(obj)
      n = str2double(obj.SpreadingFactorGUI.String{obj.SpreadingFactorGUI.Value});
    end
    function set.SpreadingFactor(obj, val)
      obj.SpreadingFactorGUI.Value = find(strcmp(obj.SpreadingFactorGUI.String, num2str(val)));
    end
    
    function n = get.OCCI_f1(obj)
      n = str2double(obj.OCCI_f1GUI.String{obj.OCCI_f1GUI.Value});
    end
    function set.OCCI_f1(obj, val)
      obj.OCCI_f1GUI.Value = find(strcmp(obj.OCCI_f1GUI.String, num2str(val)));
    end
    
    function n = get.OCCI_f4(obj)
      n = str2double(obj.OCCI_f4GUI.String{obj.OCCI_f4GUI.Value});
    end
    function set.OCCI_f4(obj, val)
      obj.OCCI_f4GUI.Value = find(strcmp(obj.OCCI_f4GUI.String, num2str(val)));
    end
    
    function n = get.NID0(obj)
      n = evalin('base', obj.NID0GUI.String);
    end
    function set.NID0(obj, val)
      obj.NID0GUI.String = mat2str(val);
    end
    
    function n = get.AdditionalDMRS(obj)
      n = logical(obj.AdditionalDMRSGUI.Value);
    end
    function set.AdditionalDMRS(obj, val) 
      obj.AdditionalDMRSGUI.Value = val;
    end
    
    function n = get.DMRSPower(obj)
      n = evalin('base', obj.DMRSPowerGUI.String);
    end
    function set.DMRSPower(obj, val)
      obj.DMRSPowerGUI.String = num2str(val);
    end
    
    function str = getCatalogPrefix(~)
      str = 'nr5g:waveformGeneratorApp:';
    end
    
    % Stack Advanced/UCI vertically:
    function cellDialogs = getDialogsPerColumn(obj)
      className = 'wirelessWaveformGenerator.nr5G_PUCCHUCI_Dialog';
      if ~isKey(obj.Parent.WaveformGenerator.pParameters.DialogsMap, className)
          cellDialogs{1} = {obj};
      else
          cellDialogs{1} = {obj ...
              obj.Parent.WaveformGenerator.pParameters.DialogsMap(className) };
      end
    end
  end
end
