classdef nr5G_PUCCH_Tables < handle
% Basic-table configuration for PUCCH tab

%   Copyright 2021 The MathWorks, Inc.

  properties (Hidden)
    pucchFig
    pxcchSingleChannelFig
    pucchTable
    addPUCCHButton
    removePUCCHButton
    selectedPXCCH = []
    
    % This is the internal representation of the entire PUCCH
    % configuration(s). It is needed to cache per-PUCCH edits at the right
    % side panel.
    pucchConfig = {}
  end
  
  methods
    function obj = nr5G_PUCCH_Tables()
    end
        
    function createPUCCHTables(obj)
      %% Spec and creation of basic PUCCH table
      
      % Format dropdown options
      formatOptions = {'Format 0', 'Format 1', 'Format 2', 'Format 3', 'Format 4'};
      
      pucchStrings = getStrings();
      if isempty(obj.pucchTable)
        spec = {pucchStrings.Label,          'char',                                   70;
                pucchStrings.Enable,         'logical',                                58;
                pucchStrings.Format,         formatOptions,                            90;
                pucchStrings.Power,          [],                                       53;
                pucchStrings.BWPID,          obj.bwpTable.RowName',                    43;
                pucchStrings.Modulation,     'char',                                   78;
                pucchStrings.StartSymbol,    cellfun(@num2str,num2cell(0:13),'UniformOutput',false),              60;
                pucchStrings.SymbolLength,   cellfun(@num2str,num2cell(1:14),'UniformOutput',false),              60;
                pucchStrings.SlotAllocation, 'char',                                   50;
                pucchStrings.Period,         [],                                       52;
                pucchStrings.PRBSet,         'char',                                   40;
                pucchStrings.NID,            [],                                       33;
                pucchStrings.RNTI,           [],                                       40;
                pucchStrings.Coding,         'logical',                                53;
             }; %#ok<*MCNPN>
              
        obj.pucchTable = uitable('Parent', obj.pucchFig, ...
                  'Position',       [10 10 obj.pucchFig.Position(3)-20 obj.pucchFig.Position(4)/3-20], ...
                  'ColumnName',     spec(1:end, 1), ...
                  'ColumnFormat',   spec(1:end, 2)', ...
                  'ColumnEditable', [true(1,5) false true(1,8)],...
                  'ColumnWidth',    spec(1:end, 3)', ...
                  'CellEditCallback', @obj.pucchTableCallback, ...
                  'CellSelectionCallback', @obj.selectPUCCHTableEntry, ...
                  'Tag',            'PUCCHTable', ...
                  'RearrangeableColumns', false);
        obj.pucchTable.Data = newPUCCHRow(obj);

        % add/remove buttons:
        addRemoveButtons(obj, obj.pucchFig, 'PUCCH');
        % customized remove callback:
        obj.removePUCCHButton.Callback = @(a, b, c) removePUCCHTableEntry(obj, [], 'PUCCH');
      end
    end
    
    function removePUCCHTableEntry(obj, ~, tag)
      % remove deleted configs from internal cache, plus take care of shifting everything up:
      obj.pucchConfig(obj.selectedPXCCH) = [];
      
      removeTableEntry(obj, [], tag); % generic remove action
      
      % update side panel with new PUCCH, not the removed one
      updateAdvancedPUCCH(obj);
      mapCache2SidePanelPUCCH(obj);

      updateGrid(obj);
    end
    
    function selectPUCCHTableEntry(obj, table, data)
      % don't perform any updates if its not needed, as this is a
      % considerable UX delay (1 sec)
      rowNames = string(obj.pucchTable.RowName);
      row = data.Indices(:, 1);
      if isempty(row)
        return;
      end
      newID = rowNames(row);
      prevID = getAdvancedPUCCHID(obj);
      if any(strcmp(newID,prevID))
        noOp = true;  
        % no update needed, this happens when editing a current PUCCH or
        % selecting multiple PUCCH rows
        % updates can be expensive as the layout may be changing
      else
        noOp = false;
      end
      
      if ~noOp
        % if updates are needed,
        % store previous edits at the right-side panel to the internal cache
        mapSidePanelPUCCH2Cache(obj);
      end
      % Still need to do base actions, like enabling delete button
      selectTableEntry(obj, table, data); % also highlight row
      
      if noOp
        return;
      end
      
      % load data of newly selected PUCCH from cache to right-side panel:
      updateAdvancedPUCCH(obj);
      mapCache2SidePanelPUCCH(obj);

      % Update RE mapping plot:
      channel = ['PUCCH ' char(newID)];
      updateREVisual(obj, channel, row);
    end
    
    function updateAdvancedPUCCH(obj)
      % update titles of right-side panels based on displayed PUCCH
      if isempty(obj.selectedPXCCH)
        % happens when a row is deleted
        rowNum = 1;
      else
        rowNum = obj.selectedPXCCH;
      end
      
      % Update titles of each panel
      classes = {'Advanced', 'UCI'};
      rowNames = obj.pucchTable.RowName;
      pucchID = rowNames{rowNum};
      for idx = 1:length(classes)
        thisClass = ['wirelessWaveformGenerator.nr5G_PUCCH' classes{idx} '_Dialog'];
        dialog = obj.getParent.WaveformGenerator.pParameters.DialogsMap(thisClass);
 
        currTitle = getTitle(dialog);
        currID = extract(string(currTitle),digitsPattern);
        currTitle = strrep(currTitle,currID,pucchID);
        dialog.setTitle(currTitle);
        if obj.getParent.WaveformGenerator.useAppContainer && isempty(obj.pxcchSingleChannelFig.Name)
          % Figure Name starts empty, even though Title is displayed
          obj.pxcchSingleChannelFig.Name = obj.getParent.WaveformGenerator.AppContainer.getDocument('figureDocumentGroup', 'pxcchSingleChannelFig').Title;
        end
        obj.pxcchSingleChannelFig.Name = strrep(obj.pxcchSingleChannelFig.Name,currID,pucchID);
      end      
    end
    
    function id = getAdvancedPUCCHID(obj)
      % Get the ID of the PUCCH that is currently displayed at the right-side pane
      
      % Look at the number in the title:
      className = 'wirelessWaveformGenerator.nr5G_PUCCHAdvanced_Dialog';
      if isKey(obj.getParent.DialogsMap, className)
        dlg = obj.getParent.DialogsMap(className);
        str = getTitle(dlg);
        id = char(extract(string(str),digitsPattern));
      else
        id = NaN;
      end
    end

    
    function data = newPUCCHRow(obj)
      % Create a new PUCCH row in the basic table, based on
      % nrWavegenPUCCH0Config defaults
      
      % Link to the 1st defined BWP
      bwpCol = 5;
      bwps = obj.pucchTable.ColumnFormat{bwpCol};
      newBWP = bwps{1};
      
      % Get the same defaults with nrWavegenPUCCH0Config;
      cfg = nrULCarrierConfig; % but get values through nrULCarrierConfig, as Enable is customized
      cfg = cfg.PUCCH{1};
      
      % Construct the new Label (smallest available PUCCH ID)
      nextID = appendNextAvailableRowName(obj, obj.pucchTable);
      cfg.Label = ['PUCCH' num2str(nextID)];
      
      % Do not show all values of Slots and PRB. Instead create indexed
      % vectors (e.g., [0:52]), using the fact that they are contiguous
      slots = cfg.SlotAllocation;
      prbs = cfg.PRBSet;
      format = '0';
      modulation = 'Z-Chu'; % This is fixed for format 0
      nid = 'N/A'; % This doesn't apply to format 0
      rnti = 'N/A'; % This doesn't apply to format 0
      coding = false; % This doesn't apply to format 0
      data = {cfg.Label, cfg.Enable, ['Format ' format], cfg.Power, newBWP, modulation, ...
              num2str(cfg.SymbolAllocation(1)), num2str(cfg.SymbolAllocation(2)), ...
              ['[' num2str(slots(1)) ':' num2str(slots(end)) ']'], cfg.Period, num2str(prbs), ...
              nid, rnti, coding};
            
      % Store new object in cache:
      cfg.BandwidthPartID = str2double(newBWP);
      obj.pucchConfig = [obj.pucchConfig {cfg}];

      if cfg.Enable
          updateGrid(obj);
      end
    end
    
    function pucchTableCallback(obj, ~, ~)
      % executes when something changes within the data of the basic PUCCH table

      % some BWP selections may have now been fixed:
      markBrokenBWPLinks(obj);
      
      displayedID = getAdvancedPUCCHID(obj);
      if isnan(displayedID) 
        % initialization
        return;
      end
      
      % Get the handle to the advanced panel to update the value of the
      % read-only property Format
      className = 'wirelessWaveformGenerator.nr5G_PUCCHAdvanced_Dialog';
      dlg = obj.getParent.DialogsMap(className);
      className = 'wirelessWaveformGenerator.nr5G_PUCCHUCI_Dialog';
      dlgUCI = obj.getParent.DialogsMap(className);
      
      % catch errors from nrWavegenPUCCHxConfig and throw in status bar
      data = obj.pucchTable.Data;
      obj.getParent.WaveformGenerator.setStatus(''); % clear possibly previous messages
      try
        
        % Preallocate values for advanced tab update flag:
        formatChanged = false;

        % error and consistency check:
        pucchConfigTmp = cell(1,size(data,1));
        for idx = 1:size(data, 1)
          % check each non-dropdown value, by reusing nrWavegenPUCCH#Config error messages:
          format = data{idx, 3}(end);
          switch format
              case '0'
                  pucchError = nrWavegenPUCCH0Config;
              case '1'
                  pucchError = nrWavegenPUCCH1Config;
              case '2'
                  pucchError = nrWavegenPUCCH2Config;
              case '3'
                  pucchError = nrWavegenPUCCH3Config;
              otherwise % format 4
                  pucchError = nrWavegenPUCCH4Config;
          end
          % Check if the format has changed
          if ~isa(obj.pucchConfig{idx},class(pucchError))
              formatChanged = true;
          end
          
          % Label
          label = data{idx, 1};
          pucchError.Label = label;
          dlg.Label = label; % Apply new Lable to the Advanced side panel
          
          % Enable
          enable = data{idx, 2};
          pucchError.Enable = enable;
          
          % Power
          power = data{idx, 4};
          data{idx, 4} = real(power); % do not leave a wrong value
          pucchError.Power = power;

          % BandwidthPartID
          pucchError.BandwidthPartID = str2double(data{idx, 5});
          
          % Modulation
          if any(strcmpi(format, {'3', '4'})) && any(strcmp(data{idx, 6},{'pi/2-BPSK','QPSK'}))
              pucchError.Modulation = data{idx, 6};
          end
          
          % Symbol Allocation
          if ~formatChanged
              firstSymbol = str2double(data{idx, 7});
              symbolLen = str2double(data{idx, 8});
              pucchError.SymbolAllocation = [firstSymbol symbolLen];
          else
              % Set default values, if a new format is chosen
              data{idx, 7} = num2str(pucchError.SymbolAllocation(1));
              data{idx, 8} = num2str(pucchError.SymbolAllocation(2));
          end
          
          % Slots
          slotAlloc = str2num(data{idx, 9}); %#ok<ST2NM>
          pucchError.SlotAllocation = slotAlloc;
          
          % Period
          period = data{idx, 10};
          data{idx, 10} = max(1, round(real(period))); % do not leave a wrong value
          pucchError.Period = period;
          
          % PRB
          prbs = str2num(data{idx, 11}); %#ok<ST2NM>
          pucchError.PRBSet = prbs;
          
          % NID, RNTI, and Coding apply only to formats 2, 3, 4
          if any(strcmpi(format, {'2', '3', '4'}))
              nid = data{idx, 12};
              if strcmp(nid, 'N/A') % Format 2, 3, or 4 was just chosen
                  data{idx, 12} = 1; % set NID to 1, as done for PUSCH
                  data{idx, 13} = pucchError.RNTI; % get RNTI from default object
                  data{idx, 14} = pucchError.Coding; % get Coding from default object
              else
                  data{idx, 12} = min(1023, max(0, round(real(nid)))); % do not leave a wrong value
                  pucchError.NID = nid;
                  rnti = data{idx, 13};
                  data{idx, 13} = min(65535, max(0, round(real(rnti)))); % do not leave a wrong value
                  pucchError.RNTI = rnti;
                  coding = data{idx, 14};
                  pucchError.Coding = coding;
                  obj.pucchTable.Data{idx, 14} = coding;
              end
          else
              data{idx, 12} = 'N/A'; % NID not applicable
              data{idx, 13} = 'N/A'; % RNTI not applicable
              data{idx, 14} = false; % Coding not applicable
          end
          
          % Create cell array of PUCCH objects only if the format of at
          % least one PUCCH instances has changed
          if formatChanged
              dlg.FormatGUI.String = format; % Update the value of Format in the advanced panel
              dlgUCI.format = format; % Update the format in the UCI panel
              formatChangedGUI(dlg, 1); % Update the visibility of Advanced properties based on format
              formatChangedGUI(dlgUCI, 1); % Update the visibility of UCI properties based on format
              if strcmpi(format, '0')
                  data{idx, 6} = dlg.Modulation_f0;
              elseif strcmpi(format, '1')
                  data{idx, 6} = dlg.Modulation_f1;
              elseif strcmpi(format, '2')
                  data{idx, 6} = dlg.Modulation_f2;
              else
                  data{idx, 6} = dlg.Modulation_f34;
              end
          end
          pucchConfigTmp{idx} = pucchError;
        end
        
        % If any format has changed, update the cache for PUCCH config
        if formatChanged
            cfg = nrULCarrierConfig('PUCCH',pucchConfigTmp);
            applyPUCCHConfig(obj, cfg, 1);
        end
        
        updateCodingDependentVisibilityPUCCH(obj,data); % Update the visibility of properties based on Coding
        
        mapTablePUCCH2Cache(obj); % store edits to internal cache
        
        % If everything is ok, update grid plots with new PUCCH values:
        updateGrid(obj);

        % Update RE mapping plot:
        channel = 'PUCCH';
        selectedTAG = 'selectedPXCCH';
        if isempty(obj.(selectedTAG))
            selectedRow = 1;
        else
            selectedRow = obj.(selectedTAG);
        end
        id = obj.pucchTable.RowName{selectedRow}; % Get the row name ID
        channel = [channel ' ' id];
        updateREVisual(obj, channel, selectedRow);
      catch e
        warnInStatusBar(obj, e); % throw a warning in the status bar
      end
      obj.pucchTable.Data = data;
    end
    
    function mapTablePUCCH2Cache(obj)
      % This method maps the basic table values to the internal representation of
      % nrWavegenPUCCHxConfig objects
      data = obj.pucchTable.Data;
      for idx = 1:size(data, 1)
        pucch = obj.pucchConfig{idx};
        pucch.Label             = data{idx, 1};
        pucch.Enable            = data{idx, 2};
        pucch.Power             = data{idx, 4};
        pucch.BandwidthPartID   = str2double(data{idx, 5});
        firstSymbol             = str2double(data{idx, 7});
        symbolLen               = str2double(data{idx, 8});
        pucch.SymbolAllocation  = [firstSymbol symbolLen];
        pucch.SlotAllocation    = str2num(data{idx, 9}); %#ok<ST2NM>
        pucch.Period            = data{idx, 10};
        pucch.PRBSet            = str2num(data{idx, 11}); %#ok<ST2NM>
        
        if any(strcmpi(data{idx, 3}(end), {'2', '3', '4'}))
            pucch.NID            = data{idx, 12};
            pucch.RNTI           = data{idx, 13};
            pucch.Coding         = data{idx, 14};
            if any(strcmpi(data{idx, 3}(end), {'3', '4'}))
                pucch.Modulation = data{idx, 6};
            end
        end
        
        obj.pucchConfig{idx} = pucch;
      end
    end
    
    function mapSidePanelPUCCH2Cache(obj)
      % This method is invoked when there is a need to store the edits at
      % the right side panel and store these internally in the cache.
      
      % Advanced panel
      className = 'wirelessWaveformGenerator.nr5G_PUCCHAdvanced_Dialog';
      if ~isKey(obj.getParent.DialogsMap, className)
        return;
      end
      dlg = obj.getParent.DialogsMap(className);
      str = getTitle(dlg);
      id = extract(str,digitsPattern);
      
      rowNames = obj.pucchTable.RowName;
      idx = find(strcmp(id, rowNames));
      pucch = obj.pucchConfig{idx};
      format = obj.pucchTable.Data{idx, 3}(end);
      
      % Set common properties
      pucch.FrequencyHopping  = dlg.FrequencyHopping;
      pucch.SecondHopStartPRB = dlg.SecondHopStartPRB;
      
      % Format-specific properties
      switch format
          case '0'
              pucch.GroupHopping = dlg.PUCCHGroupHopping;
              pucch.HoppingID = dlg.HoppingID;
              pucch.InitialCyclicShift = dlg.InitialCyclicShift;
          case '1'
              pucch.GroupHopping = dlg.PUCCHGroupHopping;
              pucch.HoppingID = dlg.HoppingID;
              pucch.InitialCyclicShift = dlg.InitialCyclicShift;
              pucch.OCCI = dlg.OCCI_f1;
              pucch.DMRSPower = dlg.DMRSPower;
          case '2'
              pucch.NID0 = dlg.NID0;
              pucch.DMRSPower = dlg.DMRSPower;
          case '3'
              pucch.GroupHopping = dlg.PUCCHGroupHopping;
              pucch.HoppingID = dlg.HoppingID;
              pucch.AdditionalDMRS = dlg.AdditionalDMRS;
              pucch.DMRSPower = dlg.DMRSPower;
          otherwise % format 4
              pucch.GroupHopping = dlg.PUCCHGroupHopping;
              pucch.HoppingID = dlg.HoppingID;
              pucch.SpreadingFactor = dlg.SpreadingFactor;
              pucch.OCCI = dlg.OCCI_f4;
              pucch.AdditionalDMRS = dlg.AdditionalDMRS;
              pucch.DMRSPower = dlg.DMRSPower;
      end
      
      % UCI panel
      className = 'wirelessWaveformGenerator.nr5G_PUCCHUCI_Dialog';
      if ~isKey(obj.getParent.DialogsMap, className)
        return;
      end
      dlgUCI = obj.getParent.DialogsMap(className);
      % Format-specific properties
      switch format
          case {'0', '1', '2'}
              numUCIBits = dlgUCI.NumUCIBits_f01;
              if strcmp(format,'0')
                  if strcmpi(dlgUCI.DataSourceSR, 'User-defined')
                      pucch.DataSourceSR = dlgUCI.CustomDataSourceSR;
                  else
                      pucch.DataSourceSR = dlgUCI.DataSourceSR;
                  end
              elseif strcmp(format,'2')
                  numUCIBits = dlgUCI.NumUCIBits_f2;
              end
              pucch.NumUCIBits  = numUCIBits;
              % Data source is handled slightly different from programmatic API
              if strcmpi(dlgUCI.DataSourceUCI_f012, 'User-defined')
                  pucch.DataSourceUCI = dlgUCI.CustomDataSourceUCI_f012;
              else
                  pucch.DataSourceUCI = dlgUCI.DataSourceUCI_f012;
              end
          otherwise % format 3 or 4
              pucch.TargetCodeRate = dlgUCI.TargetCodeRate;
              pucch.NumUCIBits     = dlgUCI.NumUCIBits_f34;
              % Data source is handled slightly different from programmatic API
              if strcmpi(dlgUCI.DataSourceUCI_f34, 'User-defined')
                  pucch.DataSourceUCI = dlgUCI.CustomDataSourceUCI_f34;
              else
                  pucch.DataSourceUCI = dlgUCI.DataSourceUCI_f34;
              end
              pucch.NumUCI2Bits = dlgUCI.NumUCI2Bits;
              if strcmpi(dlgUCI.DataSourceUCI2, 'User-defined')
                  pucch.DataSourceUCI2 = dlgUCI.CustomDataSourceUCI2;
              else
                  pucch.DataSourceUCI2 = dlgUCI.DataSourceUCI2;
              end
      end
      
      obj.pucchConfig{idx} = pucch;
    end
    
    function mapCache2SidePanelPUCCH(obj)
      % This method is invoked when there is a need to update the right
      % side panel with contents corresponding to a newly selected row
      
      % Advanced panel
      className = 'wirelessWaveformGenerator.nr5G_PUCCHAdvanced_Dialog';
      if ~isKey(obj.getParent.DialogsMap, className)
        createPXCCHAdvancedTab(obj, 'PUCCH', className);
      end
      dlg = obj.getParent.DialogsMap(className);
      if ~isempty(obj.selectedPXCCH)
        idx = obj.selectedPXCCH(1);
      else
        idx = 1;
      end
      pucch = obj.pucchConfig{idx};
      format = obj.pucchTable.Data{idx, 3}(end);
      
      % Set common properties
      dlg.Label             = pucch.Label;
      dlg.FrequencyHopping  = pucch.FrequencyHopping;
      dlg.SecondHopStartPRB = pucch.SecondHopStartPRB;
      
      % Format-specific properties
      switch format
          case '0'
              dlg.PUCCHGroupHopping = pucch.GroupHopping;
              dlg.HoppingID = pucch.HoppingID;
              dlg.InitialCyclicShift = pucch.InitialCyclicShift;
          case '1'
              dlg.PUCCHGroupHopping = pucch.GroupHopping;
              dlg.HoppingID = pucch.HoppingID;
              dlg.InitialCyclicShift = pucch.InitialCyclicShift;
              dlg.OCCI_f1 = pucch.OCCI;
              dlg.DMRSPower = pucch.DMRSPower;
          case '2'
              dlg.NID0 = pucch.NID0;
              dlg.DMRSPower = pucch.DMRSPower;
          case '3'
              dlg.Modulation_f34 = pucch.Modulation;
              dlg.PUCCHGroupHopping = pucch.GroupHopping;
              dlg.HoppingID = pucch.HoppingID;
              dlg.AdditionalDMRS = pucch.AdditionalDMRS;
              dlg.DMRSPower = pucch.DMRSPower;
          otherwise % format 4
              dlg.Modulation_f34 = pucch.Modulation;
              dlg.PUCCHGroupHopping = pucch.GroupHopping;
              dlg.HoppingID = pucch.HoppingID;
              dlg.SpreadingFactor = pucch.SpreadingFactor;
              dlg.OCCI_f4 = pucch.OCCI;
              dlg.AdditionalDMRS = pucch.AdditionalDMRS;
              dlg.DMRSPower = pucch.DMRSPower;
      end
      
      % UCI panel
      className = 'wirelessWaveformGenerator.nr5G_PUCCHUCI_Dialog';
      if ~isKey(obj.getParent.DialogsMap, className)
        return;
      end
      dlgUCI = obj.getParent.DialogsMap(className);
      switch format
          case {'0', '1', '2'}
              numUCIBits = 'NumUCIBits_f01';
              if strcmp(format,'0')
                  if ischar(pucch.DataSourceSR)
                      dlgUCI.DataSourceSR = pucch.DataSourceSR;
                  else
                      dlgUCI.DataSourceSR = 'User-defined';
                      dlgUCI.CustomDataSourceSR = pucch.DataSourceSR;
                  end
                  sourceSRChangedGUI(dlgUCI, []);
              elseif strcmp(format,'2')
                  numUCIBits = 'NumUCIBits_f2';
              end
              dlgUCI.(numUCIBits)  = pucch.NumUCIBits;
              % Data source is handled slightly different from programmatic API
              if ischar(pucch.DataSourceUCI)
                  dlgUCI.DataSourceUCI_f012 = pucch.DataSourceUCI;
              else
                  dlgUCI.DataSourceUCI_f012 = 'User-defined';
                  dlgUCI.CustomDataSourceUCI_f012 = pucch.DataSourceUCI;
              end
              sourceUCI_f012ChangedGUI(dlgUCI, []);
          otherwise % format 3 or 4
              dlgUCI.TargetCodeRate  = pucch.TargetCodeRate;
              dlgUCI.NumUCIBits_f34  = pucch.NumUCIBits;
              % Data source is handled slightly different from programmatic API
              if ischar(pucch.DataSourceUCI)
                  dlgUCI.DataSourceUCI_f34 = pucch.DataSourceUCI;
              else
                  dlgUCI.DataSourceUCI_f34 = 'User-defined';
                  dlgUCI.CustomDataSourceUCI_f34 = pucch.DataSourceUCI;
              end
              sourceUCI_f34ChangedGUI(dlgUCI, []);
              dlgUCI.NumUCI2Bits = pucch.NumUCI2Bits;
              if ischar(pucch.DataSourceUCI2)
                  dlgUCI.DataSourceUCI2 = pucch.DataSourceUCI2;
              else
                  dlgUCI.DataSourceUCI2 = 'User-defined';
                  dlgUCI.CustomDataSourceUCI2 = pucch.DataSourceUCI2;
              end
              sourceUCI2ChangedGUI(dlgUCI, []);
      end
      
      % Update the side panels
      dlg.FormatGUI.String = format; % Update the value of Format in the advanced panel
      dlgUCI.format = format; % Update the format in the UCI panel
      formatChangedGUI(dlg, []); % Update the visibility of properties based on format
      formatChangedGUI(dlgUCI, []); % Update the visibility of properties based on format
    end
    
    function updateCodingDependentVisibilityPUCCH(obj,data)
      % TargetCodeRate, NumUCIBits, NumUCI2Bits, and DataSourceUCI2 in UCI
      % only applicable when Coding = true
      % Adjust visuals if current row is displayed in "advanced"
      displayedID = getAdvancedPUCCHID(obj);
      if isnan(displayedID)
        % initialization
        return;
      end
        
      % Get Coding value:
      rowNames = obj.pucchTable.RowName;
      idx = find(strcmp(displayedID, rowNames));
      codingCol = 14;
      coding = data{idx, codingCol}; 
      format = data{idx, 3}(end); 
      if any(strcmp(format,{'0','1'}))
          % No format 0 or 1 property depends on Coding
          return;
      end
      
      % Get UCI property panels:
      className = 'wirelessWaveformGenerator.nr5G_PUCCHUCI_Dialog';
      dlgUCI = obj.getParent.DialogsMap(className);
      if strcmp(format,{'2'})
          oldVis = strcmp(dlgUCI.NumUCIBits_f2GUI.Visible, 'on');
          props = [dlgUCI.NumUCIBits_f2Label, dlgUCI.NumUCIBits_f2GUI];
      else
          oldVis = strcmp(dlgUCI.NumUCIBits_f34GUI.Visible, 'on');
          % Only formats 3 and 4 have TargetCodeRate, NumUCI2Bits, and DataSourceUCI2
          props = [dlgUCI.NumUCIBits_f34Label, dlgUCI.NumUCIBits_f34GUI, ...
                   dlgUCI.TargetCodeRateLabel, dlgUCI.TargetCodeRateGUI, ...
                   dlgUCI.NumUCI2BitsLabel, dlgUCI.NumUCI2BitsGUI, ...
                   dlgUCI.DataSourceUCI2Label, dlgUCI.DataSourceUCI2GUI];
      end
      
      if xor(oldVis, coding)
        % do re-layout only when it is needed, because it is expensive
        set(props, 'Visible', uiservices.logicaltoonoff(coding));
        if ~strcmp(format,{'2'}) && coding
            numUCI2BitsChangedGUI(dlgUCI);
        end
        layoutUIControls(dlgUCI);
        layoutPanels(dlgUCI);
      end
    end
    
    function layoutPUCCHTables(obj, margin, verScrollBarSpace)
      % This method executes everytime the App/tiles resize. Need to
      % maintain relative placement given the new "Position" of the figure
      
      if ~isempty(obj.pucchTable) && ishghandle(obj.pucchTable)
        figPos = obj.pucchTable.Parent.Position;
        if all(figPos([3 4])>margin)
            
          pucchWidth = sum([obj.pucchTable.ColumnWidth{:}])+32+verScrollBarSpace;
          obj.pucchTable.Position       = [...
            10 figPos(4)*0.52  pucchWidth         figPos(4)*0.475-10];
          
          layoutAddRemoveButtons(obj, obj.pucchTable, 'PUCCH');
        end
               
      end
    end
    
    function pucch = getPUCCHConfig(obj)
      % Get an equivalent nrULCarrierConfig.PUCCH configuration based on
      % the so far PUCCH configurations (including the internally cached ones)
      
      % Update cache with latest edits
      mapTablePUCCH2Cache(obj);
      mapSidePanelPUCCH2Cache(obj);
      
      % Get all nrWavegenPUCCHxConfig objects
      pucch = obj.pucchConfig;
    end
    
    function applyPUCCHConfig(obj, cfg, formatChanged)
      % Used on New, Open Session, and every time the PUCCH format changes
      % Map a cell array of nrWavegenPUCCHxConfig objects to the basic
      % PUCCH table, the internal cache, and the right-side panel
      
      if isempty(obj.pucchTable)
        return; % App initialization
      end
      
      if nargin==2 || ~formatChanged
        % If this method is called on New or Open Session, restore default
        % value to avoid wrong mapping between cache and selected rows
        obj.selectedPXCCH = [];
      end
      
      pucch = cfg.PUCCH;
      
      % Update internal cache:
      obj.pucchConfig = pucch;
      % Update table:
      data = cell(numel(pucch), 14);
      for idx = 1:numel(pucch)
        data{idx, 1}  = pucch{idx}.Label;
        data{idx, 2}  = pucch{idx}.Enable;
        if isa(pucch{idx},'nrWavegenPUCCH0Config')
            format  = '0';
        elseif isa(pucch{idx},'nrWavegenPUCCH1Config')
            format  = '1';
        elseif isa(pucch{idx},'nrWavegenPUCCH2Config')
            format  = '2';
        elseif isa(pucch{idx},'nrWavegenPUCCH3Config')
            format  = '3';
        else % format 4
            format  = '4';
        end
        data{idx, 3}  = ['Format ' format];
        data{idx, 4}  = pucch{idx}.Power;
        data{idx, 5}  = num2str(pucch{idx}.BandwidthPartID);
        
        % Modulation
        if any(strcmpi(format, {'3', '4'}))
            data{idx, 6} = pucch{idx}.Modulation;
        else
            className = 'wirelessWaveformGenerator.nr5G_PUCCHAdvanced_Dialog';
            if ~isKey(obj.getParent.DialogsMap, className)
                createPXCCHAdvancedTab(obj, 'PUCCH', className);
            end
            dlg = obj.getParent.DialogsMap(className);
            if strcmpi(format, '0')
                data{idx, 6} = dlg.Modulation_f0;
            elseif strcmpi(format, '1')
                data{idx, 6} = dlg.Modulation_f1;
            else % format 2
                data{idx, 6} = dlg.Modulation_f2;
            end
        end
        
        data{idx, 7}  = num2str(pucch{idx}.SymbolAllocation(1));
        data{idx, 8}  = num2str(pucch{idx}.SymbolAllocation(2));
        
        % Try to establish a [0:N] pattern if possible:
        if ~isscalar(pucch{idx}.SlotAllocation) && ...
                isequal(pucch{idx}.SlotAllocation, min(pucch{idx}.SlotAllocation):max(pucch{idx}.SlotAllocation))
          data{idx, 9} = ['[' num2str(min(pucch{idx}.SlotAllocation)) ':' num2str(max(pucch{idx}.SlotAllocation)) ']'];
        else
          data{idx, 9}  = mat2str(pucch{idx}.SlotAllocation);
        end
        
        data{idx, 10} = pucch{idx}.Period;

        % Try to establish a [0:N] pattern if possible:
        if ~isscalar(pucch{idx}.PRBSet) && isequal(pucch{idx}.PRBSet, min(pucch{idx}.PRBSet):max(pucch{idx}.PRBSet))
          data{idx, 11} = ['[' num2str(min(pucch{idx}.PRBSet)) ':' num2str(max(pucch{idx}.PRBSet)) ']'];
        else
          data{idx, 11} = mat2str(pucch{idx}.PRBSet);
        end
        
        if any(strcmpi(format, {'2', '3', '4'}))
            data{idx, 12} = pucch{idx}.NID;
            data{idx, 13} = pucch{idx}.RNTI;
            data{idx, 14} = pucch{idx}.Coding;
        else
            data{idx, 12} = 'N/A';
            data{idx, 13} = 'N/A';
            data{idx, 14} = false;
        end
      end
      obj.pucchTable.Data = data;
      obj.pucchTable.RowName = num2cell(num2str((1:numel(pucch))'));

      % Update side panel:
      mapCache2SidePanelPUCCH(obj);
    end
  end
end

function s = getStrings()

    tableName = 'PUCCHTable';
    def = {'Label' 'Enable' 'Format' 'Power' 'BWPID' 'Modulation' 'StartSymbol' 'SymbolLength' 'SlotAllocation' 'Period' 'PRBSet' 'NID' 'RNTI' 'Coding'};

    for i = 1:numel(def)
        s.(def{i}) = getString(message(['nr5g:waveformGeneratorApp:' tableName def{i}]));
    end

end
