classdef nr5G_PUSCHPTRS_Dialog < wirelessWaveformGenerator.nr5G_PDSCHPTRS_Dialog
% Interface to PUSCH PTRS properties

%   Copyright 2020-2021 The MathWorks, Inc.
  
  properties (Dependent)
    NumPTRSSamples
    NumPTRSGroups
    NID
  end  
  
  properties (Hidden)
    NumPTRSSamplesType = 'popupmenu'
    NumPTRSSamplesDropDown = {'2', '4'}
    NumPTRSSamplesLabel
    NumPTRSSamplesGUI
    
    NumPTRSGroupsType = 'popupmenu'
    NumPTRSGroupsDropDown = {'2', '4', '8'}
    NumPTRSGroupsLabel
    NumPTRSGroupsGUI
    
    NIDType = 'edit'    
    NIDLabel
    NIDGUI
  end
  
  methods
    function obj = nr5G_PUSCHPTRS_Dialog(parent, fig)
      obj@wirelessWaveformGenerator.nr5G_PDSCHPTRS_Dialog(parent, fig); % call base constructor
      
      % Callbacks needed either for validation of edit field
      obj.NIDGUI.Callback = @(a,b) nidChangedGUI(obj);
    end
    
    function adjustSpec(obj)
      % customization given the PXSCH max label length
      obj.LabelWidth = 150;
      
      % Change from PDSCH base class
      obj.TitleString = 'PUSCH 1 PT-RS';
    end
    
    function adjustDialog(obj)
      adjustDialog@wirelessWaveformGenerator.nr5G_PDSCHPTRS_Dialog(obj);
      % PT-RS is turned off by default, so hide everything
      set([obj.NumPTRSSamplesLabel obj.NumPTRSSamplesGUI ...
           obj.NumPTRSGroupsLabel obj.NumPTRSGroupsGUI ...
           obj.NIDLabel obj.NIDGUI], ...
                      'Visible', 'off');
      
      % PUSCH PT-RS Port Set can also be 2-element vector (differently than PDSCH):
      set([obj.PTRSPortSetLabel obj.PTRSPortSetGUI], 'Tooltip', getMsgString(obj, 'PUSCHPTRSPortSetTT'));
    end
    
    function props = displayOrder(~)
      props = {'Enable'; 'Power'; 'TimeDensity'; 'FrequencyDensity'; ...
        'NumPTRSSamples'; 'NumPTRSGroups'; 'REOffset'; 'PTRSPortSet'; 'NID'};
    end
    
    function restoreDefaults(obj)
      % Get the same defaults with programmatic objects
      c = nrWavegenPUSCHConfig;
      obj.Power = c.PTRSPower;
      obj.Enable = c.EnablePTRS;
      
      c = nrPUSCHPTRSConfig;
      obj.TimeDensity       = c.TimeDensity;
      obj.FrequencyDensity  = c.FrequencyDensity;
      obj.REOffset          = c.REOffset;
      obj.PTRSPortSet       = c.PTRSPortSet;
      obj.NumPTRSSamples    = c.NumPTRSSamples;
      obj.NumPTRSGroups     = c.NumPTRSGroups;
      obj.NID               = c.NID;
    end
    
    function enabledChanged(obj)
      % Turn on/off all other knobs, based on the Enable flag
      oldVal       = strcmp(obj.PowerGUI.Visible, 'on'); % = was Enable on?
      % Also get the value of Transform Precoding as some properties also
      % depend on that
      oldValPrecod = strcmp(obj.NIDGUI.Visible, 'on');
      props = displayOrder(obj);
      
      currDlg = obj.Parent.WaveformGenerator.pParameters.CurrentDialog;
      currRow = find(strcmp(getAdvancedPXSCHID(currDlg), currDlg.pxschTable.RowName), 1);
      precod = currDlg.pxschTable.Data{currRow, end-1};
      
      for idx = 1:numel(props)
        % The Enable checkbox is always visible, so no-op
        if strcmp(props{idx}, 'Enable')
          continue;
        end
        
        % Some properties appear only when Transform Precoding is on
        if any(strcmp(props{idx}, {'NumPTRSSamples', 'NumPTRSGroups', 'NID'}))
          newVal = obj.Enable && precod;
        elseif any(strcmp(props{idx}, {'Power', 'PTRSPortSet', 'FrequencyDensity', 'REOffset'}))
          newVal = obj.Enable && ~precod;
        else
          newVal = obj.Enable;
        end
        set([obj.([props{idx} 'Label']) obj.([props{idx} 'GUI'])], ...
                      'Visible', uiservices.logicalToOnOff(newVal));
      end
      
      if xor(oldVal, obj.Enable) || (obj.Enable && xor(oldValPrecod, precod))
        % do re-layout only when it is needed, because it is expensive
        layoutUIControls(obj);
        layoutPanels(obj);
      end

      % Update resource grid and RE mapping
      ptrsGridChangedGUI(obj);
    end
    
    %% Validators
    function nidChangedGUI(obj, ~)
      try
        % use error throwing behavior of nrPUSCHPTRSConfig
        c = nrPUSCHPTRSConfig;
        c.NID = obj.NID;
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateAppStatusBar(obj);
    end
       
    %% Getters/setters
    function fr = get.NumPTRSSamples(obj) 
      fr = str2double(obj.NumPTRSSamplesGUI.String{obj.NumPTRSSamplesGUI.Value});
    end
    function set.NumPTRSSamples(obj, val) 
      obj.NumPTRSSamplesGUI.Value = find(strcmp(obj.NumPTRSSamplesGUI.String, num2str(val))); 
    end
    
    function fr = get.NumPTRSGroups(obj) 
      fr = str2double(obj.NumPTRSGroupsGUI.String{obj.NumPTRSGroupsGUI.Value});
    end
    function set.NumPTRSGroups(obj, val) 
      obj.NumPTRSGroupsGUI.Value = find(strcmp(obj.NumPTRSGroupsGUI.String, num2str(val))); 
    end
    
    function n = get.NID(obj)
      n = evalin('base', obj.NIDGUI.String);
    end
    function set.NID(obj, val)
      obj.NIDGUI.String = mat2str(val);
    end
  end
end
