classdef nr5G_PXSCHAdvanced_Dialog < wirelessWaveformGenerator.Dialog
% Interface to common PXSCH properties that are not present in the basic table

%   Copyright 2020-2021 The MathWorks, Inc.

  properties (Dependent)
    Label
    XOverhead
    RVSequence
    DataSource
    CustomDataSource
  end  
  
  properties (Hidden)
    configFcn = @struct
    
    LabelType = 'edit'
    LabelLabel
    LabelGUI
    XOverheadType = 'popupmenu'
    XOverheadDropDown = {'0', '6', '12', '18'}
    XOverheadLabel
    XOverheadGUI
    RVSequenceType = 'edit'
    RVSequenceLabel
    RVSequenceGUI
    
    DataSourceType = 'popupmenu'
    DataSourceDropDown = {'PN9-ITU','PN9','PN11', 'PN15', 'PN23', 'User-defined'}
    DataSourceLabel
    DataSourceGUI
    
    CustomDataSourceType = 'edit'
    CustomDataSourceLabel
    CustomDataSourceGUI
  end
  
  methods
    function obj = nr5G_PXSCHAdvanced_Dialog(parent, fig)
      obj@wirelessWaveformGenerator.Dialog(parent, fig); % call base constructor
      
      % Callbacks needed either for validation of edit fields or for
      % changes to visibility of other controls
      obj.RVSequenceGUI.Callback =  @(a,b) rvSeqChangedGUI(obj);
      obj.DataSourceGUI.Callback =  @(a,b) sourceChangedGUI(obj);
      
      sourceChangedGUI(obj, []); % set default visibility
    end
        
    function rvSeqChangedGUI(obj, ~)
      % validation
      try
        % use error throwing behavior of nrWavegenPDSCHConfig
        c = nrWavegenPDSCHConfig;
        c.RVSequence = obj.RVSequence;
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateAppStatusBar(obj);
    end
    
    function sourceChangedGUI(obj, ~)
      % Custom data source edit field appears only under user-defined data source
      oldVis = strcmp(obj.CustomDataSourceGUI.Visible, 'on');
      newVis = strcmpi(obj.DataSource, 'User-defined');
      
      if xor(oldVis, newVis)
        % do re-layout only when it is needed, because it is expensive
        set([obj.CustomDataSourceLabel obj.CustomDataSourceGUI], ...
          'Visible', uiservices.logicaltoonoff(newVis));
        layoutUIControls(obj);
        layoutPanels(obj);
      end
    end

    function updateGrids(obj)
      % A property that affects the resource grid or the RE mapping is changed.
      dlg = obj.Parent.WaveformGenerator.pParameters.CurrentDialog;

      % Do live grid update:
      dlg.updateGrid();

      % Do live RE mapping update:
      str = getTitle(obj);
      pxschID = extract(str,digitsPattern);
      channel = [str(1:length('PDSCH ')) pxschID{1}];
      selectedRow = find(strcmpi(pxschID,dlg.pxschTable.RowName)); % Get the row name ID
      dlg.updateREVisual(channel,selectedRow);
    end

    function updateAppStatusBar(obj)
      % Update the status bar to display any warning related to the current
      % configuration or to clear any previous warning that no longer apply
      dlg = obj.Parent.WaveformGenerator.pParameters.CurrentDialog;
      updateStatusBar(dlg);
    end
   
    function adjustSpec(obj)
      % customization given the PXSCH max label length
      obj.LabelWidth = 150;
    end
    
    %% Getters/setters
    function n = get.Label(obj)
      n = obj.LabelGUI.String;
    end
    function set.Label(obj, val)
      obj.LabelGUI.String = val;
    end

    function fr = get.XOverhead(obj) 
      fr = str2double(obj.XOverheadGUI.String{obj.XOverheadGUI.Value});
    end
    function set.XOverhead(obj, val) 
      obj.XOverheadGUI.Value = find(strcmp(obj.XOverheadGUI.String, num2str(val))); 
    end
    
    function fr = get.DataSource(obj) 
      fr = obj.DataSourceGUI.String{obj.DataSourceGUI.Value};
    end
    function set.DataSource(obj, val) 
      obj.DataSourceGUI.Value = find(strcmp(obj.DataSourceGUI.String, val)); 
    end
    
    function n = get.CustomDataSource(obj)
      n = evalin('base', obj.CustomDataSourceGUI.String);
    end
    function set.CustomDataSource(obj, val)
      obj.CustomDataSourceGUI.String = mat2str(val);
    end
    
    function n = get.RVSequence(obj)
      n = evalin('base', obj.RVSequenceGUI.String);
    end
    function set.RVSequence(obj, val)
      obj.RVSequenceGUI.String = mat2str(val);
    end
    
    function str = getCatalogPrefix(~)
      str = 'nr5g:waveformGeneratorApp:';
    end
  end
end
