classdef nr5G_PXSCH_Tables < handle
% Basic-table configuration for PXSCH tab

%   Copyright 2020-2021 The MathWorks, Inc.

  properties (Hidden)
    pxschFig
    pxschSingleChannelFig
    pxschTable
    addPXSCHButton
    removePXSCHButton
    selectedPXSCH = []      
    
    % This is the internal representation of the entire PXSCH
    % configuration(s). It is needed to cache per-PXSCH edits at the right
    % side panel.
    pxschConfig = {}
  end
  
  methods
    function obj = nr5G_PXSCH_Tables()
    end
        
    function createPXSCHTables(obj)
      %% Spec and creation of basic PXSCH table
      
      % Modulation options differ between PDSCH and PUSCH. 
      % Also TranformPrecoding is only present in PUSCH
      pxschStrings = getStrings();
      modOptions = {'QPSK', '16QAM', '64QAM', '256QAM'};
      if strcmpi(PXSCHfigureName(obj), 'PUSCH')
        transPrecod = {pxschStrings.TransformPrecoding, 'logical', 58};
        modOptions = ['pi/2-BPSK' modOptions];
      else
        transPrecod = {};
      end
      
      if isempty(obj.pxschTable)
        spec = [{pxschStrings.Enable,          'logical',                               51;        
                 pxschStrings.Power,          [],                                       51;
                 pxschStrings.BWPID,          obj.bwpTable.RowName',                    42;
                 pxschStrings.Modulation,     modOptions,                               76;
                 pxschStrings.NumLayers,      num2cell(num2str((1:4)'))',               53;
                 pxschStrings.MappingType,    {'A', 'B'},                               62;
                 pxschStrings.StartSymbol,    cellfun(@num2str,num2cell(0:13),'UniformOutput',false),              56;
                 pxschStrings.SymbolLength,   cellfun(@num2str,num2cell(1:14),'UniformOutput',false),              56;
                 pxschStrings.SlotAllocation, 'char',                                   47;
                 pxschStrings.Period,         [],                                       51;
                 pxschStrings.PRBSet,         'char',                                   44;
                 pxschStrings.NID,            [],                                       32;
                 pxschStrings.RNTI,           [],                                       40;
                 pxschStrings.Coding,         'logical',                                53;
                 pxschStrings.TargetCodeRate, [],                                       72;};
                 transPrecod;
                {pxschStrings.EnablePTRS,     'logical',                                52;
             }]; %#ok<*MCNPN>
              
        obj.pxschTable = uitable('Parent', obj.pxschFig, ...
                  'Position',       [10 10 obj.pxschFig.Position(3)-20 obj.pxschFig.Position(4)/3-20], ...
                  'ColumnName',     spec(1:end, 1), ...
                  'ColumnFormat',   spec(1:end, 2)', ...
                  'ColumnEditable', true,...
                  'ColumnWidth',    spec(1:end, 3)', ...
                  'CellEditCallback', @obj.pxschTableCallback, ...
                  'CellSelectionCallback', @obj.selectPXSCHTableEntry, ...
                  'Tag',            'PXSCHTable', ...
                  'RearrangeableColumns', false);
        obj.pxschTable.Data = newPXSCHRow(obj);

        % add/remove buttons:
        addRemoveButtons(obj, obj.pxschFig, 'PXSCH');
        % customized remove callback:
        obj.removePXSCHButton.Callback = @(a, b, c) removePXSCHTableEntry(obj, [], 'PXSCH');
      end
    end
    
    function removePXSCHTableEntry(obj, ~, tag)
      % remove deleted configs from internal cache, plus take care of shifting everything up:
      obj.pxschConfig(obj.selectedPXSCH) = [];
      
      removeTableEntry(obj, [], tag); % generic remove action
      
      % update side panel with new PXSCH, not the removed one
      updateAdvancedPXSCH(obj);
      mapCache2SidePanel(obj);

      updateGrid(obj);
    end
    
    function selectPXSCHTableEntry(obj, table, data)
      % don't perform any updates if its not needed, as this is a
      % considerable UX delay (1 sec)
      rowNames = obj.pxschTable.RowName;
      row = data.Indices(:, 1);
      if isempty(row)
        return;
      end
      newID = rowNames{row};
      prevID = getAdvancedPXSCHID(obj);
      if isequal(newID, prevID)
        noOp = true;  
        % no update needed, this happens when editing a current PDSCH
        % updates can be expensive as the layout may be changing
      else
        noOp = false;
      end
      
      if ~noOp
        % if updates are needed,
        % store previous edits at the right-side panel to the internal cache
        mapSidePanel2Cache(obj);
      end
      % Still need to do base actions, like enabling delete button
      selectTableEntry(obj, table, data); % also highlight row
      
      if noOp
        return;
      end
      
      % load data of newly selected PDSCH from cache to right-side panel:
      updateAdvancedPXSCH(obj);
      mapCache2SidePanel(obj);
      
      if strcmpi(PXSCHfigureName(obj), 'PDSCH')
        % Check if TBScaling needs to be disabled given corresponding coding value
        updateCodingDependentVisibility(obj, row);
      else
        % Transform precoding affects a lot of visibilities in PUSCH DMRS and PTRS panels
        updatePrecodingDependentVisibility(obj, row);
      end
      % Also PTRS Enable must be synchronized with table
      updatePTRSEnable(obj, row);

      % Update RE mapping plot:
      channel = PXSCHfigureName(obj);
      channel = [channel ' ' newID];
      updateREVisual(obj, channel, row);
    end
    
    function updateAdvancedPXSCH(obj)
      % update titles of right-side panels based on displayed PXSCH
      if isempty(obj.selectedPXSCH)
        % happens when a row is deleted
        rowNum = 1;
      else
        rowNum = obj.selectedPXSCH;
      end
      
      % Update titles of each panel
      classes = {'Advanced', 'DMRS', 'PTRS'};
      rowNames = obj.pxschTable.RowName;
      pxschID = rowNames{rowNum};
      for idx = 1:length(classes)
        thisClass = ['wirelessWaveformGenerator.nr5G_' PXSCHfigureName(obj) classes{idx} '_Dialog'];
        dialog = obj.getParent.WaveformGenerator.pParameters.DialogsMap(thisClass);

        currTitle = getTitle(dialog);

        currID = extract(string(currTitle),digitsPattern);
        currTitle = strrep(currTitle,currID,pxschID);
        dialog.setTitle(currTitle);
        if idx == 1
          obj.pxschSingleChannelFig.Name = strrep(strrep(currTitle, '(', '- '), ')', '');
        end
      end      
    end
    
    function id = getAdvancedPXSCHID(obj)
      % Get the ID of the PXSCH that is currently displayed at the right-side pane
      
      % Look at the number in the title:
      className = ['wirelessWaveformGenerator.nr5G_' PXSCHfigureName(obj) 'Advanced_Dialog'];
      if isKey(obj.getParent.DialogsMap, className)
        dlg = obj.getParent.DialogsMap(className);       
        str = getTitle(dlg);
        id = char(extract(string(str),digitsPattern));
      else
        id = NaN;
      end
    end

    
    function data = newPXSCHRow(obj)
      % Create a new PXSCH row in the basic table, based on
      % nrWavegenPDSCHConfig defaults
      
      % link to the 1st defined BWP
      bwpCol = 3;
      bwps = obj.pxschTable.ColumnFormat{bwpCol};
      newBWP = bwps{1};
      
      % get the same defaults with nrWavegenPXSCHConfig;
      if strcmpi(PXSCHfigureName(obj), 'PDSCH') % PDSCH
        cfg = nrWavegenPDSCHConfig;
        precod = {};
      else                                      % PUSCH
        cfg = nrULCarrierConfig; % Use the nrWavegenPUSCHConfig from nrULCarrierConfig, because defaults differ slightly
        cfg = cfg.PUSCH{1};
        precod = {logical(cfg.TransformPrecoding)};
      end
      % Do not show all values of Slots and PRB. Instead create indexed
      % vectors (e.g., [0:52]), using the fact that they are contiguous
      slots = cfg.SlotAllocation;
      prbs = 0:(270-1);
      cfg.PRBSet = prbs;
      data = [{cfg.Enable, cfg.Power, newBWP, cfg.Modulation, num2str(cfg.NumLayers), ...
              cfg.MappingType, num2str(cfg.SymbolAllocation(1)), num2str(cfg.SymbolAllocation(2)), ...
              ['[' num2str(slots(1)) ':' num2str(slots(end)) ']'], cfg.Period, ...
              ['[' num2str(prbs(1))   ':' num2str(prbs(end)) ']'], 1, cfg.RNTI, ...
              cfg.Coding, cfg.TargetCodeRate} precod, {logical(cfg.EnablePTRS)}];
      
      % construct the new Label (smallest available PXCCH ID)
      nextID = appendNextAvailableRowName(obj, obj.pxschTable);
      cfg.Label = [PXSCHfigureName(obj) '' num2str(nextID)];
            
      % store new object in cache:
      cfg.BandwidthPartID = str2double(newBWP);
      cfg.NID = 1;
      obj.pxschConfig = [obj.pxschConfig {cfg}];

      if cfg.Enable
          updateGrid(obj);
      end
    end
    
    function pxschTableCallback(obj, ~, ~)
      % executes when something changes within the data of the basic PXSCH table
      
      % some BWP selections may have now been fixed:
      markBrokenBWPLinks(obj);

      isPUSCH = strcmpi(PXSCHfigureName(obj), 'PUSCH');
       
      % catch errors from nrWavegenPXSCHConfig and throw in status bar
      data = obj.pxschTable.Data;
      obj.getParent.WaveformGenerator.setStatus(''); % clear possibly previous messages
      try

        % error and consistency check:
        for idx = 1:size(data, 1)
          % check each non-dropdown value, by reusing nrWavegenPXSCHConfig error messages:
          pxschError = nrWavegenPUSCHConfig;
          
          % Enable
          pxschError.Enable = data{idx,1};

          % Power
          power = data{idx, 2};
          data{idx, 2} = real(power); % do not leave a wrong value
          pxschError.Power = power;

          % BandwidthPartID
          pxschError.BandwidthPartID = str2double(data{idx, 3});
          
          % Mapping type
          pxschError.MappingType = data{idx,6};

          % Symbol Allocation
          firstSymbol = str2double(data{idx, 7});
          symbolLen = str2double(data{idx, 8});
          pxschError.SymbolAllocation = [firstSymbol symbolLen];
          
          % Slots
          slotAlloc = str2num(data{idx, 9}); %#ok<ST2NM>
          pxschError.SlotAllocation = slotAlloc;
          
          % Period
          period = data{idx, 10};
          data{idx, 10} = max(1, round(real(period))); % do not leave a wrong value
          pxschError.Period = period;
          
          % PRB
          prbs = str2num(data{idx, 11}); %#ok<ST2NM>
          pxschError.PRBSet = prbs;
          
          % NID
          nid = data{idx, 12};
          data{idx, 12} = min(1023, max(0, round(real(nid)))); % do not leave a wrong value
          pxschError.NID = nid;
          
          % RNTI
          rnti = data{idx, 13};
          data{idx, 13} = min(65535, max(0, round(real(rnti)))); % do not leave a wrong value
          pxschError.RNTI = rnti;
          
          % Disable Target Code Rate when Coding is turned off
          coding = data{idx, 14};
          pxschError.Coding = coding;
          codingChanged = false;
          if ~isequal(obj.pxschConfig{idx}.Coding,pxschError.Coding)
              codingChanged = true;
          end
          % Coding rate
          if ~coding
            data{idx, 15} = 'N/A'; % code rate not applicable
          else
            codeRate = data{idx, 15};
            if strcmp(codeRate, 'N/A') % Coding was just enabled
              data{idx, 15} = obj.pxschConfig{idx}.TargetCodeRate; % get code rate from internal cache
            else
              pxschError.TargetCodeRate = codeRate;
            end
          end
          
          % Transform precoding
          if isPUSCH
            trPrecod = data{idx, 16};
            pxschError.TransformPrecoding = trPrecod;
            tpChanged = false;
            if ~isequal(obj.pxschConfig{idx}.TransformPrecoding,pxschError.TransformPrecoding)
                tpChanged = true;
            end
            if trPrecod
              % Fix NumLayers to 1
              data{idx, 5} = '1'; % char because it is a dropdown
            else
              % Can't allow 'pi/2-BPSK' when transform precoding is off
              if strcmpi(data{idx, 4}, 'pi/2-BPSK')
                data{idx, 4} = 'QPSK';
              end
            end
          end

          % Enable PT-RS
          pxschError.EnablePTRS = data{idx,end};
          ptrsChanged = false;
          if ~isequal(obj.pxschConfig{idx}.EnablePTRS,pxschError.EnablePTRS)
            ptrsChanged = true;
          end

          % Apply changes to property display, if needed
          % Check if TB Scaling, XOverhead need to be disabled given coding value
          if codingChanged
            updateCodingDependentVisibility(obj, idx);
          end
          % Transform precoding affects a lot of visibilities in PUSCH DMRS and PTRS panels
          if isPUSCH && tpChanged
              updatePrecodingDependentVisibility(obj, idx);
          end
          % PTRS Enable in right-side panel must be synchronized with basic table
          if ptrsChanged || (isPUSCH && tpChanged)
            updatePTRSEnable(obj, idx);
          end
        end
        mapTableToCache(obj); % store edits to internal cache
        
        % If everything is ok, update grid plots with new PXSCH values:
        updateGrid(obj);

        % Update RE mapping plot:
        channel = PXSCHfigureName(obj);
        selectedTAG = 'selectedPXSCH';
        if isempty(obj.(selectedTAG))
          selectedRow = 1;
        else
          selectedRow = obj.(selectedTAG);
        end
        id = obj.pxschTable.RowName{selectedRow}; % Get the row name ID
        channel = [channel ' ' id];
        updateREVisual(obj, channel, selectedRow);
      catch e
        warnInStatusBar(obj, e); % throw a warning in the status bar
      end
      obj.pxschTable.Data = data;
    end
    
    function mapTableToCache(obj)
      % This method maps the basic table values to the internal representation of
      % nrWavegenPXSCHConfig objects
      data = obj.pxschTable.Data;
      for idx = 1:size(data, 1)
        pxsch = obj.pxschConfig{idx};
        pxsch.Enable            = data{idx, 1};
        pxsch.Power             = data{idx, 2};
        pxsch.BandwidthPartID   = str2double(data{idx, 3});
        pxsch.Modulation        = data{idx, 4};
        pxsch.NumLayers         = str2double(data{idx, 5});
        pxsch.MappingType       = data{idx, 6};
        firstSymbol             = str2double(data{idx, 7});
        symbolLen               = str2double(data{idx, 8});
        pxsch.SymbolAllocation  = [firstSymbol symbolLen];
        pxsch.SlotAllocation    = str2num(data{idx, 9}); %#ok<ST2NM>
        pxsch.Period            = data{idx, 10};
        pxsch.PRBSet            = str2num(data{idx, 11}); %#ok<ST2NM>
        pxsch.NID               = data{idx, 12};
        pxsch.RNTI              = data{idx, 13};
        pxsch.Coding            = data{idx, 14};
        
        % Take care of disabled coding
        if ~strcmp(data{idx, 15}, 'N/A')
          pxsch.TargetCodeRate = data{idx, 15};
        end
        
        if strcmpi(PXSCHfigureName(obj), 'PUSCH')  % Trans. precoding not present in PDSCH
          pxsch.TransformPrecoding = data{idx, 16};
        end

        pxsch.EnablePTRS = logical(data{idx, end});
        
        obj.pxschConfig{idx} = pxsch;
      end
    end
    
    function mapSidePanel2Cache(obj)
      % This method is invoked when there is a need to store the edits at
      % the right side panel and store these internally in the cache.
      
      % Advanced panel
      className = ['wirelessWaveformGenerator.nr5G_' PXSCHfigureName(obj) 'Advanced_Dialog'];
      if ~isKey(obj.getParent.DialogsMap, className)
        return;
      end
      dlg = obj.getParent.DialogsMap(className);
      str = getTitle(dlg);
      id = extract(str,digitsPattern);
      
      rowNames = obj.pxschTable.RowName;
      idx = find(strcmp(id, rowNames));
      if isempty(idx)
          return; % Row has been deleted
      end
      pxsch = obj.pxschConfig{idx};
      
      pxsch.Label       = dlg.Label;
      pxsch.RVSequence  = dlg.RVSequence;
      pxsch.XOverhead   = dlg.XOverhead;
      % Data source is handled slightly different from programmatic API
      if strcmpi(dlg.DataSource, 'User-defined')
        pxsch.DataSource = dlg.CustomDataSource;
      else
        pxsch.DataSource = dlg.DataSource;
      end
      
      % Advanced panel differs between PDSCH and PUSCH
      isPUSCH = strcmpi(PXSCHfigureName(obj), 'PUSCH');
      if ~isPUSCH
        pxsch.VRBToPRBInterleaving      = dlg.VRBToPRBInterleaving;
        pxsch.VRBBundleSize             = dlg.VRBBundleSize;
        pxsch.ReservedCORESET           = dlg.ReservedCORESET;
        pxsch.ReservedPRB{1}.PRBSet     = dlg.ReservedPRB;
        pxsch.ReservedPRB{1}.SymbolSet  = dlg.ReservedSymbols;
        pxsch.ReservedPRB{1}.Period     = dlg.ReservedPeriod;
        pxsch.TBScaling                 = dlg.TBScaling;
      else
        pxsch.TransmissionScheme  = dlg.TransmissionScheme;
        pxsch.NumAntennaPorts     = dlg.NumAntennaPorts;
        pxsch.TPMI                = dlg.TPMI;
        pxsch.FrequencyHopping    = dlg.FrequencyHopping;
        pxsch.SecondHopStartPRB   = dlg.SecondHopStartPRB;
%         pxsch.BetaOffsetACK       = dlg.BetaOffsetACK;
%         pxsch.BetaOffsetCSI1      = dlg.BetaOffsetCSI1;
%         pxsch.BetaOffsetCSI2      = dlg.BetaOffsetCSI2;
%         pxsch.UCIScaling          = dlg.UCIScaling;
%         pxsch.DisableULSCH        = dlg.DisableULSCH;
      end
      
      % DMRS panel
      className = ['wirelessWaveformGenerator.nr5G_' PXSCHfigureName(obj) 'DMRS_Dialog'];
      dlg = obj.getParent.DialogsMap(className);
      pxsch.DMRSPower = dlg.Power;
      pxsch.DMRS.DMRSConfigurationType    = dlg.DMRSConfigurationType;
      pxsch.DMRS.DMRSTypeAPosition        = dlg.DMRSTypeAPosition;
      pxsch.DMRS.DMRSAdditionalPosition   = dlg.DMRSAdditionalPosition;
      pxsch.DMRS.DMRSLength               = dlg.DMRSLength;
      pxsch.DMRS.CustomSymbolSet          = dlg.CustomSymbolSet;
      pxsch.DMRS.DMRSPortSet              = dlg.DMRSPortSet;
      pxsch.DMRS.NIDNSCID                 = dlg.NIDNSCID;
      pxsch.DMRS.NSCID                    = dlg.NSCID;
      pxsch.DMRS.NumCDMGroupsWithoutData  = dlg.NumCDMGroupsWithoutData;
      % DMRS panel also has some slight differences between DMRS and PTRS
      if ~isPUSCH
        pxsch.DMRS.DMRSReferencePoint     = dlg.DMRSReferencePoint;
      else
        pxsch.DMRS.GroupHopping           = dlg.GroupHopping;
        pxsch.DMRS.SequenceHopping        = dlg.SequenceHopping;
        pxsch.DMRS.NRSID                  = dlg.NRSID;
      end
      
      % PTRS panel
      className = ['wirelessWaveformGenerator.nr5G_' PXSCHfigureName(obj) 'PTRS_Dialog'];
      dlg = obj.getParent.DialogsMap(className);
      pxsch.PTRSPower             = dlg.Power;
      pxsch.EnablePTRS            = dlg.Enable;
      pxsch.PTRS.TimeDensity      = dlg.TimeDensity;
      pxsch.PTRS.FrequencyDensity = dlg.FrequencyDensity;
      pxsch.PTRS.REOffset         = dlg.REOffset;
      pxsch.PTRS.PTRSPortSet      = dlg.PTRSPortSet;
      if isPUSCH
        % Properties only present for PUSCH
        pxsch.PTRS.NumPTRSSamples = dlg.NumPTRSSamples;
        pxsch.PTRS.NumPTRSGroups  = dlg.NumPTRSGroups;
        pxsch.PTRS.NID            = dlg.NID;
      end
      
      obj.pxschConfig{idx} = pxsch;
    end
    
    function mapCache2SidePanel(obj)
      % This method is invoked when there is a need to update the right
      % side panel with contents corresponding to a newly selected row
      
      % Advanced panel
      className = ['wirelessWaveformGenerator.nr5G_' PXSCHfigureName(obj) 'Advanced_Dialog'];
      if ~isKey(obj.getParent.DialogsMap, className)
        createPXSCHAdvancedTab(obj, PXSCHfigureName(obj), className);
      end
      dlg = obj.getParent.DialogsMap(className);
      if ~isempty(obj.selectedPXSCH)
        idx = obj.selectedPXSCH(1);
      else
        idx = 1;
      end
      pxsch = obj.pxschConfig{idx};
      
      dlg.Label       = pxsch.Label;
      dlg.RVSequence  = pxsch.RVSequence;
      dlg.XOverhead   = pxsch.XOverhead;
      
      % Data source is handled slightly different from programmatic API
      if ischar(pxsch.DataSource)
        dlg.DataSource = pxsch.DataSource;
      else
        dlg.DataSource = 'User-defined';
        dlg.CustomDataSource = pxsch.DataSource;
      end
      sourceChangedGUI(dlg, []);
      
      % Advanced panel differs between PDSCH and PUSCH
      isPUSCH = strcmpi(PXSCHfigureName(obj), 'PUSCH');
      if ~isPUSCH
        dlg.VRBToPRBInterleaving  = pxsch.VRBToPRBInterleaving;
        dlg.VRBBundleSize         = pxsch.VRBBundleSize;
        dlg.ReservedCORESET       = pxsch.ReservedCORESET;
        dlg.ReservedPRB           = pxsch.ReservedPRB{1}.PRBSet;
        dlg.ReservedSymbols       = pxsch.ReservedPRB{1}.SymbolSet;
        dlg.ReservedPeriod        = pxsch.ReservedPRB{1}.Period;
        dlg.TBScaling             = pxsch.TBScaling;
      else
        dlg.TransmissionScheme    = pxsch.TransmissionScheme;
        dlg.NumAntennaPorts       = pxsch.NumAntennaPorts;
        dlg.TPMI                  = pxsch.TPMI;
        dlg.FrequencyHopping      = pxsch.FrequencyHopping;
        dlg.SecondHopStartPRB     = pxsch.SecondHopStartPRB;
%         dlg.BetaOffsetACK         = pxsch.BetaOffsetACK;
%         dlg.BetaOffsetCSI1        = pxsch.BetaOffsetCSI1;
%         dlg.BetaOffsetCSI2        = pxsch.BetaOffsetCSI2;
%         dlg.UCIScaling            = pxsch.UCIScaling;
%         dlg.DisableULSCH          = pxsch.DisableULSCH;
      end
      
      % DMRS panel
      className = ['wirelessWaveformGenerator.nr5G_' PXSCHfigureName(obj) 'DMRS_Dialog'];
      dlg = obj.getParent.DialogsMap(className);
      dlg.Power = pxsch.DMRSPower;
      dlg.DMRSConfigurationType    = pxsch.DMRS.DMRSConfigurationType;
      dlg.DMRSTypeAPosition        = pxsch.DMRS.DMRSTypeAPosition;
      dlg.DMRSAdditionalPosition   = pxsch.DMRS.DMRSAdditionalPosition;
      dlg.DMRSLength               = pxsch.DMRS.DMRSLength;
      dlg.CustomSymbolSet          = pxsch.DMRS.CustomSymbolSet;
      dlg.DMRSPortSet              = pxsch.DMRS.DMRSPortSet;
      dlg.NIDNSCID                 = pxsch.DMRS.NIDNSCID;
      dlg.NSCID                    = pxsch.DMRS.NSCID;
      dlg.NumCDMGroupsWithoutData  = pxsch.DMRS.NumCDMGroupsWithoutData;
      % DMRS panel also has some slight differences between DMRS and PTRS
      if ~isPUSCH
        dlg.DMRSReferencePoint     = pxsch.DMRS.DMRSReferencePoint;
      else
        dlg.GroupHopping           = pxsch.DMRS.GroupHopping;
        dlg.SequenceHopping        = pxsch.DMRS.SequenceHopping;
        dlg.NRSID                  = pxsch.DMRS.NRSID;
      end
      
      % PTRS panel
      className = ['wirelessWaveformGenerator.nr5G_' PXSCHfigureName(obj) 'PTRS_Dialog'];
      dlg = obj.getParent.DialogsMap(className);
      dlg.Power = pxsch.PTRSPower;
      dlg.Enable = pxsch.EnablePTRS;
      dlg.TimeDensity      = pxsch.PTRS.TimeDensity;
      dlg.FrequencyDensity = pxsch.PTRS.FrequencyDensity;
      dlg.REOffset         = pxsch.PTRS.REOffset;
      dlg.PTRSPortSet      = pxsch.PTRS.PTRSPortSet;
      if isPUSCH
        % Properties only present for PUSCH
        dlg.NumPTRSSamples = pxsch.PTRS.NumPTRSSamples;
        dlg.NumPTRSGroups  = pxsch.PTRS.NumPTRSGroups;
        dlg.NID            = pxsch.PTRS.NID;
      end
    end

    
    function updateCodingDependentVisibility(obj, idx)
      % TB Scaling, XOverhead in advanced properties only applicable when Coding = true
      % Adjust visuals if current row is displayed in "advanced"
        
      % Get Coding value:
      codingCol = 14;
      coding = obj.pxschTable.Data{idx, codingCol};
      
      % Get Advanced-property panel:
      className = ['wirelessWaveformGenerator.nr5G_' PXSCHfigureName(obj) 'Advanced_Dialog'];
      dlg = obj.getParent.DialogsMap(className);
      oldVis = strcmp(dlg.XOverheadGUI.Visible, 'on');
      
      if xor(oldVis, coding)
        % do re-layout only when it is needed, because it is expensive
        
        % PDSCH & PUSCH both have XOverhead
        props = [dlg.XOverheadLabel, dlg.XOverheadGUI];
        if strcmpi(PXSCHfigureName(obj), 'PDSCH')
          % Only PDSCH has TBScaling
          props = [props dlg.TBScalingLabel, dlg.TBScalingGUI];
        end
        set(props, 'Visible', uiservices.logicaltoonoff(coding));
        layoutUIControls(dlg);
        layoutPanels(dlg);
      end
    end
    
    function updatePrecodingDependentVisibility(obj, idx)
      % Transform precoding affects visibility in DMRS and PTRS panels 
        
      % Get Transform Precoding value:
      precodingCol = 16;
      precoding = obj.pxschTable.Data{idx, precodingCol};
      
      % Get DMRS dialog object:
      classNameDMRS = ['wirelessWaveformGenerator.nr5G_' PXSCHfigureName(obj) 'DMRS_Dialog'];
      dlgDMRS = obj.getParent.DialogsMap(classNameDMRS);
      oldVis = strcmp(dlgDMRS.GroupHoppingGUI.Visible, 'on');
      
      if xor(oldVis, precoding)
        % do re-layout only when it is needed, because it is expensive
        
        % Properties that should only appear when Transform Precoding is on:
        props = [dlgDMRS.GroupHoppingLabel, dlgDMRS.GroupHoppingGUI ...
                 dlgDMRS.SequenceHoppingLabel, dlgDMRS.SequenceHoppingGUI ...
                 dlgDMRS.NRSIDLabel, dlgDMRS.NRSIDGUI];
        set(props, 'Visible', uiservices.logicaltoonoff(precoding));
        
        % Properties that should not appear when Transform Precoding is on:
        props = [dlgDMRS.NIDNSCIDLabel, dlgDMRS.NIDNSCIDGUI ...
                 dlgDMRS.NSCIDLabel, dlgDMRS.NSCIDGUI];
        set(props, 'Visible', uiservices.logicaltoonoff(~precoding));
        
        layoutUIControls(dlgDMRS);
        layoutPanels(dlgDMRS);
        
        % PTRS changes from Transform Precoding changes are handled by
        % updatePTRSEnable -> set.Enable
      end
    end
    
    function updatePTRSEnable(obj, idx)
      % PTRS Enable must be synchronized between table and advanced properties
      
      enable = obj.pxschTable.Data{idx, end};
      
      className = ['wirelessWaveformGenerator.nr5G_' PXSCHfigureName(obj) 'PTRS_Dialog'];
      dlg = obj.getParent.DialogsMap(className);
      dlg.Enable = enable;
    end
    
    function layoutPXSCHTables(obj, margin, verScrollBarSpace)
      % This method executes everytime the App/tiles resize. Need to
      % maintain relative placement given the new "Position" of the figure
      
      if ~isempty(obj.pxschTable) && ishghandle(obj.pxschTable)
        figPos = obj.pxschTable.Parent.Position;
        if all(figPos([3 4])>margin)
            
          pxschWidth = sum([obj.pxschTable.ColumnWidth{:}])+32+verScrollBarSpace;
          obj.pxschTable.Position       = max(0, [...
            10 figPos(4)*0.52  pxschWidth         figPos(4)*0.475-10]);
          
          layoutAddRemoveButtons(obj, obj.pxschTable, 'PXSCH');
        end
               
      end
    end
    
    function pxsch = getPXSCHConfig(obj)
      % Get an equivalent nrXLCarrierConfig.PXSCH configuration based on
      % the so far PXSCH configurations (including the internally cached ones)
      
      % Update cache with latest edits
      mapTableToCache(obj);
      mapSidePanel2Cache(obj);
      
      % Get all nrWavegenPXSCHConfig objects"
      pxsch = obj.pxschConfig;
    end
    
    function applyPXSCHConfig(obj, cfg)
      % Used on New and Open Session
      % Map a cell array of nrWavegenPXSCHConfig objects to the basic PXSCH
      % table, the internal cache, and the right-side panel
      
      if isempty(obj.pxschTable)
        return; % App initialization
      end
      
      obj.selectedPXSCH = []; % Restore default value to avoid wrong mapping between cache and selected rows
      
      pxsch = cfg.(PXSCHfigureName(obj));
      
      % Update internal cache:
      obj.pxschConfig = pxsch;
      isPUSCH = strcmpi(PXSCHfigureName(obj), 'PUSCH');
      % Update table:
      data = cell(numel(pxsch), 16+isPUSCH);
      for idx = 1:numel(pxsch)
        data{idx, 1}  = pxsch{idx}.Enable;
        data{idx, 2}  = pxsch{idx}.Power;
        data{idx, 3}  = num2str(pxsch{idx}.BandwidthPartID);
        data{idx, 4}  = pxsch{idx}.Modulation;
        data{idx, 5}  = num2str(pxsch{idx}.NumLayers);
        data{idx, 6}  = pxsch{idx}.MappingType;
        data{idx, 7}  = num2str(pxsch{idx}.SymbolAllocation(1));
        data{idx, 8}  = num2str(pxsch{idx}.SymbolAllocation(2));
        
        % Try to establish a [0:N] pattern if possible:
        if ~isscalar(pxsch{idx}.SlotAllocation) && ...
                isequal(pxsch{idx}.SlotAllocation, min(pxsch{idx}.SlotAllocation):max(pxsch{idx}.SlotAllocation))
          data{idx, 9} = ['[' num2str(min(pxsch{idx}.SlotAllocation)) ':' num2str(max(pxsch{idx}.SlotAllocation)) ']'];
        else
          data{idx, 9}  = mat2str(pxsch{idx}.SlotAllocation);
        end
        
        data{idx, 10} = pxsch{idx}.Period;

        % Try to establish a [0:N] pattern if possible:
        if ~isscalar(pxsch{idx}.PRBSet) && isequal(pxsch{idx}.PRBSet, min(pxsch{idx}.PRBSet):max(pxsch{idx}.PRBSet))
          data{idx, 11} = ['[' num2str(min(pxsch{idx}.PRBSet)) ':' num2str(max(pxsch{idx}.PRBSet)) ']'];
        else
          data{idx, 11} = mat2str(pxsch{idx}.PRBSet);
        end
        
        data{idx, 12} = pxsch{idx}.NID;
        data{idx, 13} = pxsch{idx}.RNTI;
        data{idx, 14} = pxsch{idx}.Coding;
        if pxsch{idx}.Coding
          data{idx, 15} = pxsch{idx}.TargetCodeRate;
        else
          data{idx, 15} = 'N/A';
        end
        % Transform Precoding value is only applicable for PUSCH
        if isPUSCH
          data{idx, 16} = pxsch{idx}.TransformPrecoding;
        end
        data{idx, end} = pxsch{idx}.EnablePTRS;
      end
      obj.pxschTable.Data = data;
      obj.pxschTable.RowName = num2cell(num2str((1:numel(pxsch))'));

      % Update side panel:
      mapCache2SidePanel(obj);
    end
  end
end

function s = getStrings()

    tableName = 'PXSCHTable';
    def = {'TransformPrecoding' 'Enable' 'Power' 'BWPID' 'Modulation' 'NumLayers' 'MappingType' 'StartSymbol' 'SymbolLength' 'SlotAllocation' 'Period' 'PRBSet' 'NID' 'RNTI' 'Coding' 'TargetCodeRate' 'EnablePTRS'};

    for i = 1:numel(def)
        s.(def{i}) = getString(message(['nr5g:waveformGeneratorApp:' tableName def{i}]));
    end

end