classdef nr5G_SRS_Tables < handle
% SRS tab for fully-customizable 5G Uplink wavegen

%   Copyright 2020-2021 The MathWorks, Inc.

  properties (Hidden)
    srsFig
    srsTable
    addSRSButton
    removeSRSButton
    selectedSRS = []

    % This is the internal representation of the entire SRS
    % configuration(s). It is needed to cache per-SRS edits.
    srsConfig = {}
  end
  
  methods
    function obj = nr5G_SRS_Tables()
    end
    
    function createSRSTables(obj)
      % Spec and creation of "SRS" table
      srsStrings = getStrings();
      if isempty(obj.srsTable)
        spec = {srsStrings.Enable,            'logical',                                            52;
                srsStrings.Power,             [],                                                   53;
                srsStrings.BWPID,             vec2cell(1:size(obj.bwpTable.Data)),                  43;        
                srsStrings.SRSPorts,          vec2cell(nrWavegenSRSConfig.NumSRSPorts_Options),     45;
                srsStrings.SRSSymbols,        vec2cell(nrWavegenSRSConfig.NumSRSSymbols_Options),   63;
                srsStrings.SymbolStart,       vec2cell(nrWavegenSRSConfig.SymbolStart_Options),     57;
                srsStrings.KTC,               vec2cell(nrWavegenSRSConfig.KTC_Options),             40;
                srsStrings.KBarTC,            vec2cell(nrWavegenSRSConfig.KBarTC_Options),          57;
                srsStrings.CyclicShift,       vec2cell(nrWavegenSRSConfig.CyclicShift_Options),     50;
                srsStrings.FrequencyStart,    [],                                                   45;
                srsStrings.NRRC,              [],                                                   45;
                srsStrings.CSRS,              [],                                                   48;
                srsStrings.BSRS,              vec2cell(nrWavegenSRSConfig.BSRS_Options),            48;
                srsStrings.BHop,              vec2cell(nrWavegenSRSConfig.BHop_Options),            45;
                srsStrings.Repetition,        vec2cell(nrWavegenSRSConfig.Repetition_Options),      50;
                srsStrings.SlotAllocation,    'char',                                               50;
                srsStrings.Period,            [],                                                   52;
                srsStrings.GroupSeqHopping,   nrWavegenSRSConfig.GroupSeqHopping_Values,            77;
                srsStrings.NSRSID,            [],                                                   57;
                srsStrings.SRSForPositioning, 'logical',                                            77;
        }; %#ok<*MCNPN>
              
        obj.srsTable = uitable('Parent', obj.srsFig, ...
                  'Position',       [10 10 obj.srsFig.Position(3)-20 obj.srsFig.Position(4)/3-20], ...
                  'ColumnName',     spec(1:end, 1), ...
                  'ColumnFormat',   spec(1:end, 2)', ...
                  'ColumnEditable', true,...
                  'RowName',        {'1'}, ...
                  'ColumnWidth',    spec(1:end, 3)', ...
                  'CellEditCallback', @obj.srsTableCallback, ...
                  'CellSelectionCallback', @obj.selectSRSTableEntry, ...
                  'Tag',            'SRSTable', ...
                  'RearrangeableColumns', false);
        obj.srsTable.Data = newSRSRow(obj);

        % add/remove buttons:
        addRemoveButtons(obj, obj.srsFig, 'SRS');
        % customized remove callback:
        obj.removeSRSButton.Callback = @(a, b, c) removeSRSTableEntry(obj, [], 'SRS');
                
        srsTableCallback(obj);
      end
    end

    function removeSRSTableEntry(obj, ~, tag)
      % remove deleted configs from internal cache:
      obj.srsConfig(obj.selectedSRS) = [];
      
      removeTableEntry(obj, [], tag); % generic remove action

      updateGrid(obj);
    end

    function selectSRSTableEntry(obj, table, data)
      % don't perform any updates if it's not needed, as this can be a
      % considerable UX delay
      rowNames = obj.srsTable.RowName;
      row = data.Indices(:, 1);
      if isempty(row)
        return;
      end
      newID = rowNames{row};
      prevID = num2str(obj.selectedSRS);
      if isequal(newID, prevID)
        noOp = true;
        % no update needed, this happens when clicking on the current SRS
      else
        noOp = false;
      end
      
      % Still need to do base actions, like enabling delete button
      selectTableEntry(obj, table, data); % also highlight row
      
      if ~noOp
          % Update RE mapping plot:
          channel = ['SRS ' newID];
          updateREVisual(obj, channel, row);
      end
    end
  
    function data = newSRSRow(obj)
      % link to the 1st defined BWP
      bwpCol = 3;
      bwps = obj.srsTable.ColumnFormat{bwpCol};
      newBWP = bwps{1};
      
      appendNextAvailableRowName(obj, obj.srsTable); % take care of row name
      
      % get the same defaults with nrWavegenSRSConfig;
      cfg = nrULCarrierConfig; % but get values through nrULCarrierConfig, as Enable is customized
      c = cfg.SRS{1};
      data = {c.Enable, c.Power, newBWP, num2str(c.NumSRSPorts), num2str(c.NumSRSSymbols), ...
            num2str(c.SymbolStart), num2str(c.KTC), num2str(c.KBarTC), num2str(c.CyclicShift), c.FrequencyStart, ...
            c.NRRC, c.CSRS, num2str(c.BSRS), num2str(c.BHop), num2str(c.Repetition), ...
            mat2str(c.SlotAllocation), c.Period, c.GroupSeqHopping, c.NSRSID, c.SRSPositioning };

      % store new object in cache:
      c.BandwidthPartID = str2double(newBWP);
      obj.srsConfig = [obj.srsConfig {c}];

      if c.Enable
          updateGrid(obj);
      end
    end
    
    function srsTableCallback(obj, ~, ~)
      % called when a table value has been edited
      
      % some BWP selections may have now been fixed (corrected BWP ID), see if red link can go away:
      markBrokenBWPLinks(obj);

      obj.getParent.WaveformGenerator.setStatus(''); % clear possibly previous messages

      data = obj.srsTable.Data;

      try  % check each non-dropdown value, by reusing nrWavegenSRSConfig error messages.
           % caught errors are thrown as warnings in status bar

        % error and consistency check:
        for idx = 1:size(data, 1)
          % check each non-dropdown value, by reusing nrWavegenSRSConfig error messages:
          srs = nrWavegenSRSConfig;

          % Enable
          srs.Enable = data{idx, 1};

          % Power
          power = data{idx, 2};
          data{idx, 2} = real(power); % do not leave a wrong value
          srs.Power = power;

          % BandwitdhPartID
          srs.BandwidthPartID = str2double(data{idx, 3});

          % NumSRSSymbols
          srs.NumSRSSymbols = str2double(data{idx, 5});

          % SymbolStart
          srs.SymbolStart = str2double(data{idx, 6});

          % KTC
          srs.KTC = str2double(data{idx, 7});

          % KBarTC
          srs.KBarTC = str2double(data{idx, 8});

          % FrequencyStart
          freqStart = data{idx, 10};
          data{idx, 10} = max(0, min(271, round(real(freqStart)))); % do not leave a wrong value
          srs.FrequencyStart = freqStart;

          % NRRC
          nrrc = data{idx, 11};
          data{idx, 11} = max(0, min(67, round(real(nrrc)))); % do not leave a wrong value
          srs.NRRC = nrrc;

          % CSRS
          csrs = data{idx, 12};
          data{idx, 12} = max(0, min(63, round(real(csrs)))); % do not leave a wrong value
          srs.CSRS = csrs;

          % BSRS
          srs.BSRS = str2double(data{idx, 13});

          % BHop
          srs.BHop = str2double(data{idx, 14});

          % Repetition
          srs.Repetition = str2double(data{idx, 15});

          % SlotAllocation
          slotAlloc = str2num(data{idx, 16}); %#ok<ST2NM>
          srs.SlotAllocation = slotAlloc;

          % Period
          srs.Period = data{idx, 17};

          % GroupSeqHopping
          srs.GroupSeqHopping = data{idx, 18};

          % NSRSID
          nsrsid = data{idx, 19};
          data{idx, 19} = max(0, min(1023, round(real(nsrsid)))); % do not leave a wrong value
          srs.NSRSID = nsrsid;

          % SRSPositioning
          srs.SRSPositioning = data{idx, 20};
        end
        obj.srsConfig = getSRSConfig(obj); % store edits to internal cache

        obj.srsTable.Data = data; % accept good values

        % If everything is ok, update grid plots with new SRS values:
        updateGrid(obj);

        % Update RE mapping plot:
        channel = 'SRS';
        selectedTAG = 'selectedSRS';
        if isempty(obj.(selectedTAG))
            selectedRow = 1;
        else
            selectedRow = obj.(selectedTAG);
        end
        id = obj.srsTable.RowName{selectedRow}; % Get the row name ID
        channel = [channel ' ' id];
        updateREVisual(obj, channel, selectedRow);
      catch e
        warnInStatusBar(obj, e); % throw a warning in the status bar
      end
    end
    
    function layoutSRSTables(obj, margin, verScrollBarSpace)
      % This method executes everytime the App/tiles resize. Need to
      % maintain relative placement given the new "Position" of the figure
      
      if ~isempty(obj.srsTable) && ishghandle(obj.srsTable)
        pos = obj.srsTable.Parent.Position;
        if all(pos([3 4])>margin)                   
          srsWidth = sum([obj.srsTable.ColumnWidth{:}])+32+verScrollBarSpace;
          obj.srsTable.Position       = [10 pos(4)*0.52  srsWidth         pos(4)*0.475-10];
          
          layoutAddRemoveButtons(obj, obj.srsTable, 'SRS');
        end
      end
    end
    
    function srs = getSRSConfig(obj)
      % Convert SRS table to a cell of nrWavegenSRSConfig objects
      data = obj.srsTable.Data;
      
      srs = cell(1, size(data, 1));
      for idx = 1:size(data, 1)
        c = nrWavegenSRSConfig;
%         c.Label = ['SRS' num2str(idx)]; % SRS doesn't have a Label entry yet in the app
        c.Enable = data{idx, 1};
        c.Power = data{idx, 2};
        c.BandwidthPartID = str2double(data{idx, 3});

        c.NumSRSPorts = str2double(data{idx, 4});
        c.NumSRSSymbols = str2double(data{idx, 5});
        c.SymbolStart = str2double(data{idx, 6});
        c.KTC = str2double(data{idx, 7});
        c.KBarTC = str2double(data{idx, 8});
        c.CyclicShift = str2double(data{idx, 9});
        c.FrequencyStart = data{idx, 10};
        c.NRRC = data{idx, 11};
        c.CSRS = data{idx, 12};
        c.BSRS = str2double(data{idx, 13});
        c.BHop = str2double(data{idx, 14});
        c.Repetition = str2double(data{idx, 15});
        
        % Period:
        c.SlotAllocation = str2num(data{idx, 16}); %#ok<ST2NM>
        c.Period = data{idx, 17};
        
        c.GroupSeqHopping = data{idx, 18};
        c.NSRSID = data{idx, 19};
        c.SRSPositioning = data{idx, 20};
        
        srs{idx} = c;
      end
    end
    
    function applySRSConfig(obj, cfg)
      % Used on New and Open Session
      if isempty(obj.srsTable)
        return; % App initialization
      end
      
      % Convert SRS table to a cell of nrWavegenSRSConfig objects
      srs = cfg.SRS;
      % Update internal cache:
      obj.srsConfig = srs;
      data = cell(numel(srs), size(obj.scsTable.Data, 2));
      for idx = 1:numel(srs)
        data{idx, 1} = srs{idx}.Enable;
        data{idx, 2} = srs{idx}.Power;
        data{idx, 3} = num2str(srs{idx}.BandwidthPartID);
        data{idx, 4} = num2str(srs{idx}.NumSRSPorts);
        data{idx, 5} = num2str(srs{idx}.NumSRSSymbols);
        data{idx, 6} = num2str(srs{idx}.SymbolStart);
        data{idx, 7} = num2str(srs{idx}.KTC);
        data{idx, 8} = num2str(srs{idx}.KBarTC);
        data{idx, 9} = num2str(srs{idx}.CyclicShift);
        
        data{idx, 10} = srs{idx}.FrequencyStart;
        data{idx, 11} = srs{idx}.NRRC;
        data{idx, 12} = srs{idx}.CSRS;
        
        data{idx, 13} = num2str(srs{idx}.BSRS);
        data{idx, 14} = num2str(srs{idx}.BHop);
        data{idx, 15} = num2str(srs{idx}.Repetition);

        % Period:
        data{idx, 16} = mat2str(srs{idx}.SlotAllocation);
        data{idx, 17} = srs{idx}.Period;
        
        data{idx, 18} = srs{idx}.GroupSeqHopping;
        data{idx, 19} = srs{idx}.NSRSID;
        data{idx, 20} = srs{idx}.SRSPositioning;
      end
      obj.srsTable.Data = data;
      
      obj.srsTable.RowName = num2cell(num2str((1:numel(srs))'));
    end 
  end
end

function c = vec2cell(num)
  c = cellfun(@num2str,num2cell(num), 'UniformOutput',false);
end

function s = getStrings()

    tableName = 'SRSTable';
    def = {'Enable' 'Power' 'BWPID' 'SRSPorts' 'SRSSymbols' 'SymbolStart' 'KTC' 'KBarTC' 'CyclicShift' 'FrequencyStart' 'NRRC' 'CSRS' 'BSRS' 'BHop' 'Repetition' 'SlotAllocation' 'Period' 'GroupSeqHopping' 'NSRSID' 'SRSForPositioning'};

    for i = 1:numel(def)
        s.(def{i}) = getString(message(['nr5g:waveformGeneratorApp:' tableName def{i}]));
    end

end
