classdef nr5G_SSB_DataSource < wirelessWaveformGenerator.Dialog
% Panel offering data-source configuration for SSB tab

%   Copyright 2020 The MathWorks, Inc.
  
  properties (Dependent)
    DataSource
    DMRSTypeAPosition
    CellBarred
    IntraFreqReselection
    PDCCHConfigSIB1
    SubcarrierSpacingCommon
    Payload
  end  
  
  properties (Hidden)
    TitleString = getString(message('nr5g:waveformGeneratorApp:SSDataSourceTitle'))
    configFcn = @struct
    
    DataSourceType = 'popupmenu'
    DataSourceDropDown = {'MIB', 'PN9-ITU', 'PN9', 'PN11', 'PN15', 'PN23', 'User-defined'}
    DataSourceGUI
    DataSourceLabel
    DMRSTypeAPositionType = 'popupmenu'
    DMRSTypeAPositionDropDown = {'2', '3'}
    DMRSTypeAPositionGUI
    DMRSTypeAPositionLabel
    CellBarredType = 'checkbox'
    CellBarredGUI
    CellBarredLabel
    IntraFreqReselectionType = 'checkbox'
    IntraFreqReselectionGUI
    IntraFreqReselectionLabel
    PDCCHConfigSIB1Type = 'edit'
    PDCCHConfigSIB1GUI
    PDCCHConfigSIB1Label
    SubcarrierSpacingCommonType = 'popupmenu'
    SubcarrierSpacingCommonDropDown = {'15 kHz', '30 kHz'}
    SubcarrierSpacingCommonGUI
    SubcarrierSpacingCommonLabel
    PayloadType = 'edit'
    PayloadGUI
    PayloadLabel
  end
  
  methods
    function obj = nr5G_SSB_DataSource(parent, fig)
      obj@wirelessWaveformGenerator.Dialog(parent, fig); % call base constructor
      
      if ~obj.Parent.WaveformGenerator.useAppContainer
        obj.TitleGUI.Callback = @(a, b) keepPanelPresent(obj); % Turn off panel folding; tab would be too empty
      else
        cb = findall(obj.TitleGUI, 'Tag', 'PanelUnfoldedLabel');
        cb.ValueChangedFcn = @(a, b) keepPanelPresent(obj);
        img = findall(obj.TitleGUI, 'Tag', 'PanelUnfoldedImage');
        img.ImageClickedFcn = @(a, b) keepPanelPresent(obj);
      end
      
      % Callbacks needed either for validation of edit fields or for
      % changes to visibility of other controls
      obj.DataSourceGUI.Callback        = @(a,b) dataSourceChangedGUI(obj, []);
      obj.PDCCHConfigSIB1GUI.Callback   = @(a,b) configChangedGUI(obj);
      obj.PayloadGUI.Callback           = @(a,b) payloadChangedGUI(obj);
      
      % This is not visible by default
      set([obj.PayloadLabel obj.PayloadGUI], 'Visible', false);
      
      if ~(ispc || ismac) % Linux
        obj.ValueWidth = 140; % make sure all checkboxes fit
      end
    end
    
    function adjustSpec(obj)
      % This is needed so that the SSB panel(s) do not horizontally fill the entire App
      obj.panelFixedSize = true;
    end
    
    function adjustDialog(obj)
      % make sure all tags are unique; the same controls can be present in PDSCH
      obj.DataSourceGUI.Tag         = 'SSBDataSource';
      obj.DMRSTypeAPositionGUI.Tag  = 'SSBDMRSTypeAPosition';
    end
    
    function props = displayOrder(~)
      props = {'DataSource'; 'DMRSTypeAPosition'; 'CellBarred'; 'IntraFreqReselection'; ...
              'PDCCHConfigSIB1'; 'SubcarrierSpacingCommon'; 'Payload'};
    end
    
    function restoreDefaults(obj)
      c = nrWavegenSSBurstConfig;  % Get defaults from nrWavegenSSBurstConfig
      
      % Data source is handled slightly different from programmatic API
      if ischar(c.DataSource)
        obj.DataSource = c.DataSource;
        obj.Payload = zeros(1, 24);
      else
        obj.DataSource = 'User-defined';
        obj.Payload = c.DataSource;
      end
      
      obj.DMRSTypeAPosition       = c.DMRSTypeAPosition;
      obj.CellBarred              = c.CellBarred;
      obj.IntraFreqReselection    = c.IntraFreqReselection;
      obj.PDCCHConfigSIB1         = c.PDCCHConfigSIB1;
      obj.SubcarrierSpacingCommon = c.SubcarrierSpacingCommon;
    end
    
    function ssb = addSSBDataSourceConfig(obj, ssb)
      % Append the contents of this panel to an nrWavegenSSBurstConfig configuration
      
      % Data source is handled slightly different from programmatic API
      if ~strcmpi(obj.DataSource, 'User-defined')
        ssb.DataSource = obj.DataSource;
      else
        ssb.DataSource = obj.Payload;
      end
      
      ssb.DMRSTypeAPosition       = obj.DMRSTypeAPosition;
      ssb.CellBarred              = obj.CellBarred;
      ssb.IntraFreqReselection    = obj.IntraFreqReselection;
      ssb.PDCCHConfigSIB1         = obj.PDCCHConfigSIB1;
      ssb.SubcarrierSpacingCommon = obj.SubcarrierSpacingCommon;
    end
    
    function dataSourceChangedGUI(obj, ~)
      % Determine the visibility of all items in this panel based on DataSource type
      
      mib     = strcmp(obj.DataSource, 'MIB');
      set([obj.DMRSTypeAPositionLabel obj.DMRSTypeAPositionGUI ...
           obj.CellBarredLabel obj.CellBarredGUI obj.IntraFreqReselectionLabel ...
           obj.IntraFreqReselectionGUI obj.PDCCHConfigSIB1Label obj.PDCCHConfigSIB1GUI ...
           obj.SubcarrierSpacingCommonLabel obj.SubcarrierSpacingCommonGUI], 'Visible', mib);
         
      payload = strcmp(obj.DataSource, 'User-defined');
      set([obj.PayloadLabel obj.PayloadGUI], 'Visible', payload);
         
      layoutUIControls(obj);
      layoutPanels(obj);
    end

    %% Validators
    function configChangedGUI(obj)
      try
        val = obj.PDCCHConfigSIB1;
        % use error throwing behavior of nrWavegenSSBConfig
        c = nrWavegenSSBurstConfig;
        c.PDCCHConfigSIB1 = val;
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateAppStatusBar(obj);
    end
    
    function payloadChangedGUI(obj)
      try
        val = obj.Payload;
        validateattributes(val, {'double'}, {'binary', 'row'}, ...
          '', 'user-defined payload');
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateAppStatusBar(obj);
    end

    function updateAppStatusBar(obj)
      % Update the status bar to display any warning related to the current
      % configuration or to clear any previous warning that no longer apply
      dlg = obj.Parent.WaveformGenerator.pParameters.CurrentDialog;
      updateStatusBar(dlg);
    end
    
    %% Getters/setters
    function fr = get.DataSource(obj) 
      fr = obj.DataSourceGUI.String{obj.DataSourceGUI.Value};
    end
    function set.DataSource(obj, val) 
      obj.DataSourceGUI.Value = find(strcmp(obj.DataSourceGUI.String, val)); 
    end
    
    function fr = get.DMRSTypeAPosition(obj) 
      fr = str2double(obj.DMRSTypeAPositionGUI.String{obj.DMRSTypeAPositionGUI.Value});
    end
    function set.DMRSTypeAPosition(obj, val) 
      obj.DMRSTypeAPositionGUI.Value = find(strcmp(obj.DMRSTypeAPositionGUI.String, num2str(val))); 
    end
    
    function v = get.CellBarred(obj) 
      v = logical(obj.CellBarredGUI.Value);
    end
    function set.CellBarred(obj, val) 
      obj.CellBarredGUI.Value = val;
    end
    
    function v = get.IntraFreqReselection(obj) 
      v = logical(obj.IntraFreqReselectionGUI.Value);
    end
    function set.IntraFreqReselection(obj, val) 
      obj.IntraFreqReselectionGUI.Value = val;
    end
    
    function n = get.PDCCHConfigSIB1(obj)
      n = evalin('base', obj.PDCCHConfigSIB1GUI.String);
    end
    function set.PDCCHConfigSIB1(obj, val)
      obj.PDCCHConfigSIB1GUI.String = num2str(val);
    end
    
    function scs = get.SubcarrierSpacingCommon(obj) 
      scs = obj.SubcarrierSpacingCommonGUI.String{obj.SubcarrierSpacingCommonGUI.Value};
      scs = str2double(scs(1:end-4)); % drop ' kHz'
    end
    function set.SubcarrierSpacingCommon(obj, val) 
      obj.SubcarrierSpacingCommonGUI.Value = find(startsWith(obj.SubcarrierSpacingCommonGUI.String, num2str(val)));
    end
    
    function n = get.Payload(obj)
      n = evalin('base', obj.PayloadGUI.String);
    end
    function set.Payload(obj, val)
      obj.PayloadGUI.String = mat2str(val);
    end
    
    function str = getCatalogPrefix(~)
      str = 'nr5g:waveformGeneratorApp:';
    end
  end
end
