classdef nr5G_SSB_Dialog < wirelessWaveformGenerator.Dialog
% Panel offering main properties of SSB tab

%   Copyright 2020-2021 The MathWorks, Inc.
  
  properties (Dependent)
    Enable
    BlockPattern
    TransmittedBlocks
    Period
    FrequencyOffset
    KSSB
    NCRBSSB
    Power
  end  
  
  properties (Hidden)
    TitleString = getString(message('nr5g:waveformGeneratorApp:SSBurstTitle'))
    configFcn = @struct
    
    EnableType = 'checkbox'
    EnableGUI
    EnableLabel
    BlockPatternType = 'popupmenu'
    BlockPatternDropDown = {'Case A (15 kHz)', 'Case B (30 kHz)', 'Case C (30 kHz)'}
    BlockPatternGUI
    BlockPatternLabel
    TransmittedBlocksType = 'edit'
    TransmittedBlocksGUI
    TransmittedBlocksLabel
    PeriodType = 'popupmenu'
    PeriodDropDown = {'5', '10', '20', '40', '80', '160'}
    PeriodGUI
    PeriodLabel
    FrequencyOffsetType = 'popupmenu'
    FrequencyOffsetDropDown = {'Carrier center', 'Custom'}
    FrequencyOffsetGUI
    FrequencyOffsetLabel
    KSSBType = 'edit'
    KSSBGUI
    KSSBLabel
    NCRBSSBType = 'edit'
    NCRBSSBGUI
    NCRBSSBLabel
    PowerType = 'edit'
    PowerGUI
    PowerLabel
  end
  
  methods
    function obj = nr5G_SSB_Dialog(parent, fig)
      obj@wirelessWaveformGenerator.Dialog(parent, fig); % call base constructor
      
       % Turn off panel folding; tab would be too empty
      if ~obj.Parent.WaveformGenerator.useAppContainer
        obj.TitleGUI.Callback = @(a, b) keepPanelPresent(obj); 
      else
        % Turn off panel folding; panels are stacked horizontally, not vertically
        cb = findall(obj.TitleGUI, 'Tag', 'PanelUnfoldedLabel');
        cb.ValueChangedFcn = @(a, b) keepPanelPresent(obj);
        img = findall(obj.TitleGUI, 'Tag', 'PanelUnfoldedImage');
        img.ImageClickedFcn = @(a, b) keepPanelPresent(obj);
      end
      
      % Callbacks needed either for validation of edit fields or for
      % changes to visibility of other controls
      obj.PowerGUI.Callback             = @(a,b) powerChangedGUI(obj);
      obj.TransmittedBlocksGUI.Callback = @(a,b) txBlocksChangedGUI(obj);
      obj.FrequencyOffsetGUI.Callback   = @(a,b) frequencySourceChangedGUI(obj);
      obj.NCRBSSBGUI.Callback           = @(a,b) numCRBChangedGUI(obj);
      obj.KSSBGUI.Callback              = @(a,b) kSSBChangedGUI(obj);
      % Other controls that only need a live grid update
      set([obj.EnableGUI, obj.BlockPatternGUI, obj.PeriodGUI], ...
                      'Callback', @(a,b) updateGrid(obj.Parent.WaveformGenerator.pParameters.CurrentDialog));
      
      % These are turned off by default, we are using an explicit setting to center the SSB
      set([obj.NCRBSSBGUI obj.NCRBSSBLabel obj.KSSBLabel obj.KSSBGUI], 'Visible', 'off');
    end        
    
    function adjustSpec(obj)
      % This is needed so that the SSB panel(s) do not horizontally fill the entire App
      obj.panelFixedSize = true;
    end
    
    function adjustDialog(obj)
      % Make sure all tags are unique. Otherwise there is a conflict with
      % other channels/signals
      obj.EnableGUI.Tag = 'SSBEnable';
    end
    
    function props = displayOrder(~)
      props = {'Enable'; 'Power'; 'BlockPattern'; 'TransmittedBlocks'; 'Period'; ...
                'FrequencyOffset'; 'NCRBSSB'; 'KSSB'};
    end
    
    function restoreDefaults(obj)
      c = nrWavegenSSBurstConfig;  % Get defaults from nrWavegenSSBurstConfig
      obj.Enable            = c.Enable;
      obj.BlockPattern      = c.BlockPattern;
      obj.TransmittedBlocks = c.TransmittedBlocks;
      obj.Period = c.Period;
      % The interface is slightly different from the programmatic API
      if isempty(c.NCRBSSB)
        obj.FrequencyOffset = 'Carrier center';
      else
        obj.FrequencyOffset = 'Custom';
      end
      obj.KSSB              = c.KSSB;
      obj.NCRBSSB           = c.NCRBSSB;
      obj.Power             = c.Power;
    end
    
    function ssb = getSSBConfig(obj)
      % Map GUI elements of SS Burst tab to an equivalent nrWavegenSSBurstConfig object:
      ssb = nrWavegenSSBurstConfig;
      
      ssb.Enable = obj.Enable;
      ssb.BlockPattern = obj.BlockPattern;
      ssb.TransmittedBlocks = obj.TransmittedBlocks;
      ssb.Period = obj.Period;
      ssb.Power = obj.Power;
      
      % NCRBSSB/KSSB are handled slighlty differently from programmaric API
      if strcmp(obj.FrequencyOffset, 'Carrier center')
        ssb.NCRBSSB = [];
      else
        ssb.NCRBSSB = obj.NCRBSSB;
      end
      ssb.KSSB = obj.KSSB;
    end
    
    %% Validators
    function powerChangedGUI(obj)
      try
        val = obj.Power;
        % use error throwing behavior of nrWavegenSSBConfig
        c = nrWavegenSSBurstConfig;
        c.Power = val;
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateAppStatusBar(obj);
    end
    
    function txBlocksChangedGUI(obj)
      % The App also supports vector of integer indices, e.g., [3 7 51]
      try
        val = str2num(obj.TransmittedBlocksGUI.String); %#ok<ST2NM>
        if any(val>1) || length(val) == 1
          validateattributes(val,{'double'},{'integer', 'row', 'positive', '<=' 64}, '', 'Transmitted Blocks');
        else
          % binary vector; length must be 4, 8 or 64
          validateattributes(val,{'double'},{'binary','row'}, '', 'Transmitted Blocks');
          if ~any(length(val)==[4 8 64])
            error(message('nr5g:nrWaveformGenerator:InvalidTxBlocks'));
          end
        end
        updateGrid(obj.Parent.WaveformGenerator.pParameters.CurrentDialog);
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateAppStatusBar(obj);
    end
    
    function numCRBChangedGUI(obj)
      try
        val = obj.NCRBSSB;
        % use error throwing behavior of nrWavegenSSBConfig
        c = nrWavegenSSBurstConfig;
        c.NCRBSSB = val;
        
        updateGrid(obj.Parent.WaveformGenerator.pParameters.CurrentDialog);
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateAppStatusBar(obj);
    end
    
    function kSSBChangedGUI(obj)
      try
        val = obj.KSSB;
        % use error throwing behavior of nrWavegenSSBConfig
        c = nrWavegenSSBurstConfig;
        c.KSSB = val;
        
        updateGrid(obj.Parent.WaveformGenerator.pParameters.CurrentDialog);
      catch e
        obj.errorFromException(e);
      end
      % Update message in status bar
      updateAppStatusBar(obj);
    end
    
    function frequencySourceChangedGUI(obj, ~)
      % Update visibility of NCRBSSB and KSSB fields based on explicit
      % control on frequency centering
      
      custom = strcmpi(obj.FrequencyOffset, 'Custom');
      set([obj.NCRBSSBGUI obj.NCRBSSBLabel obj.KSSBLabel obj.KSSBGUI], ...
                      'Visible', uiservices.logicalToOnOff(custom));
      if custom
        obj.NCRBSSB = 0; % Not using [] default, as we have FrequencyOffset = 'Carrier center' for that
      else
        obj.NCRBSSB = [];
      end
                    
      updateGrid(obj.Parent.WaveformGenerator.pParameters.CurrentDialog);
      
      layoutUIControls(obj);
      layoutPanels(obj);
    end

    function updateAppStatusBar(obj)
      % Update the status bar to display any warning related to the current
      % configuration or to clear any previous warning that no longer apply
      dlg = obj.Parent.WaveformGenerator.pParameters.CurrentDialog;
      updateStatusBar(dlg);
    end
    
    
    %% Getters/setters
    function v = get.Enable(obj) 
      v = logical(obj.EnableGUI.Value);
    end
    function set.Enable(obj, val) 
      obj.EnableGUI.Value = val;
    end
    
    function fr = get.BlockPattern(obj) 
      fr = obj.BlockPatternGUI.String{obj.BlockPatternGUI.Value};
      str = 'Case X';
      fr = fr(1:length(str));
    end
    function set.BlockPattern(obj, val) 
      obj.BlockPatternGUI.Value = find(startsWith(obj.BlockPatternGUI.String, val)); 
    end
    
    function n = get.TransmittedBlocks(obj)
      % Also support vector of integer indices, e.g., [3 7 51]
      n = evalin('base', obj.TransmittedBlocksGUI.String);
      lengths = [4 8 64];
      if any(n>1) || ~any(length(n)==lengths)
        % Instead of a binary c
        maxL = lengths(min(3, find(max(n) > [-1 lengths], 1, 'last')));
        temp = zeros(1, maxL);
        temp(n) = 1;
        n = temp;
      end
    end
    function set.TransmittedBlocks(obj, val)
      obj.TransmittedBlocksGUI.String = mat2str(val);
    end
    
    function fr = get.Period(obj) 
      fr = str2double(obj.PeriodGUI.String{obj.PeriodGUI.Value});
    end
    function set.Period(obj, val) 
      obj.PeriodGUI.Value = find(strcmp(obj.PeriodGUI.String, num2str(val))); 
    end
    
    function fr = get.FrequencyOffset(obj) 
      fr = obj.FrequencyOffsetGUI.String{obj.FrequencyOffsetGUI.Value};
    end
    function set.FrequencyOffset(obj, val) 
      obj.FrequencyOffsetGUI.Value = find(strcmp(obj.FrequencyOffsetGUI.String, val)); 
    end
    
    function n = get.KSSB(obj)
      n = evalin('base', obj.KSSBGUI.String);
    end
    function set.KSSB(obj, val)
      obj.KSSBGUI.String = num2str(val);
    end
    
    function n = get.NCRBSSB(obj)
      n = evalin('base', obj.NCRBSSBGUI.String);
    end
    function set.NCRBSSB(obj, val)
      obj.NCRBSSBGUI.String = mat2str(val);
    end
    
    function n = get.Power(obj)
      n = evalin('base', obj.PowerGUI.String);
    end
    function set.Power(obj, val)
      obj.PowerGUI.String = num2str(val);
    end
    
    function str = getCatalogPrefix(~)
      str = 'nr5g:waveformGeneratorApp:';
    end
    
    % Stack the 2 panels horizontally
    function cellDialogs = getDialogsPerColumn(obj)
      cellDialogs{1} = {obj};
      cellDialogs{2} = {obj.Parent.WaveformGenerator.pParameters.DialogsMap('wirelessWaveformGenerator.nr5G_SSB_DataSource')};
    end
  end
end
