classdef nr5G_TM_Dialog < wirelessWaveformGenerator.nr5G_Dialog
%

%   Copyright 2019-2021 The MathWorks, Inc.
  
  properties (Dependent)
    TMN               % Test Model Name/Number
  end  
  
  properties (Hidden)
    TitleString = getString(message('nr5g:waveformGeneratorApp:nrTMTitle'))
    
    TMNType = 'popupmenu'
    TMNDropDown
    TMNGUI
    TMNLabel
  end
  
  methods
    function obj = nr5G_TM_Dialog(parent)
      obj@wirelessWaveformGenerator.nr5G_Dialog(parent); % call base constructor
             
      % Sync GUI controls with dialog state
      windowingChangedGUI(obj, []);
      srChangedGUI(obj, []);
      phaseCompChangedGUI(obj, []);

      obj.TMNGUI.Callback    = @(a,b) TMNChanged(obj);

      % Enable toolbar in grid fig if using Java version
      useJava = ~obj.Parent.WaveformGenerator.useAppContainer;
      if useJava
        % Enable interactive toolbars for grid fig:
        figName = 'Resource Grid (BWP#1)';
        ax = obj.getVisualAxes(figName);
        ax.Toolbar.Visible = 'on'; % interactive toolbar
      end
    end
    
    function adjustSpec(obj)
      obj.TMNDropDown = obj.getTMNText('FR1');
      
      % For MATLAB script generation
      obj.configGenFcn  = @nrDLCarrierConfig;
      obj.configGenVar  = 'cfgDLTM';
    end
    
    function str = getIconDrawingCommand(obj)
        str = ['disp([' obj.configGenVar '.Label newline ...' newline ...
                '''SCS: '' num2str(' obj.configGenVar '.BandwidthParts{1}.SubcarrierSpacing) ''kHz'' newline ...' newline ...
                '''Bandwidth: '' num2str(' obj.configGenVar '.ChannelBandwidth) ''MHz'' newline ...' newline ...
                '''Subframes: '' num2str(' obj.configGenVar '.NumSubframes) ]);'];
    end
    
    function [blockName, maskTitleName, waveNameText] = getMaskTextWaveName(~)
        blockName = '5G NR Test Model';
        maskTitleName = '5G NR Test Model Waveform Generator';
        waveNameText = 'a 5G NR test model';
    end
    
    function tmnDropdown = getTMNText(obj, fr)
      % construct customer visible text from TMNs, e.g., "NR-FR1-TM1.1 (Full-band, uniform QPSK)"
      
      tmnDropdown = [];
      
      if strcmp(fr, 'FR1')
        tmnsFR1 = {'1.1', '1.2', '2', '2a', '3.1', '3.1a', '3.2', '3.3'};
        for idx = 1:length(tmnsFR1)
          tm = tmnsFR1{idx};
          tmNoDot = replace(tm, '.', ''); % message IDs cannot have dots
          tmnDropdown{idx} = ['NR-FR1-TM' tm ' ' ...
            obj.getMsgString(['TMN' tmNoDot])]; %#ok<AGROW>
        end
        
      else % FR2
        tmnsFR2 = {'1.1', '2', '2a', '3.1', '3.1a'};        
        for idx = 1:length(tmnsFR2)
          tm = tmnsFR2{idx};
          tmNoDot = replace(tm, '.', ''); % message IDs cannot have dots
          tmnDropdown{idx} = ['NR-FR2-TM' tm ' ' ...
            obj.getMsgString(['TMN' tmNoDot])]; %#ok<AGROW>
        end
      end
    end

    function props = displayOrder(~)
      props = {'FrequencyRange'; 'TMN'; 'ChannelBandwidth'; 'SubcarrierSpacing'; 'DuplexMode'; ...
               'NCellID'; ...
               'WindowingSource'; 'WindowingPercent'; 'SampleRateSource'; ...
               'SampleRate'; 'PhaseCompensation'; 'CarrierFrequency'};
    end
    

    % Reset the dialog's backing parameters
    function restoreDefaults(obj)
      obj.TMN               = 'NR-FR1-TM1.1';
      
      restoreDefaults@wirelessWaveformGenerator.nr5G_Dialog(obj);  
      obj.DuplexMode = 'TDD'; % The standard only specified NR-TM FDD modes for FR1, so TDD is a better common choice across FR1 and FR2 
    
      obj.NCellID = 1;      % Cell identity
      restoreMods(obj);     % Set modulation related parameters

    end
    
    % Apply 'configuration' that may be presented from previously saved 'dialog state' 
    function applyConfiguration(obj, cfg)
        
          % Direct name based field/property mapping
          % from 'cfg' contents to matching dialog properties  
          applyConfiguration@wirelessWaveformGenerator.nr5G_Dialog(obj,cfg);
           
          % Update GUI controls that depend on dialog properties
          windowingChangedGUI(obj, []);
          srChangedGUI(obj, []);
          phaseCompChangedGUI(obj, []);

    end
    
    % Get parameter 'configuration' for the dialog state
    function cfg = getConfiguration(obj)
        
      % Get preset parameter definitions
      pinnedversion = "15.7.0";  % Version of NR-TM definition in use
      refObj = wirelessWaveformGenerator.internal.hNRReferenceWaveformGenerator(obj.TMN, obj.ChannelBandwidth, obj.SubcarrierSpacing, obj.DuplexMode,obj.NCellID,pinnedversion);
      cfg = refObj.Config;
      
      % Set the modulation related parameters
      cfg = getConfigurationMods(obj,cfg); 

    end
    
    % Get 'configuration' that will be persistent in a save
    function config = getConfigurationForSave(obj)
      config.waveform = getConfigurationForSave@wirelessWaveformGenerator.Dialog(obj);
      config.filtering = getConfigurationForSave(obj.Parent.FilteringDialog);
    end
    
    % Generate waveform from the configuration
    function waveform = generateWaveform(obj)
      cfg = getConfiguration(obj);
      [waveform, obj.gridSet] = nrWaveformGenerator(cfg);
    end
    
    function frChanged(obj, ~)
      % update list of TMs for this frequency range:
      obj.TMNGUI.Value = 1; % to avoid out-of-bound from previous setting
      obj.TMNGUI.String = obj.getTMNText(obj.FrequencyRange);
    
      frChanged@wirelessWaveformGenerator.nr5G_Dialog(obj);
    end

    function scsChanged(obj, ~)
      scsChanged@wirelessWaveformGenerator.nr5G_Dialog(obj);

      updateGrid(obj);
      updateChannelView(obj);
    end

    function TMNChanged(obj)
      updateGrid(obj);
    end
    
    function tmn = get.TMN(obj) 
      tmn = obj.TMNGUI.String{obj.TMNGUI.Value};
      % trim everything after 1st space, e.g., (Full-band, uniform QPSK)
      idx = strfind(tmn, ' ');
      tmn = tmn(1:idx(1)-1);
    end
    function set.TMN(obj, val) 
      % add space in the end, as string '2' is contained within '2a' (false positive)
      obj.TMNGUI.Value = find(contains(obj.TMNGUI.String, [val ' '])); 
    end
    
    function cellDialogs = getDialogsPerColumn(obj)
      cellDialogs{1} = {obj obj.Parent.FilteringDialog};
    end
  end
end