classdef nr5G_UL_Dialog < wirelessWaveformGenerator.nr5G_Full_Base_Dialog & ...
                          wirelessWaveformGenerator.nr5G_PUCCH_Tables & ...
                          wirelessWaveformGenerator.nr5G_PXSCH_Tables & ...
                          wirelessWaveformGenerator.nr5G_SRS_Tables
% Fully-customizable 5G Uplink extension for Wireless Waveform Generator App

%   Copyright 2020-2021 The MathWorks, Inc.
  
  properties (Dependent)
    
  end  
  
  properties (Hidden)
    TitleString = getString(message('nr5g:waveformGeneratorApp:nrULTitle'))
    
    % Cache default nrULCarrierConfig for UX speedup. Constructor takes 0.5 sec
    % because of numerous properties and validations (validatestring). CTOR
    % is exercised in updateGrid -> getConfiguration -> nrULCarrierConfig
    cachedDefaultCfg = nrULCarrierConfig

    mainULFig
    ssBurstFig
  end
  
  methods
  
    function obj = nr5G_UL_Dialog(parent)
      obj@wirelessWaveformGenerator.nr5G_Full_Base_Dialog(parent, false); % call base constructor
      
      updateGrid(obj); % paint grid for default vals
    end
  
    function names = figureNames(~)
      names = {'Main', 'PUSCH', 'PUCCH', 'SRS'};
    end
   function tag = figTag(~, name)
      switch name
        case 'Main'
          tag = 'mainULFig';
        case 'PUSCH'
          tag = 'pxschFig';
        case 'PUCCH'
          tag = 'pucchFig';
        case 'SRS'
          tag = 'srsFig';
      end
    end
    function name = PXSCHfigureName(~)
      name = 'PUSCH';
    end
    
    function tag = mainFigTag(~)
      tag = 'mainULFig';
    end

    function tag = pxschExtraFigTag(~)
      tag = 'puschSingleChannelFig';
    end

    function tag = pxcchExtraFigTag(~)
      tag = 'pxcchSingleChannelFig';
    end
    
    function createFigures(obj)
      % Create figures for all 3 tabs:
      obj.createFig('PUSCH',                              'pxschFig',               'puschFig');
      obj.createFig('PUCCH',                              'pucchFig');
      
%       if ~obj.Parent.WaveformGenerator.useAppContainer
        obj.createFig('PUSCH 1 - Advanced Configuration',   'pxschSingleChannelFig',  'puschSingleChannelFig');
        if obj.Parent.WaveformGenerator.useAppContainer
          obj.Parent.WaveformGenerator.AppContainer.getDocument('figureDocumentGroup', pxschExtraFigTag(obj)).Visible = false;
        else
          obj.pxschSingleChannelFig.Visible = false;
        end
	      obj.createFig('PUCCH 1 - Advanced Configuration',    'pxcchSingleChannelFig');
        if obj.Parent.WaveformGenerator.useAppContainer
          obj.Parent.WaveformGenerator.AppContainer.getDocument('figureDocumentGroup', pxcchExtraFigTag(obj)).Visible = false;
        else
          obj.pxcchSingleChannelFig.Visible = false;
        end
%       end
      obj.createFig('SRS',                                'srsFig');
    end

    function createTables(obj)
      % Create tables for all figures:
      createSCSTable(obj);
      createBWPTable(obj);
      
      createPXSCHTables(obj);
      
      createPUCCHTables(obj);
      
      createSRSTables(obj);
    end
    
    function setExtraConfigFigVisibility(obj, visibility)
      % This method turns on or off all tabs if the waveform type changes
      % (e.g., Full DL -> WLAN)
      
      
      appObj = obj.Parent.WaveformGenerator;
      if appObj.useAppContainer
        if ~visibility
          % exiting this waveform type, need to close documents
          appObj.AppContainer.closeDocument('figureDocumentGroup', 'mainULFig');
          appObj.AppContainer.closeDocument('figureDocumentGroup', 'puschFig');
          doc = appObj.AppContainer.getDocument("figureDocumentGroup", 'puschSingleChannelFig');
          if ~isempty(doc) && doc.Visible
            appObj.AppContainer.closeDocument('figureDocumentGroup', 'puschSingleChannelFig');
          end
          appObj.AppContainer.closeDocument('figureDocumentGroup', 'pucchFig');
          doc = appObj.AppContainer.getDocument("figureDocumentGroup", 'pxcchSingleChannelFig');
          if ~isempty(doc) && doc.Visible
            appObj.AppContainer.closeDocument('figureDocumentGroup', 'pxcchSingleChannelFig');
          end
          appObj.AppContainer.closeDocument('figureDocumentGroup', 'srsFig');
        end
      else
      
        figs = [obj.pxschFig obj.pucchFig obj.srsFig];
        if ~visibility
	      % do not enable the PXSCH or PUCCH right panel if starting this waveform type
          figs = [figs, obj.pxschSingleChannelFig, obj.pxcchSingleChannelFig];
        end
        set(figs, 'Visible', visibility);
        
        if visibility && any(strcmp(obj.lastFigInFocus, {PXSCHfigureName(obj), 'PUCCH'}))
          % We need some time for the figure to disappear before laying out
          % the tiles. This is for the case Downlink 5G (PXSCH tab) -> other
          % waveform type (e.g., WLAN) -> Downlink 5G (must be Main tab)
          pause(0.5);
        end
      end
    end
    
    function cleanupDlg(obj)
      % Dialog-specific cleanup when app is closing
      % Custom 5G UL has additional UI objects that need deletion when the
      % app is closing.
      delete([obj.pxschFig; obj.pxschSingleChannelFig; obj.pucchFig; obj.pxcchSingleChannelFig; obj.srsFig])
    end

    function cfg = getConfiguration(obj)
      % Map all graphical content into an equivalent nrULCarrierConfig object
      
      % Top level:
      cfg = obj.cachedDefaultCfg; % default nrULCarrierConfig object
      cfg.Label             = obj.Label;
      cfg.FrequencyRange    = obj.FrequencyRange;
      cfg.ChannelBandwidth  = obj.ChannelBandwidth;
      cfg.NCellID           = obj.NCellID;
      cfg.NumSubframes      = obj.NumSubframes;
      
      % Windowing controls are slightly different than programmatic API
      if strcmp(obj.WindowingSource, 'Auto')
        cfg.WindowingPercent = [];
      else
        cfg.WindowingPercent = obj.WindowingPercent;
      end
      
      if strcmp(obj.SampleRateSource, 'Auto')
        cfg.SampleRate = [];
      else
        cfg.SampleRate = obj.SampleRate;
      end
      
      if obj.PhaseCompensation
        cfg.CarrierFrequency   = obj.CarrierFrequency;
      else
        cfg.CarrierFrequency   = 0;
      end
      
      % SCS carriers & BWP:
      cfg.SCSCarriers = getCarrierConfig(obj);
      cfg.BandwidthParts = getBWPConfig(obj);
      
      % PUSCH
      cfg.PUSCH = getPXSCHConfig(obj);
      
      % PUCCH
      cfg.PUCCH = getPUCCHConfig(obj);

      % SRS
      cfg.SRS = getSRSConfig(obj);
    end
    
    function figResizedCallback(obj, ~)
      % This method executes when the App or its tile resize. The tables
      % (+ text labels) need to be repositioned.
      
      margin = 20;
      verScrollBarSpace = 20;
      
      layoutSCSBWPTables(obj, margin, verScrollBarSpace);
      
      layoutPXSCHTables(obj, margin, verScrollBarSpace);
      
      layoutPUCCHTables(obj, margin, verScrollBarSpace);
      
      layoutSRSTables(obj, margin, verScrollBarSpace);
    end
    
    function str = getIconDrawingCommand(obj)
        str = ['disp([''Freq. range: '' ' obj.configGenVar '.FrequencyRange newline ...' newline ...
                '''Bandwidth: '' num2str(' obj.configGenVar '.ChannelBandwidth) ''MHz'' newline ...' newline ...
                '''Subframes: '' num2str(' obj.configGenVar '.NumSubframes) ]);'];
    end
    
    function [blockName, maskTitleName, waveNameText] = getMaskTextWaveName(obj)
        blockName = ['5G NR ' obj.Parent.WaveformGenerator.pCurrentWaveformType];
        maskTitleName = ['5G NR ' obj.Parent.WaveformGenerator.pCurrentWaveformType ' Waveform Generator'];
        waveNameText = 'a 5G NR uplink' ;
    end
    
    function markAllBrokenLinks(obj)
      % Update all broken links by highlighting red color in invalid rows
      markBrokenBWPLinks(obj);
      markBrokenSCSLinks(obj);
    end
    
    function markBrokenBWPLinks(obj)
      % paint red the rows that do not link to a valid BWP
      
      markBrokenLinks(obj, obj.bwpTable, obj.pxschTable, 3);
      markBrokenLinks(obj, obj.bwpTable, obj.srsTable, 3);
      markBrokenLinks(obj, obj.bwpTable, obj.pucchTable, 5);
    end
    
    function updateBWPoptions(obj)
      updateBWPoptionsBase(obj, obj.pxschTable, 3);
      updateBWPoptionsBase(obj, obj.pucchTable, 5);
      updateBWPoptionsBase(obj, obj.srsTable, 3);
    end
    
    function adjustSpec(obj)
      adjustSpec@wirelessWaveformGenerator.nr5G_Full_Base_Dialog(obj);
      
      obj.panelFixedSize = true; % prevents panel containing top-level 
                                % properties (NumSubframes etc.) from
                                % filling horizontal space and covering
                                % SCS, BWP tables
                                
      % For MATLAB Script generation:
      obj.configGenFcn  = @nrULCarrierConfig;
      obj.configGenVar  = 'cfgUL';
    end
    
    function adjustDialog(obj)
      adjustDialog@wirelessWaveformGenerator.nr5G_Full_Base_Dialog(obj);
      
      obj.Parent.WaveformGenerator.pParametersFig.Tag = 'WavegenFigUL'; % For testing, to avoid conflict with DL
    end
    
    function updatePXCCHSidePanel(obj, flag)
        obj.pxcchExtraTile = flag;
        
        if obj.Parent.WaveformGenerator.useAppContainer
          obj.Parent.WaveformGenerator.AppContainer.getDocument('figureDocumentGroup', pxcchExtraFigTag(obj)).Visible = flag;
        else
          obj.pxcchSingleChannelFig.Visible = flag;
        end
    end
    
    function restoreDefaults(obj)
      obj.pxschExtraTile = false;
      obj.selectedPXSCH = [];
      obj.selectedPXCCH = [];
      obj.selectedSRS = [];
      className = ['wirelessWaveformGenerator.nr5G_' PXSCHfigureName(obj) 'Advanced_Dialog'];
      if isKey(obj.Parent.DialogsMap, className) && ~isempty(obj.pxschTable) && ishghandle(obj.pxschTable)
        updateAdvancedPXSCH(obj);  % reset the PXSCH num to 1.
      end
      
      cfg = obj.cachedDefaultCfg; % default nrULCarrierConfig object
      cfg.Label = 'Carrier1';
      
      % One difference is that carriers are prepared for equal span in
      % different SCS (maximum occupancy).
      def15kHzNRB = 270;
      cfg.SCSCarriers{1}.NSizeGrid   = def15kHzNRB;
      cfg.SCSCarriers{1}.NStartGrid  = 3;
      cfg.BandwidthParts{1}.NSizeBWP = def15kHzNRB;
      cfg.BandwidthParts{1}.NStartBWP = 3;
      
      cfg.PUSCH{1}.PRBSet = 0:(def15kHzNRB-1);
      cfg.PUSCH{1}.NID = 1; % avoid showing [] in the table
      cfg.PUSCH{1}.EnablePTRS = logical(cfg.PUSCH{1}.EnablePTRS);
      cfg.PUSCH{1}.TransformPrecoding = logical(cfg.PUSCH{1}.TransformPrecoding);
      
      cfg.PUCCH{1}.Label = 'PUCCH1';

      applyConfiguration(obj, cfg);
    end
    
    function applyConfiguration(obj, cfg)
      % Used on New and Open Session.
      % Map an nrULCarrierConfig object to all tables and UI elements
      applyConfiguration@wirelessWaveformGenerator.nr5G_Full_Base_Dialog(obj, cfg);
      
      applyPXSCHConfig(obj, cfg);
      applyPUCCHConfig(obj, cfg);
      applySRSConfig(obj, cfg);
    end

    function w = getColumnWeights(obj, ~)
      % The right-side column (PDSCH Advanced config or Spectrum Analyzer)
      % is a bit smaller.
      if obj.Parent.WaveformGenerator.useAppContainer
        c = 0.7807;
        w = [nan; c; 1-c];
      else
        w = [0.725; 0.275];
      end
    end

    function [tileCount, tileCoverage, tileOccupancy] = getTileLayout(obj, numScopes)
      [tileCount, tileCoverage, tileOccupancy] = getTileLayout@wirelessWaveformGenerator.nr5G_Full_Base_Dialog(obj, numScopes);
     
      figs = {'puschFig', 'pucchFig', 'srsFig'};
      for idx = 1:numel(figs)
        tileID = 1;
        documentID = ['figureDocumentGroup_' figs{idx}];
        str = struct('showOrder', 1+idx, 'id', documentID);
        tileOccupancy(tileID).children = [tileOccupancy(tileID).children str];
      end

      % Right-side Advanced PUCCH panel:
      tileID = tileID + 1;
      if obj.pxcchExtraTile
        documentID = 'figureDocumentGroup_pxcchSingleChannelFig';
        str = struct('showOrder', 1, 'id', documentID);
        tileOccupancy(tileID).children = [tileOccupancy(tileID).children str];
        tileOccupancy(tileID).showingChildId = documentID;
        tileOccupancy(1).showingChildId = 'figureDocumentGroup_pucchFig';

        tileCoverage = [1 2; 3 2];
      end
    end
    
    % Get GUI table from channel name
    function t = getChannelTable(dialog,channel)
        switch channel
            case 'PUSCH'
                t = dialog.pxschTable;
            case 'PUCCH'
                t = dialog.pucchTable;
            case 'SRS'
                t = dialog.srsTable;
            otherwise
                t = [];
        end
    end

  end

end