function [sym,info] = nrCSIRS(carrier,csirs,varargin)
%nrCSIRS Channel state information reference signal
%   [SYM,INFO] = nrCSIRS(CARRIER,CSIRS) returns Channel State Information
%   Reference Signal (CSI-RS) symbols SYM as defined in TS 38.211 Section
%   7.4.1.5, given carrier specific configuration object CARRIER and CSI-RS
%   specific configuration object CSIRS for one or more zero-power (ZP) or
%   non-zero-power (NZP) CSI-RS resource configurations. The function also
%   provides some additional information INFO regarding the CSI-RS
%   resources. Note that when multiple CSI-RS resources are configured,
%   irrespective of the input resource order, the output SYM is provided in
%   the order of ZP-CSI-RS resources followed by NZP-CSI-RS resources. With
%   this, for the resources of same configuration, the priority is given to
%   NZP-CSI-RS over ZP-CSI-RS while mapping the CSI-RS symbols to the grid.
%
%   CARRIER is a carrier specific configuration object as described in
%   <a href="matlab:help('nrCarrierConfig')">nrCarrierConfig</a> with the following properties:
%
%   SubcarrierSpacing     - Subcarrier spacing in kHz
%   CyclicPrefix          - Cyclic prefix type
%   NSlot                 - Absolute slot number
%   NFrame                - Absolute system frame number
%   NSizeGrid             - Size of the carrier resource grid in terms of
%                           number of resource blocks (RBs)
%   NStartGrid            - Starting RB index of the carrier resource grid
%                           relative to common resource block 0 (CRB 0)
%
%   CSIRS is a CSI-RS specific configuration object as described in
%   <a href="matlab:help('nrCSIRSConfig')">nrCSIRSConfig</a> with the following properties specified for one or more
%   CSI-RS resources:
%
%   CSIRSType             - Type of a CSI-RS resource whether it is ZP or
%                           NZP
%   CSIRSPeriod           - CSI-RS resource slot periodicity and offset
%                           in terms of number of slots
%   RowNumber             - Row number corresponding to a CSI-RS
%                           resource as defined in
%                           TS 38.211 Table 7.4.1.5.3-1
%   Density               - CSI-RS resource frequency density
%   SymbolLocations       - Time-domain locations of a CSI-RS resource
%   SubcarrierLocations   - Frequency-domain locations of a CSI-RS resource
%   NumRB                 - Number of RBs allocated for a CSI-RS resource
%   RBOffset              - Starting RB index of CSI-RS resource allocation
%                           relative to carrier resource grid
%   NID                   - Scrambling identity
%
%   Note that each element of CSIRSType configures one CSI-RS resource.
%
%   INFO is a structure containing the following fields:
%   ResourceOrder    - Order of the CSI-RS resources in terms of CSIRSType
%                      indices for which the output is generated
%   KBarLBar         - Frequency and time domain locations of lowest REs
%                      corresponding to all CDM groups
%   CDMGroupIndices  - CDM group indices
%   KPrime           - Frequency domain indexing within a CDM group
%   LPrime           - Time domain indexing within a CDM group
%
%   Note that the above fields except 'ResourceOrder' are provided in the
%   same order as the input configuration object CSIRS and represent the
%   time and frequency domain locations of CSI-RS within a slot as defined
%   in TS 38.211 Table 7.4.1.5.3-1 for each resource.
%
%   [SYM,INFO] = nrCSIRS(...,NAME,VALUE,...) specifies additional options
%   as NAME,VALUE pairs to allow control over the data type and format of
%   the output symbols:
%
%   'OutputDataType'       - 'double' for double precision (default)
%                            'single' for single precision
%
%   'OutputResourceFormat' - 'concatenated' for output of all CSI-RS
%                            resources concatenated into a single column (default)
%                            'cell' for cell array output with each cell
%                            corresponding to an individual CSI-RS resource
%
%   % Example 1:
%   % Generate CSI-RS symbols for the default configurations generated by
%   % nrCarrierConfig and nrCSIRSConfig.
%
%   carrier = nrCarrierConfig;
%   csirs = nrCSIRSConfig;
%   [sym,info] = nrCSIRS(carrier,csirs);
%
%   % Example 2:
%   % Generate CSI-RS symbols of single data type for the default
%   % configurations generated by nrCarrierConfig and nrCSIRSConfig.
%
%   carrier = nrCarrierConfig;
%   csirs = nrCSIRSConfig;
%   [sym,info] = nrCSIRS(carrier,csirs,'OutputDataType','single');
%
%   Example 3:
%   %  Configure CSI-RS used for tracking, which consists of four periodic
%   %  NZP-CSI-RS resources in two consecutive slots with two periodic
%   %  NZP-CSI-RS resources in each slot with periodicity set to 20. The
%   %  offset for the first two resources is set to 0 and offset for next
%   %  two resources is set to 1. Set row number to 1 and density to
%   %  'three' for all resources. Then generate the CSI-RS symbols for the 
%   %  two slots 0 and 1.
%
%   carrier = nrCarrierConfig; % Slot number 0
%   NSlotSymb = carrier.SymbolsPerSlot;
%
%   csirs = nrCSIRSConfig;
%   csirs.CSIRSType = {'nzp','nzp','nzp','nzp'};
%   csirs.CSIRSPeriod = {[20 0],[20 0],[20 1],[20 1]};
%   csirs.RowNumber = [1 1 1 1];
%   csirs.Density = {'three','three','three','three'};
%   csirs.SymbolLocations = {6,10,6,10};
%   csirs.SubcarrierLocations = {0,0,0,0};
%
%   % Generate CSI-RS symbols and indices for slot 0 and map the
%   % symbols to the slot grid
%   [ind0,~] = nrCSIRSIndices(carrier,csirs);
%   [sym0,~] = nrCSIRS(carrier,csirs);
%   slotgrid0 = complex(zeros(12*carrier.NSizeGrid,NSlotSymb,max(csirs.NumCSIRSPorts)));
%   slotgrid0(ind0) = sym0;
%
%   % Generate CSI-RS symbols and indices for slot 1 and map the
%   % symbols to the slot grid
%   carrier.NSlot = 1; % Slot number 1
%   [ind1,~] = nrCSIRSIndices(carrier,csirs);
%   [sym1,~] = nrCSIRS(carrier,csirs);
%   slotgrid1 = complex(zeros(12*carrier.NSizeGrid,NSlotSymb,max(csirs.NumCSIRSPorts)));
%   slotgrid1(ind1) = sym1;
%
%   % Concatenate two slot grids
%   grid = [slotgrid0 slotgrid1];
%
%   % Plot the grid
%   imagesc(abs(grid(:,:,1)));
%   axis xy;
%   title('CSI-RS as tracking reference signal');
%   xlabel('OFDM symbols');
%   ylabel('Subcarriers');
%
%   See also nrCarrierConfig, nrCSIRSConfig, nrCSIRSIndices.

%   Copyright 2019-2021 The MathWorks, Inc.

    %#codegen
    narginchk(2,6);

    % PV pair check
    coder.extrinsic('nr5g.internal.parseOptions');

    % Validate inputs
    fcnName = 'nrCSIRS';
    validateattributes(carrier, {'nrCarrierConfig'}, {'scalar'}, fcnName, 'Carrier specific configuration object');
    validateattributes(csirs, {'nrCSIRSConfig'}, {'scalar'}, fcnName, 'CSI-RS specific configuration object');
    opts = coder.const(nr5g.internal.parseOptions(fcnName,{'OutputDataType','OutputResourceFormat'},varargin{:}));

    % Get CSI-RS locations within a slot
    [csirslocations,csirsParams] = nr5g.internal.getCSIRSLocations(carrier,csirs);

    % Extract the following properties of carrier and cast the necessary ones to double
    absNSlot   = double(carrier.NSlot);  % Absolute slot number
    NFrame     = double(carrier.NFrame); % Absolute frame number
    NFrameSlot = carrier.SlotsPerFrame;  % Number of slots per frame

    % Get the relative slot number and relative frame number
    [nslot,sfn] = nr5g.internal.getRelativeNSlotAndSFN(absNSlot,NFrame,NFrameSlot);

    % Generate ZP-CSI-RS and NZP-CSI-RS symbols
    numCSIRSRes = numel(csirslocations);
    csirsSym = coder.nullcopy(cell(1,numCSIRSRes));
    for resIdx = 1:numCSIRSRes

        % Extract the slot periodicity and offset
        if isnumeric(csirsParams.CSIRSPeriod{resIdx})
            Tcsi_rs = double(csirsParams.CSIRSPeriod{resIdx}(1));
            Toffset = double(csirsParams.CSIRSPeriod{resIdx}(2));
        else
            if strcmpi(csirsParams.CSIRSPeriod{resIdx},'on')
                Tcsi_rs = 1;
            else
                Tcsi_rs = 0;
            end
            Toffset = 0;
        end

        % Schedule CSI-RS based on slot periodicity and offset
        if (Tcsi_rs ~= 0) && (mod(NFrameSlot*sfn + nslot - Toffset, Tcsi_rs) == 0)
            % Generate CSI-RS symbols
            csirsSym{resIdx} = generateCSIRSsym(carrier,csirsParams,resIdx,csirslocations{resIdx},nslot,opts);
        else
            csirsSym{resIdx} = zeros(0,1,opts.OutputDataType);
        end

    end

    % Find the order of CSI-RS resources
    resourceOrder = [find(strcmpi(csirsParams.CSIRSType,'zp')) find(strcmpi(csirsParams.CSIRSType,'nzp'))];

    % Reorder the generated symbols in the order of ZP-CSI-RS resources
    % followed by NZP-CSI-RS resources
    reOrderedSym = coder.nullcopy(cell(1,numCSIRSRes));
    for resIdx = 1:numCSIRSRes
        reOrderedSym{resIdx} = csirsSym{resourceOrder(resIdx)};
    end

    if strcmpi(opts.OutputResourceFormat,'cell')
        sym = reOrderedSym;
    else
        sym = zeros(0,1,opts.OutputDataType);
        for resIdx = 1:numel(csirslocations)
            sym = [sym;reOrderedSym{resIdx}]; %#ok<AGROW>
        end
    end

    % Output info
    info = struct();
    info.ResourceOrder = resourceOrder;

    % Initialize the fields (for codegen purpose) 
    KBarLBar = coder.nullcopy(cell(1,numCSIRSRes));
    CDMGroupIndices = coder.nullcopy(cell(1,numCSIRSRes));
    KPrime = coder.nullcopy(cell(1,numCSIRSRes));
    LPrime = coder.nullcopy(cell(1,numCSIRSRes));

    for resIdx = 1:numCSIRSRes
        KBarLBar{resIdx} = csirslocations{resIdx}.kbar_lbar;
        CDMGroupIndices{resIdx} = csirslocations{resIdx}.CDMGroupIndex_j;
        KPrime{resIdx} = csirslocations{resIdx}.k_prime;
        LPrime{resIdx} = csirslocations{resIdx}.l_prime;
    end
    info.KBarLBar = KBarLBar;
    info.CDMGroupIndices = CDMGroupIndices;
    info.KPrime = KPrime;
    info.LPrime = LPrime;

end

function sym = generateCSIRSsym(carrier,csirsParams,resIdx,csirslocations,nslot,opts)
%   SYM = generateCSIRSsym(CARRIER,CSIRSPARAMS,RESIDX,CSIRSLOCATIONS,NSLOT,OPTS)
%   returns CSI-RS symbols SYM defined in TS 38.211 Sections 7.4.1.5.2 and
%   7.4.1.5.3 given the following inputs:
%
%   CARRIER        - Carrier specific configuration object
%   CSIRSPARAMS    - Structure containing CSI-RS specific parameters
%   RESIDX         - CSI-RS resource index
%   CSIRSLOCATIONS - Frequency and time domain locations of CSI-RS within
%                    a slot
%   NSLOT          - Slot number
%   OPTS           - Additional options to control the data type of symbols

    % Extract the following parameters for a CSI-RS resource based
    % on the resource index
    csirsType = csirsParams.CSIRSType{resIdx};
    X         = csirsParams.NumCSIRSPorts(resIdx);
    density   = csirsParams.Density{resIdx};
    if strcmpi(csirsType,'nzp')
        nID = csirsParams.NID(resIdx);
    else
        nID = 0; % For codegen purpose
    end
    numRB     = csirsParams.NumRB(resIdx);

    % CSI-RS RB offset relative to CRB 0
    RBStart   = csirsParams.RBOffset(resIdx) + double(carrier.NStartGrid);

    % Number of subcarriers per resource block
    NscRB = 12;

    % Set of RBs across which NZP-CSI-RS resource spans
    n = 0:numRB-1;

    % Get the equivalent CRB numbers of RBs across which CSI-RS resource
    % spans
    nCRB = RBStart + n;

    % Based on density, extract the set of CRBs in which CSI-RS needs to
    % be transmitted
    if strcmpi(density,'one')
        rho = 1;
    elseif strcmpi(density,'three')
        rho = 3;
    else % 'dot5even' or 'dot5odd'
        rho = 0.5;
    end

    if (rho ~= 0.5)
        prbSet = nCRB;
    else
        if strcmpi(density,'dot5even')
            prbSet = nCRB(mod(nCRB,2) == 0);
        else
            prbSet = nCRB(mod(nCRB,2) == 1);
        end
    end

    % Calculate CDM group size
    L = numel(csirslocations.k_prime)*numel(csirslocations.l_prime);

    if strcmpi(csirsType,'zp') % ZP CSI-RS

        csirsSym = zeros(numel(prbSet)*ceil(rho)*L*X,1,opts.OutputDataType);

    else % NZP-CSI-RS

        % Get the value of alpha based on rho value
        if X == 1
            alpha = rho;
        else
            alpha = 2*rho;
        end

        % Get the orthogonal sequences based on CDM type
        [wf,wt] = getwf_wt(L);
        wf_wt = coder.nullcopy(cell(1,L));
        for s = 1:L
            wf_wt{s} = wf(s,:)'*wt(s,:);
        end

        % Initialize csirsSym
        csirsSym = complex(zeros(0,1,opts.OutputDataType));
        for cdmGrpIdx = 1:numel(unique(csirslocations.CDMGroupIndex_j)) % Note that each CDM group is active for L number of ports

            % Get k_bar and l_bar values of a CDM group using CDM group index
            kbar = csirslocations.kbar_lbar{cdmGrpIdx}(1);
            lbar = csirslocations.kbar_lbar{cdmGrpIdx}(2);

            % Calculate the OFDM symbol numbers in which NZP-CSI-RS is present
            l = lbar + csirslocations.l_prime;
            % Calculate pseudo-random sequence (PRBS) generator initialization
            % sequence
            cinit = mod((2^10)*(carrier.SymbolsPerSlot*nslot+l+1)*(2*nID+1)+nID,2^31);

            % Calculate m_prime values
            if rho == 3
                m_prime = floor(prbSet*alpha) + csirslocations.k_prime(1) + floor((kbar+[0;4;8])*rho/NscRB);
            else
                m_prime = floor(prbSet*alpha) + csirslocations.k_prime(:) + floor(kbar*rho/NscRB);
            end

            % Initialize reference-signal sequence by considering the
            % maximum value of m_prime
            numModSym = max(m_prime(:))+1;
            r = coder.nullcopy(complex(zeros(numModSym,length(l),opts.OutputDataType)));
            for symNum = 1:numel(l)
                % Generate the pseudo-random sequence
                c = nrPRBS(cinit(symNum),2*numModSym);

                % Generate the reference-signal sequence defined in TS 38.211 Section 7.4.1.5
                r(:,symNum) = nrSymbolModulate(c,"QPSK",'OutputDataType',opts.OutputDataType);
            end

            % Extract the reference symbols based on m_prime values
            r_mPrime = repmat(r(m_prime+1,:),1,L);

            % Apply orthogonal weights to the reference symbols
            weighted_csirsSym = repmat([wf_wt{:}],size(r_mPrime,1)/size([wf_wt{:}],1),1).*r_mPrime;

            csirsSym = [csirsSym; weighted_csirsSym(:)]; %#ok<AGROW>

        end
    end
    sym = csirsSym;
end

function [wf,wt] = getwf_wt(cdmGroupSize)
%   [WF,WT] = getwf_wt(CDMGROUPSIZE) returns orthogonal sequences WF and WT
%   given by TS 38.211 Tables 7.4.1.5.3-2 and 7.4.1.5.3-5 for a CDM group
%   size CDMGROUPSIZE.

    switch cdmGroupSize
        case 1 %'noCDM'
            % Table 7.4.1.5.3-2
            %    wf(0)     s
            wf = +1;     % 0

            %    wt(0)     s
            wt = +1;     % 0

        case 2 %'FD-CDM2'
            % Table 7.4.1.5.3-3
            %     wf(0) wf(1)    s
            wf = [+1    +1;    % 0
                  +1    -1];   % 1

            %     wt(0)     s
            wt = [+1;     % 0
                  +1];    % 1

        case 4 %'CDM4'
            % Table 7.4.1.5.3-4
            %     wf(0) wf(1)    s
            wf = [+1    +1;    % 0
                  +1    -1;    % 1
                  +1    +1;    % 2
                  +1    -1];   % 3
              
            %     wt(0) wt(1)    s
            wt = [+1    +1;    % 0
                  +1    +1;    % 1
                  +1    -1;    % 2
                  +1    -1];   % 3

        otherwise %'CDM8'
            % Table 7.4.1.5.3-5
            %     wf(0) wf(1)    s
            wf = [+1    +1;    % 0
                  +1    -1;    % 1
                  +1    +1;    % 2
                  +1    -1;    % 3
                  +1    +1;    % 4
                  +1    -1;    % 5
                  +1    +1;    % 6
                  +1    -1];   % 7

            %     wt(0) wt(1) wt(2) wt(3)    s
            wt = [+1    +1    +1    +1;    % 0
                  +1    +1    +1    +1;    % 1
                  +1    -1    +1    -1;    % 2
                  +1    -1    +1    -1;    % 3
                  +1    +1    -1    -1;    % 4
                  +1    +1    -1    -1;    % 5
                  +1    -1    -1    +1;    % 6
                  +1    -1    -1    +1];   % 7
    end

end
