function [ind,info] = nrCSIRSIndices(carrier,csirs,varargin)
%nrCSIRSIndices CSI-RS resource element indices
%   [IND,INFO] = nrCSIRSIndices(CARRIER,CSIRS) returns the indices of the
%   Channel State Information Reference Signal (CSI-RS) resource elements
%   (REs) as defined in TS 38.211 Section 7.4.1.5.3, given carrier specific
%   configuration object CARRIER and CSI-RS specific configuration object
%   CSIRS for one or more zero-power (ZP) or non-zero-power (NZP) CSI-RS
%   resource configurations. The function also provides some additional
%   information INFO regarding the CSI-RS resources. Note that when
%   multiple CSI-RS resources are configured, irrespective of the input
%   resource order, the output IND is provided in the order of ZP-CSI-RS
%   resources followed by NZP-CSI-RS resources. With this, for the
%   resources of same configuration, the priority is given to NZP-CSI-RS
%   over ZP-CSI-RS while mapping the CSI-RS symbols to the grid.
%
%   CARRIER is a carrier specific configuration object as described in
%   <a href="matlab:help('nrCarrierConfig')">nrCarrierConfig</a> with the following properties:
%
%   SubcarrierSpacing     - Subcarrier spacing in kHz
%   CyclicPrefix          - Cyclic prefix type
%   NSlot                 - Absolute slot number
%   NFrame                - Absolute system frame number
%   NSizeGrid             - Size of the carrier resource grid in terms of
%                           number of resource blocks (RBs)
%   NStartGrid            - Starting RB index of the carrier resource grid
%                           relative to common resource block 0 (CRB 0)
%
%   CSIRS is a CSI-RS specific configuration object as described in
%   <a href="matlab:help('nrCSIRSConfig')">nrCSIRSConfig</a> with the following properties specified for one or more
%   CSI-RS resources:
%
%   CSIRSType             - Type of a CSI-RS resource whether it is ZP or
%                           NZP
%   CSIRSPeriod           - CSI-RS resource slot periodicity and offset
%                           in terms of number of slots
%   RowNumber             - Row number corresponding to a CSI-RS
%                           resource as defined in
%                           TS 38.211 Table 7.4.1.5.3-1
%   Density               - CSI-RS resource frequency density
%   SymbolLocations       - Time-domain locations of a CSI-RS resource
%   SubcarrierLocations   - Frequency-domain locations of a CSI-RS resource
%   NumRB                 - Number of RBs allocated for a CSI-RS resource
%   RBOffset              - Starting RB index of CSI-RS resource allocation
%                           relative to carrier resource grid
%
%   Note that each element of CSIRSType configures one CSI-RS resource.
%
%   INFO is a structure containing the following fields:
%   ResourceOrder    - Order of the CSI-RS resources in terms of CSIRSType
%                      indices for which the output is generated
%   KBarLBar         - Frequency and time domain locations of lowest REs
%                      corresponding to all CDM groups
%   CDMGroupIndices  - CDM group indices
%   KPrime           - Frequency domain indexing within a CDM group
%   LPrime           - Time domain indexing within a CDM group
%
%   Note that the above fields except 'ResourceOrder' are provided in the
%   same order as the input configuration object CSIRS and represent the
%   time and frequency domain locations of CSI-RS within a slot as defined
%   in TS 38.211 Table 7.4.1.5.3-1 for each resource.
%
%   [IND,INFO] = nrCSIRSIndices(...,NAME,VALUE,...) specifies additional
%   options as NAME,VALUE pairs to allow control over the format of the
%   indices:
%
%   'IndexStyle'           - 'index' for linear indices (default)
%                            'subscript' for [subcarrier, symbol, antenna] 
%                            subscript row form
%
%   'IndexBase'            - '1based' for 1-based indices (default) 
%                            '0based' for 0-based indices
%
%   'OutputResourceFormat' - 'concatenated' for output of all CSI-RS
%                            resources concatenated into a single column (default)
%                            'cell' for cell array output with each cell
%                            corresponding to an individual CSI-RS resource
%
%   % Example 1:
%   % Generate CSI-RS indices for the default configurations generated by
%   % nrCarrierConfig and nrCSIRSConfig.
%
%   carrier = nrCarrierConfig;
%   csirs = nrCSIRSConfig;
%   [ind,info] = nrCSIRSIndices(carrier,csirs);
%
%   Example 2:
%   %  Configure CSI-RS used for tracking, which consists of four periodic
%   %  NZP-CSI-RS resources in two consecutive slots with two periodic
%   %  NZP-CSI-RS resources in each slot with periodicity set to 20. The
%   %  offset for the first two resources is set to 0 and offset for next
%   %  two resources is set to 1. Set row number to 1 and density to
%   %  'three' for all resources. Then generate the CSI-RS indices for the 
%   %  two slots 0 and 1.
%
%   carrier = nrCarrierConfig;
%   csirs = nrCSIRSConfig;
%   csirs.CSIRSType = {'nzp','nzp','nzp','nzp'};
%   csirs.CSIRSPeriod = {[20 0],[20 0],[20 1],[20 1]};
%   csirs.RowNumber = [1 1 1 1];
%   csirs.Density = {'three','three','three','three'};
%   csirs.SymbolLocations = {6,10,6,10};
%   csirs.SubcarrierLocations = {0,0,0,0};
%
%   % Generate CSI-RS indices for slot 0
%   [ind0,~] = nrCSIRSIndices(carrier,csirs);
%
%   % Generate CSI-RS indices for slot 1
%   carrier.NSlot = 1; % Slot number 1
%   [ind1,~] = nrCSIRSIndices(carrier,csirs);
%
%   Example 3:
%   %  Configure CSI-RS for two resources (NZP and ZP) and generate CSI-RS
%   %  indices with slot number as 1 in carrier configuration and indexing
%   %  style as 'subscript' and output resource format as 'cell'.
%
%   carrier = nrCarrierConfig('NSlot',1);
%   csirs = nrCSIRSConfig;
%   csirs.CSIRSType = {'nzp','zp'};
%   csirs.CSIRSPeriod = {[5 1],[5 1]};
%   csirs.RowNumber = [3 5];
%   csirs.Density = {'one','one'};
%   csirs.SymbolLocations = {13,9};
%   csirs.SubcarrierLocations = {6,4};
%
%   [ind,info] = nrCSIRSIndices(carrier,csirs,...
%                'IndexStyle','subscript','OutputResourceFormat','cell');
%
%   % See whether the output is generated as cell or not
%   iscell(ind)
%
%   % Display few indices of first CSI-RS resource in the output
%   ind{1}(1:5,:)
%
%   % See whether the output is generated in the order of ZP-CSI-RS
%   % resources followed by NZP-CSI-RS resources
%   info.ResourceOrder
%
%   See also nrCarrierConfig, nrCSIRSConfig, nrCSIRS.

%   Copyright 2019-2021 The MathWorks, Inc.

    %#codegen
    narginchk(2,8);

    % PV pair check
    coder.extrinsic('nr5g.internal.parseOptions');

    % Validate inputs
    fcnName = 'nrCSIRSIndices';
    validateattributes(carrier, {'nrCarrierConfig'}, {'scalar'}, fcnName, 'Carrier specific configuration object');
    validateattributes(csirs, {'nrCSIRSConfig'}, {'scalar'}, fcnName, 'CSI-RS specific configuration object');
    opts = coder.const(nr5g.internal.parseOptions(fcnName,{'IndexStyle','IndexBase','OutputResourceFormat'},varargin{:}));
    opts.MultiColumnIndex = false; % Output indices in a single column
    
    % Get CSI-RS locations within a slot
    [csirslocations,csirsParams] = nr5g.internal.getCSIRSLocations(carrier,csirs);

    % Extract the following properties of carrier and cast the necessary ones to double
    absNSlot   = double(carrier.NSlot);  % Absolute slot number
    NFrame     = double(carrier.NFrame); % Absolute frame number
    NFrameSlot = carrier.SlotsPerFrame;  % Number of slots per frame

    % Get the relative slot number and relative frame number
    [nslot,sfn] = nr5g.internal.getRelativeNSlotAndSFN(absNSlot,NFrame,NFrameSlot);    

    % Generate ZP-CSI-RS and NZP-CSI-RS indices
    numCSIRSRes = numel(csirslocations);
    csirsInd = coder.nullcopy(cell(1,numCSIRSRes));
    for resIdx = 1:numCSIRSRes

        % Extract the slot periodicity and offset
        if isnumeric(csirsParams.CSIRSPeriod{resIdx})
            Tcsi_rs = double(csirsParams.CSIRSPeriod{resIdx}(1));
            Toffset = double(csirsParams.CSIRSPeriod{resIdx}(2));
        else
            if strcmpi(csirsParams.CSIRSPeriod{resIdx},'on')
                Tcsi_rs = 1;
            else
                Tcsi_rs = 0;
            end
            Toffset = 0;
        end

        k = zeros(0,1);
        l = zeros(0,1);
        p = zeros(0,1);

        % Schedule CSI-RS based on slot periodicity and offset
        if (Tcsi_rs ~= 0) && (mod(NFrameSlot*sfn + nslot - Toffset, Tcsi_rs) == 0)
            % Generate CSI-RS indices
            [k,l,p] = generateCSIRSIndices(carrier,csirsParams,resIdx,csirslocations{resIdx});
        end

        % Apply options
        gridsize = [double(carrier.NSizeGrid)*12, carrier.SymbolsPerSlot, csirsParams.NumCSIRSPorts(resIdx)];
        tempInd = nr5g.internal.applyIndicesOptions(gridsize,opts,k,l,p);
        csirsInd{resIdx} = tempInd;

    end

    % Find the order of CSI-RS resources
    resourceOrder = [find(strcmpi(csirsParams.CSIRSType,'zp')) find(strcmpi(csirsParams.CSIRSType,'nzp'))];

    % Reorder the generated indices in the order of ZP-CSI-RS resources
    % followed by NZP-CSI-RS resources
    reOrderedInd = coder.nullcopy(cell(1,numCSIRSRes));
    for resIdx = 1:numCSIRSRes
        reOrderedInd{resIdx} = csirsInd{resourceOrder(resIdx)};
    end

    if strcmpi(opts.OutputResourceFormat,'cell')
        ind = reOrderedInd;
    else
        ind = uint32(zeros(0,1));
        for resIdx = 1:numel(csirslocations)
            ind = [ind;reOrderedInd{resIdx}]; %#ok<AGROW>
        end
    end

    % Output info
    info = struct();
    info.ResourceOrder = resourceOrder;

    % Initialize the fields (for codegen purpose) 
    KBarLBar = coder.nullcopy(cell(1,numCSIRSRes));
    CDMGroupIndices = coder.nullcopy(cell(1,numCSIRSRes));
    KPrime = coder.nullcopy(cell(1,numCSIRSRes));
    LPrime = coder.nullcopy(cell(1,numCSIRSRes));

    for resIdx = 1:numCSIRSRes
        KBarLBar{resIdx} = csirslocations{resIdx}.kbar_lbar;
        CDMGroupIndices{resIdx} = csirslocations{resIdx}.CDMGroupIndex_j;
        KPrime{resIdx} = csirslocations{resIdx}.k_prime;
        LPrime{resIdx} = csirslocations{resIdx}.l_prime;
    end
    info.KBarLBar = KBarLBar;
    info.CDMGroupIndices = CDMGroupIndices;
    info.KPrime = KPrime;
    info.LPrime = LPrime;

end

function [k,l,p] = generateCSIRSIndices(carrier,csirsParams,resIdx,csirslocations)
%   [K,L,P] = generateCSIRSIndices(CARRIER,CSIRSPARAMS,RESIDX,CSIRSLOCATIONS)
%   returns CSI-RS indices [K,L,P] as defined in TS 38.211 Section 7.4.1.5.3,
%   given the following inputs:
%
%   CARRIER        - Carrier specific configuration object
%   CSIRSPARAMS    - Structure containing CSI-RS specific parameters
%   RESIDX         - CSI-RS resource index
%   CSIRSLOCATIONS - Frequency and time domain locations of CSI-RS within
%                    a slot

    % Extract the following parameters for a CSI-RS resource based
    % on the resource index
    X        = csirsParams.NumCSIRSPorts(resIdx);
    density  = csirsParams.Density{resIdx};
    numRB    = csirsParams.NumRB(resIdx);
    RBOffset = csirsParams.RBOffset(resIdx); % CSI-RS RB offset relative to carrier resource grid

    % CSI-RS RB offset relative to CRB 0
    RBStart  = RBOffset + double(carrier.NStartGrid);

    % Number of subcarriers per resource block
    NscRB = 12;

    % Initialize k,l,p
    k = zeros(0,1);
    l = zeros(0,1);
    p = zeros(0,1);

    % Calculate CDM group size
    L = X/numel(unique(csirslocations.CDMGroupIndex_j));

    % ZP-CSI-RS/NZP-CSI-RS indices generation
    for cdmGrpIdx = 0:numel(unique(csirslocations.CDMGroupIndex_j))-1 % Note that each CDM group is active for L number of ports

        % Get k_bar and l_bar values of a CDM group using CDM group index
        kbar = csirslocations.kbar_lbar{cdmGrpIdx+1}(1);
        lbar = csirslocations.kbar_lbar{cdmGrpIdx+1}(2);

        % Set of RBs across which CSI-RS resource spans
        n = 0:numRB-1;

        % Get the equivalent CRB numbers of RBs across which CSI-RS
        % resource spans
        nCRB = RBStart + n;

        % Based on density, extract the set of CRBs in which CSI-RS needs
        % to be transmitted
        if strcmpi(density,'one') || strcmpi(density,'three')
            prbSet = n;
        else
            if strcmpi(density,'dot5even')
                prbSet = n(mod(nCRB,2) == 0);
            else
                prbSet = n(mod(nCRB,2) == 1);
            end
        end

        % Generate the frequency and time locations of CSI-RS per port
        if strcmpi(density,'three')
            k1 = repmat((prbSet*NscRB+RBOffset*NscRB)+kbar+[0;4;8]+csirslocations.k_prime(1),1,numel(csirslocations.l_prime));
        else
            k1 = repmat((prbSet*NscRB+RBOffset*NscRB)+kbar+csirslocations.k_prime(:),1,numel(csirslocations.l_prime));
        end
        l1 = repmat(lbar + csirslocations.l_prime,numel(k1)/numel(csirslocations.l_prime),1);

        % CSI-RS antenna ports corresponding to a CDM group
        ports = (0:L-1) + (cdmGrpIdx)*L;

        % Reproduce the indices for all ports corresponding to a CDM group
        k_temp = repmat(k1(:),1,numel(ports));
        l_temp = repmat(l1(:),1,numel(ports));
        p_temp = repmat(ports,numel(k1),1);

        k = [k; k_temp(:)]; %#ok<AGROW>
        l = [l; l_temp(:)]; %#ok<AGROW>
        p = [p; p_temp(:)]; %#ok<AGROW>

    end
end
