classdef nrDLSCH < matlab.System
%nrDLSCH Downlink Shared Channel (DL-SCH)
%   DLSCH = nrDLSCH creates a Downlink Shared Channel System object, DLSCH.
%   This object takes in a transport block and processes it through the
%   components of the downlink shared channel (CRC, code-block
%   segmentation, LDPC encoding, and rate matching). This object implements
%   the following aspects of 3GPP TR 38.212:
%   * Section 7.2 Downlink shared channel and paging channel
%   The object supports both single and multiple codewords.
%
%   DLSCH = nrDLSCH(Name,Value) creates a DL-SCH object, DLSCH, with the
%   specified property Name set to the specified Value. You can specify
%   additional name-value pair arguments in any order as
%   (Name1,Value1,...,NameN,ValueN).
%
%   setTransportBlock method syntaxes
%
%   setTransportBlock(DLSCH,TRBLK),
%   setTransportBlock(DLSCH,TRBLK,TBID), and
%   setTransportBlock(DLSCH,TRBLK,TBID,HARQID) all load transport block
%   TRBLK into DLSCH for the specified transport block TBID and HARQ
%   process HARQID. Input TBID is optional, with a default value of 0 and
%   possible values of 0 and 1. Input HARQID is optional, with a default
%   value of 0. This method must be used prior to the step method so that
%   there is data to process. TRBLK must be a column vector for a single
%   transport block loaded per invocation (when TBID is specified) or a
%   cell array of one or two elements, with each element being a column
%   vector, when TBID is not specified.
%
%   Step method syntaxes
%
%   PDSCHBITS = step(DLSCH,MODULATION,NLAYERS,OUTCWLEN,RV), and
%   PDSCHBITS = step(DLSCH,MODULATION,NLAYERS,OUTCWLEN,RV,HARQID)
%   both apply the DL-SCH processing chain to the transport block(s) that
%   was(were) previously loaded using the setTransportBlock method.
%   MODULATION is one of {'QPSK','16QAM','64QAM','256QAM'} character arrays
%   or string specifying the modulation scheme. NLAYERS is a scalar between
%   1 and 8 specifying the number of transmission layers. For NLAYERS>4, a
%   two-codeword transmission is assumed and the parameters MODULATION and
%   OUTCWLEN can be two-valued tuples to signify the parameters for the two
%   codewords individually. OUTCWLEN is a positive integer scalar or vector
%   of length two that specifies the output codeword length in bits. RV is
%   an integer value between 0 and 3 to specify which redundancy version is
%   to be used with this transmission. For two codewords, RV must be a
%   two-element vector. HARQID is an integer scalar between 0 and 15,
%   specifying the ID of the HARQ process used with the current
%   transmission.  HARQID input is enabled when MultipleHARQProcesses
%   property is true, else there is only one HARQ process used. PDSCHBITS
%   is a column vector of length OUTCWLEN and is the encoded, rate-matched
%   output.  For two codewords, PDSCHBITS is a cell array of two elements.
%
%   Usage of the setTransportBlock method for loading new data makes it
%   natural for using this object with Hybrid ARQ processing. Once a
%   transport block(s) is(are) loaded, it will be retained for the
%   specified HARQ process until it is overwritten using the
%   setTransportBlock method.
%
%   System objects may be called directly like a function instead of using
%   the step method. For example, y = step(obj,x) and y = obj(x) are
%   equivalent.
%
%   getTransportBlock method syntaxes
%
%   TRBLK = getTransportBlock(DLSCH,TBID), and
%   TRBLK = getTransportBlock(DLSCH,TBID,HARQID) both return the transport
%   block that was loaded into DL-SCH using the setTransportBlock method
%   for the specified transport block TBID and the HARQ process HARQID.
%   Input HARQID is optional and defaults to a value of 0 when not
%   specified.
%
%   nrDLSCH methods:
%
%   setTransportBlock - Load next transport block(s) into the DL-SCH for
%                       processing. Retransmission is facilitated by
%                       calling or not calling this method between
%                       successive step function calls.
%   step              - Encode the data contained in the internal buffer to
%                       produce the next codeword for subsequent PDSCH
%                       processing (see above)
%   release           - Allow property value and input characteristics
%                       changes
%   clone             - Create a DL-SCH encoder object with same property
%                       values
%   isLocked          - Locked status (logical)
%   getTransportBlock - Get the transport block that was loaded into the
%                       DL-SCH for processing.
%
%   nrDLSCH properties:
%
%   MultipleHARQProcesses     - Enable multiple HARQ processes
%   TargetCodeRate            - Target code rate
%   LimitedBufferSize         - Limited buffer size (Nref) for rate
%                               matching
%
%   Example 1:
%   % Use the DL-SCH Encoder and Decoder system objects back to back.
%
%   targetCodeRate = 526 / 1024;
%   modulation = 'QPSK';
%   nlayers = 2;
%   trBlkLen = 5120;
%   outCWLen = 10240;
%   rv = 0;
%
%   % Construct and configure encoder system object
%   encDL = nrDLSCH;
%   encDL.TargetCodeRate = targetCodeRate;
%
%   % Construct and configure decoder system object
%   decDL = nrDLSCHDecoder;
%   decDL.TargetCodeRate = targetCodeRate;
%   decDL.TransportBlockLength = trBlkLen;
%
%   % Construct random data and send it through the encoder and decoder,
%   % back to back.
%   trBlk = randi([0 1],trBlkLen,1);
%   setTransportBlock(encDL,trBlk);
%   codedTrBlock = encDL(modulation,nlayers,outCWLen,rv);
%   rxSoftBits = (1.0 - 2.0*double(codedTrBlock));
%   [decbits,blkerr] = decDL(rxSoftBits,modulation,nlayers,rv);
%
%   % Check that the decoder output matches the original data
%   isequal(decbits,trBlk)
%
%   Example 2:
%   % Configure DL-SCH Encoder for use with multiple HARQ processes
%
%   encDL = nrDLSCH;
%   encDL.MultipleHARQProcesses = true;
%   encDL.TargetCodeRate = 490/1024;
%   NDI = true;                         % New Data Indicator
%   tbID = 0;
%   harqID = 7;
%   rv = 2;
%   modulation = '16QAM';
%   nlayers = 2;
%   trBlkLen = 5120;
%   outCWLen = 10496;
%
%   if NDI
%       trBlk = randi([0 1],trBlkLen,1);
%       % Latch new data into DL-SCH
%       setTransportBlock(encDL,trBlk,tbID,harqID);
%   end
%   codedTrBlock = encDL(modulation,nlayers,outCWLen,rv,harqID);
%
%   % Check that output has proper size
%   isequal(length(codedTrBlock),outCWLen)
%
%   See also nrDLSCHDecoder, nrDLSCHInfo, nrPDSCH.

%   Copyright 2018-2021 The MathWorks, Inc.

%#codegen

    % Public, non-tunable logical property
    properties (Nontunable)
        %MultipleHARQProcesses Enable multiple HARQ processes
        %   Enable multiple HARQ processes when set to true. A maximum of
        %   16 processes can be enabled. When set to false, a single
        %   process is used. In both cases, input data is buffered to
        %   enable retransmissions in case of failures. The default value
        %   of this property is false.
        MultipleHARQProcesses (1, 1) logical = false;
    end

    % Public, tunable properties
    properties
        %TargetCodeRate Target Code Rate
        %   Specify target code rate as a numeric scalar or vector of
        %   length two, each value between 0 and 1.0. The default value of
        %   this property is 526/1024.
        TargetCodeRate = 526 / 1024;
    end

    % Public, nontunable properties
    properties (Nontunable)
        %LimitedBufferSize Limited buffer size
        %   Specify the size of the internal buffer used for rate matching
        %   per codeblock as a scalar, positive integer. The default value
        %   of this property is 25344 that corresponds to the maximum
        %   codeword length.
        LimitedBufferSize = 25344;
    end

    % Private state property
    properties(Access = protected)
        % Maximum HARQ processes = 16
        pTBdata = {{int8(0),int8(0)}; {int8(0),int8(0)}; {int8(0),int8(0)}; ...
                   {int8(0),int8(0)}; {int8(0),int8(0)}; {int8(0),int8(0)}; ...
                   {int8(0),int8(0)}; {int8(0),int8(0)}; {int8(0),int8(0)}; ...
                   {int8(0),int8(0)}; {int8(0),int8(0)}; {int8(0),int8(0)}; ...
                   {int8(0),int8(0)}; {int8(0),int8(0)}; {int8(0),int8(0)}; ...
                   {int8(0),int8(0)} };
    end

    % Private properties
    properties (Access=private)
        % Copies of public properties, scalar expanded, if necessary.
        pTargetCodeRate;
    end

    % Public methods
    methods
        % Constructor
        function obj = nrDLSCH(varargin)
            % Set property values from any name-value pairs input to the
            % constructor
            setProperties(obj,nargin,varargin{:});
        end

        function set.TargetCodeRate(obj,value)
            % real scalar, or vector of length 2, 0<r<1
            propName = 'TargetCodeRate';
            validateattributes(length(value(:)),{'numeric'}, ...
                {'scalar','>',0,'<',3}, ...
                [class(obj) '.' propName],'Length of TargetCodeRate');

            validateattributes(value, {'numeric'}, ...
                {'real','<',1,'>',0},[class(obj) '.' propName], propName);

            obj.TargetCodeRate = value;
        end

        function set.LimitedBufferSize(obj,value)
            % scalar, integer > 0
            propName = 'LimitedBufferSize';
            validateattributes(value,{'numeric'}, ...
                {'scalar','integer','>',0}, ...
                [class(obj) '.' propName],propName);

            obj.LimitedBufferSize = value;
        end

    end

    methods(Access = protected)

        function num = getNumInputsImpl(obj)
            % Number of Inputs based on property for varargin in step
            num = 4 + double(obj.MultipleHARQProcesses);
        end

        % setupImpl
        function setupImpl(obj, varargin)
            % Always scalar expand the properties
            setPrivateProperties(obj);
        end

        function cwout = stepImpl(obj,modulation,nlayers,outCWLen,rv,varargin)
        % Supported syntaxes:
        %   cwout = step(obj,modulation,nlayers,outCWLen,rv)
        %   cwout = step(obj,modulation,nlayers,outCWLen,rv,harqID)

            narginchk(5,6);
            fcnName = [class(obj) '/step'];

            % Validate inputs
            validateattributes(nlayers,{'numeric'}, ...
                {'scalar','integer','<=',8,'>=',1},fcnName,'NLAYERS');
            coder.internal.prefer_const(nlayers);
            coder.internal.assert(coder.internal.isConst(nlayers),'nr5g:nrDLSCH:nLayersNeedsToBeConstant');

            if nlayers>4  % Key check
                is2CW = true;
                nl1 = floor(nlayers/2);
                nl2 = ceil(nlayers/2);
            else
                is2CW = false;
                nl1 = nlayers;
            end

            modlist = {'QPSK','16QAM','64QAM','256QAM'};
            if iscell(modulation)
                modLen = length(modulation);
                validateattributes(modLen,{'numeric'}, ...
                    {'scalar','>',0,'<',3},fcnName, ...
                    'Length of MODULATION specified as a cell');

                modSch = cell(1,modLen);
                for idx = 1:modLen
                    modSch{idx} = validatestring(modulation{idx}, ...
                        modlist,fcnName,'MODULATION');
                end
            else
                modSch = validatestring(modulation,modlist,fcnName,'MODULATION');
            end
            % Scalar expand, if necessary
            if iscell(modSch)
                if is2CW && length(modSch)==1
                    modScheme = {modSch{1},modSch{1}};
                else
                    modScheme = modSch;
                end
            else
                if is2CW
                    modScheme = {modSch,modSch};
                else
                    modScheme = {modSch};
                end
            end

            validateattributes(length(outCWLen(:)),{'numeric'}, ...
                {'scalar','>',0,'<',3},fcnName,'Length of OUTCWLEN');
            validateattributes(outCWLen,{'numeric'}, ...
                {'integer','>=',0},fcnName,'OUTCWLEN');
            if is2CW && isscalar(outCWLen) % Scalar expand, if necessary
                outCWLength = outCWLen.*ones(1,2);
            else
                outCWLength = outCWLen;
            end

            % vector RV value
            coder.internal.errorIf( is2CW && length(rv)~=2, ...
                'nr5g:nrDLSCH:InvalidRVLength');
            if is2CW
                % vector RV value
                validateattributes(rv,{'numeric'}, ...
                    {'integer','>=',0,'<=',3},fcnName,'RV')
            else
                % Check RV input is a scalar
                nr5g.internal.validateParameters('RV',rv,fcnName);
            end

            if nargin==5  % step(obj,modulation,nlayers,outCWLen,rv)
                harqID = 0;
            else    % step(obj,modulation,nlayers,outCWLen,rv,harqID)
                harqID = varargin{1};
                nr5g.internal.validateParameters('HARQID',harqID,fcnName);
            end

            if is2CW
                % Empty in, empty out
                if isempty(obj.pTBdata{harqID+1}{1}) && ...
                        isempty(obj.pTBdata{harqID+1}{2})
                    cw1 = zeros(0,1,'int8');
                    cw2 = zeros(0,1,'int8');
                    cwout = {cw1, cw2};
                else
                    % Process first codeword
                    cw1 = DLSCHEncode(obj,modScheme{1},nl1,outCWLength(1), ...
                            rv(1),harqID,1);
                    % Process second codeword
                    cw2 = DLSCHEncode(obj,modScheme{2},nl2,outCWLength(2), ...
                        rv(2),harqID,2);
                    cwout = {cw1, cw2};
                end
            else    % Single codeword
                % Empty in, empty out
                if isempty(obj.pTBdata{harqID+1}{1})
                    cwout = zeros(0,1,'int8');
                else
                    % Process first codeword
                    cwout = DLSCHEncode(obj,modScheme{1},nl1,outCWLength(1), ...
                        rv(1),harqID,1);
                end
            end
        end

        function codeword = DLSCHEncode(obj,modulation,nlayers,outCWLen, ...
                rv,harqID,tbIdx)
        % Encode DLSCH for a single codeword
        % Note: Code Block Group (CBG) transmissions are not supported yet

            % Get transport block
            trBlk = obj.pTBdata{harqID+1}{tbIdx};

            % Create the informational output 'chinfo'
            chinfo = nrDLSCHInfo(length(trBlk),obj.pTargetCodeRate(tbIdx));

            % Transport block CRC attachment
            crced = nrCRCEncode(trBlk,chinfo.CRC);

            % Code block segmentation and code block CRC attachment
            segmented = nrCodeBlockSegmentLDPC(crced,chinfo.BGN);

            % Channel coding
            encoded = nrLDPCEncode(segmented,chinfo.BGN);

            % Rate matching: with buffer limit enabled
            codeword = nrRateMatchLDPC(encoded,outCWLen,rv,modulation, ...
                nlayers,obj.LimitedBufferSize);

        end

        function resetImpl(~)
            % Must be a no-op, if setTransportBlock method is used
            % as else the whole object gets reset, since setup calls reset.
        end

        function processTunedPropertiesImpl(obj)
            % Perform calculations if tunable properties change while
            % system is running
            setPrivateProperties(obj);
        end

        function setPrivateProperties(obj)
            % Scalar expand relevant properties
            if isscalar(obj.TargetCodeRate)
                obj.pTargetCodeRate = obj.TargetCodeRate.*ones(1,2);
            else
                obj.pTargetCodeRate = obj.TargetCodeRate;
            end
        end

        function s = saveObjectImpl(obj)
            s = saveObjectImpl@matlab.System(obj);
            if isLocked(obj)
                s.pTBdata         = obj.pTBdata;
                s.pTargetCodeRate = obj.pTargetCodeRate;
            end
        end

        function loadObjectImpl(obj,s,wasLocked)
            if wasLocked
                obj.pTBdata         = s.pTBdata;
                obj.pTargetCodeRate = s.pTargetCodeRate;
            end
            % Call the base class method
            loadObjectImpl@matlab.System(obj,s);
        end

    end

    methods % Public
        function setTransportBlock(obj,trBlk,varargin)
        %setTransportBlock Set DL-SCH transport block data
        %
        %   setTransportBlock(DLSCH,TRBLK),
        %   setTransportBlock(DLSCH,TRBLK,TBID), and
        %   setTransportBlock(DLSCH,TRBLK,TBID,HARQID) all load transport
        %   block TRBLK into DLSCH for the specified transport block TBID
        %   and HARQ process HARQID. Input TBID is optional, with a default
        %   value of 0 and possible values of 0 and 1. HARQID defaults to a
        %   value of 0 when not specified. When MultipleHARQProcesses
        %   property is set to true, HARQID must be an integer in [0, 15].
        %
        %   This method must be used prior to the step method so that there
        %   is data to process. TRBLK must be a column vector for a single
        %   transport block loaded per invocation (when TBID is specified)
        %   or a cell array of one or two elements, with each element being
        %   a column vector, when TBID is not specified.

            narginchk(2,4);
            fcnName = [class(obj) '/setTransportBlock'];
            if nargin > 2
                tbID = varargin{1};
                validateattributes(tbID,{'numeric'}, ...
                    {'integer','scalar','>=',0,'<=',1},fcnName,'TBID')
            else
                tbID = 0;
            end
            if nargin > 3
                harqID = varargin{2};
                nr5g.internal.validateParameters('HARQID',harqID,fcnName);
            else
                harqID = 0;
            end

            % Allow both cell and vector as input for TB
            if iscell(trBlk)
                coder.internal.errorIf(numel(trBlk)>2 || numel(trBlk)==0, ...
                        'nr5g:nrDLSCH:InvalidInputCellLength');
                if length(trBlk)==2
                    validateattributes(trBlk{1},{'double','int8','logical'}, ...
                        {'binary','column'},fcnName,'TRBLK 0');
                    validateattributes(trBlk{2},{'double','int8','logical'}, ...
                        {'binary','column'},fcnName,'TRBLK 1');

                    obj.pTBdata{harqID+1}{1} = cast(trBlk{1},'int8');
                    obj.pTBdata{harqID+1}{2} = cast(trBlk{2},'int8');

                else
                    validateattributes(trBlk{1},{'double','int8','logical'}, ...
                        {'binary','column'},fcnName,'TrBlk');

                    obj.pTBdata{harqID+1}{tbID+1} = cast(trBlk{1},'int8');
                end
            else
                validateattributes(trBlk,{'double','int8','logical'}, ...
                    {'binary','column'},fcnName,'TrBlk');

                obj.pTBdata{harqID+1}{tbID+1} = cast(trBlk,'int8');
            end

        end

        function trBlk = getTransportBlock(obj,tbID,varargin)
        %getTransportBlock Get DL-SCH transport block data
        %
        %   TRBLK = getTransportBlock(DLSCH,TBID) returns the
        %   transport block that was loaded into DL-SCH using the
        %   setTransportBlock method for the specified transport block
        %   TBID. TBID must be either 0 or 1.
        %
        %   TRBLK = getTransportBlock(...,HARQID) returns the transport
        %   block for the specified HARQ process HARQID. HARQID defaults to
        %   a value of 0 when not specified. When MultipleHARQProcesses
        %   property is set to true, HARQID must be an integer in [0, 15].

            fcnName = [class(obj) '/getTransportBlock'];
            validateattributes(tbID,{'numeric'}, ...
                {'integer','scalar','>=',0,'<=',1},fcnName,'TBID')

            if nargin > 2
                harqID = varargin{1};
                nr5g.internal.validateParameters('HARQID',harqID,fcnName);
            else
                harqID = 0;
            end

            % Get data
            trBlk = obj.pTBdata{harqID+1}{tbID+1};
        end

    end % end methods

end % end classdef
