function [sym,info] = nrPRACH(carrier,prach,varargin)
%nrPRACH Physical random access channel
%   [SYM,INFO] = nrPRACH(CARRIER,PRACH) returns physical random access
%   channel (PRACH) symbols, SYM, as defined in TS 38.211 Section 6.3.3,
%   for the specified carrier and PRACH configurations. SYM is a complex
%   column vector. SYM is empty if the current PRACH preamble is not active
%   in the current subframe or 60 kHz slot, as described in TS 38.211
%   Section 6.3.3.2. The function also returns additional information,
%   INFO, as a structure with these fields:
%
%   RootSequence    - Physical root Zadoff-Chu sequence index or indices
%   CyclicShift     - Cyclic shift or shifts of Zadoff-Chu sequence
%   CyclicOffset    - For restricted set mode, cyclic shift or shifts
%                     corresponding to a Doppler shift of 1/T_SEQ
%   NumCyclicShifts - Number of cyclic shifts which corresponds to a single
%                     PRACH preamble sequence
%
%   CARRIER is a carrier configuration object, <a href="matlab:help('nrCarrierConfig')">nrCarrierConfig</a>.
%   Only these object properties are relevant for this function:
%
%   SubcarrierSpacing - Subcarrier spacing in kHz (15, 30, 60, 120, 240)
%   NSizeGrid         - Number of resource blocks in carrier resource
%                       grid (1...275)
%
%   PRACH is a PRACH configuration object, <a href="matlab:help('nrPRACHConfig')">nrPRACHConfig</a>.
%   Only these object properties are relevant for this function:
%
%   FrequencyRange       - Frequency range (used in combination with
%                          DuplexMode to select a configuration table from
%                          TS 38.211 Table 6.3.3.2-2 to 6.3.3.2-4)
%   DuplexMode           - Duplex mode (used in combination with
%                          FrequencyRange to select a configuration table
%                          from TS 38.211 Table 6.3.3.2-2 to 6.3.3.2-4)
%   ConfigurationIndex   - Configuration index, as defined in TS 38.211
%                          Tables 6.3.3.2-2 to 6.3.3.2-4
%   SubcarrierSpacing    - PRACH subcarrier spacing in kHz
%   LRA                  - Length of the Zadoff-Chu preamble sequence
%   SequenceIndex        - Logical root sequence index
%   PreambleIndex        - Scalar preamble index within cell
%   RestrictedSet        - Type of restricted set
%   ZeroCorrelationZone  - Cyclic shift configuration index
%   ActivePRACHSlot      - Active PRACH slot number within a subframe or a
%                          60 kHz slot
%   NPRACHSlot           - PRACH slot number
%
%   [SYM,INFO] = nrPRACH(...,NAME,VALUE) specifies additional options
%   as NAME,VALUE pairs to allow control over the data type of the output
%   symbols:
%
%   'OutputDataType'       - 'double' for double precision (default)
%                            'single' for single precision
%
%   If the value of PRACH.PreambleIndex is such that an insufficient
%   quantity of cyclic shifts are available at the configured logical root
%   index PRACH.SequenceIndex, the function increments the logical root
%   index number. As such, the physical root used, INFO.RootSequence,
%   differs from the physical root configured by PRACH.SequenceIndex. The
%   INFO.CyclicShift field represents the cyclic shift corresponding to
%   PRACH.PreambleIndex. Similarly, for high speed mode (i.e., restricted
%   set type A or B), if no valid cyclic shift exists for the current value
%   of PRACH.SequenceIndex, the function increments the logical root index
%   number.
%
%   Example:
%   % Generate PRACH symbols for the default configurations of
%   % nrCarrierConfig and nrPRACHConfig.
%
%   prach = nrPRACHConfig;
%   carrier = nrCarrierConfig;
%   [sym,info] = nrPRACH(carrier,prach);
%
%   See also nrCarrierConfig, nrPRACHConfig, nrPRACHIndices.

%   Copyright 2019-2021 The MathWorks, Inc.

    %#codegen
    narginchk(2,4);
    
    % PV pair check
    coder.extrinsic('nr5g.internal.parseOptions');
    
    % Validate inputs
    fcnName = 'nrPRACH';
    validateattributes(carrier, {'nrCarrierConfig'}, {'scalar'}, fcnName, 'Carrier specific configuration object');
    validateattributes(prach, {'nrPRACHConfig'}, {'scalar'}, fcnName, 'PRACH configuration object');
    opts = coder.const(nr5g.internal.parseOptions(fcnName,{'OutputDataType'},varargin{:}));
    
    % Validate the PRACH configuration
    prach = validateConfig(prach);
    % Validate the carrier against the PRACH
    nr5g.internal.prach.validatePRACHAndCarrier(carrier, prach)
    
    % Set up empty output vectors
    sym = complex(zeros(0,1,opts.OutputDataType));
    info = struct();
    
    % Get the parameters RootSequence, CyclicShift, CyclicOffset, and NumCyclicShifts
    [numCyclicShifts, rootSequence, cyclicShift, cyclicOffset] = ...
        nr5g.internal.prach.getPreambleSeqParameters(prach);
    info.RootSequence = rootSequence;
    info.CyclicShift = cyclicShift;
    info.CyclicOffset = cyclicOffset;
    info.NumCyclicShifts = numCyclicShifts;
    
    % If PRACH is active in this subframe:
    if nr5g.internal.prach.isActive(prach)
        
        % Create Zadoff-Chu sequence, apply appropriate cyclic shift,
        % perform DFT.
        x_u = zadoffChuSeq(rootSequence(1), prach.LRA);
        C_v = cyclicShift;
        x_uv = circshift(x_u, [-C_v 0]);
        y_uv = fft(x_uv);
        
        % Generate the output symbol vector
        L = prach.PRACHDuration(1); % Number of repeated symbols
        % One time occasion of preamble format C2 spans 6 OFDM symbols,
        % according to TS 38.211 Tables 6.3.3.2-2 to 6.3.3.2-4. However,
        % note that the number of OFDM symbols per PRACH slot appears as
        % parts of the expressions for N_u in TS 38.211 Table 6.3.3.1-2.
        % This is 4 for preamble format C2. That is, considering that the
        % cyclic prefix for preamble format C2 is approximately one OFDM
        % symbol long, there is a gap of one OFDM symbol between the first
        % and the second time occasion of the PRACH preamble for format C2.
        % Thus, the last OFDM symbol in each time occasion for preamble
        % format C2 is empty. You can refer to TDoc R1-1805220 Figures 1
        % and 2 for a visual representation of this concept.
        % This is represented here by considering a length of the preamble
        % that is one symbol shorter than the value of PRACHDuration.
        if strcmpi(prach.Format,'C2')
            L = L - 1;
        end
        
        sym = cast(repmat(y_uv(1:prach.LRA,1), L(1), 1), opts.OutputDataType);
    end
end
