function [ind,info] = nrPRACHIndices(carrier,prach,varargin)
%nrPRACHIndices Physical random access channel indices
%   [IND,INFO] = nrPRACHIndices(CARRIER,PRACH) returns the matrix IND
%   containing physical random access channel (PRACH) resource element
%   indices, as defined in TS 38.211 Section 5.3.2, for the specified
%   carrier and PRACH configurations. IND is empty if the current PRACH
%   preamble is not active in the current subframe or 60 kHz slot, as
%   described in TS 38.211 Section 6.3.3.2. The function also returns
%   additional information, INFO, as a structure with the field:
%
%   PRBSet              - Physical resource block (PRB) indices occupied by
%                         PRACH preamble for PUSCH (0-based)
%
%   CARRIER is a carrier configuration object, <a href="matlab:help('nrCarrierConfig')">nrCarrierConfig</a>.
%   Only these object properties are relevant for this function:
%
%   SubcarrierSpacing     - Subcarrier spacing in kHz
%   NSizeGrid             - Size of the carrier resource grid in terms of
%                           number of resource blocks (RBs)
%
%   PRACH is a PRACH configuration object, <a href="matlab:help('nrPRACHConfig')">nrPRACHConfig</a>.
%   Only these object properties are relevant for this function:
%
%   FrequencyRange       - Frequency range (used in combination with
%                          DuplexMode to select a configuration table from
%                          TS 38.211 Table 6.3.3.2-2 to 6.3.3.2-4)
%   DuplexMode           - Duplex mode (used in combination with
%                          FrequencyRange to select a configuration table
%                          from TS 38.211 Table 6.3.3.2-2 to 6.3.3.2-4)
%   ConfigurationIndex   - Configuration index, as defined in TS 38.211
%                          Tables 6.3.3.2-2 to 6.3.3.2-4
%   SubcarrierSpacing    - PRACH subcarrier spacing in kHz
%   LRA                  - Length of the Zadoff-Chu preamble sequence
%   RBOffset             - Starting RB index of PRACH allocation relative to
%                          carrier resource grid
%   FrequencyStart       - Frequency offset of lowest PRACH transmission
%                          occasion in frequency domain with respect to PRB 0
%   FrequencyIndex       - Index of the PRACH transmission occasions in
%                          frequency domain
%   RBSetOffset          - Starting RB index of the uplink RB set for this
%                          PRACH transmission occasion
%   ActivePRACHSlot      - Active PRACH slot number within a subframe or a
%                          60 kHz slot
%   NPRACHSlot           - PRACH slot number
%
%   [IND,INFO] = nrPRACHIndices(...,NAME,VALUE) specifies additional
%   options as NAME,VALUE pairs to allow control over the format of the
%   indices:
%
%   'IndexStyle'           - 'index' for linear indices (default)
%                            'subscript' for [subcarrier, symbol, antenna] 
%                            subscript row form
%
%   'IndexBase'            - '1based' for 1-based indices (default) 
%                            '0based' for 0-based indices
%
%   Example:
%   % Generate PRACH indices for the default configurations of
%   % nrCarrierConfig and nrPRACHConfig.
%
%   prach = nrPRACHConfig;
%   carrier = nrCarrierConfig;
%   [ind,info] = nrPRACHIndices(carrier,prach);
%
%   See also nrCarrierConfig, nrPRACHConfig, nrPRACH, nrPRACHGrid.

%   Copyright 2019-2021 The MathWorks, Inc.

    %#codegen
    narginchk(2,6);
    
    % PV pair check
    coder.extrinsic('nr5g.internal.parseOptions');
    
    % Validate inputs
    fcnName = 'nrPRACHIndices';
    validateattributes(carrier, {'nrCarrierConfig'}, {'scalar'}, fcnName, 'Carrier specific configuration object');
    validateattributes(prach, {'nrPRACHConfig'}, {'scalar'}, fcnName, 'PRACH configuration object');
    opts = coder.const(nr5g.internal.parseOptions(fcnName,{'IndexStyle','IndexBase'},varargin{:}));
    
    % Validate the PRACH configuration
    prach = validateConfig(prach);
    % Validate the carrier against the PRACH
    nr5g.internal.prach.validatePRACHAndCarrier(carrier, prach)
    
    % Initialize empty linear indices and output vectors
    kk = []; ll = []; pp = [];
    PRBSet = zeros(0,1);
    info = struct();
    
    % Get the grid size
    carrierNRB = double(carrier.NSizeGrid);
    carrierSCS = carrier.SubcarrierSpacing;
    gridsize = nr5g.internal.prach.gridSize(carrierNRB, carrierSCS, prach, 1);
    
    % If PRACH is active in this subframe:
    if nr5g.internal.prach.isActive(prach)
        
        % Get frequency locations
        table = nr5g.internal.prach.getTable6332x(1);
        tableIndex = table.LRA == prach.LRA & ...
                     table.PRACHSubcarrierSpacing == prach.SubcarrierSpacing & ...
                     table.PUSCHSubcarrierSpacing == carrierSCS;
        
        K = double(carrierSCS / prach.SubcarrierSpacing);
        [k1, PRBSet] = getFreqParameters(carrier, prach, tableIndex);
        errorFiller = {prach.LRA,sprintf('%g',PRBSet(1)),sprintf('%g',PRBSet(end)),sprintf('%g',carrierNRB),prach.RBOffset,prach.FrequencyStart,'FrequencyIndex',prach.FrequencyIndex};
        if any(prach.LRA == [571 1151])
            errorFiller{end-1} = 'RBSetOffset';
            errorFiller{end} = prach.RBSetOffset;
        end
        coder.internal.errorIf(PRBSet(end)>=carrierNRB,'nr5g:nrPRACH:InvalidPRBSet',errorFiller{:});
        
        %kbar Additional parameter needed for the OFDM baseband signal
        % generation (TS 38.211 Section 5.3.2) and given by TS 38.211 Table
        % 6.3.3.2-1.
        kbar = table.kbar(tableIndex);
        
        % Get frequency locations k
        coder.varsize('k',[1 1151],[0 1]);
        first = K*k1 + kbar; % PRACH-specific frequency shift
        k = 0:(prach.LRA-1);
        zeroFreq = double(gridsize(1)/2); % Set the origin of the frequency domain to the middle of the grid
        k = k + zeroFreq(1) + first(1);
        
        % Get time locations
        L = prach.PRACHDuration(1);
        % One time occasion of preamble format C2 spans 6 OFDM symbols,
        % according to TS 38.211 Tables 6.3.3.2-2 to 6.3.3.2-4. However,
        % note that the number of OFDM symbols per PRACH slot appears as
        % parts of the expressions for N_u in TS 38.211 Table 6.3.3.1-2.
        % This is 4 for preamble format C2. That is, considering that the
        % cyclic prefix for preamble format C2 is approximately one OFDM
        % symbol long, there is a gap of one OFDM symbol between the first
        % and the second time occasion of the PRACH preamble for format C2.
        % Thus, the last OFDM symbol in each time occasion for preamble
        % format C2 is empty. You can refer to TDoc R1-1805220 Figures 1
        % and 2 for a visual representation of this concept.
        % This is represented here by considering a length of the preamble
        % that is one symbol shorter than the value of PRACHDuration.
        if strcmpi(prach.Format,'C2')
            L = L - 1;
        end
        
        l = prach.SymbolLocation(1) + (0:L-1) - double(gridsize(2))*prach.ActivePRACHSlot; % Consider a single PRACH slot
        
        % Get antenna ports locations
        % Note that only one antenna port is used for PRACH. However, this
        % value can be later used for beamforming
        p = zeros(size(k));
        
        % Retrieve 0-based linear indices
        [ll,kk] = meshgrid(l, k);
        pp = repmat(p(:), 1, L);
    end
    
    ind = nr5g.internal.applyIndicesOptions(gridsize, opts, kk, ll, pp);
    info.PRBSet = PRBSet;
end

function [k1, PRBSet] = getFreqParameters(carrier, prach, tableIndex)
    % Get the additional parameter 'k1' and the set of PRBs for PUSCH
    % occupied by PRACH.
    % The former is used to generate the frequency locations, as discussed
    % in TS 38.211 Section 5.3.2, whereas the latter is used in the
    % informational output to give the corresponding set of PRBs for the
    % PUSCH that the PRACH occupies.
    
    %NRB Number of resource blocks occupied which is given by the
    % parameter allocation expressed in number of RBs for PUSCH.
    % TS 38.211 Table 6.3.3.2-1.
    table = nr5g.internal.prach.getTable6332x(1);
    NRB = table.NRBAllocation(tableIndex);
    
    %PRBSet Set of PRBs for PUSCH occupied by PRACH (0-based)
    coder.varsize('PRBSet',[1 96],[0 1]);
    if any(prach.LRA==[139,839])
        rbsetOffset = prach.FrequencyIndex*NRB;
    else % LRA==571,1151
        rbsetOffset = prach.RBSetOffset;
    end
    PRBFirst = prach.RBOffset + prach.FrequencyStart + rbsetOffset;
    PRBSet = PRBFirst(1) + (0:NRB-1);
    
    %k0 Auxiliary parameter for the computation of k1
    % Note that information about the entire list of carriers is not
    % available at this stage and so we assume that mu==mu0. That is, k0 = 0.
    k0 = 0;
    
    % Get k1
    k1 = k0 + (PRBSet(1) - double(carrier.NSizeGrid)/2)*12;
end
