function sym = nrPRS(carrier,prs,varargin)
%nrPRS Positioning reference signal
%   SYM = nrPRS(CARRIER,PRS) returns the positioning reference signal (PRS)
%   symbols, SYM, as defined in TS 38.211 Section 7.4.1.7.2, given the
%   carrier specific configuration object CARRIER and positioning reference
%   signal configuration object PRS. The function also handles the
%   conditions related to the mapping of PRS resources to slots, as defined
%   in TS 38.211 Section 7.4.1.7.4.
%
%   CARRIER is a carrier specific configuration object, as described in
%   <a href="matlab:help('nrCarrierConfig')">nrCarrierConfig</a>. Only these object properties are relevant for this
%   function:
%
%   SubcarrierSpacing - Subcarrier spacing in kHz
%                       (15 (default), 30, 60, 120, 240)
%   CyclicPrefix      - Cyclic prefix ('normal' (default), 'extended')
%   NSizeGrid         - Number of resource blocks in carrier resource
%                       grid (1...275) (default 52)
%   NStartGrid        - Start of carrier resource grid relative to common
%                       resource block 0 (CRB 0) (0...2199) (default 0)
%   NSlot             - Absolute slot number (default 0)
%   NFrame            - Absolute system frame number (default 0)
%
%   PRS is a positioning reference signal configuration object to specify
%   one or more PRS resources in a PRS resource set, as described in
%   <a href="matlab:help('nrPRSConfig')">nrPRSConfig</a>. Only these object properties are relevant for this
%   function:
%
%   PRSResourceSetPeriod  - PRS resource set slot periodicity
%                           (TPRSPeriod) and slot offset (TPRSOffset)
%                           ('on' (default), 'off', [TPRSPeriod TPRSOffset])
%   PRSResourceOffset     - Slot offset of each PRS resource relative
%                           to PRS resource set slot offset (TPRSOffset)
%                           (0...511) (default 0)
%   PRSResourceRepetition - PRS resource repetition factor
%                           (1 (default), 2, 4, 6, 8, 16, 32)
%   PRSResourceTimeGap    - Slot offset between two consecutive repeated
%                           instances of a PRS resource (1 (default), 2, 4, 8, 16, 32)
%   MutingPattern1        - Muting bit pattern option-1 (default [])
%   MutingBitRepetition   - Number of consecutive instances of a PRS
%                           resource set corresponding to a single element
%                           of MutingPattern1 binary vector (1 (default), 2, 4, 8)
%   MutingPattern2        - Muting bit pattern option-2 (default [])
%   NumPRSSymbols         - Number of OFDM symbols allocated for each
%                           PRS resource (0...12) (default 12)
%   SymbolStart           - Starting OFDM symbol of each PRS resource
%                           in a slot (0...13) (default 0)
%   NumRB                 - Number of physical resource blocks (PRBs)
%                           allocated for all PRS resources (0...275)
%                           (default 52)
%   RBOffset              - Starting PRB index of all PRS resources
%                           relative to the carrier resource grid
%                           (0...274) (default 0)
%   CombSize              - Comb size of all PRS resources
%                           (2 (default), 4, 6, 12)
%   NPRSID                - Sequence identity of each PRS resource
%                           (0...4095) (default 0)
%
%   Note that the following five properties can be specified as scalars or
%   vectors, which are unique to each PRS resource in a PRS resource set:
%   1. PRSResourceOffset
%   2. NumPRSSymbols
%   3. SymbolStart
%   4. REOffset
%   5. NPRSID
%   The number of configured PRS resources is considered as the maximum of
%   lengths of above five mentioned properties. For the above five
%   properties, when the value is specified as a vector, the length must be
%   equal to the number of configured PRS resources. When the property is
%   specified as a scalar, the same value is used for all the PRS resources
%   in a PRS resource set.
%
%   SYM = nrPRS(...,NAME,VALUE,...) specifies additional options as
%   NAME,VALUE pairs to allow control over the data type and format of the
%   output symbols:
%
%   'OutputDataType'       - 'double' for double precision (default)
%                            'single' for single precision
%
%   'OutputResourceFormat' - 'concatenated' for output of all PRS
%                            resources concatenated into a single column (default)
%                            'cell' for cell array output with each cell
%                            corresponding to an individual PRS resource
%
%   % Example 1:
%   % Generate PRS symbols for the default configurations generated by
%   % nrCarrierConfig and nrPRSConfig.
%
%   carrier = nrCarrierConfig;
%   prs = nrPRSConfig;
%   sym = nrPRS(carrier,prs);
%
%   % Example 2:
%   % Create a carrier configuration object with 60 kHz subcarrier spacing
%   % and extended cyclic prefix. Configure a PRS resource set having four
%   % PRS resources. Generate the PRS symbols of single data type with the
%   % output resource format as 'cell'.
%
%   % Set carrier parameters
%   carrier = nrCarrierConfig;
%   carrier.SubcarrierSpacing = 60;
%   carrier.CyclicPrefix = 'extended';
%   carrier.NSlot = 0;
%
%   % Set PRS parameters
%   prs = nrPRSConfig;
%
%   % Set the properties which are common to all the PRS resources in a
%   % resource set
%   prs.PRSResourceSetPeriod = [640 0];
%   prs.PRSResourceRepetition = 8;
%   prs.PRSResourceTimeGap = 4;
%   prs.MutingPattern1 = [1 0 0 1];
%   prs.MutingBitRepetition = 2;
%   prs.MutingPattern2 = [1 0 1 0 1 0 1 0];
%   prs.NumRB = 52;
%   prs.RBOffset = 0;
%   prs.CombSize = 2;
%
%   % Set the properties which are unique to each PRS resource in a 
%   % resource set (these can be scalars or vectors)
%   prs.PRSResourceOffset = [0 40 80 120];
%   prs.NumPRSSymbols = [6 4 12 2];
%   prs.SymbolStart = 0;
%   prs.REOffset = [0 1 0 1];
%   prs.NPRSID = 15;
%
%   % Generate PRS symbols
%   sym = nrPRS(carrier,prs,'OutputDataType','single','OutputResourceFormat','cell')
%
%   See also nrCarrierConfig, nrPRSConfig, nrPRSIndices.

%   Copyright 2020-2021 The MathWorks, Inc.

%#codegen

    narginchk(2,6);

    % PV pair check
    coder.extrinsic('nr5g.internal.parseOptions');

    % Validate the inputs and get the slot schedule of all the PRS
    % resources in a resource set
    [prs,PRSPresence,numRes,nSlot] = nr5g.internal.validateAndSchedulePRS(carrier,prs);

    % Parse options
    opts = coder.const(nr5g.internal.parseOptions('nrPRS',...
        {'OutputDataType','OutputResourceFormat'},varargin{:}));

    % Get the number of resource elements allocated for all the PRS
    % resources within a PRB
    numREsPerRB = 12/prs.CombSize;
    mIndPerRB  = (0:numREsPerRB-1)';

    % Get the equivalent CRB numbers (0-based) of the PRBs across which all
    % the PRS resources span
    nCRB = prs.RBOffset + double(carrier.NStartGrid) + (0:prs.NumRB-1);

    % Generate PRS symbols
    prsSym = coder.nullcopy(cell(1,numRes));
    numPRSSymPerRes = zeros(1,numRes);
    for resIdx = 1:numRes
        % Get the number of OFDM symbols allocated for a PRS resource
        LPRS = prs.NumPRSSymbols(resIdx);

        if (~PRSPresence(resIdx)) || (LPRS == 0) || isempty(nCRB)
            prsSym{resIdx} = zeros(0,1,opts.OutputDataType);
        else
            % Get the PRS sequence identity
            nPRSID = prs.NPRSID(resIdx);

            % Calculate the OFDM symbols allocated for a PRS resource
            l = prs.SymbolStart(resIdx) + (0:LPRS-1);

            % Calculate the pseudo-random binary sequence (PRBS) generator
            % sequence initialization
            tempNPRSID = mod(nPRSID,1024);
            cinit = mod((2^22)*floor(nPRSID/1024) +...
                ((2^10)*(carrier.SymbolsPerSlot*nSlot + l + 1)*(2*tempNPRSID + 1)) + ...
                tempNPRSID,2^31);

            % Get the PRS sequence indices (m values, as defined in
            % TS 38.211 Section 7.4.1.7.3)
            mValuesTemp = reshape(mIndPerRB+(nCRB*numREsPerRB),[],1);
            mValues = (2*mValuesTemp(1)):(2*mValuesTemp(end) + 1);

            % Generate PRBS
            c = zeros(numel(mValues),LPRS);
            for l_idx = 1:LPRS
                c_temp = nrPRBS(cinit(l_idx),2*(mValuesTemp(end)+1));
                c(:,l_idx) = c_temp(mValues + 1);
            end
            % Generate the PRS sequence, as defined in TS 38.211
            % Section 7.4.1.7.2
            prsSym{resIdx} = nrSymbolModulate(c(:),"QPSK",'OutputDataType',opts.OutputDataType);

            % Store the number of PRS symbols for each PRS resource
            numPRSSymPerRes(resIdx) = numel(prsSym{resIdx});
        end
    end

    % Apply OutputResourceFormat option on the generated PRS symbols
    if strcmpi(opts.OutputResourceFormat,'cell')
        sym = prsSym;
    else
        prsSymIndPerRes = cumsum([0 numPRSSymPerRes]) + 1;
        sym = complex(zeros(prsSymIndPerRes(end)-1,1,opts.OutputDataType));
        for resIdx = 1:numRes
            sym(prsSymIndPerRes(resIdx):prsSymIndPerRes(resIdx+1)-1,1) = prsSym{resIdx};
        end
    end
end
