classdef nrPUCCH0Config < nr5g.internal.BWPSizeStart & nr5g.internal.pucch.Format0ConfigBase
    %nrPUCCH0Config NR PUCCH format 0 configuration
    %   PUCCH = nrPUCCH0Config creates a physical uplink control channel
    %   (PUCCH) configuration object, which contains all the parameters of
    %   PUCCH format 0. This object provides the properties related to TS
    %   38.211 Sections 6.3.2.1 to 6.3.2.3. The object also configures the
    %   bandwidth part (BWP) containing the PUCCH and the number of
    %   resource blocks (RBs) occupied by the PUCCH within the BWP. The
    %   default nrPUCCH0Config object configures a PUCCH format 0 allocated
    %   in the first RB of the BWP and the last OFDM symbol in the slot of
    %   14 OFDM symbols.
    %
    %   PUCCH = nrPUCCH0Config(Name,Value) creates a physical uplink
    %   control channel configuration object, PUCCH, with the specified
    %   property Name set to the specified Value. You can specify
    %   additional name-value pair arguments in any order as
    %   (Name1,Value1,...,NameN,ValueN).
    %
    %   nrPUCCH0Config properties:
    %
    %   NSizeBWP           - Size of BWP in terms of number of physical
    %                        resource blocks (PRBs) (1...275) (default [])
    %   NStartBWP          - Starting PRB index of BWP relative to common
    %                        resource block 0 (CRB 0) (0...2473) (default [])
    %   SymbolAllocation   - OFDM symbol allocation of PUCCH within a slot
    %                        (default [13 1])
    %   PRBSet             - PRBs allocated for PUCCH within the BWP
    %                        (default 0)
    %   FrequencyHopping   - Frequency hopping configuration
    %                        ('neither' (default), 'intraSlot', 'interSlot')
    %   SecondHopStartPRB  - Starting PRB of second hop relative to the
    %                        BWP (0...274) (default 1)
    %   GroupHopping       - Group hopping configuration
    %                        ('neither' (default), 'enable', 'disable')
    %   HoppingID          - Hopping identity (0...1023) (default [])
    %   InitialCyclicShift - Initial cyclic shift (0...11) (default 0)
    %
    %   Example:
    %   % Display the default properties of PUCCH format 0.
    %
    %   pucch = nrPUCCH0Config
    %
    %   See also nrPUCCH, nrPUCCHIndices, nrPUCCH1Config, nrPUCCH2Config,
    %   nrPUCCH3Config, nrPUCCH4Config.

    %   Copyright 2020-2021 The MathWorks, Inc.

    %#codegen

    % Hidden properties
    properties (Hidden)
        % Custom property list to change the order of display properties
        CustomPropList = {'NSizeBWP','NStartBWP','SymbolAllocation','PRBSet',...
            'FrequencyHopping','SecondHopStartPRB','GroupHopping','HoppingID',...
            'InitialCyclicShift'};
    end

    methods
        function obj = nrPUCCH0Config(varargin)
            % Get the value of NStartBWP from the name-value pairs
            nStartBWP = nr5g.internal.parseProp('NStartBWP',[],varargin{:});
            % Get the value of NSizeBWP from the name-value pairs
            nSizeBWP = nr5g.internal.parseProp('NSizeBWP',[],varargin{:});

            % Add sets for enum properties with different sized values
            obj = obj@nr5g.internal.pucch.Format0ConfigBase( ...
                'NStartBWP', nStartBWP, ...
                'NSizeBWP', nSizeBWP, ...
                varargin{:});
        end
        
        % Validate configuration
        function validateConfig(obj)
            % Call Format0ConfigBase validator
            validateConfig@nr5g.internal.pucch.Format0ConfigBase(obj);
        end
    end

end
