classdef nrPUCCH3Config < nr5g.internal.BWPSizeStart & nr5g.internal.pucch.Format3ConfigBase
    %nrPUCCH3Config NR PUCCH format 3 configuration
    %   PUCCH = nrPUCCH3Config creates a physical uplink control channel
    %   (PUCCH) configuration object, which contains all the parameters of
    %   PUCCH format 3. This object provides the properties related to TS
    %   38.211 Sections 6.3.2.1, 6.3.2.2, 6.3.2.6, and 6.4.1.3.3. The
    %   object also configures the bandwidth part (BWP) containing the
    %   PUCCH and the number of resource blocks (RBs) occupied by the PUCCH
    %   within the BWP. The default nrPUCCH3Config object configures a
    %   PUCCH format 3 allocated in the first RB of the BWP and spanning
    %   over 14 OFDM symbols in a slot.
    %
    %   PUCCH = nrPUCCH3Config(Name,Value) creates a physical uplink
    %   control channel configuration object, PUCCH, with the specified
    %   property Name set to the specified Value. You can specify
    %   additional name-value pair arguments in any order as
    %   (Name1,Value1,...,NameN,ValueN).
    %
    %   nrPUCCH3Config properties:
    %
    %   NSizeBWP           - Size of BWP in terms of number of physical
    %                        resource blocks (PRBs) (1...275) (default [])
    %   NStartBWP          - Starting PRB index of BWP relative to common
    %                        resource block 0 (CRB 0) (0...2473) (default [])
    %   Modulation         - Modulation scheme ('QPSK' (default), 'pi/2-BPSK')
    %   SymbolAllocation   - OFDM symbol allocation of PUCCH within a slot
    %                        (default [0 14])
    %   PRBSet             - PRBs allocated for PUCCH within the BWP
    %                        (default 0)
    %   FrequencyHopping   - Frequency hopping configuration
    %                        ('neither' (default), 'intraSlot', 'interSlot')
    %   SecondHopStartPRB  - Starting PRB of second hop relative to the
    %                        BWP (0...274) (default 1)
    %   GroupHopping       - Group hopping configuration
    %                        ('neither' (default), 'enable', 'disable')
    %   HoppingID          - Hopping identity (0...1023) (default [])
    %   NID                - Data scrambling identity (0...1023) (default [])
    %   RNTI               - Radio network temporary identifier (0...65535)
    %                        (default 1)
    %   AdditionalDMRS     - Additional demodulation reference signal (DM-RS)
    %                        configuration flag (0 (default), 1)
    %
    %   Example:
    %   % Display the default properties of PUCCH format 3.
    %
    %   pucch = nrPUCCH3Config
    %
    %   See also nrPUCCH, nrPUCCHDMRS, nrPUCCHDMRSIndices,
    %   nrPUCCHIndices, nrPUCCH0Config, nrPUCCH1Config, nrPUCCH2Config,
    %   nrPUCCH4Config.

    %   Copyright 2020-2021 The MathWorks, Inc.

    %#codegen

    properties (Hidden)
        % Custom property list to change the order of display properties
        CustomPropList = {'NSizeBWP','NStartBWP','Modulation',...
            'SymbolAllocation','PRBSet','FrequencyHopping','SecondHopStartPRB',...
            'GroupHopping','HoppingID','NID','RNTI','AdditionalDMRS'};
    end

    methods
        function obj = nrPUCCH3Config(varargin)
            % Get the value of NStartBWP from the name-value pairs
            nStartBWP = nr5g.internal.parseProp('NStartBWP',[],varargin{:});
            % Get the value of NSizeBWP from the name-value pairs
            nSizeBWP = nr5g.internal.parseProp('NSizeBWP',[],varargin{:});

            % Add sets for enum properties with different sized values
            obj = obj@nr5g.internal.pucch.Format3ConfigBase( ...
                'NStartBWP', nStartBWP, ...
                'NSizeBWP', nSizeBWP, ...
                varargin{:});
        end
        
        % Validate configuration
        function validateConfig(obj)
            % Call Format3ConfigBase validator
            validateConfig@nr5g.internal.pucch.Format3ConfigBase(obj);
        end
    end

end
