function W = nrPUSCHCodebook(nlayers,nports,TPMI,varargin)
%nrPUSCHCodebook Codebook for PUSCH precoding
%   W = nrPUSCHCodebook(NLAYERS,NPORTS,TPMI) returns the PUSCH precoding
%   matrix W defined for codebook-based transmission in TS 38.211 Section
%   6.3.1.5 for number of layers NLAYERS (1...4), number of antenna ports
%   NPORTS (1,2,4) and Transmitted Precoding Matrix Indicator TPMI (0...27,
%   valid range depends on NLAYERS and NPORTS). Transform precoding is
%   disabled, but can be enabled by an optional input described below.
%   
%   W is the NLAYERS-by-NPORTS precoding matrix for codebook-based
%   transmission of the PUSCH. W=1 for NLAYERS=1 and NPORTS=1 otherwise it
%   is selected from TS 38.211 Tables 6.3.1.5-1...7, depending on NLAYERS,
%   NPORTS and TPMI.
%
%   Note that W is the transpose of the precoding matrix defined in the
%   specification. nrLayerMap produces an output of size M-by-NLAYERS, i.e.
%   one column per layer, and the orientation of W here allows the
%   precoding to be performed by matrix multiplication.
%
%   W = nrPUSCHCodebook(NLAYERS,NPORTS,TPMI,TPRECODE) allows transform
%   precoding to be enabled or disabled via the parameter TPRECODE (false,
%   true), which affects the precoding matrix used in the case of NLAYERS=1
%   and NPORTS=4 (see TS 38.211 Section 6.3.1.4).
%    
%   Example:
%   % Create 64QAM modulation symbols, split onto two layers, and precode
%   % using a precoding matrix for four antenna ports.
%
%   modulation = '64QAM';
%   nlayers = 2;
%   nports = 4;
%   TPMI = 7;
%   
%   data = randi([0 1],600,1);
%   d = nrSymbolModulate(data,modulation);
%   y = nrLayerMap(d,nlayers);
%
%   W = nrPUSCHCodebook(nlayers,nports,TPMI);
%   z = y * W;
%
%   See also nrLayerMap, nrLayerDemap.
 
%   Copyright 2018-2020 The MathWorks, Inc.

%#codegen
    
    % Initialize precoding matrix tables
    persistent matrixTables;
    if (isempty(matrixTables))
        matrixTables = initializeMatrixTables();
    end
    
    narginchk(3,4);
    
    % Parse and validate inputs
    if (nargin>3)
        transformPrecode = varargin{1};
    else
        transformPrecode = false;
    end
    fcnName = 'nrPUSCHCodebook';
    validateattributes(nlayers,{'numeric'}, ...
        {'scalar','integer','>=',1,'<=',4},fcnName,'NLAYERS');
    validateattributes(nports,{'numeric'}, ...
        {'scalar','integer'},fcnName,'NPORTS');
    coder.internal.errorIf(~any(nports==[1 2 4]), ...
        'nr5g:nrPUSCHCodebook:InvalidNPorts',nports);
    validateattributes(TPMI,{'numeric'}, ...
        {'scalar','integer','>=',0,'<=',27},fcnName,'TPMI');
    validateattributes(transformPrecode,{'numeric','logical'}, ...
        {'scalar'},fcnName,'TPRECODE');
    coder.internal.errorIf(nlayers>nports, ...
        'nr5g:nrPUSCHCodebook:TooManyLayers',nlayers,nports);
    
    if (nlayers==1) % single-layer transmission
        
        if (nports==1) % single antenna port
            
            % Section 6.3.1.5
            allW = 1;
            
        elseif (nports==2) % two antenna ports
            
            % Table 6.3.1.5-1
            allW = matrixTables.W12;
            
        else % four antenna ports
            
            if (transformPrecode) % transform precoding enabled
                
                % Table 6.3.1.5-2
                allW = matrixTables.W14_tp;
                
            else % transform precoding disabled
                
                % Table 6.3.1.5-3
                allW = matrixTables.W14_notp;
                
            end
            
        end
        
    elseif (nlayers==2) % two-layer transmission
        
        if (nports==2) % two antenna ports
            
            % Table 6.3.1.5-4
            allW = matrixTables.W22;
            
        else % four antenna ports
            
            % Table 6.3.1.5-5
            allW = matrixTables.W24;
            
        end
        
    elseif (nlayers==3) % three-layer transmission using four antenna ports
        
        % Table 6.3.1.5-6
        allW = matrixTables.W34;
        
    else % four-layer transmission using four antenna ports
        
        % Table 6.3.1.5-7
        allW = matrixTables.W44;
        
    end
    
    maxTPMI = (size(allW,1) / nlayers) - 1;
    coder.internal.errorIf(TPMI>maxTPMI, ...
        'nr5g:nrPUSCHCodebook:InvalidTPMI',TPMI,maxTPMI);
    W = allW(double(TPMI)*nlayers + (1:nlayers),:);
    
end

function matrixTables = initializeMatrixTables()

    matrixTables.W12 = initializeW12();
    matrixTables.W14_tp = initializeW14_tp();
    matrixTables.W14_notp = initializeW14_notp();
    matrixTables.W22 = initializeW22();
    matrixTables.W24 = initializeW24();
    matrixTables.W34 = initializeW34();
    matrixTables.W44 = initializeW44();

end

function W12 = initializeW12()

    % Table 6.3.1.5-1
    %              TPMI
    W12 = [1   0;  %  0
           0   1;  %  1
           1   1;  %  2
           1  -1;  %  3
           1  1j;  %  4
           1 -1j]; %  5
    W12 = W12 / sqrt(2);
    
end

function W14_tp = initializeW14_tp()

    % Table 6.3.1.5-2
    %                         TPMI
    W14_tp = [1   0   0   0;  %  0
              0   1   0   0;  %  1
              0   0   1   0;  %  2
              0   0   0   1;  %  3
              1   0   1   0;  %  4
              1   0  -1   0;  %  5
              1   0  1j   0;  %  6
              1   0 -1j   0;  %  7
              0   1   0   1;  %  8
              0   1   0  -1;  %  9
              0   1   0  1j;  % 10
              0   1   0 -1j;  % 11
              1   1   1  -1;  % 12
              1   1  1j  1j;  % 13
              1   1  -1   1;  % 14
              1   1 -1j -1j;  % 15
              1  1j   1  1j;  % 16
              1  1j  1j   1;  % 17
              1  1j  -1 -1j;  % 18
              1  1j -1j  -1;  % 19
              1  -1   1   1;  % 20
              1  -1  1j -1j;  % 21
              1  -1  -1  -1;  % 22
              1  -1 -1j  1j;  % 23
              1 -1j   1 -1j;  % 24
              1 -1j  1j  -1;  % 25
              1 -1j  -1  1j;  % 26
              1 -1j -1j   1]; % 27
    W14_tp = W14_tp / 2;
    
end

function W14_notp = initializeW14_notp()
    
    % Table 6.3.1.5-3
    %                           TPMI
    W14_notp = [1   0   0   0;  %  0
                0   1   0   0;  %  1
                0   0   1   0;  %  2
                0   0   0   1;  %  3
                1   0   1   0;  %  4
                1   0  -1   0;  %  5
                1   0  1j   0;  %  6
                1   0 -1j   0;  %  7
                0   1   0   1;  %  8
                0   1   0  -1;  %  9
                0   1   0  1j;  % 10
                0   1   0 -1j;  % 11
                1   1   1   1;  % 12
                1   1  1j  1j;  % 13
                1   1  -1  -1;  % 14
                1   1 -1j -1j;  % 15
                1  1j   1  1j;  % 16
                1  1j  1j  -1;  % 17
                1  1j  -1 -1j;  % 18
                1  1j -1j   1;  % 19
                1  -1   1  -1;  % 20
                1  -1  1j -1j;  % 21
                1  -1  -1   1;  % 22
                1  -1 -1j  1j;  % 23
                1 -1j   1 -1j;  % 24
                1 -1j  1j   1;  % 25
                1 -1j  -1  1j;  % 26
                1 -1j -1j  -1]; % 27
    W14_notp = W14_notp / 2;
    
end

function W22 = initializeW22()

    % Table 6.3.1.5-4
    %              TPMI
    W22 = [1   0;  %  0
           0   1;
           1   1;  %  1
           1  -1;
           1  1j;  %  2
           1 -1j];
    W22(1:2,:) = W22(1:2,:) / sqrt(2);
    W22(3:end,:) = W22(3:end,:) / 2;

end

function W24 = initializeW24()

    % Table 6.3.1.5-5
    %                      TPMI
    W24 = [1   0   0   0;  %  0
           0   1   0   0;
           1   0   0   0;  %  1
           0   0   1   0;
           1   0   0   0;  %  2
           0   0   0   1;
           0   1   0   0;  %  3
           0   0   1   0;
           0   1   0   0;  %  4
           0   0   0   1;
           0   0   1   0;  %  5
           0   0   0   1;
           1   0   1   0;  %  6
           0   1   0 -1j;
           1   0   1   0;  %  7
           0   1   0  1j;
           1   0 -1j   0;  %  8
           0   1   0   1;
           1   0 -1j   0;  %  9
           0   1   0  -1;
           1   0  -1   0;  % 10
           0   1   0 -1j;
           1   0  -1   0;  % 11
           0   1   0  1j;
           1   0  1j   0;  % 12
           0   1   0   1;
           1   0  1j   0;  % 13
           0   1   0  -1;
           1   1   1   1;  % 14
           1   1  -1  -1;
           1   1  1j  1j;  % 15
           1   1 -1j -1j;
           1  1j   1  1j;  % 16
           1  1j  -1 -1j;
           1  1j  1j  -1;  % 17
           1  1j -1j   1;
           1  -1   1  -1;  % 18
           1  -1  -1   1;
           1  -1  1j -1j;  % 19
           1  -1 -1j  1j;
           1 -1j   1 -1j;  % 20
           1 -1j  -1  1j;
           1 -1j  1j   1;  % 21
           1 -1j -1j  -1];
    W24(1:28,:) = W24(1:28,:) / 2;
    W24(29:end,:) = W24(29:end,:) / (2*sqrt(2));
    
end

function W34 = initializeW34()

    % Table 6.3.1.5-6
    %                     TPMI
    W34 = [1   0   0   0; %  0
           0   1   0   0;
           0   0   1   0;
           1   0   1   0; %  1
           0   1   0   0;
           0   0   0   1;
           1   0  -1   0; %  2
           0   1   0   0;
           0   0   0   1;
           1   1   1   1; %  3
           1  -1   1  -1;
           1   1  -1  -1;
           1   1  1j  1j; %  4
           1  -1  1j -1j;
           1   1 -1j -1j;
           1  -1   1  -1; %  5
           1   1   1   1;
           1  -1  -1   1;
           1  -1  1j -1j; %  6
           1   1  1j  1j;
           1  -1 -1j  1j];
    W34(1:9,:) = W34(1:9,:) / 2;
    W34(10:end,:) = W34(10:end,:) / (2*sqrt(3));
    
end

function W44 = initializeW44()

    % Table 6.3.1.5-7
    %                     TPMI
    W44 = [1   0   0   0; %  0
           0   1   0   0;
           0   0   1   0;
           0   0   0   1;
           1   0   1   0; %  1
           1   0  -1   0;
           0   1   0   1;
           0   1   0  -1;
           1   0  1j   0; %  2
           1   0 -1j   0;
           0   1   0  1j;
           0   1   0 -1j;
           1   1   1   1; %  3
           1  -1   1  -1;
           1   1  -1  -1;
           1  -1  -1   1;
           1   1  1j  1j; %  4
           1  -1  1j -1j;
           1   1 -1j -1j;
           1  -1 -1j  1j];
    W44(1:4,:) = W44(1:4,:) / 2;
    W44(5:12,:) = W44(5:12,:) / (2*sqrt(2));
    W44(13:end,:) = W44(13:end,:) / 4;
    
end
