function cw = nrPUSCHDescramble(in,nid,rnti,varargin)
%nrPUSCHDescramble Physical uplink shared channel descrambling
%   CW = nrPUSCHDescramble(...) returns a vector of soft bits CW resulting 
%   from performing the inverse of physical uplink shared channel (PUSCH)
%   scrambling as defined in TS 38.211 Section 6.3.1.1.
%
%   CW = nrPUSCHDescramble(IN,NID,RNTI) performs PUSCH descrambling given
%   soft bit vector IN, scrambling identity NID (0...1023) and Radio
%   Network Temporary Identifier RNTI (0...65535). Note that this syntax
%   will only descramble the data bits, as the placeholder bit locations
%   for any UCI, if present, are unknown.
%
%   CW = nrPUSCHDescramble(...,NRAPID) performs PUSCH descrambling
%   considering the random access preamble index, NRAPID, used to
%   initialize the scrambling sequence for msgA on PUSCH.
%
%   CW = nrPUSCHDescramble(...,XIND,YIND) performs PUSCH descrambling
%   accounting for the uplink control information (UCI) placeholder 'x' bit
%   locations, XIND and the UCI placeholder 'y' bit locations YIND. XIND
%   and YIND are 1-based column vectors within the codeword to indicate the
%   respective placeholder locations. The descrambling of input codeword IN
%   at the locations of XIND is ignored, and locations of YIND are
%   descrambled with the previous values of the scrambling sequence.
%
%   Example 1:
%   % Perform PUSCH scrambling, symbol modulation, symbol demodulation and 
%   % descrambling of a codeword containing 3000 data bits.
%
%   cw = randi([0 1],3000,1);
%   ncellid = 42;
%   rnti = 101;
%   modulation = '16QAM';
%
%   scrambled = nrPUSCHScramble(cw,ncellid,rnti);
%   sym = nrSymbolModulate(scrambled,modulation);
%   demod = nrSymbolDemodulate(sym,modulation);
%
%   descrambled = nrPUSCHDescramble(demod,ncellid,rnti);
%   rxcw = double(descrambled<0);
%   isequal(cw,rxcw)
%
%   Example 2:
%   % Perform PUSCH scrambling, symbol modulation, symbol demodulation and
%   % descrambling of a codeword with placeholders.
%
%   cw = [1 -2 -1 -1]'; % 1 bit with value 1, encoding with '16QAM'
%   nid = 100;
%   rnti = 65350;
%   modulation = '16QAM';
%   xind = find(cw == -1);
%   yind = find(cw == -2);
%
%   scrambled = nrPUSCHScramble(cw,nid,rnti);
%   sym = nrSymbolModulate(scrambled,modulation);
%   demod = nrSymbolDemodulate(sym,modulation);
%
%   descrambled = nrPUSCHDescramble(demod,nid,rnti,xind,yind);
%   isequal(descrambled(1)<0,cw(1))
%
%   See also nrPUSCHScramble, nrPUSCHPRBS.
 
%   Copyright 2018-2021 The MathWorks, Inc.

%#codegen
    
    narginchk(3,6);
    if ~mod(nargin,2)
        % The fourth input is NRAPID, used to initialize the scrambling
        % sequence for msgA on PUSCH
        nrapid = varargin{1};
        firstoptarg = 2;
    else
        % The scrambling sequence initialization only uses NID and RNTI
        nrapid = [];
        firstoptarg = 1;
    end
    
    % Validate inputs
    fcnName = 'nrPUSCHDescramble';
    if (isempty(in) && isnumeric(in))
        in = zeros([0 1],'like',in);
    end
    validateattributes(in,{'double','single'},...
        {'finite','column'},fcnName,'IN');
    validateattributes(nid,{'numeric'},{'scalar', ...
        'integer','>=',0,'<=',1023},fcnName,'NID');
    validateattributes(rnti,{'numeric'},{'scalar', ...
        'integer','>=',0,'<=',65535},fcnName,'RNTI');
    if ~isempty(nrapid)
        validateattributes(nrapid,{'numeric'}, {'scalar', ...
            'integer','>=',0,'<=',63},fcnName,'NRAPID');
    end

    % Create scrambling sequence
    opts.MappingType = 'signed';
    opts.OutputDataType = class(in);
    cwLength = length(in);
    c = nrPUSCHPRBS(nid,rnti,nrapid,cwLength,opts);

    % Find the locations of data and UCI placeholder bits
    if nargin > (3 + firstoptarg - 1)
        numReqInputs = 3 + firstoptarg + 1;
        narginchk(numReqInputs,numReqInputs);
        xInd = varargin{firstoptarg};
        yInd = varargin{firstoptarg+1};
        validateInputWithEmpty(xInd,{'numeric'},...
            {'column','integer','positive','<=',cwLength},fcnName,'XIND');
        validateInputWithEmpty(yInd,{'numeric'},...
            {'column','integer','positive','<=',cwLength},fcnName,'YIND');

        % Get the unique 'x' and 'y' placeholder locations
        map = zeros(1,cwLength);
        map(yInd(:)) = 3;
        map(xInd(:)) = 2;
        dataIndex = find(map == 0);
        xIndSort = find(map == 2);
        yIndSort = find(map == 3);

        % Find the UCI placeholder locations which require special handling
        % Get the start and last value of consecutive 'y' placeholder
        % indices
        consInd = diff([0 diff(yIndSort)==1 0]);
        startInd = yIndSort(consInd>0);
        lastInd = yIndSort(consInd<0);

        % Check for the 'y' placeholder locations which has 'x' placeholder
        % locations previous to it
        numYInd = numel(startInd);
        if numYInd
            logicalMatrix = sum(repmat(startInd-1,numel(xIndSort)+1,1) == ...
                repmat([0;xIndSort(:)],1,numel(startInd)),1) == 1;
        else
            logicalMatrix = false(1,numYInd);
        end
        yc = yIndSort; % The placeholder locations to be used in scrambling sequence
        for i = 1:numYInd
            si = find(yIndSort == startInd(i));
            li = find(yIndSort == lastInd(i));
            if logicalMatrix(i)
                yc(si(1):li(1)) = -1;
            else
                yc(si(1):li(1)) = yc(si(1));
            end
        end
        yIndSort = yIndSort(yc ~= -1);
        yc = yc(yc ~= -1);
        if any(yc == 1)
            % Ignore 'y' placeholder location, if present at the starting
            % location
            yc(1) = [];
            yIndSort(1) = [];
        end
        % Check if locations previous to yc include any 'x' placeholder
        % locations
        if ~isempty(yc)
            % Get the locations which has 'x' placeholder locations
            % previous to yc
            map(yc-1) = map(yc-1)-1;
            mapInd = (map == 1);
            yMapInd = mapInd(yIndSort-1);
            % Get the effective 'y' placeholder locations
            yc = yc(~yMapInd);
            yIndSort = yIndSort(~yMapInd);
        end
    else
        % Assume no placeholders
        dataIndex = 1:cwLength;
        yIndSort = zeros(1,0);
        yc = zeros(1,0);
    end

    % Descramble input soft bits, ignore 'x' and take the previous for 'y'
    % placeholder bit locations, if any
    cw = in;
    cw(dataIndex) = in(dataIndex) .* c(dataIndex);
    % Descramble soft bits at 'y' placeholder locations
    cw(yIndSort) = cw(yIndSort).*c(yc-1); % yc-1 are previous bit locations
end

function validateInputWithEmpty(in,classes,attributes,fcnName,varname)
%Validates input

    if ~isempty(in)
        % Check for type and attributes
        validateattributes(in,classes,attributes,fcnName,varname);
    else
        % Check for type when input is empty
        validateattributes(in,classes,{'2d'},fcnName,varname);
    end

end
