function out = nrPUSCHScramble(cw,nid,rnti,varargin)
%nrPUSCHScramble Physical uplink shared channel scrambling
%   OUT = nrPUSCHScramble(CW,NID,RNTI) returns a column vector OUT
%   resulting from performing PUSCH scrambling according to TS 38.211
%   Section 6.3.1.1 given codeword bit vector CW, scrambling identity NID
%   (0...1023) and Radio Network Temporary Identifier RNTI (0...65535).
%
%   CW represents a UL-SCH codeword as described in TS 38.212 Section
%   6.2.7. Bit values should be represented by 0 and 1. Placeholders for
%   UCI with tags 'x' and 'y' should be represented by values -1 and -2
%   respectively.
%
%   OUT = nrPUSCHScramble(...,NRAPID) performs PUSCH scrambling considering
%   the random access preamble index, NRAPID, used to initialize the
%   scrambling sequence for msgA on PUSCH.
%    
%   Example:
%   % Perform PUSCH scrambling of a codeword containing 5000 bits.
%
%   cw = randi([0 1],5000,1);
%   ncellid = 42;
%   rnti = 101;
%   scrambled = nrPUSCHScramble(cw,ncellid,rnti);
%
%   See also nrPUSCHDescramble, nrPUSCHPRBS.
 
%   Copyright 2018-2021 The MathWorks, Inc.

%#codegen
    
    narginchk(3,4);
    if nargin==4
        % The fourth input is NRAPID, used to initialize the scrambling
        % sequence for msgA on PUSCH
        nrapid = varargin{1};
    else
        % The scrambling sequence initialization only uses NID and RNTI
        nrapid = [];
    end
    
    % Validate inputs
    fcnName = 'nrPUSCHScramble';
    if (isempty(cw) && isnumeric(cw))
        cw = zeros([0 1],'like',cw);
    end
    validateattributes(cw,{'double','int8'}, ...
        {'column'},fcnName,'CW');
    validateattributes(nid,{'numeric'},{'scalar', ...
        'integer','>=',0,'<=',1023},fcnName,'NID');
    validateattributes(rnti,{'numeric'},{'scalar', ...
        'integer','>=',0,'<=',65535},fcnName,'RNTI');
    if ~isempty(nrapid)
        validateattributes(nrapid,{'numeric'}, {'scalar', ...
            'integer','>=',0,'<=',63},fcnName,'NRAPID');
    end

    % Create scrambling sequence
    cwLen = length(cw);
    c = nrPUSCHPRBS(nid,rnti,nrapid,cwLen);

    % Get the data locations excluding the UCI placeholders having tag 'x'
    % (input value is -1) and tag 'y' (input value is -2)
    xLogical = (cw == -1);
    yLogical = (cw == -2);
    dataLogical = ~(xLogical | yLogical);

    % Scramble all input data bits (excluding UCI placeholders)
    out = true(cwLen,1);
    out(dataLogical) = xor(cw(dataLogical),c(dataLogical));

    % Replace UCI placeholders having tag 'y'. If the first input value is
    % 'y' placeholder, assume that the previous bit is 0
    yidx = find(yLogical);
    yfirst = (yidx==1);
    out(yidx(yfirst)) = 0;
    start = any(yfirst)+1;
    for idx = start:numel(yidx)
        out(yidx(idx)) = out(yidx(idx) - 1);
    end

end
