classdef nrSCSCarrierConfig < nr5g.internal.ConfigBase
%nrSCSCarrierConfig NR SCS carrier configuration
%   CARRIER = nrSCSCarrierConfig creates a carrier configuration object for
%   a specific OFDM numerology. You can use this object in the SCSCarriers
%   property of the <a href="matlab:help('nrDLCarrierConfig')">nrDLCarrierConfig</a> and <a
%   href="matlab:help('nrULCarrierConfig')">nrULCarrierConfig</a> objects to
%   specify the SCS carrier(s) of a 5G waveform generated by <a
%   href="matlab:help('nrWaveformGenerator')">nrWaveformGenerator</a>.
%   This object contains parameters defining the carrier subcarrier spacing
%   (SCS), bandwidth, and offset from point A. Point A is the center of
%   subcarrier 0 of common resource block 0 (CRB 0). By default, the object
%   defines a 10 MHz carrier with a 15 kHz subcarrier spacing (52 resource
%   blocks).
%
%   CARRIER = nrSCSCarrierConfig(Name,Value) creates a carrier
%   configuration object with the specified property Name set to the
%   specified Value. You can specify additional name-value pair
%   arguments in any order as (Name1,Value1,...,NameN,ValueN).
%
%   nrSCSCarrierConfig properties:
%
%   SubcarrierSpacing - Subcarrier spacing in kHz
%                       (15 (default), 30, 60, 120, 240)
%   NSizeGrid         - Number of resource blocks in carrier resource
%                       grid (1...275) (default 52)
%   NStartGrid        - Start of carrier resource grid relative to CRB 0
%                       (0...2199) (default 0)
%
%   Example 1:
%   % Create a default object specifying a 10 MHz carrier at 15 kHz
%   % subcarrier spacing.
%
%   carrier = nrSCSCarrierConfig
%   cfgDL = nrDLCarrierConfig;
%   cfgDL.SCSCarriers = {carrier};
%
%   Example 2: 
%   % Create two SCS carrier objects; one default and one
%   % specifying a 100 MHz carrier at 30 kHz subcarrier spacing.
%
%   carrier1 = nrSCSCarrierConfig; 
%   carrier2 = nrSCSCarrierConfig;
%   carrier2.SubcarrierSpacing = 30;
%   carrier2.NSizeGrid = 273;
%   % add two objects to an nrDLCarrierConfig configuration:
%   cfgDL = nrDLCarrierConfig;
%   cfgDL.SCSCarriers = {carrier1, carrier2};
%
%   See also nrDLCarrierConfig, nrULCarrierConfig, nrWaveformGenerator,
%   nrWavegenBWPConfig.

%   Copyright 2020 The MathWorks, Inc.

%#codegen
  
  properties
    %SubcarrierSpacing Subcarrier spacing in kHz
    %   Specify the subcarrier spacing of the carrier in kHz. The value
    %   must be one of {15, 30, 60, 120, 240}. The default value is 15.
    SubcarrierSpacing = 15;
        
    %NSizeGrid Number of resource blocks in carrier resource grid
    %   Specify the size of carrier resource grid in number of resource
    %   blocks. The value must be an integer in the range 1...275. The
    %   default value is 52 which corresponds to the maximum number of
    %   resource blocks for a 10 MHz carrier with a 15 kHz subcarrier
    %   spacing.
    NSizeGrid = 52;

    %NStartGrid Start of carrier resource grid relative to CRB 0
    %   Specify the starting resource block of the carrier resource
    %   grid relative to common resource block 0 (CRB 0). The value
    %   must be an integer in the range 0...2199. The default value is 0.
    NStartGrid = 0;
  end
  
  methods
    function obj = nrSCSCarrierConfig(varargin)
       obj@nr5g.internal.ConfigBase(varargin{:});
    end
    
    function obj = set.SubcarrierSpacing(obj,val)
      prop = 'SubcarrierSpacing';
      validateattributes(val, {'numeric'}, ...
          {'real','scalar'}, ...
          [class(obj) '.' prop], prop);
      flag = ~any(val == [15 30 60 120 240]);
      coder.internal.errorIf(flag,'nr5g:nrCarrierConfig:InvalidSCS',sprintf('%g',double(val)));
      obj.(prop) = val;
    end
    
    function obj = set.NSizeGrid(obj,val)
        prop = 'NSizeGrid';
        validateattributes(val, {'numeric'}, ...
            {'scalar','integer','positive','<=',275}, ...
            [class(obj) '.' prop], prop);
        obj.(prop) = val;
    end

    function obj = set.NStartGrid(obj,val)
        prop = 'NStartGrid';
        validateattributes(val, {'numeric'}, ...
            {'scalar','integer','nonnegative','<=',2199}, ...
            [class(obj) '.' prop], prop);
        obj.(prop) = val;
    end
  end
  
end

