function [sym, info] = nrSRS(carrier,srs,varargin)
%nrSRS Uplink sounding reference signal
%   [SYM,INFO] = nrSRS(CARRIER,SRS) returns a complex matrix containing
%   uplink sounding reference signal (SRS) values corresponding to the
%   carrier-specific configuration object CARRIER and SRS-specific
%   configuration object SRS as defined in TS 38.211 section 6.4.1.4.2. The
%   symbols for each antenna port are in the columns of SYM, with the
%   number of columns determined by the number of configured transmission
%   antenna ports. The function also provides additional information INFO
%   regarding the SRS generation process.
%   
%   CARRIER is a carrier-specific configuration object as described in
%   <a href="matlab:help('nrCarrierConfig')">nrCarrierConfig</a> with properties:
%
%   SubcarrierSpacing     - Subcarrier spacing in kHz
%   CyclicPrefix          - Cyclic prefix (CP) type
%   NSlot                 - Absolute slot number
%   NFrame                - Absolute system frame number
%
%   SRS is an SRS-specific configuration object as described in
%   <a href="matlab:help('nrSRSConfig')">nrSRSConfig</a> with properties:
%   
%   NumSRSPorts       - Number of SRS antenna ports (1,2,4)
%   NumSRSSymbols     - Number of consecutive OFDM symbols allocated to the 
%                       SRS (1,2,4,8,12)
%   SymbolStart       - Index of the first SRS symbol in a slot (default 13)
%   KTC               - Transmission comb number (2,4,8). The SRS is 
%                       transmitted every KTC subcarriers
%   CyclicShift       - Cyclic shift number offset (0...NCSmax-1). The 
%                       maximum number of cyclic shifts is NCSmax = 6 if
%                       KTC = 8, NCSmax = 12 if KTC = 4, and NCSmax = 8 if
%                       KTC = 2
%   CSRS              - Bandwidth configuration index C_SRS (0...63). It 
%                       controls the SRS bandwidth and frequency hopping,
%                       as defined in TS 38.211 Table 6.4.1.4.3-1
%   BSRS              - Bandwidth configuration index B_SRS (0...3). It 
%                       controls the SRS bandwidth and frequency
%                       hopping, as defined in TS 38.211 Table 6.4.1.4.3-1
%   SRSPeriod         - Slot periodicity and offset of the SRS resource 
%                       ('on','off',[Tsrs Toffset]). Accepted values for
%                       Tsrs are 1, 2, 4, 5, 8, 10, 16, 20, 32, 40, 64, 80,
%                       160, 320, 640, 1280, 2560. The value of Toffset
%                       must be within the range (0...Tsrs-1). When
%                       SRSPeriod = 'on', the function returns symbols
%                       regardless of the slot and frame numbers. When
%                       SRSPeriod = 'off', the function returns an empty
%                       array of symbols. If SRSPeriod = [Tsrs Toffset],
%                       the function returns a nonempty array of symbols
%                       only for the candidate slots specified in TS 38.211
%                       Section 6.4.1.4.4
%   GroupSeqHopping   - Group or sequence numbers hopping configuration. 
%                       ('neither','groupHopping','sequenceHopping')
%   NSRSID            - SRS scrambling identity (0...65535). It determines
%                       the group number when GroupSeqHopping is set to
%                       'neither'. Otherwise, it initializes the
%                       pseudorandom binary sequence for group or sequence
%                       hopping
%
%   INFO is a structure containing the fields:
%   SeqGroup   - Base sequence group number per OFDM symbol (u)
%   NSeq       - Base sequence number per OFDM symbol (v)
%   Alpha      - Reference signal cyclic shift per port (alpha_p)
%   SeqLength  - Zadoff Chu sequence length (MRSSC)
% 
%   [SYM,INFO] = nrSRS(...,NAME,VALUE) specifies additional options
%   as NAME,VALUE pairs to allow control over the data type and format of
%   the output symbols:
%
%   'OutputDataType'       - 'double' for double precision (default)
%                            'single' for single precision
%
%   Example: 
%   % Generate SRS symbols for a 2-port SRS transmission of 4 OFDM symbols.  
%   carrier = nrCarrierConfig;
% 
%   srs = nrSRSConfig;
%   srs.NumSRSPorts = 2;
%   srs.NumSRSSymbols = 4;
%   srs.SymbolStart = 8;
%   srs.CSRS = 5; 
%   srs.BSRS = 0;
% 
%   [sym,info] = nrSRS(carrier,srs);
%
%   See also nrCarrierConfig, nrSRSConfig, nrSRSIndices.

%   Copyright 2019-2021 The MathWorks, Inc.

%#codegen

    narginchk(2,4);
    
    validateInputs(carrier, srs);
    
    % PV pair check
    coder.extrinsic('nr5g.internal.parseOptions');
    opts = coder.const(nr5g.internal.parseOptions('nrSRS',{'OutputDataType'},varargin{:}));

    % Carrier configuration parameters 
    NSlot = mod(double(carrier.NSlot),carrier.SlotsPerFrame);
    SymPerSlot = carrier.SymbolsPerSlot;
    
    % SRS config parameters
    NPorts = double(srs.NumSRSPorts);
    NSym   = double(srs.NumSRSSymbols);
    TStart = double(srs.SymbolStart);
    KTC    = double(srs.KTC);
    NID    = double(srs.NSRSID);
    NCS    = double(srs.CyclicShift);

    % Cyclic shift(s) alpha per port
    alphap = getCyclicShift(NPorts,KTC,NCS);
    
    % Number of RBs allocated to the SRS
    mSRSb = srs.NRBPerTransmission;

    % Set group and sequence numbers (u,v) per OFDM symbol
    switch lower(srs.GroupSeqHopping)
        case 'grouphopping'
            u = groupNumberHopping(NSlot,SymPerSlot,NID,NSym,TStart);
            v = 0;
        case 'sequencehopping'
            u = mod(NID,30);
            v = sequenceNumberHopping(NSlot,SymPerSlot,NID,NSym,TStart,KTC,mSRSb);
        otherwise % 'neither'
            u = mod(NID,30)*ones(1,NSym);
            v = 0;
    end
    
    % Determine if the slot must contain SRS (TS 38.211 Section 6.4.1.4.4)
    srsSlot = nr5g.internal.isRSCandidateSlot(carrier.NSlot,carrier.NFrame,carrier.SlotsPerFrame,srs.SRSPeriod);
    
    % SRS length in subcarriers
    seqLen = mSRSb*12/KTC;
    
    if srsSlot
        % Generate low-PAPR sequence per OFDM symbol only in candidate slots
        seqT = nrLowPAPRS(u,v,alphap,seqLen,'OutputDataType',opts.OutputDataType);
        seq = permute(seqT(:,:,:),[1 3 2]);
        
        % Vectorize in time domain and let columns store SRS seqs per antenna port
        sym = reshape(seq,[],NPorts);        
    else
        sym = zeros(0,NPorts,opts.OutputDataType);
        seqLen = 0;
    end
        
    % Collect information 
    info.SeqGroup = u .* ones(1,NSym);
    info.NSeq = v .* ones(1,NSym);
    info.Alpha = alphap;
    info.SeqLength = seqLen; 
end

% Cyclic shift (alpha) per port
function alphap = getCyclicShift(NPorts,KTC,NCS)

    % Max number of cyclic shifts based on comb type
    maxcs = [8 12 6];
    nCSmax = maxcs(log2(KTC));
    
    % Port numbers from number of ports
    p = 1000 + (0:NPorts-1);
    
    % Number of cyclic shift(s) per port
    nCSp = mod(NCS + nCSmax*(p-1000)/NPorts,nCSmax);

    % Cyclic shift(s) per port
    alphap = 2*pi * nCSp / nCSmax;
end

% Calculate the group number (u) of low PAPR sequence for SRS group hopping
function u = groupNumberHopping(NSlot,SymbolsPerSlot,NID,NSym,SymbolStart)

    % Index of the first bit of the PRBS   
    firstBitIndex = 8*(NSlot*SymbolsPerSlot + SymbolStart);

    % Length of pseudorandom binary sequence
    PRBSLen = 8*NSym;

    % Pseudorandom binary sequence
    PRBS = nrPRBS(NID, [firstBitIndex PRBSLen]); 

    % Reorganize PRBS indices for OFDM symbol processing
    PRBSm = reshape(PRBS,8,NSym);

    % Group number (u) per OFDM symbol by converting 8-bit blocks of the
    % PRBS to mod-30 decimal representation
    m = (0:7)';
    f = mod(2.^m'*PRBSm, 30);
    u = mod(f + NID, 30);
end

% Calculate base sequences (v) of low PAPR sequence for SRS sequence number hopping
function v = sequenceNumberHopping(NSlot,SymbolsPerSlot,nID,NSym,SymbolStart,KTC,mb)

    % SRS occupies less than 6 RBs
    if mb / KTC < 6
        v = zeros(1,NSym);
    else 
        % Index of the first bit of the PRBS
        firstBitIndex = NSlot*SymbolsPerSlot + SymbolStart;
        
        % Sequence number sequence
        v = double(nrPRBS(nID, [firstBitIndex NSym])');
    end
end

function validateInputs(carrier,srs)
    fcnName = 'nrSRS';
    validateattributes(carrier, {'nrCarrierConfig'}, {'scalar'}, fcnName, 'Carrier-specific configuration object');
    validateattributes(srs, {'nrSRSConfig'}, {'scalar'}, fcnName, 'SRS-specific configuration object');
    
    validateConfig(srs);
    
    % Cross validation of properties
    SymbolStart = double(srs.SymbolStart);
    NumSRSSymbols = double(srs.NumSRSSymbols);
    if SymbolStart + NumSRSSymbols > carrier.SymbolsPerSlot
        cpOffset = 2*(carrier.SymbolsPerSlot==12);
        coder.internal.error('nr5g:nrSRS:InconsistentTimeAllocation', SymbolStart,NumSRSSymbols,SymbolStart+NumSRSSymbols-1,13-cpOffset,carrier.CyclicPrefix);
    end
end
