classdef nrSRSConfig < nr5g.internal.nrSRSConfigBase
    %nrSRSConfig Sounding reference signal (SRS) configuration object
    %   CFGSRS = nrSRSConfig creates an SRS configuration object. The
    %   object contains parameters of the SRS defined in TS 38.211 Section
    %   6.4.1.4. The default nrSRSConfig object represents a single-port,
    %   single-symbol configuration that places the SRS at the end of the
    %   slot. This configuration is narrowband and without frequency
    %   hopping (BHop>=BSRS).
    %
    %   CFGSRS = nrSRSConfig(Name,Value) creates an SRS configuration object
    %   with the specified property Name set to the specified Value. You
    %   can specify additional name-value arguments in any order as
    %   (Name1,Value1,...,NameN,ValueN).
    %
    %   nrSRSConfig properties (configurable):
    %
    %   NumSRSPorts       - Number of SRS antenna ports (1(default), 2, 4)
    %   NumSRSSymbols     - Number of consecutive OFDM symbols allocated 
    %                       to the SRS (1(default), 2, 4, 8, 12)
    %   SymbolStart       - Index of the first SRS symbol in a slot (default 13)
    %   KTC               - Transmission comb number (2(default), 4, 8). 
    %                       The SRS is transmitted every KTC subcarriers
    %   KBarTC            - Transmission comb offset in subcarriers 
    %                       (0...KTC-1) (default 0)
    %   CyclicShift       - Cyclic shift number offset NCS (default 0)
    %   FrequencyStart    - Frequency-domain starting position of the SRS 
    %                       in physical resource blocks (PRBs) relative to
    %                       the carrier resource grid (0...271) (default 0)
    %   NRRC              - Frequency domain position. Additional frequency
    %                       offset to FrequencyStart specified in blocks of
    %                       4 RBs (0...67) (default 0)
    %   CSRS              - Bandwidth configuration index C_SRS (0...63) (default 0) 
    %   BSRS              - Bandwidth configuration index B_SRS (0...3) (default 0) 
    %   BHop              - Frequency hopping index B_hop (0...3) (default 0)
    %   Repetition        - Repetition factor (1(default), 2, 4, 8, 12)
    %   SRSPeriod         - Slot periodicity (Tsrs) and offset (Toff) of 
    %                       the SRS ('on'(default), 'off', [Tsrs Toffset])
    %   ResourceType      - Time-domain behavior of the SRS resource 
    %                       ('periodic'(default), 'semi-persistent', 'aperiodic')
    %   GroupSeqHopping   - Group or sequence number hopping 
    %                       ('neither'(default), 'groupHopping', 'sequenceHopping')
    %   NSRSID            - Scrambling identity (0...65535) (default 0)
    %   SRSPositioning    - Enable SRS for user positioning (true, false(default))
    %   
    %   nrSRSConfig properties (read-only):
    %
    %   NRB                - Maximum number of RBs allocated to the SRS
    %   NRBPerTransmission - Number of RBs allocated to the SRS in an OFDM symbol.
    %
    %   Constant properties: 
    %   
    %   BandwidthConfigurationTable - Table containing the SRS configuration
    %                                 parameters m_SRS and N in TS 38.211
    %                                 Table 6.4.1.4.3-1.
    %   SubcarrierOffsetTable       - Table containing subcarrier offsets
    %                                 per OFDM symbol and comb number for
    %                                 SRS user positioning specified in 
    %                                 TS 38.211 Table 6.4.1.4.3-2.
    %
    % Example:
    % % Create an SRS configuration object with 4 OFDM symbols.
    %
    % srs = nrSRSConfig('NumSRSSymbols',4,'SymbolStart',8);
    %
    % See also nrSRS, nrSRSIndices.
    
    % Copyright 2019-2020 The MathWorks, Inc.
        
    %#codegen
    
    % Public, tunable properties
    properties
        %SRSPeriod Slot periodicity and offset
        %   Specify the slot periodicity and offset values of the SRS. The
        %   possible options are ('on', 'off', [Tsrs Toffset]). When this
        %   property is set to 'on', then the SRS is present in all slots.
        %   When it is set to 'off', the resource is absent in all slots.
        %   For explicit values of SRSPeriod [Tsrs Toffset], the slot
        %   periodicity (Tsrs) and offset (Toffset) values are considered
        %   to determine the presence of the SRS in a given slot according
        %   to TS 38.211 Section 6.4.1.4.4. Tsrs must be one of 1, 2, 4, 5,
        %   8, 10, 16, 20, 32, 40, 64, 80, 160, 320, 640, 1280, 2560; and
        %   Toffset must be lower than Tsrs. SRSPeriod = 'on' is equivalent
        %   to SRSPeriod = [1 0]. The default value of SRSPeriod is 'on'.
        SRSPeriod = 'on';
        
        %ResourceType Time-domain behavior of SRS
        %   Specify the time-domain behavior of the SRS. The value must be
        %   one of {'periodic', 'semi-persistent', 'aperiodic'}. Downlink
        %   control information (DCI) triggers aperiodic SRS transmissions.
        %   For an 'aperiodic' resource type, SRSPeriod determines the
        %   periodicity and offset of the DCI triggering signal. Set
        %   ResourceType to 'aperiodic' to disable inter-slot frequency
        %   hopping. The default value is 'periodic'.
        ResourceType = nrSRSConfig.getDefault('ResourceType');
    end
    
    % Read-only properties
    properties (SetAccess = private)
        
        %NRB Number of RBs allocated to the SRS
        %   Number of RBs allocated to the SRS transmission. When frequency
        %   hopping is enabled, NRB denotes the hopping bandwidth or number
        %   of RBs over which the SRS signal hops across multiple time slots.
        NRB;
        
        %NRBPerTransmission Number of RBs allocated to the SRS at each OFDM symbol 
        % When frequency hopping is enabled, NRBPerTransmission specifies
        % the allocated bandwidth in RBs at each SRS transmission.
        NRBPerTransmission;
    end

    % Constant, hidden properties
    properties (Constant,Hidden)
        SRSPeriod_Options        = [1,2,4,5,8,10,16,20,32,40,64,80,160,320,640,1280,2560];
        SRSPeriod_CharOpt        = {'on','off'};
        ResourceType_Values      = {'periodic','semi-persistent','aperiodic'};
    end
    
    % Constant public properties 
    properties (Constant)
        %BandwidthConfigurationTable SRS Bandwidth configuration table
        % It contains the information specified in TS 38.211 Table 6.4.1.4.3-1.
        BandwidthConfigurationTable = getBandwidthConfigTable;
        
        %SubcarrierOffsetTable Subcarrier offset for SRS user positioning
        % It contains the information specified in TS 38.211 Table 6.4.1.4.3-2.
        SubcarrierOffsetTable = getOffsetKTable;
    end    
    
    properties (Hidden)
    CustomPropList = {'NumSRSPorts'; 'NumSRSSymbols'; 'SymbolStart'; ...
        'KTC'; 'KBarTC'; 'CyclicShift'; 'FrequencyStart'; 'NRRC'; 'CSRS'; 'BSRS'; 'BHop'; ...
        'Repetition'; 'SRSPeriod'; 'ResourceType'; 'GroupSeqHopping'; 'NSRSID'; 'SRSPositioning'; ...
        'NRB'; 'NRBPerTransmission'; 'BandwidthConfigurationTable'; 'SubcarrierOffsetTable'};
    end
    
    methods
        
        % Constructor
        function obj = nrSRSConfig(varargin)
            % Support name-value pair arguments when constructing object
            obj@nr5g.internal.nrSRSConfigBase(...
                'ResourceType',nrSRSConfig.getDefault('ResourceType'),...
                varargin{:}); % Set variable-size properties for codegen compatibility
        end
        
        % Self-validate and set properties
        function obj = set.SRSPeriod(obj,val)
            propName = 'SRSPeriod';
            validateattributes(val,{'numeric','char','string'},...
                         {'nonempty'},[class(obj) '.' propName],propName);
            
            if ischar(val) || (isstring(val) && numel(val) == 1)
                % Character array or string scalar
                temp = validatestring(val,obj.SRSPeriod_CharOpt,[class(obj) '.' propName],propName);
                obj.(propName) = ''; % For codegen compatibility
            elseif isnumeric(val)
                validateattributes(val,{'numeric'},...
                    {'integer','vector','numel',2},[class(obj) '.' propName],propName);
                if ~any(val(1) == obj.SRSPeriod_Options)
                    coder.internal.error('nr5g:nrSRS:InvalidSlotPeriodicity',val(1));
                end
                if val(2) >= val(1) || val(2) < 0
                    coder.internal.error('nr5g:nrSRS:InvalidSlotOffset',val(2),val(1));
                end
                temp = val;
            end
            obj.(propName) = temp;
        end
        
        function obj = set.ResourceType(obj,val)
            propName = 'ResourceType';
            validateattributes(val,{'char','string'},...
                         {'nonempty'},[class(obj) '.' propName],propName);
            val = validatestring(val,obj.ResourceType_Values,[class(obj) '.' propName],propName);
            obj.(propName) = ''; % For codegen compatibility
            obj.(propName) = val;
        end
        
        % Read-only properties getters
        function val = get.NRB(obj)
            if obj.BHop < obj.BSRS % Frequency hopping cases
                val = nr5g.internal.SRSBandwidthConfiguration(obj.CSRS,obj.BHop);
            else
                val = nr5g.internal.SRSBandwidthConfiguration(obj.CSRS,obj.BSRS);
            end
        end
        
        function val = get.NRBPerTransmission(obj)
            val = nr5g.internal.SRSBandwidthConfiguration(obj.CSRS,obj.BSRS);
        end
    end
    
    methods (Static, Access = private)
        % Default values of variable-size char properties. This allows to
        % localize the defaults required for codegen compatibility
        function out = getDefault(propName)
            switch propName
                case 'ResourceType'
                    out = 'periodic';
            end
        end
    end
end

%% Local functions
% Get bandwidth configuration table for MSRS
function t = getBandwidthConfigTable
    % Create TS 38.211 Table 6.4.1.4.3-1: SRS bandwidth configuration
    
    confTableArray = nr5g.internal.SRSBandwidthConfiguration;
    
    columnNames = ["C_SRS", "m_SRS_0", "N_0", "m_SRS_1", "N_1", "m_SRS_2", "N_2", "m_SRS_3", "N_3"];
    
    % Package array in a table 
    t = array2table(confTableArray,"VariableNames",columnNames);
    t.Properties.VariableNames = columnNames;
    t.Properties.Description = 'TS 38.211 Table 6.4.1.4.3-1: SRS bandwidth configuration';
end

function t = getOffsetKTable
    % Create TS 38.211 Table 6.4.1.4.3-2: The offset k_offset for SRS as a function of K_TC and l'
    
    confTableArray = nr5g.internal.SRSOffsetK;
    
    columnNames = ["K_TC", "NSRS_symb = 1", "NSRS_symb = 2", "NSRS_symb = 4", "NSRS_symb = 8", "NSRS_symb = 12"];
     
    % Package array into a table 
    t = cell2table(confTableArray,"VariableNames",columnNames);
    t.Properties.Description = 'TS 38.211 Table 6.4.1.4.3-2: The offset k_offset for SRS as a function of K_TC and l''';
end
