function [ind,info] = nrSRSIndices(carrier,srs,varargin)
%nrSRSIndices Uplink SRS resource element indices 
%   [IND,INFO] = nrSRSIndices(CARRIER,SRS) returns a matrix containing
%   resource element indices for the uplink sounding reference signal (SRS)
%   corresponding to the carrier configuration object CARRIER and the SRS
%   configuration object SRS, as defined in TS 38.211 section 6.4.1.4.3. By
%   default, the indices are returned in 1-based linear form that directly
%   index the elements of a resource matrix. The number of columns in IND
%   is equal to the number of antenna ports configured. These indices are
%   ordered as the SRS modulation symbols should be mapped.
% 
%   CARRIER is a carrier-specific configuration object as described in
%   <a href="matlab:help('nrCarrierConfig')">nrCarrierConfig</a> with properties:
%
%   SubcarrierSpacing     - Subcarrier spacing in kHz
%   CyclicPrefix          - Cyclic prefix (CP) type
%   NSlot                 - Absolute slot number
%   NFrame                - Absolute system frame number
%   NSizeGrid             - Size of the carrier resource grid in terms of
%                           number of resource blocks (RBs)
%
%   SRS is an SRS-specific configuration object as described in
%   <a href="matlab:help('nrSRSConfig')">nrSRSConfig</a> with properties:
%
%   NumSRSPorts       - Number of SRS antenna ports (1,2,4)
%   NumSRSSymbols     - Number of consecutive OFDM symbols allocated to the 
%                       SRS (1,2,4,8,12)
%   SymbolStart       - Index of the first SRS symbol in a slot (default 13)
%   KTC               - Transmission comb number (2,4,8). The SRS is
%                       transmitted every KTC subcarriers in the frequency
%                       domain
%   KBarTC            - Transmission comb offset in subcarriers (0...KTC-1)
%   CyclicShift       - Cyclic shift number (0...NCSmax-1). The maximum
%                       number of cyclic shifts is NCSmax = 6 if KTC = 8,
%                       NCSmax = 12 if KTC = 4, and NCSmax = 8 if KTC = 2
%   FrequencyStart    - Starting position of the SRS in frequency in
%                       physical resource blocks (PRBs) when NRRC = 0.
%   NRRC              - Frequency domain position (0...67). Additional 
%                       frequency offset to FrequencyStart specified in
%                       blocks of 4 RBs. The resulting location of the SRS
%                       in frequency depends on FrequencyStart and the
%                       parameters in TS 38.211 Table 6.4.1.4.3-1
%   CSRS              - Bandwidth configuration index C_SRS (0...63). It 
%                       controls the SRS bandwidth and frequency hopping,
%                       as defined in TS 38.211 Table 6.4.1.4.3-1
%   BSRS              - Bandwidth configuration index B_SRS (0...3). It 
%                       controls the SRS bandwidth and frequency
%                       hopping, as defined in TS 38.211 Table 6.4.1.4.3-1
%   BHop              - Frequency hopping configuration (0...3). Set 
%                       BHop >= BSRS to disable frequency hopping
%   Repetition        - Repetition factor (1,2,4,8,12). When frequency
%                       hopping is enabled (BHop<BSRS), Repetition must be
%                       <= NumSRSSymbols. When frequency hopping is
%                       disabled, Repetition is ignored.
%   SRSPeriod         - Slot periodicity and offset of the SRS resource 
%                       ('on','off',[Tsrs Toffset]). Accepted values for
%                       Tsrs are 1, 2, 4, 5, 8, 10, 16, 20, 32, 40, 64, 80,
%                       160, 320, 640, 1280, 2560. The value of Toffset
%                       must be within the range (0...Tsrs-1). When
%                       SRSPeriod = 'on', the function returns indices
%                       regardless of the slot and frame numbers. When
%                       SRSPeriod = 'off', the function returns an empty
%                       array of indices. If SRSPeriod = [Tsrs Toffset],
%                       the function returns a nonempty array of indices
%                       only for the candidate slots specified in TS 38.211
%                       Section 6.4.1.4.4.
%   ResourceType      - Time domain behavior of SRS resource configuration.
%                       ('periodic', 'semi-persistent', 'aperiodic'). When
%                       ResourceType = 'aperiodic', the SRSPeriod parameter
%                       is interpreted as the periodicity and offset of the
%                       downlink control information signal triggering
%                       aperiodic SRS transmissions.
%   SRSPositioning    - Enable SRS for user positioning (true/false). A 
%                       value of true corresponds to the higher-layer
%                       parameter 'SRS-PosResource-r16' and false
%                       corresponds to 'SRS-Resource'.
%
%   INFO is a structure containing the fields:
%   SubcarrierOffset  - Frequency starting position of the SRS in 
%                       subcarriers per antenna port and OFDM symbol (k_0).
%                       SubcarrierOffset is a matrix of size
%                       NumSRSPorts-by-NumSRSSymbols.
%   FreqIndex         - Frequency position index (n_b) per OFDM symbol 
%                       for b = 0...BSRS. FreqIndex is a matrix of size
%                       (BSRS+1)-by-NumSRSSymbols.
%   HoppingOffset     - Hopping offset (F_b) per OFDM symbol for 
%                       b = (BHop+1)...BSRS. HoppingOffset is a matrix of
%                       size (BSRS-BHop)-by-NumSRSSymbols for BHop < BSRS.
%   PRBSet            - Resource blocks allocated for SRS per OFDM symbol.
%                       PRBSet is a matrix of size NRB-by-NumSRSSymbols,
%                       where NRB is the number of RBs per OFDM symbol.
%    
%   IND = nrSRSIndices(...,NAME,VALUE,...) specifies additional options as
%   NAME,VALUE pairs to allow control over the format of the indices:
%
%   'IndexStyle'     - 'index' for linear indices (default)
%                      'subscript' for [subcarrier, symbol, antenna] 
%                       subscript row form
%
%   'IndexBase'      - '1based' for 1-based indices (default) 
%                      '0based' for 0-based indices
%
%   Example: 
%   % Generate indices for a 2-port SRS transmission of 4 OFDM symbols.
%   
%   carrier = nrCarrierConfig;
% 
%   srs = nrSRSConfig;
%   srs.NumSRSPorts = 2;
%   srs.NumSRSSymbols = 4;
%   srs.SymbolStart = 8;
%   srs.FrequencyStart = 2;
%   srs.CSRS = 5; 
%   srs.BSRS = 0;
% 
%   indices = nrSRSIndices(carrier,srs);
%
%   See also nrCarrierConfig, nrSRSConfig, nrSRS.

%   Copyright 2019-2021 The MathWorks, Inc.

%#codegen

    narginchk(2,6);
    
    validateInputs(carrier, srs);
    
    % PV pair check
    coder.extrinsic('nr5g.internal.parseOptions');
    opts = coder.const(nr5g.internal.parseOptions('nrSRSIndices',{'IndexStyle','IndexBase'},varargin{:}));
    
    % Number of subcarriers per resource block
    NRBsc = 12; 
    
    % Lowest subcarrier per antenna port with respect to the grid origin 
    k0bar = frequencyPositionOffset(srs);
    
    % Frequency position indices and hopping offset
    [nb,Fb] = frequencyPositionIndex(carrier,srs);
        
    % NRBs for all b = 0:BSRS
    mSRSb = nr5g.internal.SRSBandwidthConfiguration(srs.CSRS,0:srs.BSRS);
    
    % Frequency starting position per antenna port (row) and OFDM symbol (column)
    k0 = k0bar + NRBsc*mSRSb'*nb;
    
    % Validate location of the SRS within the resource grid
    FSRS = [floor(min(k0(:))/NRBsc), floor(max(k0(:))/NRBsc+mSRSb(end))-1];
    lastCRB = carrier.NSizeGrid-1;
    if FSRS(2)-FSRS(1) > lastCRB
        coder.internal.error('nr5g:nrSRS:InconsistentBandwidthConfiguration',lastCRB,FSRS(1),FSRS(2),"CSRS");
    elseif FSRS(2) > lastCRB
        coder.internal.error('nr5g:nrSRS:InconsistentBandwidthConfiguration',lastCRB,FSRS(1),FSRS(2),"FrequencyStart");        
    end
    
    % Calculate 0-based subscripts     
    NPorts = double(srs.NumSRSPorts);
    NSym   = double(srs.NumSRSSymbols);
    KTC    = double(srs.KTC);
    
    % Determine if the slot must contain SRS (TS 38.211 Section 6.4.1.4.4)
    srsSlot = nr5g.internal.isRSCandidateSlot(carrier.NSlot,carrier.NFrame,carrier.SlotsPerFrame,srs.SRSPeriod);
    
    % SRS length in subcarriers
    MSRS = mSRSb(end) * NRBsc / KTC;

    if srsSlot
        K = permute(k0,[3 2 1]) + (0:MSRS-1)' * KTC;
        L = repmat( double(srs.SymbolStart) + (0:NSym-1), [MSRS,1,NPorts] );
        P = repmat( 0:NPorts-1, [NSym*MSRS, 1] );
    else
        K = []; L = []; P = [];
    end
    
    % Apply indices options
    gridsize = [double(carrier.NSizeGrid)*NRBsc,carrier.SymbolsPerSlot, NPorts];
    opts.MultiColumnIndex = true; % Output indices in multiple columns for multiport
    ind = nr5g.internal.applyIndicesOptions(gridsize,opts,K(:),L(:),P(:));
    
    % Create info structure
    info = struct();
    info.SubcarrierOffset = k0;
    info.FreqIndex = nb; 
    info.HoppingOffset = Fb;
    info.PRBSet = unique(floor(K(:,:,1)/12),'rows');
end

% Calculate the frequency position offset k0bar measured in subcarriers for
% each antenna port based on the comb number, comb offset, number of cyclic
% shifts and frequency start.
function k0bar = frequencyPositionOffset(srs)

    NPorts = double(srs.NumSRSPorts);
    KTC    = double(srs.KTC);
    kBarTC = double(srs.KBarTC);
    nShift = double(srs.FrequencyStart);
    NCS    = srs.CyclicShift;

    % Number of subcarriers per resource block
    NRBsc = 12; 

    % Max number of cyclic shifts based on comb type    
    maxcs = [8 12 6];
    nCSmax = maxcs(log2(KTC));

    % All antenna ports (AP) have the same frequency offset by default
    kTC = kBarTC*ones(1,NPorts);

    % If there are 4 APs and the number of cyclic shifts belongs to the
    % upper half of the range (0...nCSmax-1)
    if (NCS >= nCSmax/2) && NPorts == 4
        OddAP = logical( mod(0:NPorts-1,2) );
        % Only for odd port numbers 
        kTC(OddAP) = mod(kBarTC + KTC/2, KTC); 
    end
    
    if srs.SRSPositioning
        kOff = nr5g.internal.SRSOffsetK(srs.KTC,srs.NumSRSSymbols);
    else
        kOff = zeros(1,srs.NumSRSSymbols);
    end
    
    k0bar = nShift*NRBsc + mod(kTC(:)+kOff,KTC);
end

% Calculate the frequency position index n_b and hopping offset Fb of the SRS
% (see TS 38.211 Section 6.4.1.4.3)
function [nb,Fb] = frequencyPositionIndex(carrier,srs)

    CSRS = srs.CSRS; 
    BSRS = double(srs.BSRS);
    BHop = double(srs.BHop);
    NRRC = double(srs.NRRC);

    % Number of OFDM symbols containing SRS symbols in the current slot 
    NSym = srs.NumSRSSymbols; 

    % Bandwidth configuration
    b = 0:BSRS;
    [mSRSb,Nb] = nr5g.internal.SRSBandwidthConfiguration(CSRS,b);
    
    nb = repmat(mod(floor(4*NRRC./mSRSb), Nb),1,NSym);
    
    Fb = [];
    if BHop < BSRS % Frequency hopping        
        idx = BHop < b;
        
        % Number of SRS transmissions (SRS symbol counter)
        nSRS = numberSRSTransmissions(carrier,srs);
    
        % Function Fb
        Fb = hoppingOffset(b(idx),nSRS,CSRS,BHop);
        nb(idx,:) = mod(Fb+nb(idx,:), Nb(idx));
    end
    
end

% Returns the 0-based number of unique SRS transmissions nSRS measured in
% OFDM symbols. nSRS is measured from the origin of the radio frame for
% periodic and semi-persistent resource type configurations, and from the
% first SRS OFDM symbol transmitted in the current slot for aperiodic
% resource type configurations.
function nSRS = numberSRSTransmissions(carrier,srs)

    [Tsrs,Toff] = nr5g.internal.getRSPeriodicityAndOffset(srs.SRSPeriod);

    % Number of slots per frame 
    Nsf = carrier.SlotsPerFrame;

    NSym = double(srs.NumSRSSymbols);
    Rep  = double(srs.Repetition);
    
    % OFDM sym number in SRS resource
    LPrime = 0:NSym-1;
    
    % Relative frame and slot numbers
    NFrame = double(carrier.NFrame);
    NSlot  = double(carrier.NSlot);
    [NSlot,NFrame] = nr5g.internal.getRelativeNSlotAndSFN(NSlot,NFrame,carrier.SlotsPerFrame);

    % Number of SRS transmissions (repeated transmissions count as 1)
    if strcmpi(srs.ResourceType,'aperiodic')
        nSRS = floor(LPrime/Rep); 
    else
        nSRS = (NSym/Rep)*(Nsf*NFrame + NSlot - Toff)/Tsrs + floor(LPrime/Rep);
    end
end

% Calculate the frequency hopping offset function Fb as described in TS
% 38.211 Section 6.4.1.4.3. The parameter b (0...3) denotes the subscript
% of the function Fb and nSRS the number of SRS transmissions.
function Fb = hoppingOffset(b,nSRS,CSRS,BHop)

    LenB = length(b);
    Fb = zeros(LenB, length(nSRS));
    for i = 1:LenB
        % Bandwidth configuration parameters
        [~,Nbp] = nr5g.internal.SRSBandwidthConfiguration(CSRS, BHop:b(i));

        Nb = Nbp(end);

        % Nbhop = 1 regardless of the value of Nb
        Nbp(1) = 1;
        
        NbOdd = mod(Nb,2);
        if NbOdd % Function Fb for odd values of Nb
            Fb(i,:) = floor(Nb/2)*floor(nSRS/prod(Nbp(1:end-1)));
        else
            argument = mod(nSRS, prod(Nbp))/prod(Nbp(1:end-1));
            Fb(i,:) = Nb/2*floor(argument) + floor(argument/2);
        end
    end
end

function validateInputs(carrier,srs)
    fcnName = 'nrSRSIndices';
    validateattributes(carrier, {'nrCarrierConfig'}, {'scalar'}, fcnName, 'Carrier-specific configuration object');
    validateattributes(srs, {'nrSRSConfig'}, {'scalar'}, fcnName, 'SRS-specific configuration object');
    
    validateConfig(srs);
    
    % Cross validation of properties
    SymbolStart = double(srs.SymbolStart);
    NumSRSSymbols = double(srs.NumSRSSymbols);
    if (SymbolStart + NumSRSSymbols) > carrier.SymbolsPerSlot
        cpOffset = 2*(carrier.SymbolsPerSlot==12);
        coder.internal.error('nr5g:nrSRS:InconsistentTimeAllocation', SymbolStart,NumSRSSymbols,SymbolStart+NumSRSSymbols-1,13-cpOffset,carrier.CyclicPrefix);
    end 
end
