function tbs = nrTBS(modulation,nlayers,nPRB,NREPerPRB,tcr,varargin)
%nrTBS Transport block size determination
%   TBS = nrTBS(MODULATION,NLAYERS,NPRB,NREPERPRB,TCR) returns the
%   transport block size, TBS, associated with each codeword for a shared
%   channel transmission, as defined in TS 38.214 Sections 5.1.3.2 and
%   6.1.4.2, with the inputs:
%   MODULATION - Modulation scheme for each codeword. It must be specified
%                as one of {'pi/2-BPSK', 'QPSK', '16QAM', '64QAM', '256QAM'}.
%                Modulation scheme can be specified as a character array or
%                string scalar for one codeword or two codewords. In case
%                of two codewords, the same modulation scheme is applied to
%                both codewords. Alternatively, a cell array of character
%                vectors or string array can be used to specify different
%                modulation schemes for each codeword.
%   NLAYERS    - Number of transmission layers (1...4 for one codeword,
%                5...8 for two codewords).
%   NPRB       - Number of physical resource blocks (PRBs) allocated for
%                the physical shared channel. The value must be a scalar
%                nonnegative integer. The nominal value of NPRB is in the
%                range of 0...275.
%   NREPERPRB  - Number of resource elements (REs) allocated for the data
%                transmission in the physical shared channel, within one
%                PRB for one slot. The value must be a scalar nonnegative
%                integer. Note that this value does not include any
%                additional overhead.
%   TCR        - Target code rate for each codeword. It is a scalar for one
%                codeword or a two-element vector for two codewords, with
%                each value between 0 and 1. Alternatively, two codewords
%                can also be configured with single target code rate.
%   Note that the additional overhead and the scaling factor, used for TBS
%   calculation are considered as 0 and 1, respectively.
%
%   TBS = nrTBS(MODULATION,NLAYERS,NPRB,NREPERPRB,TCR,XOH) specifies the
%   additional overhead, which controls the number of REs available for the
%   data transmission in the shared channel, within one PRB for one slot.
%   XOH must be a scalar nonnegative integer. The nominal value of XOH is
%   one of {0, 6, 12, 18}, provided by the higher-layer parameter xOverhead
%   in PDSCH-ServingCellConfig IE or PUSCH-ServingCellConfig IE. Note that
%   the scaling factor is considered as 1.
%
%   TBS = nrTBS(MODULATION,NLAYERS,NPRB,NREPERPRB,TCR,XOH,TBSCALING) also
%   specifies the scaling factor(s) used in the calculation of intermediate
%   number of information bits, N_info, as defined in TS 38.214
%   Section 5.1.3.2. TBSCALING is a scalar for one codeword or a
%   two-element vector for two codewords, with each value must be greater
%   than 0 and less than or equal to 1. Alternatively, two codewords can
%   also be configured with single scaling factor. The nominal value of
%   TBSCALING is one of {0.25, 0.5, 1}, as defined in TS 38.214
%   Table 5.1.3.2-2.
%
%   Example 1:
%   % Determine the transport block size associated with a data
%   % transmission in a shared channel for one codeword. Specify the
%   % modulation scheme as '16QAM', number of transmission layers as 4,
%   % number of resource blocks allocated for the shared channel as 52,
%   % number of REs allocated for the shared channel within one PRB for one
%   % slot without accounting for the additional overhead as 120. Set the
%   % target code rate as 0.48, additional overhead as 6, and scaling
%   % factor as 0.25.
%
%   modulation = '16QAM';
%   nlayers = 4;
%   nPRB = 52;
%   NREPerPRB = 120;
%   tcr = 0.48;
%   xOh = 6;
%   tbScaling = 0.25;
%
%   % Calculate TBS
%   tbs = nrTBS(modulation,nlayers,nPRB,NREPerPRB,tcr,xOh,tbScaling)
%
%   Example 2:
%   % Determine the payload size of each transport block for a shared
%   % channel transmission with two codewords. Specify the modulation
%   % schemes as 'QPSK' and '64QAM', number of transmission layers as 8,
%   % number of resource blocks allocated for the shared channel as 106,
%   % number of REs allocated for shared channel within one PRB for one
%   % slot having no additional overhead as 100. Set the target code rates
%   % for two codewords as 0.3701 and 0.4277.
%
%   modulation = {'QPSK','64QAM'};
%   nlayers = 8;
%   nPRB = 106;
%   NREPerPRB = 100;
%   tcr = [0.3701 0.4277];
%
%   % Calculate TBS
%   tbs = nrTBS(modulation,nlayers,nPRB,NREPerPRB,tcr)
%
%   See also nrDLSCH, nrULSCH.

%   Copyright 2020 The MathWorks, Inc.

%#codegen

    narginchk(5,7);
    if nargin == 5
        % nrTBS(modulation,nlayers,nPRB,NREPerPRB,tcr)
        xOh = 0;       % Default value
        tbScaling = 1; % Default value
    elseif nargin == 6
        % nrTBS(modulation,nlayers,nPRB,NREPerPRB,tcr,xOh)
        xOh = varargin{1};
        tbScaling = 1; % Default value
    else
        % nrTBS(modulation,nlayers,nPRB,NREPerPRB,tcr,xOh,tbScaling)
        xOh = varargin{1};
        tbScaling = varargin{2};
    end

    % Validate the inputs
    [ncw,Qm,nLayers,R,S] = validateInputs(modulation,nlayers,nPRB,NREPerPRB,tcr,xOh,tbScaling);

    % Calculate the number of REs available for the data transmission in
    % the physical shared channel, within one PRB for one slot
    NREPrime = double(NREPerPRB) - double(xOh);

    % Output zero(s) for TBS, when no REs are available for the data
    % transmission
    if ~NREPerPRB || ~nPRB || NREPrime <= 0
        tbs = zeros(1,ncw);
        return;
    end

    % Calculate the total number of REs allocated for the physical shared
    % channel
    NRE = min(156,NREPrime)*double(nPRB);

    % Calculate the intermediate number of information bits (N_info)
    Ninfo = S.*NRE.*R.*Qm.*nLayers;

    % Calculate the transport block size(s) associated with a data
    % transmission in the physical shared channel with one or two codewords
    tbs = getTBS(Ninfo,R);

end

function [ncw,Qm,nLayers,R,S] = validateInputs(modulation,nlayers,nPRB,NREPerPRB,tcr,xOh,tbScaling)

    % Validate inputs
    fcnName = 'nrTBS';
    % modulation
    modlist = {'pi/2-BPSK', 'QPSK', '16QAM', '64QAM', '256QAM'};
    validateattributes(modulation,{'cell','char','string'},...
        {'nonempty'},fcnName,'MODULATION');
    numModSchemes = numel(modulation);
    if ischar(modulation) || (isstring(modulation) && numModSchemes == 1)
        % Character array or string scalar
        modscheme = validatestring(modulation,modlist,fcnName,'MODULATION');
        tempQm = nr5g.internal.getQm(modscheme);
    else
        % Cell array or string array

        % Convert string array to cell array
        tempVal = convertStringsToChars(modulation);
        validateattributes(numModSchemes,{'double'},{'scalar','<=',2},fcnName,'length of MODULATION');
        tempQm = zeros(numModSchemes,1);
        for idx = 1:numModSchemes
            modscheme = validatestring(tempVal{idx},modlist,fcnName,'MODULATION');
            tempQm(idx) = nr5g.internal.getQm(modscheme);
        end
    end

    % nlayers
    validateattributes(nlayers,{'numeric'},...
        {'scalar','integer','positive','<=',8},fcnName,'NLAYERS');

    % nPRB
    validateattributes(nPRB,{'numeric'},...
        {'scalar','integer','nonnegative'},fcnName,'NPRB');

    % NREPerPRB
    validateattributes(NREPerPRB,{'numeric'},...
        {'scalar','integer','nonnegative'},fcnName,'NREPERPRB');

    % tcr
    validateattributes(tcr,{'double','single'},...
        {'nonempty','real','>',0,'<',1},fcnName,'TCR');
    validateattributes(numel(tcr),{'double'},{'scalar','<=',2},fcnName,'length of TCR');

    % xOh
    validateattributes(xOh,{'numeric'},...
        {'scalar','integer','nonnegative'},fcnName,'XOH');

    % tbScaling
    validateattributes(tbScaling,...
        {'double','single'},{'nonempty','real','>',0,'<=',1},fcnName,'TBSCALING');
    validateattributes(numel(tbScaling),{'double'},{'scalar','<=',2},fcnName,'length of TBSCALING');

    % Get the number of codewords
    nlayersD = double(nlayers);
    ncw = ceil(nlayersD/4);

    % Number of layers for each codeword
    nLayers = floor((nlayersD + (0:ncw-1)')/ncw);

    % Apply scalar expansion for tempQm, tcr, and tbScaling
    Qm = applyScalarExpansion(tempQm,ncw);
    R = applyScalarExpansion(double(tcr(:)),ncw);
    S = applyScalarExpansion(double(tbScaling(:)),ncw);
end

function out = applyScalarExpansion(in,ncw)
% applyScalarExpansion Applies scalar expansion
%   OUT = applyScalarExpansion(IN,NCW) returns the output OUT of length
%   equals to the number of codewords NCW by repeating the input IN for NCW
%   times, if IN is a scalar.

    if numel(in) == 1
        out = repmat(in,ncw,1);
    else
        out = in(1:ncw);
    end
end

function tbs = getTBS(Ninfo,R)
% getTBS returns the calculated transport block size(s) for the shared channel
%   TBS = getTBS(NINFO,R) returns the TBS, as defined in TS 38.214
%   Section 5.1.3.2 and Section 6.1.4.2, with the intermediate number of
%   information bits, NINFO, and target code rate, R.

    % Get the number of codewords
    ncw = numel(Ninfo);

    % Initialize the output
    tbs = zeros(1,ncw);

    % Calculate transport block size, as per TS 38.214 Section 5.1.3.2
    for cwIdx = 1:ncw
        Ninfo_cw = Ninfo(cwIdx);
        R_cw = R(cwIdx);
        if Ninfo_cw <= 3824
            n = max(3,floor(log2(Ninfo_cw))-6);
            % Calculate quantized intermediate number of information bits
            % (Nd_info)
            NdInfo = max(24,(2^n)*floor(Ninfo_cw/(2^n)));
            % Get the TBS value using TS 38.214 Table 5.1.3.2-1
            tbs(cwIdx) = getTBSFromTable(NdInfo);
        else
            n = floor(log2(Ninfo_cw - 24)) - 5;
            % Calculate quantized intermediate number of information bits
            % (Nd_info)
            NdInfo = max(3840,(2^n)*round((Ninfo_cw - 24)/(2^n)));
            if R_cw <= 1/4
                C = ceil((NdInfo + 24)/3816);
            else
                if NdInfo > 8424
                    C = ceil((NdInfo + 24)/8424);
                else
                    C = 1;
                end
            end
            % Calculate TBS
            tbs(cwIdx) = 8*C*ceil((NdInfo + 24)/(8*C)) - 24;
        end
    end
end

function tbs = getTBSFromTable(NdInfo)
% getTBSFromTable returns the TBS value using the standard table
%   TBS = getTBSFromTable(NDINFO) returns the TBS value using TS 38.214
%   Table 5.1.3.2-1, with the quantized intermediate number of information
%   bits, NDINFO.

    % Capture TS 38.214 Table 5.1.3.2-1
    tbsTable = [  24;  32;  40;  48;  56;  64;  72;  80;  88;  96; 104; 112; 120; 128; 136; 144; 152; 160; 168; 176; 184; 192; 208; 224; 240; 256; 272; 288; 304; 320;...
                 336; 352; 368; 384; 408; 432; 456; 480; 504; 528; 552; 576; 608; 640; 672; 704; 736; 768; 808; 848; 888; 928; 984;1032;1064;1128;1160;1192;1224;1256;...
                1288;1320;1352;1416;1480;1544;1608;1672;1736;1800;1864;1928;2024;2088;2152;2216;2280;2408;2472;2536;2600;2664;2728;2792;2856;2976;3104;3240;3368;3496;...
                3624;3752;3824];

    % Find the closest TBS value that is not less than NdInfo
    tbsIndex = find(tbsTable >= NdInfo, 1);
    tbs = tbsTable(tbsIndex(1));
end
