classdef nrULSCH < matlab.System
%nrULSCH Uplink Shared Channel (UL-SCH)
%   ULSCH = nrULSCH creates an Uplink Shared Channel System object, ULSCH.
%   This object takes in a transport block and processes it through the
%   components of the uplink shared channel (CRC, code-block segmentation,
%   LDPC encoding, and rate matching).
%   This object implements the following aspects of 3GPP TR 38.212:
%   * Section 6.2 Uplink shared channel
%   ** Sections 6.2.1 to 6.2.6, *without* the 6.2.7 Data and control
%   multiplexing
%
%   ULSCH = nrULSCH(Name,Value) creates a UL-SCH object, ULSCH, with the
%   specified property Name set to the specified Value. You can specify
%   additional name-value pair arguments in any order as
%   (Name1,Value1,...,NameN,ValueN).
%
%   setTransportBlock method syntaxes
%
%   setTransportBlock(ULSCH,TRBLK), and
%   setTransportBlock(ULSCH,TRBLK,HARQID) load transport block TRBLK into
%   ULSCH for the specified HARQ process HARQID. Input HARQID is optional,
%   with a default value of 0. This method must be used prior to the step
%   method so that there is data to process. TRBLK must be a column vector
%   for the single transport block loaded per invocation.
%
%   Step method syntaxes
%
%   CODEDBITS = step(ULSCH,MODULATION,NLAYERS,OUTCWLEN,RV), and
%   CODEDBITS = step(ULSCH,MODULATION,NLAYERS,OUTCWLEN,RV,HARQID), both
%   apply the UL-SCH processing chain to the transport block that was
%   previously loaded using the setTransportBlock method. MODULATION is one
%   of {'pi/2-BPSK','QPSK','16QAM','64QAM','256QAM'} character arrays or
%   string specifying the modulation scheme. NLAYERS is a scalar between 1
%   and 4 specifying the number of transmission layers. OUTCWLEN is a
%   positive integer scalar that specifies the output codeword length in
%   bits. RV is an integer scalar between 0 and 3 to specify which
%   redundancy version is to be used with this transmission. HARQID is an
%   integer scalar between 0 and 15, specifying the ID of the HARQ process
%   used with the current transport block. HARQID input is enabled when
%   MultipleHARQProcesses property is true, else there is only one HARQ
%   process used.
%   CODEDBITS is a column vector of length OUTCWLEN and is the encoded,
%   rate-matched output.
%
%   Usage of the setTransportBlock method for loading new data makes it
%   natural for using this object with Hybrid ARQ processing. Once a
%   transport block is loaded, it will be retained for the specified HARQ
%   process until it is overwritten using the setTransportBlock method.
%
%   System objects may be called directly like a function instead of using
%   the step method. For example, y = step(obj,x) and y = obj(x) are
%   equivalent.
%
%   getTransportBlock method syntaxes
%
%   TRBLK = getTransportBlock(ULSCH), and
%   TRBLK = getTransportBlock(ULSCH,HARQID) both return the transport block
%   that was loaded into UL-SCH using the setTransportBlock method for the
%   specified HARQ process HARQID. Input HARQID is optional and defaults to
%   a value of 0 when not used.
%
%   nrULSCH methods:
%
%   setTransportBlock - Load next transport block into the UL-SCH for
%                       processing. Retransmission is facilitated by
%                       calling or not calling this method between
%                       successive step function calls.
%   step              - Encode the data contained in the internal buffer to
%                       produce the next codeword for subsequent PUSCH
%                       processing (see above)
%   release           - Allow property value and input characteristics
%                       changes
%   clone             - Create a UL-SCH encoder object with same property
%                       values
%   isLocked          - Locked status (logical)
%   getTransportBlock - Get the transport block that was loaded into the
%                       UL-SCH for processing.
%
%   nrULSCH properties:
%
%   MultipleHARQProcesses     - Enable multiple HARQ processes
%   TargetCodeRate            - Target code rate
%   LimitedBufferRateMatching - Enable limited buffer rate matching
%   LimitedBufferSize         - Limited buffer size (Nref)
%
%   Example 1:
%   % Use a UL-SCH Encoder and Decoder system object back to back.
%
%   targetCodeRate = 567/1024;
%   modulation = '64QAM';
%   nlayers = 1;
%   trBlkLen = 5120;
%   outCWLen = 10240;
%   rv = 0;
%
%   % Construct and configure encoder system object
%   encUL = nrULSCH;
%   encUL.MultipleHARQProcesses = false;
%   encUL.TargetCodeRate = targetCodeRate;
%
%   % Construct and configure decoder system object
%   decUL = nrULSCHDecoder;
%   decUL.MultipleHARQProcesses = false;
%   decUL.TargetCodeRate = targetCodeRate;
%   decUL.TransportBlockLength = trBlkLen;
%
%   % Construct random data and send it through encoder and decoder, back
%   % to back.
%   trBlk = randi([0 1],trBlkLen,1);
%   setTransportBlock(encUL,trBlk);
%   codedTrBlock = encUL(modulation,nlayers,outCWLen,rv);
%   rxSoftBits = 1.0 - 2.0*double(codedTrBlock);
%   [decbits,blkerr] = decUL(rxSoftBits,modulation,nlayers,rv);
%
%   % Check that the decoder output matches the original data
%   isequal(decbits,trBlk)
%
%   Example 2:
%   % Configure UL-SCH Encoder for use with multiple HARQ processes
%
%   encUL = nrULSCH;
%   encUL.MultipleHARQProcesses = true;
%   NDI = true;                         % New Data Indicator
%   harqID = 2;
%   rv = 3;
%   modulation = 'QPSK';
%   nlayers = 3;
%   trBlkLen = 5120;
%   outCWLen = 10002;
%
%   if NDI
%       trBlk = randi([0 1],trBlkLen,1);
%       % Latch new data into UL-SCH
%       setTransportBlock(encUL,trBlk,harqID);
%   end
%   codedTrBlock = encUL(modulation,nlayers,outCWLen,rv,harqID);
%
%   % Check that output has proper size
%   isequal(length(codedTrBlock),outCWLen)
%
%   See also nrULSCHDecoder, nrULSCHInfo, nrPUSCH.

%   Copyright 2018-2021 The MathWorks, Inc.

%#codegen

    % Public, non-tunable property
    properties (Nontunable)
        %MultipleHARQProcesses Enable multiple HARQ processes
        %   Enable multiple HARQ processes when set to true. A maximum of
        %   16 processes can be enabled. When set to false, a single
        %   process is used. In both cases, input data is buffered to
        %   enable retransmissions in case of failures. The default value
        %   of this property is false.
        MultipleHARQProcesses (1, 1) logical = false;
    end

    % Public, tunable properties
    properties
        %TargetCodeRate Target Code Rate
        %   Specify the target code rate as a numeric scalar value between
        %   0 and 1.0. The default value of this property is 526/1024.
        TargetCodeRate = 526 / 1024;
    end

    % Public, non-tunable logical property
    properties (Nontunable)
        %LimitedBufferRateMatching Enable limited buffer for rate matching
        %   If set to true, the size of the internal buffer used for rate
        %   matching can be specified by the LimitedBufferSize property. If
        %   set to false, the internal buffer size used is the full coded
        %   length. The default value of this property is false.
        LimitedBufferRateMatching (1, 1) logical = false;

        %LimitedBufferSize Limited buffer size
        %   Specify the size of the internal buffer used for rate matching
        %   per codeblock as a scalar, positive integer, when
        %   LimitedBufferRateMatching is set to true. The default value of
        %   this property is 25344 that corresponds to the maximum
        %   codeword length.
        LimitedBufferSize = 25344;
    end

    % Private state property
    properties(Access = protected)
        pTBdata = num2cell(zeros(16,1,'int8')); % Maximum HARQ processes = 16
    end

    % Public methods
    methods
        % Constructor
        function obj = nrULSCH(varargin)
            % Set property values from any name-value pairs input to the
            % constructor
            setProperties(obj,nargin,varargin{:});
        end

        function set.TargetCodeRate(obj,value)
            % real scalar, 0<r<1
            propName = 'TargetCodeRate';
            validateattributes(value, {'numeric'}, ...
                {'real','scalar','<',1,'>',0}, ...
                [class(obj) '.' propName], propName);

            obj.TargetCodeRate = value;
        end

        function set.LimitedBufferSize(obj,value)
            % scalar, integer > 0
            propName = 'LimitedBufferSize';
            validateattributes(value, {'numeric'}, ...
                {'scalar','integer','>',0}, ...
                [class(obj) '.' propName], propName);

            obj.LimitedBufferSize = value;
        end

    end

    methods(Access = protected)

        function num = getNumInputsImpl(obj)
            % Number of Inputs based on property for varargin in step
            num = 4 + double(obj.MultipleHARQProcesses);
        end

       function cwout = stepImpl(obj,modulation,nlayers,outCWLen,rv,varargin)
        % Supported syntaxes:
        %   cwout = step(obj,modulation,nlayers,outCWLen,rv)
        %   cwout = step(obj,modulation,nlayers,outCWLen,rv,harqID)

            narginchk(5,6);
            fcnName = [class(obj) '/step'];
            if nargin==5    % step(obj,modulation,nlayers,outCWLen,rv)
                harqID = 0;
            else        % step(obj,modulation,nlayers,outCWLen,rv,harqID)
                harqID = varargin{1};
                nr5g.internal.validateParameters('HARQID',harqID,fcnName);
            end

            % Validate inputs
            validateattributes(nlayers, {'numeric'}, ...
                {'scalar','integer','<=',4,'>=',1},fcnName,'NLAYERS');

            modlist = {'pi/2-BPSK','QPSK','16QAM','64QAM','256QAM'};
            modScheme = validatestring(modulation,modlist,fcnName,'MODULATION');

            validateattributes(outCWLen, {'numeric'}, ...
                {'scalar','integer','>=',0},fcnName,'OUTCWLEN');

            nr5g.internal.validateParameters('RV',rv,fcnName);

            % Empty in, empty out
            if isempty(obj.pTBdata{harqID+1})
                cwout = zeros(0,0,'int8');
            else
                % Process codeword
                cwout = ULSCHEncode(obj,modScheme,nlayers,outCWLen,rv,harqID);
            end
        end

        function codeword = ULSCHEncode(obj,modulation,nlayers,outCWLen,rv,harqID)
        % Encode ULSCH for a single codeword
        %   Code Block Group (CBG) transmissions are not supported yet

            % Get transport block
            trBlk = obj.pTBdata{harqID+1};

            % Create the informational output
            chinfo = nrULSCHInfo(length(trBlk),obj.TargetCodeRate);

            % Transport block CRC attachment
            crced = nrCRCEncode(trBlk,chinfo.CRC);

            % Code block segmentation and code block CRC attachment
            segmented = nrCodeBlockSegmentLDPC(crced,chinfo.BGN);

            % Channel coding
            encoded = nrLDPCEncode(segmented,chinfo.BGN);

            % Rate matching: with buffer limit enabled
            if obj.LimitedBufferRateMatching
                codeword = nrRateMatchLDPC(encoded,outCWLen,rv,modulation, ...
                    nlayers,obj.LimitedBufferSize);
            else % full buffer
                codeword = nrRateMatchLDPC(encoded,outCWLen,rv,modulation, ...
                    nlayers);
            end

        end

        function resetImpl(~)
            % Must be a no-op, if setTransportBlock method is used
            % as else the whole object gets reset, since setup calls reset.
        end

        function flag = isInactivePropertyImpl(obj,prop)
            flag = false;
            switch prop
                case 'LimitedBufferSize'
                    if ~obj.LimitedBufferRateMatching
                        flag = true;
                    end
            end
        end

        function s = saveObjectImpl(obj)
            s = saveObjectImpl@matlab.System(obj);
            if isLocked(obj)
                s.pTBdata  = obj.pTBdata;
            end
        end

        function loadObjectImpl(obj,s,wasLocked)
            if wasLocked
                obj.pTBdata  = s.pTBdata;
            end
            % Call the base class method
            loadObjectImpl@matlab.System(obj,s);
        end

    end

    methods % Public

        function setTransportBlock(obj,trBlk,varargin)
        %setTransportBlock Set UL-SCH transport block data
        %
        %   setTransportBlock(ULSCH,TRBLK), and
        %   setTransportBlock(ULSCH,TRBLK,HARQID) load transport block
        %   TRBLK into ULSCH for the specified HARQ process HARQID. HARQID
        %   defaults to a value of 0 when not specified. When
        %   MultipleHARQProcesses property is set to true, HARQID must be
        %   an integer in [0, 15].
        %
        %   This method must be used prior to the step method so that there
        %   is data to process. TRBLK must be a column vector for the
        %   single transport block loaded per invocation.

            fcnName = [class(obj) '/setTransportBlock'];
            if nargin > 2
                harqID = varargin{1};
                nr5g.internal.validateParameters('HARQID',harqID,fcnName);
            else
                harqID = 0;
            end

            validateattributes(trBlk,{'double','int8','logical'}, ...
                {'binary','column'},fcnName,'TRBLK');

            % Set data
            obj.pTBdata{harqID+1} = cast(trBlk,'int8');
        end

        function trBlk = getTransportBlock(obj,varargin)
        %getTransportBlock Get UL-SCH transport block data
        %
        %   TRBLK = getTransportBlock(ULSCH), and
        %   TRBLK = getTransportBlock(ULSCH,HARQID) both return the
        %   transport block that was loaded into UL-SCH using the
        %   setTransportBlock method for the specified HARQ process HARQID.
        %   HARQID defaults to a value of 0 when not specified and must be
        %   an integer in [0, 15] otherwise.

            if nargin > 1
                harqID = varargin{1};
                nr5g.internal.validateParameters('HARQID',harqID,...
                    [class(obj) '/getTransportBlock']);
            else
                harqID = 0;
            end

            % Get data
            trBlk = obj.pTBdata{harqID+1};
        end

    end % end methods

end % end classdef
