classdef nrULSCHDecoder < matlab.System
%nrULSCHDecoder Uplink Shared Channel (UL-SCH) Decoder
%   ULSCHDEC = nrULSCHDecoder creates an Uplink Shared Channel Decoder
%   System object, ULSCHDEC. This object takes demultiplexed data from
%   PUSCH and processes it through the components of the uplink shared
%   channel (UL-SCH) decoder (rate recovery, LDPC decoding, desegmentation,
%   and CRC decoding). It decodes signals that were encoded according to
%   3GPP TR 38.212:
%   * Section 6.2 Uplink shared channel
%   ** Sections 6.2.1 to 6.2.6, *without* the 6.2.7 Data and control
%   multiplexing
%
%   ULSCHDEC = nrULSCHDecoder(Name,Value) creates a UL-SCH decoder object,
%   ULSCHDEC, with the specified property Name set to the specified Value.
%   You can specify additional name-value pair arguments in any order as
%   (Name1,Value1,...,NameN,ValueN).
%
%   Step method syntaxes
%
%   TRBLKOUT = step(ULSCHDEC,RXSOFTBITS,MODULATION,NLAYERS,RV), and
%   TRBLKOUT = step(ULSCHDEC,RXSOFTBITS,MODULATION,NLAYERS,RV,HARQID),
%   both apply the UL-SCH decoding chain to the RXSOFTBITS input.
%   RXSOFTBITS is a column vector of received log-likelihood ratio (LLR)
%   values corresponding to the received codeword. MODULATION is a one of
%   {'pi/2-BPSK','QPSK','16QAM','64QAM','256QAM'} character arrays or
%   strings specifying the modulation scheme. NLAYERS is a scalar between 1
%   and 4 specifying the number of transmission layers. RV is an integer
%   scalar between 0 and 3 specifying which redundancy version was used
%   with this transmission. HARQID is an integer scalar between 0 and 15,
%   specifying the ID of the HARQ process used with the transport block.
%   HARQID input is enabled when MultipleHARQProcesses property is true,
%   else there is only one HARQ process in use.
%   The output TRBLKOUT is a column vector of length TransportBlockLength,
%   a public property of nrULSCHDecoder, representing the decoded bits.
%
%   The object uses soft buffer state retention to combine the different
%   redundancy version received codewords for an individual HARQ process.
%   When multiple processes are enabled, independent buffers per process
%   are maintained.
%
%   [TRBLKOUT,BLKERR] = step(ULSCHDEC,...) also returns an error flag
%   BLKERR to indicate if the transport block was decoded in error or not
%   (true indicates an error).
%
%   System objects may be called directly like a function instead of using
%   the step method. For example, y = step(obj,x) and y = obj(x) are
%   equivalent.
%
%   nrULSCHDecoder methods:
%
%   step            - Decode demultiplexed data from PUSCH output to
%                     retrieve the transport block (see above)
%   release         - Allow property value and input characteristics changes
%   clone           - Create a UL-SCH decoder object with same property values
%   isLocked        - Locked status (logical)
%   <a href="matlab:help nrULSCHDecoder/reset">reset</a>           - Reset buffers for all HARQ processes
%   resetSoftBuffer - Reset soft buffer for specified HARQ process
%
%   nrULSCHDecoder properties:
%
%   MultipleHARQProcesses     - Enable multiple HARQ processes
%   TargetCodeRate            - Target code rate
%   TransportBlockLength      - Length of decoded transport block (in bits)
%   LimitedBufferRateRecovery - Enable limited buffer rate recovery
%   LimitedBufferSize         - Limited buffer size (Nref)
%   LDPCDecodingAlgorithm     - LDPC decoding algorithm
%   ScalingFactor             - Scaling factor for normalized min-sum LDPC 
%                               decoding
%   Offset                    - Offset for offset min-sum LDPC decoding
%   MaximumLDPCIterationCount - Maximum number of LDPC decoding iterations
%
%   Example 1:
%   % Use a UL-SCH Encoder and Decoder system object back to back.
%
%   targetCodeRate = 602/1024;
%   modulation = 'QPSK';
%   nlayers = 2;
%   trBlkLen = 5120;
%   outCWLen = 8704;
%   rv = 0;
%
%   % Construct and configure encoder system object
%   encUL = nrULSCH;
%   encUL.MultipleHARQProcesses = false;
%   encUL.TargetCodeRate = targetCodeRate;
%
%   % Construct and configure decoder system object
%   decUL = nrULSCHDecoder;
%   decUL.MultipleHARQProcesses = false;
%   decUL.TargetCodeRate = targetCodeRate;
%   decUL.TransportBlockLength = trBlkLen;
%
%   % Construct random data and send it through encoder and decoder, back
%   % to back.
%   trBlk = randi([0 1],trBlkLen,1);
%   setTransportBlock(encUL,trBlk);
%   codedTrBlock = encUL(modulation,nlayers,outCWLen,rv);
%   rxSoftBits = 1.0 - 2.0*double(codedTrBlock);
%   [decbits,blkerr] = decUL(rxSoftBits,modulation,nlayers,rv);
%
%   % Check that the decoder output matches the original data
%   isequal(decbits,trBlk)
%
%   Example 2:
%   % Use UL-SCH Encoder and Decoder system objects with different
%   % transport block length processing
%
%   encUL = nrULSCH('MultipleHARQProcesses',true);
%   harqID = 1;
%   modSch = 'QPSK';
%   nlayers = 1;
%   rv = 0;
%
%   trBlkLen1 = 5120;
%   trBlk1 = randi([0 1],trBlkLen1,1,'int8');
%   setTransportBlock(encUL,trBlk1,harqID);
%   outCWLen1 = 10240;
%   codedTrBlock1 = step(encUL,modSch,nlayers,outCWLen1,rv,harqID);
%
%   decUL = nrULSCHDecoder('MultipleHARQProcesses',true);
%   decUL.TransportBlockLength = trBlkLen1;
%
%   rxBits1 = awgn(1-2*double(codedTrBlock1),5);
%   [decBits1,blkErr1] = step(decUL,rxBits1,modSch,nlayers,rv,harqID);
%
%   % Switch to a different transport block length for same HARQ process
%   trBlkLen2 = 4400;
%   trBlk2 = randi([0 1],trBlkLen2,1,'int8');
%   setTransportBlock(encUL,trBlk2,harqID);
%   outCWLen2 = 8800;
%   codedTrBlock2 = step(encUL,modSch,nlayers,outCWLen2,rv,harqID);
%
%   rxBits2 = awgn(1-2*double(codedTrBlock2),8);
%   decUL.TransportBlockLength = trBlkLen2;
%   if blkErr1
%       % Reset decoder if there was an error for first transport block
%       resetSoftBuffer(decUL,harqID);
%   end
%   [decBits2,blkErr2] = step(decUL,rxBits2,modSch,nlayers,rv,harqID);
%   [blkErr1 blkErr2]
%
%   See also nrULSCH, nrULSCHInfo, nrPUSCHDecode.

%   Copyright 2018-2021 The MathWorks, Inc.

%#codegen

    % Public, non-tunable property
    properties (Nontunable)
        %MultipleHARQProcesses Enable multiple HARQ processes
        %   Enable multiple HARQ processes when set to true. A maximum of
        %   16 processes can be enabled. When set to false, a single
        %   process is used. In both cases, for each process, the rate
        %   recovered input is buffered and combined with previous
        %   receptions of that process, before passing on for LDPC
        %   decoding. The default value of this property is false.
        MultipleHARQProcesses (1, 1) logical = false;
    end

    % Public, tunable properties
    properties
        %TargetCodeRate Target Code Rate
        %   Specify the target code rate as a numeric scalar value between
        %   0 and 1.0. The default value of this property is 526/1024.
        TargetCodeRate = 526 / 1024;
        %TransportBlockLength Size of transport block decoded output
        %   Specify the transport block size (in bits) to be output from
        %   the decoder. The default value of this property is 5120.
        TransportBlockLength = 5120;
    end

    % Public, non-tunable property
    properties (Nontunable)
        %LimitedBufferRateRecovery Enable limited buffer for rate recovery
        %   If set to true, the size of the internal buffer used for rate
        %   recovery can be specified by the LimitedBufferSize property. If
        %   set to false, the internal buffer size used is the full coded
        %   length. The default value of this property is false.
        LimitedBufferRateRecovery (1, 1) logical = false;

        %LimitedBufferSize Limited buffer size for rate recovery
        %   Specify the size of the internal buffer used for rate recovery
        %   as a scalar, positive integer, when LimitedBufferRateRecovery
        %   is set to true. The default value of this property is 25344
        %   that corresponds to the maximum codeword length.
        LimitedBufferSize = 25344;
        %LDPCDecodingAlgorithm LDPC decoding algorithm
        %   Specify the LDPC decoding algorithm as one of 'Belief propagation', 
        %   'Layered belief propagation', 'Normalized min-sum', or
        %   'Offset min-sum'. The default value of this property is
        %   'Belief propagation'.
        LDPCDecodingAlgorithm = 'Belief propagation';

        %ScalingFactor Scaling factor for normalized min-sum decoding
        %   Specify the scaling factor as a scalar real value greater than
        %   0 and less than or equal to 1. The default value of this
        %   property is 0.75. This property only applies when
        %   LDPCDecodingAlgorithm is set to 'Normalized min-sum'.
        ScalingFactor = 0.75;
        %Offset Offset for offset min-sum decoding
        %   Specify the offset as a scalar real value greater than
        %   or equal to 0. The default value of this property is 0.5. This
        %   property only applies when LDPCDecodingAlgorithm is set
        %   to 'Offset min-sum'.
        Offset = 0.5;

        %MaximumLDPCIterationCount Maximum LDPC decoding iterations
        %   Specify the maximum number of LDPC decoding iterations as a
        %   scalar positive integer. The default value of this property is
        %   12.
        MaximumLDPCIterationCount = 12;
    end

    % Private state property
    properties(Access = protected)
        pCWSoftBuffer = num2cell(zeros(16,1));  % Maximum HARQ processes = 16
    end

    % Supported values for LDPCDecodingAlgorithm
    properties (Constant, Hidden)
        LDPCDecodingAlgorithmSet = matlab.system.StringSet({ ...
            'Belief propagation','Layered belief propagation', ...
            'Normalized min-sum','Offset min-sum'});
    end
        
    % Public methods
    methods
        % Constructor
        function obj = nrULSCHDecoder(varargin)
            % Set property values from any name-value pairs input to the
            % constructor
            setProperties(obj,nargin,varargin{:});
        end

        function resetSoftBuffer(obj,varargin)
        %resetSoftBuffer Reset soft buffer for specified HARQ process
        %
        %   resetSoftBuffer(ULSCHDEC) and
        %   resetSoftBuffer(ULSCHDEC,HARQID) resets the internal soft
        %   buffer for the HARQ process specified by HARQID. HARQID
        %   defaults to a value of 0 when not specified. When
        %   MultipleHARQProcesses property is set to true, HARQID must be
        %   an integer in [0, 15].

            narginchk(1,2);
            if nargin>1
                harqID = varargin{1};
                nr5g.internal.validateParameters('HARQID',harqID, ...
                    [class(obj) '/resetSoftBuffer']);
            else
                harqID = 0;
            end

            info = nrULSCHInfo(obj.TransportBlockLength,obj.TargetCodeRate);
            obj.pCWSoftBuffer{harqID+1} = zeros(info.N,info.C);
        end

        function set.TargetCodeRate(obj,value)
            % real scalar, 0<r<1
            propName = 'TargetCodeRate';
            validateattributes(value, {'numeric'}, ...
                {'real','scalar','<',1,'>',0}, ...
                [class(obj) '.' propName], propName);

            obj.TargetCodeRate = value;
        end

        function set.TransportBlockLength(obj,value)
            % scalar, integer > 0
            propName = 'TransportBlockLength';
            validateattributes(value, {'numeric'}, ...
                {'scalar','integer','>',0}, ...
                [class(obj) '.' propName], propName);

            obj.TransportBlockLength = value;
        end

        function set.LimitedBufferSize(obj,value)
            % scalar, integer > 0
            propName = 'LimitedBufferSize';
            validateattributes(value, {'numeric'}, ...
                {'scalar','integer','>',0}, ...
                [class(obj) '.' propName], propName);

            obj.LimitedBufferSize = value;
        end
                
        function set.ScalingFactor(obj,value)
            % scalar, 0<x<=1
            propName = 'ScalingFactor';
            validateattributes(value, {'numeric'}, ...
                {'scalar','real','>',0,'<=',1}, ...
                [class(obj) '.' propName], propName);

            obj.ScalingFactor = value;
        end

        function set.Offset(obj,value)
            % scalar, >= 0
            propName = 'Offset';
            validateattributes(value, {'numeric'}, ...
                {'scalar','real','finite','>=',0}, ...
                [class(obj) '.' propName], propName);

            obj.Offset = value;
        end
        
        function set.MaximumLDPCIterationCount(obj,value)
            % scalar, integer > 0
            propName = 'MaximumLDPCIterationCount';
            validateattributes(value, {'numeric'}, ...
                {'scalar','integer','>',0}, ...
                [class(obj) '.' propName], propName);

            obj.MaximumLDPCIterationCount = value;
        end

    end

    methods(Access = protected)

        function num = getNumInputsImpl(obj)
            % Number of Inputs based on property for varargin in step
            num = 4 + double(obj.MultipleHARQProcesses);
        end

        function setupImpl(~)
            % Has to be defined, but is empty as only calls reset
        end

        function [rxBits,blkCRCErr] = stepImpl(obj,rxSoftBits,modulation, ...
                nlayers,rv,varargin)
        % Supported syntaxes:
        %   [rxBits,blkCRCErr] = step(obj,rxSoftBits,modulation,nlayers,rv)
        %   [rxBits,blkCRCErr] = step(obj,rxSoftBits,modulation,nlayers,rv,harqID)

            narginchk(5,6);
            fcnName = [class(obj) '/step'];
            if nargin==5    % step(obj,rxSoftBits,modulation,nlayers,rv)
                harqID = 0;
            else    % step(obj,rxSoftBits,modulation,nlayers,rv,harqID)
                harqID = varargin{1};
                nr5g.internal.validateParameters('HARQID',harqID,fcnName);
            end

            % Validate inputs
            validateattributes(nlayers, {'numeric'}, ...
                {'scalar','integer','<=',4,'>=',1},fcnName,'NLAYERS');

            modlist = {'pi/2-BPSK','QPSK','16QAM','64QAM','256QAM'};
            modScheme = validatestring(modulation,modlist,fcnName,'MODULATION');

            nr5g.internal.validateParameters('RV',rv,fcnName);

            if isempty(rxSoftBits)
                rxBits = zeros(size(rxSoftBits),'int8');
                blkCRCErr = false;  % no error
            else
                % Validate input
                validateattributes(rxSoftBits, {'double','single'}, ...
                {'real','column'},fcnName,'RXSOFTBITS');

                % Process codeword
                [rxBits,blkCRCErr] = ULSCHDecode(obj,rxSoftBits, ...
                    modScheme,nlayers,rv,harqID);
            end
        end

        function [rxBits,blkCRCErr] = ULSCHDecode(obj,rxSoftBits, ...
                modScheme,nlayers,rv,harqID)
            % Decode ULSCH codeword

            trBlkLen = obj.TransportBlockLength;
            targetCodeRate = obj.TargetCodeRate;

            info = nrULSCHInfo(trBlkLen,targetCodeRate);

            % Rate recovery, with limited buffer support
            ncb = info.C;
            if obj.LimitedBufferRateRecovery
                raterecovered = nrRateRecoverLDPC(rxSoftBits,trBlkLen, ...
                    targetCodeRate,rv,modScheme,nlayers,ncb, ...
                    obj.LimitedBufferSize);
            else % full size buffer
                raterecovered = nrRateRecoverLDPC(rxSoftBits,trBlkLen, ...
                    targetCodeRate,rv,modScheme,nlayers,ncb);
            end

            % Combining
            if isequal(size(obj.pCWSoftBuffer{harqID+1}),[info.N,ncb])
                raterecoveredD = double(raterecovered) + obj.pCWSoftBuffer{harqID+1};
            else
                raterecoveredD = double(raterecovered);
            end

            % LDPC decoding: set to early terminate, within max iterations
            decoded = nrLDPCDecode(raterecoveredD,info.BGN, ...
                obj.MaximumLDPCIterationCount,'Algorithm',obj.LDPCDecodingAlgorithm, ...
                'ScalingFactor',obj.ScalingFactor,'Offset',obj.Offset);

            % Code block desegmentation and code block CRC decoding
            desegmented = nrCodeBlockDesegmentLDPC(decoded,info.BGN,trBlkLen+info.L);

            % Transport block CRC decoding
            [rxBits,blkErr] = nrCRCDecode(desegmented,info.CRC);

            % Logic to reset in case no more RVs are available is not here.
            % Calling code would reset this object in that case.
            errflg = (blkErr ~= 0); % errored
            if errflg
                obj.pCWSoftBuffer{harqID+1} = raterecoveredD;
            else
                % Flush soft buffer on CRC pass
                obj.pCWSoftBuffer{harqID+1} = [];
            end
            blkCRCErr = errflg;

        end

        function resetImpl(obj)
            % Initialize/reset properties
            info = nrULSCHInfo(obj.TransportBlockLength,obj.TargetCodeRate);
            obj.pCWSoftBuffer{1} = zeros(info.N,info.C);
            if obj.MultipleHARQProcesses
                for i = 2:16
                    obj.pCWSoftBuffer{i} = zeros(info.N,info.C);
                end
            end
        end

        function processTunedPropertiesImpl(~)
            % Validate and Perform calculations if tunable properties
            % change while system is running
            % No operation, as do not want to reset all processes.
        end

        function flag = isInactivePropertyImpl(obj,prop)
            flag = false;
            switch prop
                case 'LimitedBufferSize'
                    if ~obj.LimitedBufferRateRecovery
                        flag = true;
                    end
                case 'ScalingFactor'
                    if ~strcmp(obj.LDPCDecodingAlgorithm,'Normalized min-sum')
                        flag = true;
                    end
                case 'Offset'
                    if ~strcmp(obj.LDPCDecodingAlgorithm,'Offset min-sum')
                        flag = true;
                    end
            end
        end

        function s = saveObjectImpl(obj)
            s = saveObjectImpl@matlab.System(obj);
            if isLocked(obj)
                s.pCWSoftBuffer  = obj.pCWSoftBuffer;
            end
        end

        function loadObjectImpl(obj,s,wasLocked)
            if wasLocked
                obj.pCWSoftBuffer  = s.pCWSoftBuffer;
            end
            % Call the base class method
            loadObjectImpl@matlab.System(obj,s);
        end

    end
end
