function [culsch,cack,ccsi1,ccsi2] = nrULSCHDemultiplex(pusch,tcr,tbs,oack,ocsi1,ocsi2,cw)
%nrULSCHDemultiplex Uplink shared channel data and control demultiplexing
%   [CULSCH,CACK,CCSI1,CCSI2] = nrULSCHDemultiplex(PUSCH,TCR,TBS,OACK,OCSI1,OCSI2,CW)
%   returns the demultiplexed encoded data vector CULSCH, encoded UCI vectors
%   (CACK, CCSI1, CCSI2), post the demultiplexing of received codeword CW,
%   to undo the processing described in TS 38.212 Section 6.2.7, for given
%   physical uplink shared channel configuration object PUSCH, target code
%   rate TCR, transport block size TBS, HARQ-ACK payload length OACK, CSI
%   part 1 payload length OCSI1, and CSI part 2 payload length OCSI 2. CW
%   is a column vector of received log-likelihood ratio (LLR) values.
%
%   The presence or absence of UCI type (HARQ-ACK, CSI part 1, CSI part 2)
%   in the transmission is signaled via OACK, OCSI1, OCSI2 inputs with a
%   non-zero or zero value, respectively. The length of CW must be equal to
%   the bit capacity of the configured physical uplink shared channel.
%
%   PUSCH is the physical uplink shared channel configuration object, as
%   described in <a href="matlab:help('nrPUSCHConfig')">nrPUSCHConfig</a> with properties:
%
%   Modulation         - Modulation scheme
%                        ('QPSK' (default), 'pi/2-BPSK', '16QAM', '64QAM', '256QAM')
%   NumLayers          - Number of transmission layers (1...4) (default 1)
%   MappingType        - Mapping type of physical uplink shared channel
%                        ('A' (default), 'B')
%   SymbolAllocation   - Symbol allocation of physical uplink shared
%                        channel (default [0 14]). This property is a
%                        two-element vector. First element represents the
%                        start of OFDM symbol in a slot. Second element
%                        represents the number of contiguous OFDM symbols
%   PRBSet             - PRBs allocated for physical uplink shared channel
%                        within a BWP (0-based) (default 0:51)
%   TransformPrecoding - Flag to enable transform precoding
%                        (0 (default), 1). 0 indicates that transform
%                        precoding is disabled, and the waveform type is
%                        CP-OFDM. 1 indicates that transform precoding is
%                        enabled, and waveform type is DFT-s-OFDM
%   FrequencyHopping   - Frequency hopping configuration
%                        ('neither' (default), 'intraSlot', 'interSlot')
%   BetaOffsetACK      - Beta offset of HARQ-ACK (default 20)
%   BetaOffsetCSI1     - Beta offset of CSI part 1 (default 6.25)
%   BetaOffsetCSI2     - Beta offset of CSI part 2 (default 6.25)
%   UCIScaling         - UCI scaling factor (default 1)
%   RNTI               - Radio network temporary identifier (0...65535)
%                        (default 1)
%   DMRS               - PUSCH-specific DM-RS configuration object, as
%                        described in <a href="matlab:help('nrPUSCHDMRSConfig')">nrPUSCHDMRSConfig</a> with properties:
%       DMRSConfigurationType   - DM-RS configuration type (1 (default), 2).
%                                 When transform precoding is enabled, the
%                                 value must be 1
%       DMRSTypeAPosition       - Position of first DM-RS OFDM symbol in a
%                                 slot (2 (default), 3)
%       DMRSLength              - Number of consecutive DM-RS OFDM symbols
%                                 (1 (default), 2). When intra-slot
%                                 frequency hopping is enabled, the value
%                                 must be 1. Value of 1 indicates
%                                 single-symbol DM-RS. Value of 2 indicates
%                                 double-symbol DM-RS
%       DMRSAdditionalPosition  - Maximum number of DM-RS additional
%                                 positions (0...3) (default 0). When
%                                 intra-slot frequency hopping is enabled,
%                                 the value must be either 0 or 1
%       DMRSPortSet             - DM-RS antenna port set (0...11)
%                                 (default []). The default value implies
%                                 that the values are in the range from 0
%                                 to NumLayers-1
%       CustomSymbolSet         - Custom DM-RS symbol locations (0-based)
%                                 (default []). This property is used to
%                                 override the standard defined DM-RS
%                                 symbol locations. Each entry corresponds
%                                 to a single-symbol DM-RS
%       NumCDMGroupsWithoutData - Number of CDM groups without data
%                                 (1...3) (default 2). When transform
%                                 precoding is enabled, the value must be 2
%   EnablePTRS         - Enable or disable the PT-RS configuration
%                        (0 (default), 1)
%   PTRS               - PUSCH-specific PT-RS configuration object, as
%                        described in <a href="matlab:help('nrPUSCHPTRSConfig')">nrPUSCHPTRSConfig</a> with properties:
%       TimeDensity            - PT-RS time density (1 (default), 2, 4)
%    These properties are applicable, when transform precoding is set to 0:
%       FrequencyDensity       - PT-RS frequency density (2 (default), 4)
%       REOffset               - Resource element offset
%                                ('00' (default), '01', '10', '11')
%       PTRSPortSet            - PT-RS antenna port set (default []). The
%                                default value of empty ([]) implies the
%                                value is equal to the lowest DM-RS antenna
%                                port configured
%    These properties are applicable, when transform precoding is set to 1:
%       NumPTRSSamples         - Number of PT-RS samples (2 (default), 4)
%       NumPTRSGroups          - Number of PT-RS groups (2 (default), 4, 8)
%
%   Example:
%   % Perform UCI multiplexing and demultiplexing on PUSCH in the presence
%   % of UL-SCH. Check the demultiplexed outputs.
%
%   % Set the PUSCH parameters
%   pusch = nrPUSCHConfig;
%   pusch.FrequencyHopping = 'neither';
%   pusch.BetaOffsetACK = 20;
%   pusch.BetaOffsetCSI1 = 6.25;
%   pusch.BetaOffsetCSI2 = 6.25;
%   pusch.UCIScaling = 1;
%
%   % Set the target code rate, payload lengths of UL-SCH data, HARQ-ACK,
%   % CSI part 1, and CSI part 2
%   tcr = 0.5;
%   oack = 8;
%   tbs = 3848;
%   ocsi1 = 88;
%   ocsi2 = 100;
%
%   % Get the rate matched lengths of data, HARQ-ACK, CSI part 1, and CSI
%   % part 2
%   rmInfo = nrULSCHInfo(pusch,tcr,tbs,oack,ocsi1,ocsi2);
%
%   % Create random coded UL-SCH, HARQ-ACK, CSI part 1, and CSI part 2
%   culsch = randi([0 1],rmInfo.GULSCH,1);
%   cack = randi([0 1],rmInfo.GACK,1);
%   ccsi1 = randi([0 1],rmInfo.GCSI1,1);
%   ccsi2 = randi([0 1],rmInfo.GCSI2,1);
%
%   % Get the codeword
%   cw = nrULSCHMultiplex(pusch,tcr,tbs,culsch,cack,ccsi1,ccsi2);
%
%   % Perform UCI and UL-SCH demultiplexing from codeword cw
%   [rxculsch,rxcack,rxccsi1,rxccsi2] = ...
%   nrULSCHDemultiplex(pusch,tcr,tbs,oack,ocsi1,ocsi2,1-2*cw);
%
%   % Check the demultiplexed types
%   isequal(rxculsch<0,culsch)
%   isequal(rxcack<0,cack)
%   isequal(rxccsi1<0,ccsi1)
%   isequal(rxccsi2<0,ccsi2)
%
%   See also nrULSCHMultiplex, nrULSCHInfo, nrPUSCHDecode,
%   nrRateRecoverLDPC, nrRateRecoverPolar, nrUCIDecode, nrULSCHDecoder.

%   Copyright 2020 The MathWorks, Inc.

%#codegen

    narginchk(7,7);

    % Validate inputs
    fcnName = 'nrULSCHDemultiplex';
    coder.internal.errorIf(~(isa(pusch,'nrPUSCHConfig') && isscalar(pusch)),...
        'nr5g:nrPUSCH:InvalidPUSCHInput');
    validateConfig(pusch);
    validateattributes(tcr,{'numeric'},{'real','scalar','>',0,'<',1},fcnName,'TCR');
    validateattributes(tbs,{'numeric'},{'scalar','integer','nonnegative'},fcnName,'TBS');
    validateattributes(oack,{'numeric'},{'scalar','integer','nonnegative'},fcnName,'OACK');
    validateattributes(ocsi1,{'numeric'},{'scalar','integer','nonnegative'},fcnName,'OCSI1');
    validateattributes(ocsi2,{'numeric'},{'scalar','integer','nonnegative'},fcnName,'OCSI2');
    if ~isempty(cw)
        validateattributes(cw,{'double','single'},{'column','real'},fcnName,'CW');
    else
        validateattributes(cw,{'double','single'},{'2d'},fcnName,'CW');
    end

    % Assign empty outputs, if codeword is empty
    cwLen = length(cw);
    typecw = class(cw);
    if cwLen
        % Get the rate-match and resource information
        [rmInfo,resInfo] = nr5g.internal.pusch.getULSCHInfo(...
            pusch,tcr,tbs,oack,ocsi1,ocsi2);
        resInfo.Modulation = pusch.Modulation;
        resInfo.NumLayers = pusch.NumLayers;
        resInfo.FrequencyHopping = pusch.FrequencyHopping;

        % Get the bit capacities of UL-SCH and CSI part 2
        GULSCH = rmInfo.GULSCH;
        GCSI2 = rmInfo.GCSI2;

        % Get the bit capacity of physical uplink shared channel
        G = sum(resInfo.MULSCH)*double(pusch.NumLayers)*nr5g.internal.getQm(pusch.Modulation);

        % Check the length of the codeword
        coder.internal.errorIf(cwLen ~= G,'nr5g:nrULSCHDemultiplex:InvalidCWLen',cwLen,G);

        % Get the locations of UL-SCH and UCI type(s) in the codeword
        [~,info] = nr5g.internal.pusch.dataAndControlMapping(...
            resInfo,rmInfo.GULSCH,rmInfo.GACK,rmInfo.GCSI1,rmInfo.GCSI2,rmInfo.GACKRvd);

        % Get the locations that are accessed within the codeword
        ulschInd = info.ULSCHIndices;
        csi1Ind = info.CSI1Indices;
        csi2Ind = info.CSI2Indices;
        ackInd = info.ACKIndices;

        % Demultiplex codeword
        % Get coded UL-SCH
        if GULSCH ~= length(ulschInd) && rmInfo.GACKRvd
            % When there is puncturing of UL-SCH bits with ACK bits, figure
            % out the locations punctured by HARQ-ACK and get the coded
            % UL-SCH bits accordingly
            culsch = zeros(GULSCH,1,typecw);
            sortULSCHInd = sort([ulschInd;info.ULSCHACKIndices]);
            ulschLogicalInd = ismember(sortULSCHInd,ulschInd);
            culsch(ulschLogicalInd) = cw(ulschInd);
        else
            culsch = cw(ulschInd);
        end

        % Get coded HARQ-ACK
        cack = cw(ackInd);

        % Get coded CSI part 1
        ccsi1 = cw(csi1Ind);

        % Get coded CSI part 2
        if GCSI2 ~= length(csi2Ind) && rmInfo.GACKRvd
            % When there is puncturing of CSI part 2 bits with ACK bits,
            % figure out the locations punctured by HARQ-ACK and get the
            % coded CSI part 2 bits accordingly
            ccsi2 = zeros(GCSI2,1,typecw);
            sortCSI2Ind = sort([csi2Ind;info.CSI2ACKIndices]);
            csi2LogicalInd = ismember(sortCSI2Ind,csi2Ind);
            ccsi2(csi2LogicalInd) = cw(csi2Ind);
        else
            ccsi2 = cw(csi2Ind);
        end
    else
        % Return empty outputs for empty codeword
        culsch = zeros(0,1,typecw);
        cack = zeros(0,1,typecw);
        ccsi1 = zeros(0,1,typecw);
        ccsi2 = zeros(0,1,typecw);
    end

end
