classdef nrWavegenBWPConfig  < nr5g.internal.nrNumerologyConfig
%nrWavegenBWPConfig Bandwidth part configuration
%  BWP = nrWavegenBWPConfig creates a bandwidth part configuration object
%  that lies within a specific SCS carrier. Use this object in the
%  BandwidthParts property of the <a href="matlab:help('nrDLCarrierConfig')"
%  >nrDLCarrierConfig</a> and <a href="matlab:help('nrULCarrierConfig')"
%  >nrULCarrierConfig</a> objects
%  to specify bandwidth parts of a 5G waveform generated by <a
%  href="matlab:help('nrWaveformGenerator')">nrWaveformGenerator</a>.
%  This object contains parameters defining the subcarrier spacing of the
%  carrier containing the bandwidth part, the size of the bandwidth part,
%  and the offset from common resource block 0 (CRB 0). For 60 kHz SCS, you
%  can specify either normal or extended cyclic prefix.
%
%  BWP = nrWavegenBWPConfig(Name,Value) creates a bandwidth part
%  configuration object BWP with the specified property Name set to the
%  specified Value. You can specify additional name-value arguments in any
%  order as (Name1,Value1,...,NameN,ValueN).
%
%  nrWavegenBWPConfig properties:
%
%  BandwidthPartID    - Numeric ID of this bandwidth part
%  Label              - Alphanumeric description for this bandwidth part
%  SubcarrierSpacing  - Subcarrier spacing in kHz
%                       (15 (default), 30, 60, 120, 240)
%  CyclicPrefix       - Cyclic prefix ('normal' (default), 'extended')
%  NSizeBWP           - Size of bandwidth path (1...275) (default 52)
%  NStartBWP          - Bandwidth part offset relative to CRB 0
%                       (0...2473) (default 0)
%    
%  Example 1: 
%  % Create a customized bandwidth part for the default SCS (15 kHz)
%
%  bwp = nrWavegenBWPConfig;
%  bwp.NSizeBWP = 50;
%  bwp.NStartBWP = 12;
%  cfgDL = nrDLCarrierConfig;
%  cfgDL.BandwidthParts = {bwp};
%
%  Example 2: 
%  % Create 2 SCS carriers and 2 bandwidth parts, in 15 & 30 kHz, and place
%  % them in a downlink carrier configuration object.
%
%  scs1 = nrSCSCarrierConfig; 
%  bwp1 = nrWavegenBWPConfig;
% 
%  scs2 = nrSCSCarrierConfig('SubcarrierSpacing', 30);
%  bwp2 = nrWavegenBWPConfig('SubcarrierSpacing', 30, 'NSizeBWP', 12, 'NStartBWP', 30);
%
%  cfgDL = nrDLCarrierConfig;
%  cfgDL.SCSCarriers = {scs1, scs2};
%  cfgDL.BandwidthParts = {bwp1, bwp2};
% 
%  See also nrDLCarrierConfig, nrULCarrierConfig, nrSCSCarrierConfig,
%  nrWaveformGenerator.

% Copyright 2020 The MathWorks, Inc.

%#codegen
    
  properties
    %BandwidthPartID Bandwidth part ID
    % Specify the banwidth part ID as a scalar nonnegative integer. The
    % default is 1.
    BandwidthPartID = 1;
    
    %Label Custom alphanumeric label
    % Specify Label as a character array or string scalar. Use this
    % property to assign a description to this bandwidth part object. The
    % default is 'BWP1'.
    Label = 'BWP1';
    
    %NSizeBWP Size of bandwidth part in resource blocks
    % Specify the number of resource blocks in the bandwidth part (BWP) as
    % a scalar positive integer. It must be in the range 1...275 and less
    % than or equal to the NSizeGrid value from the SCS carrier that is of
    % the same subcarrier spacing in its container configuration object
    % (nrDLCarrierConfig or nrULCarrierConfig) in which it is used. The
    % default is 52.
    NSizeBWP = 52;
    
    %NStartBWP Start of BWP resource grid relative to CRB 0
    % Specify the starting resource block of bandwidth part (BWP) relative
    % to common resource block 0 (CRB 0) as a nonnegative integer scalar
    % greater than or equal to the NStartGrid value from the SCS carrier
    % that is of the same subcarrier spacing in its container configuration
    % object (nrDLCarrierConfig or nrULCarrierConfig) in which it is used.
    % The default is 0.
    NStartBWP = 0;
  end
    
  properties (Hidden)
      CustomPropList = {'BandwidthPartID', 'Label', 'SubcarrierSpacing', ...
        'CyclicPrefix', 'NSizeBWP', 'NStartBWP'};     
  end
  
  methods
    function obj = nrWavegenBWPConfig(varargin)
      obj@nr5g.internal.nrNumerologyConfig(varargin{:});
    end
    
    % Self-validate and set properties
    function obj = set.BandwidthPartID(obj,val)
        prop = 'BandwidthPartID';
        validateattributes(val, {'numeric'}, ...
            {'scalar','integer','nonnegative'}, ...
            [class(obj) '.' prop], prop);

        obj.(prop) = val;
    end
       
    function obj = set.Label(obj,val)
        prop = 'Label';    
        validateattributes(val, {'char', 'string'}, {'scalartext'}, ...
            [class(obj) '.' prop], prop);        
        obj.(prop) = val;
    end
    
    function obj = set.NStartBWP(obj,val)
        prop = 'NStartBWP';
        validateattributes(val, {'numeric'}, ...
            {'scalar','integer','nonnegative','<=',2199+275-1}, ...
            [class(obj) '.' prop], prop);

        obj.(prop) = val;
    end

    function obj = set.NSizeBWP(obj,val)
        prop = 'NSizeBWP';
        validateattributes(val, {'numeric'}, ...
            {'scalar','integer','positive','<=',275}, ...
            [class(obj) '.' prop], prop);

        obj.(prop) = val;
    end
  end
end

