classdef nrWavegenPDCCHConfig < nr5g.internal.wavegen.EnablePower & ...
        nr5g.internal.nrPDCCHConfigBase & nr5g.internal.wavegen.DMRSPowerCommon & ...
        nr5g.internal.wavegen.DataSourceCommon & nr5g.internal.wavegen.PXYCHConfigBase
%nrWavegenPDCCHConfig PDCCH configuration for 5G waveform generation
%   CFGPDCCH = nrWavegenPDCCHConfig creates a physical downlink control
%   channel (PDCCH) configuration object. Use this object in a <a
%   href="matlab:help('nrDLCarrierConfig')">nrDLCarrierConfig</a>
%   object that describes a 5G downlink waveform generated by <a
%   href="matlab:help('nrWaveformGenerator')">nrWaveformGenerator</a>.
%   An nrWavegenPDCCHConfig object specifies the parameters for the PDCCH as
%   per TS 38.211 Section 7.3.2 and TS 38.213 Section 10.
%
%   CFGPDCCH = nrWavegenPDCCHConfig(Name,Value) creates a PDCCH configuration
%   object, CFGPDCCH, with the specified property Name set to the
%   specified Value. You can specify additional name-value pair
%   arguments in any order as (Name1,Value1,...,NameN,ValueN).
%
%   nrWavegenPDCCHConfig properties:
%
%   Enable                - Flag turning this channel on or off (default true)
%   Label                 - Alphanumeric description for this PDCCH (default 'PDCCH1')
%   Power                 - Power scaling in dB (default 0)
%   BandwidthPartID       - ID of bandwidth part containing this PDCCH (default 1)
%   SearchSpaceID         - ID of search space containing this PDCCH
%   AggregationLevel      - PDCCH aggregation level {1, 2, 4, 8, 16}
%   AllocatedCandidate    - Candidate used for the PDCCH instance
%   CCEOffset             - Explicit CCE offset (overrides AllocatedCandidate)
%   SlotAllocation        - Time-domain location of PDCCH (in slots) (default 0)
%   Period                - Slot allocation period of PDCCH (default 1)
%   Coding                - Flag to enable DCI coding (default true)
%   DataBlockSize         - Payload size of DCI (1...140) (default 20)
%   DataSource            - Source of DCI contents (pseudo-noise (PN) or custom)
%   RNTI                  - Radio network temporary identifier
%   DMRSScramblingID      - PDCCH DM-RS scrambling identity
%   DMRSPower             - Scaling of DM-RS power in dB (default 0)
%
%   Example 1:
%   %  Create a custom nrWavegenPDCCHConfig object; link it to search space and CORESET.
%
%   coreset = nrCORESETConfig;
%   searchSpace = nrSearchSpaceConfig('CORESETID', coreset.CORESETID);
%   pdcch = nrWavegenPDCCHConfig('SearchSpaceID', searchSpace.SearchSpaceID, ...
%                    'AggregationLevel', 4, 'AllocatedCandidate', 2, ...
%                    'SlotAllocation', [0 2], 'Period', 3);
% 
%   cfg = nrDLCarrierConfig;
%   cfg.CORESET = {coreset};
%   cfg.SearchSpaces = {searchSpace};
%   cfg.PDCCH = {pdcch};
%
%   Example 2:
%   %  Create 2 CORESET used by 3 search spaces and by 4 PDCCH.
%
%   coreset1 = nrCORESETConfig('CORESETID', 1);
%   coreset2 = nrCORESETConfig('CORESETID', 2);
%   searchSpace1 = nrSearchSpaceConfig('SearchSpaceID', 1, 'CORESETID', coreset1.CORESETID);
%   searchSpace2 = nrSearchSpaceConfig('SearchSpaceID', 2, 'CORESETID', coreset1.CORESETID);
%   searchSpace3 = nrSearchSpaceConfig('SearchSpaceID', 3, 'CORESETID', coreset2.CORESETID);
%   pdcch1 = nrWavegenPDCCHConfig('RNTI', 1, 'SearchSpaceID', searchSpace1.SearchSpaceID);
%   pdcch2 = nrWavegenPDCCHConfig('RNTI', 2, 'SearchSpaceID', searchSpace2.SearchSpaceID);
%   pdcch3 = nrWavegenPDCCHConfig('RNTI', 3, 'SearchSpaceID', searchSpace2.SearchSpaceID);
%   pdcch4 = nrWavegenPDCCHConfig('RNTI', 4, 'SearchSpaceID', searchSpace3.SearchSpaceID);
% 
%   cfg = nrDLCarrierConfig;
%   cfg.CORESET = {coreset1, coreset2};
%   cfg.SearchSpaces = {searchSpace1, searchSpace2, searchSpace3};
%   cfg.PDCCH = {pdcch1, pdcch2, pdcch3, pdcch4};
%
%   See also nrDLCarrierConfig, nrSearchSpaceConfig, nrCORESETConfig,
%   nrWaveformGenerator.

%   Copyright 2020-2021 The MathWorks, Inc.

%#codegen

  properties
    %SearchSpaceID ID of search space containing this PDCCH
    % Specify SearchSpaceID as a nonnegative integer scalar to link this
    % PDCCH instance with one of the search spaces defined in the
    % SearchSpaces property of <a href="matlab:help('nrDLCarrierConfig.SearchSpaces')"
    % >nrDLCarrierConfig</a>. The default is 1.
    SearchSpaceID = 1;

    %Coding Flag to enable channel coding
    % Specify Coding as a scalar logical. Setting Coding to true enables 
    % channel coding for the downlink control information (DCI). The default is true.
    Coding = true;
    
    %DataBlockSize Payload size of DCI
    % Specify DataBlockSize in bits as a positive integer scalar no greater
    % than 140. This property specifies the length of the Downlink Control
    % Information (DCI). The default is 20 (bits).
    DataBlockSize = 20;
    
    %DataSource Source of DCI contents
    % Specify DataSource as one of {'PN9-ITU', 'PN9', 'PN11', 'PN15',
    % 'PN23'}, as a cell array containing one of the abovementioned
    % options and a numeric scalar that is the random seed (for example,
    % {'PN9',7}), or as a binary vector. If the seed is not specified, then
    % all shift registers are initialized with an active state. The default
    % is 'PN9-ITU'.
    DataSource = 'PN9-ITU';  
    
    %RNTI Radio network temporary identifier
    % Specify the RNTI as a scalar nonnegative integer no greater than
    % 65535. When this value is 0 or greater than 65519 and the type of the
    % associated search space is 'common', then the nRNTI value is inferred
    % to be 0. The default is 1.
    RNTI = 1;
  end

  % Hidden properties
  properties (Hidden)
      % Custom property list to change the order of display properties
      CustomPropList = {'Enable', 'Label', 'Power', 'BandwidthPartID', 'SearchSpaceID', ...
                        'AggregationLevel', 'AllocatedCandidate', 'CCEOffset', 'SlotAllocation', 'Period', ...
                        'Coding', 'DataBlockSize', 'DataSource', ...
                        'RNTI', 'DMRSScramblingID', 'DMRSPower'};
  end
  
  % Constant, hidden properties
  properties (Constant,Hidden)
    DataSource_Values = nr5g.internal.wavegen.DataSourceCommon.DataSource_Options;
  end

  methods
    function obj = nrWavegenPDCCHConfig (varargin)
      obj@nr5g.internal.nrPDCCHConfigBase(...
                  'Label', 'PDCCH1', ...
                  'SlotAllocation', 0, ...
                  'Period', 1, ...
                  varargin{:});
    end

    % Self-validate and set properties
    function obj = set.SearchSpaceID(obj,val)
        prop = 'SearchSpaceID';
        validateattributes(val, {'numeric'}, ...
            {'integer','scalar','nonnegative'}, ...
            [class(obj) '.' prop], prop);

        obj.(prop) = val;
    end

    function obj = set.Coding(obj,val)
        prop = 'Coding';
        validateattributes(val, {'logical', 'numeric'}, {'scalar', 'nonnan'}, ...
            [class(obj) '.' prop], prop);
        obj.(prop) = logical(val);
    end

    function obj = set.DataBlockSize(obj,val)
        prop = 'DataBlockSize';
        validateattributes(val, {'numeric'}, ...
            {'integer','scalar','positive', '<=', 140}, ...
            [class(obj) '.' prop], prop);
        obj.(prop) = val;
    end

    function obj = set.DataSource(obj,val)
      prop = 'DataSource';
      temp = getDataSource(obj, prop, val);
      obj.(prop) = temp;
    end

    function obj = set.RNTI(obj,val)
      prop = 'RNTI';
      validateattributes(val, {'numeric'}, ...
          {'scalar','integer','nonnegative','<=',65535},...
          [class(obj) '.' prop], prop);

      obj.(prop) = val;
    end

    function validateConfig(obj, coreset, searchspace, bwpStart, bwpSize)
      % validateConfig Validate the nrWavegenPDCCHConfig object
      %   Cross-checks among properties and individual object
      %   properties
      %
      %   For INTERNAL use only.            

      % CORESET, SearchSpaces validated by nrDLCarrierConfig

      mode = 'full';
      baseValidate(obj, mode, coreset, searchspace, bwpStart, bwpSize);
    end
  end


  methods (Access = protected)

    function flag = isInactiveProperty(obj, prop)
      % Controls the conditional display of properties

      flag = isInactiveProperty@nr5g.internal.nrPDCCHConfigBase(obj, prop);

      if strcmp(prop,'DataBlockSize')
          flag = ~obj.Coding;
      end
    end
  end
end
