classdef nrWavegenSRSConfig < nr5g.internal.wavegen.EnablePower & nr5g.internal.nrSRSConfigBase
    %nrWavegenSRSConfig SRS configuration object for 5G waveform generation
    %   CFGSRS = nrWavegenSRSConfig creates a sounding reference signal (SRS)
    %   configuration object. Use this object in a <a
    %   href="matlab:help('nrULCarrierConfig')">nrULCarrierConfig</a>
    %   object that describes a 5G uplink waveform generated by <a
    %   href="matlab:help('nrWaveformGenerator')">nrWaveformGenerator</a>.
    %
    %   The default nrWavegenSRSConfig object represents a single-port,
    %   single-symbol configuration that places the SRS at the end of the
    %   slot. This configuration is narrowband and without frequency
    %   hopping (BHop>=BSRS).
    %
    %   CFGSRS = nrWavegenSRSConfig(Name,Value) creates an SRS configuration
    %   object, CFGSRS, with the specified property Name set to the
    %   specified Value. You can specify additional name-value pair
    %   arguments in any order as (Name1,Value1,...,NameN,ValueN).
    %
    %   nrWavegenSRSConfig properties:
    %
    %   Enable            - Flag turning this SRS on or off
    %   Label             - Alphanumeric description for this SRS
    %   Power             - Power scaling in dB
    %   BandwidthPartID   - ID of bandwidth part containing this SRS
    %   NumSRSPorts       - Number of SRS antenna ports (1(default), 2, 4)
    %   NumSRSSymbols     - Number of consecutive OFDM symbols allocated
    %                       to the SRS (1(default), 2, 4, 8, 12)
    %   SymbolStart       - Index of the first SRS symbol in a slot (default 13))
    %   SlotAllocation    - Time-domain location of SRS (in slots) (default 0:9)
    %   Period            - Period of slot allocation (default 10)
    %   KTC               - Transmission comb number (2(default), 4, 8)
    %                       The SRS is transmitted every KTC subcarriers
    %   KBarTC            - Transmission comb offset in subcarriers
    %                       (0...KTC-1) (default 0)
    %   CyclicShift       - Cyclic shift number offset NCS (default 0)
    %   FrequencyStart    - Frequency-domain starting position of the SRS
    %                       in physical resource blocks (PRBs) relative to
    %                       the carrier resource grid (0...271) (default 0)
    %   NRRC              - Frequency domain position. Additional frequency
    %                       offset to FrequencyStart specified in blocks of
    %                       4 RBs (0...67) (default 0)
    %   CSRS              - Bandwidth configuration index C_SRS (0...63) (default 0)
    %   BSRS              - Bandwidth configuration index B_SRS (0...3) (default 0)
    %   BHop              - Frequency hopping index B_hop (0...3) (default 0)
    %   Repetition        - Repetition factor (1(default), 2, 4, 8, 12)
    %   GroupSeqHopping   - Group or sequence number hopping
    %                       ('neither'(default), 'groupHopping', 'sequenceHopping')
    %   NSRSID            - Scrambling identity (0...65535) (default 0)
    %   SRSPositioning    - Enable SRS for user positioning (true, false(default))
    %
    %   Example 1:
    %   % Create nrWavegenSRSConfig object with its default properties.
    %
    %   srs = nrWavegenSRSConfig
    %
    %   Example 2:
    %   % Create a custom nrWavegenSRSConfig object with 4 OFDM symbols and
    %   % frequency hopping enabled, pass it to nrULCarrierConfig.
    %
    %   srs = nrWavegenSRSConfig('NumSRSSymbols',4,'SymbolStart',8,'CSRS',14,'BSRS',2,'BHop',1);
    %   cfg = nrULCarrierConfig;
    %   cfg.SRS = {srs};
    %
    %   See also nrULCarrierConfig, nrWaveformGenerator.
    
    %   Copyright 2020-2021 The MathWorks, Inc.
    
    %#codegen
    properties
        %Label Custom alphanumeric label
        % Specify Label as a character array or string scalar. Use this
        % property to assign a description to this SRS object. The default
        % is 'SRS1'.
        Label = 'SRS1';
        
        %BandwidthPartID ID of bandwidth part containing this SRS
        % Specify BandwidthPartID as a nonnegative integer scalar to link
        % this SRS instance with one of the bandwidth parts defined in the
        % BandwidthParts property of <a href="matlab:help('nrULCarrierConfig.BandwidthParts')"
        % >nrULCarrierConfig</a>. The default is 1.
        BandwidthPartID = 1;
        
        %SlotAllocation Allocation of slots within a period
        % Specify SlotAllocation as a scalar or row vector of nonnegative
        % integers to specify the slot position of the SRS. Indexing is
        % 0-based. The values in SlotAllocation must be smaller than the
        % value of the Period property, otherwise they are ignored. Each
        % element of the vector corresponds to an SRS resource. The default
        % is 0:9.
        SlotAllocation = 0:9;
        
        %Period Allocation period of the SRS in slots
        % Specify the period of the SRS in slots as a positive integer
        % scalar or []. An empty value indicates no repetition (SRS in a
        % single slot only) and makes the SRS resource type aperiodic, as
        % specified in TS 38.211 Section 6.4.1.4.3. The default is 10 (slots).
        Period = 10;
    end
    
    properties (Hidden)
    CustomPropList = {'Enable'; 'Label'; 'Power'; 'BandwidthPartID'; ...
        'NumSRSPorts'; 'NumSRSSymbols'; 'SymbolStart'; 'SlotAllocation'; 'Period'; ...
        'KTC'; 'KBarTC'; 'CyclicShift'; 'FrequencyStart'; 'NRRC'; 'CSRS'; 'BSRS'; 'BHop'; ...
        'Repetition'; 'GroupSeqHopping'; 'NSRSID'; 'SRSPositioning'};
    end
    
    methods
        % Constructor
        function obj = nrWavegenSRSConfig(varargin)
            obj@nr5g.internal.nrSRSConfigBase(varargin{:});
        end
        
        % Set properties
        function obj = set.Label(obj,val)
            prop = 'Label';
            validateattributes(val, {'char', 'string'}, {'scalartext'}, ...
                [class(obj) '.' prop], prop);
            obj.(prop) = convertStringsToChars(val);
        end
        
        function obj = set.BandwidthPartID(obj,val)
            prop = 'BandwidthPartID';
            validateattributes(val, {'numeric'}, ...
                {'scalar','integer','nonnegative', 'real'}, ...
                [class(obj) '.' prop], prop);
            
            obj.(prop) = val;
        end
        
        function obj = set.SlotAllocation(obj,val)
            prop = 'SlotAllocation';
            % To allow codegen for varying length in a single function script
            temp = val;
            coder.varsize('temp',[Inf Inf],[1 1]);
            if ~(isnumeric(temp) && isempty(temp))
                validateattributes(temp,{'numeric'},{'vector','integer','nonnegative'},[class(obj) '.' prop],prop);
            end
            obj.(prop) = temp;
        end
        
        function obj = set.Period(obj,val)
            prop = 'Period';
            % To allow codegen for varying length in a single function script
            temp = val;
            coder.varsize('temp',[1 1],[1 1]);
            if ~(isnumeric(temp) && isempty(temp))
                validateattributes(temp,{'numeric'},{'scalar','integer','positive'},[class(obj) '.' prop],prop);
            end
            obj.(prop) = temp;
        end
    end
end