classdef nrWavegenSSBurstConfig < nr5g.internal.wavegen.EnablePower & nr5g.internal.ConfigBase
%nrWavegenSSBurstConfig SS Burst configuration for 5G waveform generation
%   CFGSSB = nrWavegenSSBurstConfig creates a synchronization signal (SS)
%   burst configuration object that describes the subcarrier spacing of the
%   SS burst, its placement in time and frequency, as well as its power and
%   payload. Use this object in the SSBurst property of the <a
%   href="matlab:help('nrDLCarrierConfig')">nrDLCarrierConfig</a>
%   object, which configures a 5G downlink waveform generated by 
%   <a href="matlab:help('nrWaveformGenerator')">nrWaveformGenerator</a>.
%
%   By default, the nrWavegenSSBurstConfig object configures the SS burst
%   to carry the master information block (MIB). This SS burst is formed by
%   four active SS blocks and has a periodicity of 20 ms, which is the
%   default value for initial cell selection. The default object represents
%   an SS burst in the center of a carrier with 15 kHz subcarrier spacing
%   (block pattern Case A). To change the frequency location of the SS
%   burst, set NCRBSSB and KSSB to the desired nonempty value.
%
%   CFGSSB = nrWavegenSSBurstConfig(Name,Value) creates an SS Burst
%   configuration object, CFGSSB, with the specified property Name set to
%   the specified Value. You can specify additional name-value pair
%   arguments in any order as (Name1,Value1,...,NameN,ValueN).
%
%   nrWavegenSSBurstConfig properties:
%
%   Enable                  - Flag turning SS burst on or off (true (default), false)
%   Power                   - Power scaling in dB (default 0)
%   BlockPattern            - SS block pattern 
%                             ('Case A' (default), 'Case B', 'Case C', 'Case D', 'Case E')
%   TransmittedBlocks       - Bitmap for block transmission in 5 ms half-frame (default [1 1 1 1])
%   Period                  - Period of SS burst in ms 
%                             (5, 10, 20 (default), 40, 80, 160)
%   NCRBSSB                 - RB offset from Point A (0...2199) (default [])
%   KSSB                    - Subcarrier offset (0...23) (default 0)
%   DataSource              - Source of SS burst payload (default 'MIB')
%   DMRSTypeAPosition       - First DM-RS symbol position for SIB1 PDSCH (2 (default), 3)
%   CellBarred              - Flag for cell barring (true, false (default))
%   IntraFreqReselection    - Flag for intra-frequency reselection (true, false (default))
%   PDCCHConfigSIB1         - Configuration type of SIB1 PDCCH (0...255) (default 0)
%   SubcarrierSpacingCommon - SIB1 subcarrier spacing in kHz (15 (default), 30, 60, 120)
%
%   Example 1:
%   % Create a default object specifying an SS burst at a 15 kHz SCS
%
%   ssb = nrWavegenSSBurstConfig;
%   cfgDL = nrDLCarrierConfig;
%   cfgDL.SSBurst = ssb;
%
%   Example 2: 
%   % Create an SS burst configuration object at a 120 kHz SCS, with a
%   % custom frequency offset
%
%   ssb = nrWavegenSSBurstConfig;
%   ssb.BlockPattern = 'Case D';
%   ssb.TransmittedBlocks = ones(1, 64);
%
%   cfgDL = nrDLCarrierConfig('FrequencyRange', 'FR2', 'ChannelBandwidth', 100);
%   cfgDL.SCSCarriers{1} = nrSCSCarrierConfig('SubcarrierSpacing', 120);
%   cfgDL.BandwidthParts{1} = nrWavegenBWPConfig('SubcarrierSpacing', 120);
%
%   % Custom frequency offset at a third of the carrier size:
%   ssb.NCRBSSB = round(cfgDL.SCSCarriers{1}.NSizeGrid/3);
%   ssb.KSSB = 2*4; % must be even for Case B, D; multiple of 4 for Case E
%
%   cfgDL.SSBurst = ssb;
% 
% See also nrDLCarrierConfig, nrWaveformGenerator

%   Copyright 2020 The MathWorks, Inc.
  
%#codegen

  properties    
    %BlockPattern SS block pattern
    % Specify BlockPattern as one of {'Case A', 'Case B', 'Case
    % C', 'Case D', 'Case E'}, as per TS 38.213 Section 4.1. Cases A, B,
    % and C correspond to Frequency Range 1 (FR1) and to a subcarrier spacing
    % equal to 15 kHz, 30 kHz and 30 kHz respectively. Cases D, and E,
    % correspond to Frequency Range 2 (FR2) and to a subcarrier spacing
    % equal to 120 kHz and 240 kHz respectively. The default is 'Case A'.
    BlockPattern = 'Case A';
    
    %TransmittedBlocks Bitmap indicating block transmission in a 5 ms half-frame burst
    % Specify TransmittedBlocks as a 4- or 8-bit binary vector for
    % Frequency Range 1 (FR1) and as a 64-bit binary vector for Frequency
    % Range 2 (FR2). This binary vector indicates which SS blocks are
    % activated within the SS burst. The default is [1 1 1 1].
    TransmittedBlocks = [1 1 1 1];
    
    %Period  Period of SS burst 
    % Specify the period of the SS burst in milliseconds as one of {5, 10,
    % 20, 40, 80, 160}. The default is 20.
    Period = 20;
            
    %NCRBSSB RB offset from Point A
    % Specify NCRBSSB as a scalar nonnegative integer no greater than 2199
    % or as an empty vector ([]). This property specifies the frequency
    % offset of the SS burst in resource blocks (RB) relative to Point A.
    % For block patterns Case A, B and C, the NCRBSSB units are expressed
    % in terms of 15 kHz subcarrier spacing; for Case D and E, units are
    % expressed in terms of 60 kHz subcarrier spacing. When NCRBSSB is
    % empty ([]), the SS burst is automatically placed in the center of the
    % carrier whose subcarrier spacing is denoted by BlockPattern. The
    % default is [].
    NCRBSSB = [];
    
    %KSSB Subcarrier offset
    % Specify KSSB as a scalar nonnegative integer. This property further
    % increases the frequency offset of the SS burst from Point A by KSSB
    % subcarriers. For frequency range 1 (corresponding to BlockPattern
    % Case A, B, or C) and when SubcarrierSpacingCommon is 15, the KSSB
    % range is [0 11] and units are expressed in terms of 15 kHz subcarrier
    % spacing. For frequency range 1 and when SubcarrierSpacingCommon is
    % 30, the KSSB range is [0 23] and units are expressed in terms of 15
    % kHz subcarrier spacing. For frequency range 2 (corresponding to
    % BlockPattern Case E or D) the KSSB range is [0 11] and units are
    % expressed in terms of subcarrier spacing equal to
    % SubcarrierSpacingCommon. Additionally, KSSB must be even for cases B
    % and D, while it must be a multiple of 4 for case E. This property
    % applies when NCRBSSB is not empty ([]), which corresponds to a custom
    % frequency offset. The default is 0.
    KSSB = 0;

    %DataSource Source of SS burst payload
    % Specify DataSource as one of {'MIB', 'PN9-ITU', 'PN9', 'PN11',
    % 'PN15', 'PN23'}, as a cell array containing one of the abovementioned
    % options and a numeric scalar that is the random seed (e.g.,
    % {'PN9',7}), or as a 24-bit binary vector. If the seed is not
    % specified, then all shift registers are initialized with an active
    % state. The default is 'MIB'.
    DataSource = 'MIB';
    
    %DMRSTypeAPosition First DM-RS symbol position for SIB1 PDSCH 
    % Specify the first DMRS symbol position in the System Information
    % Block Type 1 (SIB1) of the PDSCH as 2 or 3. The default is 2. This
    % property applies when DataSource is set to 'MIB'.
    DMRSTypeAPosition = 2;
    
    %CellBarred Flag for cell barring
    % Specify CellBarred as a scalar logical. A true value indicates that
    % the cell allows UEs to camp on this cell. The default is false. This
    % property applies when DataSource is set to 'MIB'.
    CellBarred = false;
    
    %IntraFreqReselection Flag for intra-frequency reselection
    % Specify IntraFreqReselection as a scalar logical. A true value
    % allows reselection of same-frequency cells. The default is false.
    % This property applies when DataSource is set to 'MIB'.
    IntraFreqReselection = false;
    
    %PDCCHConfigSIB1 Configuration of SIB1 PDCCH
    % Specify the configuration type of the System Information Block Type 1
    % (SIB1) as a scalar integer in the range 0...255. The default is 0.
    % This property applies when DataSource is set to 'MIB'.
    PDCCHConfigSIB1 = 0;
    
    %SubcarrierSpacingCommon SIB1 subcarrier spacing in kHz
    % Specify the subcarrier spacing of the System Information Block Type 1
    % (SIB1) in kHz as a numeric scalar that is either 15 or 30 for
    % frequency range 1 (FR1) and either 60 or 120 for frequency range 2
    % (FR2). The default is 15. This property applies when DataSource is
    % set to 'MIB' or when BlockPattern is Case D or E (FR2) and NCRBSSB is
    % not empty ([]), which corresponds to a custom frequency offset.
    SubcarrierSpacingCommon = 15;
  end
  
  % Constant, hidden properties
  properties (Constant,Hidden)
      BlockPattern_Values = {'Case A', 'Case B', 'Case C', 'Case D', 'Case E'};
      DataSource_Values = {'MIB', 'PN9', 'PN9-ITU', 'PN11', 'PN15', 'PN23'}
  end
  
  methods
    function obj = nrWavegenSSBurstConfig(varargin)
      obj@nr5g.internal.ConfigBase(varargin{:});
    end
    
    function obj = set.BlockPattern(obj,val)
      prop = 'BlockPattern';
      val = validateEnumProperties(obj, prop, val);

      obj.(prop) = '';
      obj.(prop) = val;
    end
    
    function obj = set.TransmittedBlocks(obj,val)
      prop = 'TransmittedBlocks';
      temp = val;
      coder.varsize('temp',[1 64],[0 1]);
      validateattributes(temp,{'numeric'},{'binary','row'}, ...
          [class(obj) '.' prop], prop);

      % Lengths other than 4, 8, 64 error at edit time.
      % Invalid lengths for a given FR error at
      % nrDLCarrierConfig.validateConfig.
      coder.internal.errorIf(~any(length(temp)==[4 8 64]), ...
          'nr5g:nrWaveformGenerator:InvalidTxBlocks');

      obj.(prop) = temp;
    end
    
    function obj = set.Period(obj,val)
      prop = 'Period';
      validateattributes(val, {'numeric'},{'real','scalar', 'integer'},...
          [class(obj) '.' prop], prop);
      coder.internal.errorIf(~any(val == [5 10 20 40 80 160]),...
          'nr5g:nrWaveformGenerator:InvalidPeriod');
      obj.(prop) = val;
    end
    
    function obj = set.NCRBSSB(obj,val)
        prop = 'NCRBSSB';
        % To allow codegen for varying length in a single function script
        temp = val;
        coder.varsize('temp',[1 1],[1 1]);
        if ~(isnumeric(temp) && isempty(temp))
            validateattributes(temp,{'numeric'},{'scalar','integer','nonnegative','<=',2199},[class(obj) '.' prop],prop);
        end
        obj.(prop) = temp;
    end
    
    function obj = set.KSSB(obj,val)
        prop = 'KSSB';
        validateattributes(val, {'numeric'}, ...
            {'scalar','integer','nonnegative','<=',23}, ...
            [class(obj) '.' prop], prop);
        obj.(prop) = val;
    end
    
    function obj = set.DataSource(obj,val)
      prop = 'DataSource';
      if iscell(val)
        srcOptions = {'PN9-ITU', 'PN9', 'PN11', 'PN15', 'PN23'};
        validateattributes(val,{'cell'},{'numel', 2},[class(obj) '.' prop],prop);
        temp2 = validatestring(val{1}, srcOptions, [class(obj) '.' prop], prop);
        validateattributes(val{2},{'numeric'},{'scalar', 'integer', 'nonnegative'},[class(obj) '.' prop],prop);
        temp = {temp2 val{2}};
      elseif ischar(val) || isstring(val)
        srcOptions = {'MIB', 'PN9-ITU', 'PN9', 'PN11', 'PN15', 'PN23'};
        temp = validatestring(val, srcOptions, [class(obj) '.' prop], prop);
      else
        temp = val;
        validateattributes(temp,{'numeric'},{'vector', 'integer','binary', 'numel', 24},[class(obj) '.' prop],prop);
      end
      obj.(prop) = temp;
    end

    function obj = set.DMRSTypeAPosition(obj,val)
      prop = 'DMRSTypeAPosition';
      validateattributes(val, {'numeric'},...
          {'real','scalar'},...
          [class(obj) '.' prop], prop);
      coder.internal.errorIf(~any(val == [2 3]),...
          'nr5g:DMRSConfigBase:InvalidDMRSTypeAPosition',sprintf('%g',double(val)));
      obj.(prop) = val;
    end
    
    function obj = set.CellBarred(obj,val)
        prop = 'CellBarred';
        validateattributes(val, {'logical', 'numeric'}, {'scalar'}, ...
            [class(obj) '.' prop], prop);        
        obj.(prop) = logical(val);
    end
    
    function obj = set.IntraFreqReselection(obj,val)
        prop = 'IntraFreqReselection';
        validateattributes(val, {'logical', 'numeric'}, {'scalar'}, ...
            [class(obj) '.' prop], prop);        
        obj.(prop) = logical(val);
    end
    
    function obj = set.PDCCHConfigSIB1(obj,val)
        prop = 'PDCCHConfigSIB1';
        validateattributes(val, {'numeric'}, ...
            {'scalar','integer','nonnegative','<=',255}, ...
            [class(obj) '.' prop], prop);
        obj.(prop) = val;
    end
    
    function obj = set.SubcarrierSpacingCommon(obj,val)
      prop = 'SubcarrierSpacingCommon';
      validateattributes(val, {'numeric'}, ...
          {'real','scalar'}, ...
          [class(obj) '.' prop], prop);
      flag = ~any(val == [15 30 60 120]);
      coder.internal.errorIf(flag,'nr5g:nrWaveformGenerator:InvalidSCSSSB',sprintf('%g',double(val)));
      obj.(prop) = val;
    end
        
    function validateConfig(obj)
      fr1 = any(strcmp(obj.BlockPattern, {'Case A', 'Case B', 'Case C'}));
      
      %% BlockPattern & TransmittedBlocks
      coder.internal.errorIf(fr1 && ~any(length(obj.TransmittedBlocks)==[4 8]), ...
          'nr5g:nrWaveformGenerator:InvalidTxBlocks');
      coder.internal.errorIf(~fr1 && length(obj.TransmittedBlocks)~=64, ...
          'nr5g:nrWaveformGenerator:InvalidTxBlocks');
      
      %% BlockPattern & SubcarrierSpacingCommon
      SCSCommonActive = (~fr1 && ~isempty(obj.NCRBSSB)) || ~isInactiveProperty(obj, 'SubcarrierSpacingCommon');
      coder.internal.errorIf( SCSCommonActive && xor(fr1, obj.SubcarrierSpacingCommon<=30), ...
          'nr5g:nrWaveformGenerator:InvalidSCSSSB', obj.SubcarrierSpacingCommon);
      
      %% BlockPattern & KSSB
      if ~isInactiveProperty(obj, 'KSSB')
          % KSSB is validated only if it is active
          coder.internal.errorIf(~fr1 && obj.KSSB>11, ...
              'nr5g:nrWaveformGenerator:InvalidKSSB', obj.KSSB);
          coder.internal.errorIf(SCSCommonActive && obj.SubcarrierSpacingCommon~=30 && obj.KSSB>11, ...
              'nr5g:nrWaveformGenerator:InvalidKSSBFR1', obj.KSSB);
          coder.internal.errorIf(any(strcmpi(obj.BlockPattern, {'Case B', 'Case D'})) && mod(obj.KSSB, 2), ...
              'nr5g:nrWaveformGenerator:InvalidKSSBMultiple', obj.KSSB);
          coder.internal.errorIf(any(strcmpi(obj.BlockPattern, 'Case E')) && mod(obj.KSSB, 4), ...
              'nr5g:nrWaveformGenerator:InvalidKSSBMultiple', obj.KSSB);
      end
    end
  end
  
	methods (Access = protected)
     function flag = isInactiveProperty(obj, prop)
      % Controls the conditional display of properties

      flag = false;
      if any(strcmp(prop, {'DMRSTypeAPosition'; 'CellBarred'; 'IntraFreqReselection'; ...
                      'PDCCHConfigSIB1'; 'SubcarrierSpacingCommon'}))
        if iscell(obj.DataSource)
          flag = true;
        else
          flag = ~strcmpi(obj.DataSource, 'MIB');
        end
      end
      
      if strcmp(prop, 'KSSB')
        flag = isempty(obj.NCRBSSB);
      end
    end
    
    function groups = getPropertyGroups(obj)
      mainPropList = {'Enable'; 'Power'; 'BlockPattern'; 'TransmittedBlocks'; 'Period'; 'NCRBSSB'; 'KSSB'};
      mainPropVal  = {obj.Enable; obj.Power; obj.BlockPattern; obj.TransmittedBlocks; obj.Period; obj.NCRBSSB; obj.KSSB};
      mainPropActive = cellfun(@(x) ~isInactiveProperty(obj, x), mainPropList);
      fr2 = any(strcmpi(obj.BlockPattern, {'Case D', 'Case E'}));
      SCSCommonAtTop = fr2 && ~isempty(obj.NCRBSSB);
      if SCSCommonAtTop
        mainPropList = [mainPropList; 'SubcarrierSpacingCommon'];
        mainPropVal = [mainPropVal; obj.SubcarrierSpacingCommon];
        mainPropActive = [mainPropActive; true];
      end
      
      mibPropList = {'DataSource'; 'DMRSTypeAPosition'; 'CellBarred'; 'IntraFreqReselection'; ...
                      'PDCCHConfigSIB1'};
      mibPropVal  = {obj.DataSource; obj.DMRSTypeAPosition; obj.CellBarred; obj.IntraFreqReselection; ...
                      obj.PDCCHConfigSIB1};
      mibPropActive = cellfun(@(x) ~isInactiveProperty(obj, x), mibPropList);

      if ~SCSCommonAtTop
        mibPropList = [mibPropList; 'SubcarrierSpacingCommon'];
        mibPropVal = [mibPropVal; obj.SubcarrierSpacingCommon];
        % SubcarrierSpacingCommon shows up in Payload group only when
        % DataSource = MIB and for FR1
        mibPropActive = [mibPropActive; mibPropActive(end)&&~SCSCommonAtTop];
      end
      
      groups = [matlab.mixin.util.PropertyGroup(cell2struct(mainPropVal(mainPropActive), mainPropList(mainPropActive))) ...
                matlab.mixin.util.PropertyGroup(cell2struct(mibPropVal(mibPropActive), mibPropList(mibPropActive)), 'Payload:')];
    end
  end
end

